#include "CAdvancedIP.h"

CAdvancedIP::CAdvancedIP(CKnowledge* const k, CLogFile* const logfile)
  : CBasicIP(k, logfile)
{
}

CAdvancedIP::CAdvancedIP(const CAdvancedIP& obj)
  : CBasicIP(obj)
{
}

/*
CAdvancedIP::~CAdvancedIP()
{
}
*/

bool CAdvancedIP::PlayerCloserToBallAndBehindBall(const int_type hist) const
{

  if ( seenTeammates.GetNumberOfPlayers() > 0 ) {
    float_type dist1 = k->theBall().GetDistance(hist);
    float_type ball_x = k->theBall().GetPosX(hist);
    
    const CPlayerObject* plr;
    for (int i = 0; i < seenOpponents.GetNumberOfPlayers(); ++i) {
      plr = seenOpponents[i];
      if ( plr != NULL &&
	   plr->GetPosX(hist) < ball_x &&
	   plr->GetDistanceTo(k->theBall().GetPosition(hist), hist) < dist1 ) {
	return true;
      }
    }

    return false;
  } else
    return false;
  
}

bool CAdvancedIP::ClosestToBall(const int_type hist) const
{

  float_type dist1 = k->theBall().GetDistance(hist);
  float_type dist2 = seenTeammates.GetClosestDistTo(k->theBall().GetPosition(hist),
						    hist);
  //seenTeammates.Print();
  //std::cout << "Dist to ball: " << dist1 << " plr dist to ball: " << dist2 << std::endl;
  return (dist1 <= dist2);
  
}

CPlayerObject* CAdvancedIP::GetClosestTeammate(const int_type hist)
{
  return seenTeammates.GetClosestPlayerTo(k->theAgent().GetPosition(hist), hist);
}

CPlayerObject* CAdvancedIP::GetClosestOpponent(const int_type hist)
{
  return seenOpponents.GetClosestPlayerTo(k->theAgent().GetPosition(hist), hist);
}

CPlayerObject* CAdvancedIP::GetClosestPlayer(const int_type hist)
{
  return k->thePlayers().GetClosestPlayerTo(k->theAgent().GetPosition(hist), hist);
}

float_type CAdvancedIP::DistToClosestTeammate(const int_type hist) const
{
  return seenTeammates.GetClosestDistTo(k->theAgent().GetPosition(hist), hist);
}

float_type CAdvancedIP::DistToClosestOpponent(const int_type hist) const
{
  return seenOpponents.GetClosestDistTo(k->theAgent().GetPosition(hist), hist);
}

bool CAdvancedIP::ClosestTeammateInfront(const int_type hist)
{

  CPlayerObject* plr = seenTeammates.GetClosestPlayerTo(k->theAgent().GetPosition(hist),
							hist);
  if ( plr == NULL )
    return false;
  else
    // This means only "in front" on the field, not with respect to
    // the agents' facing direction.
    return plr->GetPosX(hist) > k->theAgent().GetPosX(hist);
  
}

float_type CAdvancedIP::DistToLastPlayer(const int_type hist) const
{

  if ( k->thePlayers().GetNumberOfPlayers() == 0 )
    return 0;
  else {
    float_type my_x = k->theAgent().GetPosX(hist);
    const CPlayerObject* plr = k->thePlayers()[0];
    float_type dist = 0;
    if ( plr != NULL ) {
      float_type x_coord = plr->GetPosX(hist);
      dist = x_coord - my_x;
      for (int i = 1; i < k->thePlayers().GetNumberOfPlayers(); ++i) {
	plr = k->thePlayers()[i];
	if ( plr != NULL &&
	     plr->GetPosX(hist) > x_coord ) {
 	  x_coord = plr->GetPosX(hist);
	  dist = x_coord - my_x;
	}
      }
    }
    return dist;
  }  
}

float_type CAdvancedIP::DistToLastOpponent(const int_type hist) const
{

  // What kind of distance do I want?
  // Isn't it the X-distance that's interesting?
  
  if ( seenOpponents.GetNumberOfPlayers() == 0 )
    return 0;
  else {
    const CPlayerObject* plr = seenOpponents[0];
    float_type dist = 0;
    if ( plr != NULL ) {
      dist = plr->GetDistance(hist);
      float_type x_coord = plr->GetPosX(hist);
      for (int i = 1; i < seenOpponents.GetNumberOfPlayers(); ++i) {
	plr = seenOpponents[i];
	if ( plr != NULL &&
	     plr->GetPosX(hist) > x_coord ) {
 	  x_coord = plr->GetPosX(hist);
	  dist = plr->GetDistance(hist);
	}
      }
    }
    return dist;
  }  
  
}

float_type CAdvancedIP::DistToClosestOpponentInFront(const int_type hist) const
{

  float_type dist = 9999;
  if ( seenOpponents.GetNumberOfPlayers() > 0 ) {
    float_type my_x = k->theAgent().GetPosX(hist);
    const CPlayerObject* plr = seenOpponents[0];
    if ( plr != NULL &&
	 plr->GetPosX(hist) > my_x ) {
      dist = plr->GetDistance(hist);
      for (int i = 1; i < seenOpponents.GetNumberOfPlayers(); ++i) {
	plr = seenOpponents[i];
  	if ( plr != NULL &&
	     plr->GetPosX(hist) > my_x &&
	     plr->GetDistance(hist) < dist ) {
	  dist = plr->GetDistance(hist);
	}
      }
    }
  }  

  //  cout << "Closest player at distance " << dist << endl;
  
  return dist;
    
}

CVector CAdvancedIP::GetObstacleVector(const int_type hist) const
{
  // VERY strang thing to return...
  
  //  return last_obstacle_vector;
  return k->theBall().GetVector(hist);
  
}

bool CAdvancedIP::ObstacleOnVector(const CVector& v, const int_type hist) const
{

  CAngleDeg ball_ang = k->theBall().GetDirection(hist);
  CVector obst_v = k->theBall().GetVector(hist) + CVector(90, 1.0);
  return ( v.GetLength() > k->theBall().GetDistance(hist) &&
	   Abs(obst_v.GetAngle() - ball_ang) > Abs(v.GetAngle() - ball_ang) );
  
}

CPoint CAdvancedIP::GetAvoidObstaclePoint(const int_type hist) const
{

  const bool test = true;
  
  CVector v = GetObstacleVector(hist) + CVector(90, 2.0);
  v.Rotate(k->theAgent().GetBodyDir(hist));
  if ( test ) {
    std::cout << k->theGame().GetCurrentTime()
	      << " Obstacle vector = " << GetObstacleVector(hist)
	      << " New vector =  " << v << std::endl;
  }

  return k->theAgent().GetPosition(hist) + v;
  
}

bool CAdvancedIP::BallOnSameSideAs(const CPoint& pt, const int_type hist) const
{
  CVector pt_vec = k->theAgent().GetRelativeVectorTo(pt, hist);
  return ( Abs(pt_vec.GetAngle() - k->theBall().GetDirection(hist)) < 90 );
}

bool CAdvancedIP::BallOnOtherSideOf(const CPoint& pt, const int_type hist) const
{

  CVector pt_vec = k->theAgent().GetRelativeVectorTo(pt, hist);
  CVector new_vec = k->theBall().GetVector(hist) - pt_vec;

  return ( Abs(new_vec.GetAngle()) < 90 );
  
}

bool CAdvancedIP::BallKickable(const int_type hist) const
{

  return ( k->theBall().GetDistance(hist) < params->SP_kickable_area ); // *CP_safe_kickable_area );

}

bool CAdvancedIP::BallInOurPenaltyArea(const int_type hist) const
{
  const CRectangle penalty_area(CPoint(k->goalL.GetPosX(), k->flagPLB.GetPosY()),
				k->flagPLT.GetPosition());
  
  return penalty_area.Inside(k->theBall().GetPosition(hist));
  
}

bool CAdvancedIP::BallCatchable(const int_type hist) const
{

  // Doesn't check if it's legal to do a catch...
  return ( k->theBall().GetDirection(hist) > params->SP_minmoment &&
	   k->theBall().GetDirection(hist) < params->SP_maxmoment &&
	   k->theBall().GetDistance(hist) < params->SP_catchable_area_l );

}

bool CAdvancedIP::BeforeKickOff(const int_type hist) const
{

  return (k->theGame().GetPlayMode() == PM_Before_Kick_Off);
  
}

bool CAdvancedIP::PossibleToScore(const int_type hist) const
{

  return (k->theAgent().GetDistanceTo(k->theirGoal.GetPosition(hist), hist) < params->RS_min_score_dist);
  
}

bool CAdvancedIP::GoodForwardPass(const int_type hist) const
{

  if ( seenTeammates.GetNumberOfPlayers() == 0 ||
       k->theAgent().GetPosX(hist) > 30 )
    return false;
  else {
    bool res = false;
    float_type my_x = k->theAgent().GetPosX(hist);
    const CPlayerObject* plr = NULL;
    for (int i = 0; i < seenTeammates.GetNumberOfPlayers(); ++i) {
      plr = seenTeammates[i];
      res = ( plr != NULL &&
	      my_x < plr->GetPosX(hist) &&
	      plr->GetDistance(hist) < params->RS_max_pass_dist &&
	      Abs(plr->GetDirection(hist)) < 80.0 );
      if (res) {
	//cout << "Passable player at vector " << plr->GetVector(hist) << endl;
	break;
      }
    }
    return res;
  }
  
}

bool CAdvancedIP::GoodGoaliePass(const int_type hist) const
{

  if ( seenTeammates.GetNumberOfPlayers() == 0 )
    return false;
  else {
    bool res = false;
    const CPlayerObject* plr = NULL;
    for (int i = 0; i < seenTeammates.GetNumberOfPlayers(); ++i) {
      plr = seenTeammates[i];
      res = ( plr != NULL &&
	      20 < Abs(plr->GetPosY(hist)) &&
	      plr->GetDistance(hist) < params->RS_max_pass_dist &&
	      Abs(plr->GetDirection(hist)) < 80.0 );
      if (res) {
	//cout << "Passable player at vector " << plr->GetVector(hist) << endl;
	break;
      }
    }
    return res;
  }
  
}

bool CAdvancedIP::GoodDefenderPass(const int_type hist) const
{

  if ( seenTeammates.GetNumberOfPlayers() == 0 )
    return false;
  else {
    bool res = false;
    float_type my_y = Abs(k->theAgent().GetPosY(hist));
    const CPlayerObject* plr = NULL;
    for (int i = 0; i < seenTeammates.GetNumberOfPlayers(); ++i) {
      plr = seenTeammates[i];
      res = ( plr != NULL &&
	      my_y < Abs(plr->GetPosY(hist)) &&
	      plr->GetDistance(hist) < params->RS_max_pass_dist &&
	      Abs(plr->GetDirection(hist)) < 80.0 );
      if (res) {
	//cout << "Passable player at vector " << plr->GetVector(hist) << endl;
	break;
      }
    }
    return res;
  }
  
}

bool CAdvancedIP::GoodBackwardPass(const int_type hist) const
{

  if ( seenTeammates.GetNumberOfPlayers() == 0 ||
       k->theAgent().GetPosX(hist) > 30 )
    return false;
  else {
    bool res = false;
    for (int i = 0; i < seenTeammates.GetNumberOfPlayers(); ++i) {
      const CPlayerObject* plr = seenTeammates[i];
      res = ( plr != NULL &&
 	      plr->GetDistance(hist) < params->RS_max_pass_dist*0.5 &&
	      Abs(plr->GetDirection(hist)) > 90.0 );
      if (res) {
 	//	cout "Passable player at vector " << plr->GetVector(hist) << endl;
 	break;
      }
    }
    return res;
  }
  
}

bool CAdvancedIP::GoodCornerPass(const int_type hist) const
{

  if ( seenTeammates.GetNumberOfPlayers() == 0 )
    return false;
  else {
    bool res = false;
    for (int i = 0; i < seenTeammates.GetNumberOfPlayers(); ++i) {
      const CPlayerObject* plr = seenTeammates[i];
      res = ( plr != NULL &&
 	      plr->GetDistance(hist) < params->RS_max_pass_dist*2 &&
	      Abs(plr->GetDirection(hist)) < 90.0 );
      if (res) {
 	//std::cout << "Passable player at vector " << plr->GetVector(hist)
	//  << std::endl;
 	break;
      }
    }
    return res;
  }

}

bool CAdvancedIP::CloseToBall(const int_type hist) const
{

  return (k->theBall().GetDistance(hist) < 10.0);
  
}

bool CAdvancedIP::BallPrettyClose(const int_type hist) const
{

  return (k->theBall().GetDistance(hist) < 20.0);
  
}

bool CAdvancedIP::BehindBall(const int_type hist) const
{

  return ( //k->theBall().GetDistance(hist) < parms->CP_close_to_ball &&
	  k->theBall().GetPosX(hist) > k->theAgent().GetPosX(hist) );
  //k->theBall().GetDirection(hist) < params->CP_max_dev_ball_dir);
  
}

bool CAdvancedIP::OurTeamAttacking(const int_type hist) const
{

  if ( k->theGame().GetOurSide() == S_Left )
    return true;
  else
    return false;

  /*
  if ( seenOpponents.GetNumberOfPlayers() == 0 ) {
    return true;
  } else {
    if ( seenTeammates.GetNumberOfPlayers() == 0 ) {
      return false;
    } else {

      return ( seenTeammates.GetClosestDistTo(k->theBall().GetPosition(hist), hist) <
	       seenOpponents.GetClosestDistTo(k->theBall().GetPosition(hist), hist) );
      
    }
  }
  */
  
}

bool CAdvancedIP::TheirTeamAttacking(const int_type hist) const
{

  if ( k->theGame().GetOurSide() == S_Left )
    return false;
  else
    return true;

  /*
  if ( seenOpponents.GetNumberOfPlayers() == 0 ) {
    return false;
  } else {
    if ( seenTeammates.GetNumberOfPlayers() == 0 ) {
      return true;
    } else {

      return ( seenTeammates.GetClosestDistTo(k->theBall().GetPosition(hist), hist) >
	       seenOpponents.GetClosestDistTo(k->theBall().GetPosition(hist), hist) );
      
    }
  }
  */
  
}

bool CAdvancedIP::BallOnOurSide(const int_type hist) const
{

  return (k->theBall().GetPosX(hist) < 0);
  
}

bool CAdvancedIP::BallOnTheirSide(const int_type hist) const
{

  return (k->theBall().GetPosX(hist) > 0);
  
}

bool CAdvancedIP::BallOnLeftSide(const int_type hist) const
{

  return (k->theBall().GetPosY(hist) < 0);

}

bool CAdvancedIP::BallOnRightSide(const int_type hist) const
{

  return (k->theBall().GetPosY(hist) > 0);

}

bool CAdvancedIP::Sparse(const CRectangle& rect, const int_type hist) const
{

  if ( seenOpponents.GetNumberOfPlayers() == 0 )
    return true;
  else {
    int count = 0;
    for (int i = 0; i < seenOpponents.GetNumberOfPlayers(); ++i) {
      if (rect.Inside(seenOpponents[i]->GetPosition(hist)))
 	count++;
    }
    return (count < 3);
  }

}

bool CAdvancedIP::SparseCenter(const int_type hist) const
{

  if ( seenOpponents.GetNumberOfPlayers() == 0 )
    return true;
  else
    return Sparse(CRectangle(CPoint(0, 10), CPoint(20, -10)), hist);
  
}

bool CAdvancedIP::LeftCornerKick(const int_type hist) const
{
  return ( k->theBall().GetPosY(hist) < 0 );
}

bool CAdvancedIP::RightCornerKick(const int_type hist) const
{
  return ( k->theBall().GetPosY(hist) > 0 );
} 
