#include "CAgentObject.h"

const float_type CAgentObject::DEFAULT_STAMINA = 3500.0;
const float_type CAgentObject::DEFAULT_EFFORT = 1.0;
const float_type CAgentObject::DEFAULT_RECOVER = 1.0;
const t_view_width CAgentObject::DEFAULT_VIEW_WIDTH = VW_Normal;
const t_view_quality CAgentObject::DEFAULT_VIEW_QUALITY = VQ_High;

const bool CAgentObject::DEFAULT_GOALIE = false;
const int_type CAgentObject::DEFAULT_HEAR_CAPACITY_FROM_TEAMMATE = 2;
const int_type CAgentObject::DEFAULT_HEAR_CAPACITY_FROM_OPPONENT = 2;
const int_type CAgentObject::DEFAULT_CYCLES_SINCE_CATCH = 0;
const int_type CAgentObject::DEFAULT_CYCLES_SINCE_SAY = 0;
const t_sense CAgentObject::DEFAULT_LAST_SENSE_TYPE = SENSE_Sense; // ?
const CTime CAgentObject::DEFAULT_TIME = UNKNOWN_TIME;
/*
  const CTime CAgentObject::DEFAULT_LAST_SENSE_TIME = UNKNOWN_TIME;
  const CTime CAgentObject::DEFAULT_LAST_SIGHT_TIME = UNKNOWN_TIME;
  const CTime CAgentObject::DEFAULT_LAST_HEAR_TIME = UNKNOWN_TIME;
  const CTime CAgentObject::DEFAULT_LAST_HEAR_TEAMMATE_TIME = UNKNOWN_TIME;
  const CTime CAgentObject::DEFAULT_LAST_HEAR_OPPONENT_TIME = UNKNOWN_TIME;
  const CTime CAgentObject::DEFAULT_LAST_HEAR_REFEREE_TIME = UNKNOWN_TIME;
  const CTime CAgentObject::DEFAULT_LAST_ACT_TIME = UNKNOWN_TIME;
  const CTime CAgentObject::DEFAULT_LAST_NECK_MOVEMENT = UNKNOWN_TIME;
  */
const CCommandQueue CAgentObject::DEFAULT_COMMAND_QUEUE = CCommandQueue();
const CTurnNeckCommand CAgentObject::DEFAULT_NEXT_TURN_NECK = CTurnNeckCommand(NULL);
const bool CAgentObject::DEFAULT_DO_TURN_NECK = false;
const CMessages CAgentObject::DEFAULT_MESSAGES = CMessages();

CAgentObject::CAgentObject(const CParameters* const p, const int_type hist_length)
  : CPlayerObject(p, hist_length),
    stamina(max_length),
    effort(max_length),
    recover(max_length),
    dashes(max_length),
    kicks(max_length),
    turns(max_length),
    says(max_length),
    turn_necks(max_length),
    view_width(max_length),
    view_quality(max_length),
    goalie(DEFAULT_GOALIE),
    hear_capacity_from_teammate(DEFAULT_HEAR_CAPACITY_FROM_TEAMMATE),
    hear_capacity_from_opponent(DEFAULT_HEAR_CAPACITY_FROM_OPPONENT),
    cycles_since_catch(DEFAULT_CYCLES_SINCE_CATCH),
    cycles_since_say(DEFAULT_CYCLES_SINCE_SAY),
    last_sense_time(DEFAULT_TIME), //LAST_SENSE_TIME),
    last_sight_time(DEFAULT_TIME), //LAST_SIGHT_TIME),
    last_hear_time(DEFAULT_TIME), //LAST_HEAR_TIME),
    last_hear_teammate_time(DEFAULT_TIME), //LAST_HEAR_TEAMMATE_TIME),
    last_hear_opponent_time(DEFAULT_TIME), //LAST_HEAR_OPPONENT_TIME),
    last_hear_referee_time(DEFAULT_TIME), //LAST_HEAR_REFEREE_TIME),
    last_act_time(DEFAULT_TIME), //LAST_ACT_TIME),
    last_neck_movement(DEFAULT_TIME), //LAST_NECK_MOVEMENT),
    last_commands(DEFAULT_COMMAND_QUEUE),
    last_executed_commands(DEFAULT_COMMAND_QUEUE),
    next_turn_neck(DEFAULT_NEXT_TURN_NECK),
    do_turn_neck(DEFAULT_DO_TURN_NECK),
    sent_messages(DEFAULT_MESSAGES),
    received_messages(DEFAULT_MESSAGES),
    senses(0),
    sees(0)
{
  
  for (int_type i=0; i < max_length; ++i) {
    Reset(i);
  }
  
}

CAgentObject::CAgentObject(const CAgentObject& obj)
  : CPlayerObject(obj),
    stamina      (obj.stamina),
    effort       (obj.effort),
    recover      (obj.recover),
    dashes       (obj.dashes),
    kicks        (obj.kicks),
    turns        (obj.turns),
    says         (obj.says),
    turn_necks   (obj.turn_necks),
    view_width   (obj.view_width),
    view_quality (obj.view_quality),
    goalie                (obj.goalie),
    hear_capacity_from_teammate(obj.hear_capacity_from_teammate),
    hear_capacity_from_opponent(obj.hear_capacity_from_opponent),
    cycles_since_catch     (obj.cycles_since_catch),
    cycles_since_say       (obj.cycles_since_say),
    last_sense_type        (obj.last_sense_type),
    last_sense_time        (obj.last_sense_time),
    last_sight_time        (obj.last_sight_time),
    last_hear_time         (obj.last_hear_time),
    last_hear_teammate_time(obj.last_hear_teammate_time),
    last_hear_opponent_time(obj.last_hear_opponent_time),
    last_hear_referee_time (obj.last_hear_referee_time),
    last_act_time          (obj.last_act_time),
    last_neck_movement     (obj.last_neck_movement),
    last_commands          (obj.last_commands),
    last_executed_commands (obj.last_executed_commands),
    next_turn_neck         (obj.next_turn_neck),
    do_turn_neck           (obj.do_turn_neck),
    sent_messages          (obj.sent_messages),
    received_messages      (obj.received_messages),
    senses                 (obj.senses),
    sees                   (obj.sees)
{
}

CAgentObject::~CAgentObject()
{
}

void CAgentObject::CopyData(const CAgentObject& obj)
{
  CPlayerObject::CopyData(obj);
  stamina                 = obj.stamina;
  effort                  = obj.effort;
  recover                 = obj.recover;
  dashes                  = obj.dashes;
  kicks                   = obj.kicks;
  turns                   = obj.turns;
  says                    = obj.says;
  turn_necks              = obj.turn_necks;
  view_width              = obj.view_width;
  view_quality            = obj.view_quality;
  goalie                  = obj.goalie;
  hear_capacity_from_teammate = obj.hear_capacity_from_teammate;
  hear_capacity_from_opponent = obj.hear_capacity_from_opponent;
  cycles_since_catch      = obj.cycles_since_catch;
  cycles_since_say        = obj.cycles_since_say;
  last_sense_type         = obj.last_sense_type;
  last_sense_time         = obj.last_sense_time;
  last_sight_time         = obj.last_sight_time;
  last_hear_time          = obj.last_hear_time;
  last_hear_teammate_time = obj.last_hear_teammate_time;
  last_hear_opponent_time = obj.last_hear_opponent_time;
  last_hear_referee_time  = obj.last_hear_referee_time;
  last_act_time           = obj.last_act_time;
  last_neck_movement      = obj.last_neck_movement;
  last_commands           = obj.last_commands;
  last_executed_commands  = obj.last_executed_commands;
  next_turn_neck          = obj.next_turn_neck;
  do_turn_neck            = obj.do_turn_neck;
  sent_messages           = obj.sent_messages;
  received_messages       = obj.received_messages;
  senses                  = obj.senses;
  sees                    = obj.sees;
}

CAgentObject& CAgentObject::operator= (const CAgentObject& obj)
{

  if (this != &obj) {
    CopyData(obj);
  }

  return *this;
  
}

void CAgentObject::Copy(const int_type from, const int_type to)
{
  CPlayerObject::Copy(from, to);
  stamina[GetIndex(to)]      = stamina[GetIndex(from)];
  effort[GetIndex(to)]       = effort[GetIndex(from)];
  recover[GetIndex(to)]      = recover[GetIndex(from)];
  dashes[GetIndex(to)]       = dashes[GetIndex(from)];
  kicks[GetIndex(to)]        = kicks[GetIndex(from)];
  turns[GetIndex(to)]        = turns[GetIndex(from)];
  says[GetIndex(to)]         = says[GetIndex(from)];
  turn_necks[GetIndex(to)]   = turn_necks[GetIndex(from)];
  view_width[GetIndex(to)]   = view_width[GetIndex(from)];
  view_quality[GetIndex(to)] = view_quality[GetIndex(from)];
}

void CAgentObject::ResetAll()
{
  Reset();

  for (int_type i=0; i < max_length; ++i)
    Reset(i);

}

void CAgentObject::Reset()
{

  CPlayerObject::Reset();

  goalie = DEFAULT_GOALIE;
  hear_capacity_from_teammate = DEFAULT_HEAR_CAPACITY_FROM_TEAMMATE;
  hear_capacity_from_opponent = DEFAULT_HEAR_CAPACITY_FROM_OPPONENT;
  cycles_since_catch = DEFAULT_CYCLES_SINCE_CATCH;
  cycles_since_say = DEFAULT_CYCLES_SINCE_SAY;
  last_sense_time = DEFAULT_TIME; //LAST_SENSE_TIME;
  last_sight_time = DEFAULT_TIME; //LAST_SIGHT_TIME;
  last_hear_time = DEFAULT_TIME; //LAST_HEAR_TIME;
  last_hear_teammate_time = DEFAULT_TIME; //LAST_HEAR_TEAMMATE_TIME;
  last_hear_opponent_time = DEFAULT_TIME; //LAST_HEAR_OPPONENT_TIME;
  last_hear_referee_time = DEFAULT_TIME; //LAST_HEAR_REFEREE_TIME;
  last_act_time = DEFAULT_TIME; //LAST_ACT_TIME;
  last_neck_movement = DEFAULT_TIME; //LAST_NECK_MOVEMENT;
  last_commands = DEFAULT_COMMAND_QUEUE;
  last_executed_commands = DEFAULT_COMMAND_QUEUE;
  next_turn_neck = DEFAULT_NEXT_TURN_NECK;
  do_turn_neck = DEFAULT_DO_TURN_NECK;
  sent_messages = DEFAULT_MESSAGES;
  received_messages = DEFAULT_MESSAGES;
  senses = 0;
  sees = 0;

}

void CAgentObject::Reset(const int_type i)
{

  CPlayerObject::Reset(i);

  stamina[GetIndex(i)] = DEFAULT_STAMINA;
  effort[GetIndex(i)]  = DEFAULT_EFFORT;
  recover[GetIndex(i)] = DEFAULT_RECOVER;
  dashes[GetIndex(i)]       = 0;
  kicks[GetIndex(i)]        = 0;
  turns[GetIndex(i)]        = 0;
  says[GetIndex(i)]         = 0;
  turn_necks[GetIndex(i)]   = 0;
  view_width[GetIndex(i)]   = DEFAULT_VIEW_WIDTH;
  view_quality[GetIndex(i)] = DEFAULT_VIEW_QUALITY;

}

void CAgentObject::NewCycle()
{

  //  std::cout << "Turns before: " << GetTurns(PREV) << " "
  //    << GetTurns(NOW) << " " << GetTurns(NEXT) << std::endl;
  CPlayerObject::NewCycle();
  Copy(PREV, NOW);
  senses = 0;
  sees = 0;
  // std::cout << "Turns after: " << GetTurns(PREV) << " "
  //    << GetTurns(NOW) << " " << GetTurns(NEXT) << std::endl;

}

void CAgentObject::UpdateStamina(const int_type from, const int_type to)
{
#ifndef _REQUIRE_PARAMETERS_IN_OBJECTS_
  if ( !have_parameters ) {
    return;
  }
#endif
    
  // If the stamina is greater than the threshold for
  // increasing the effort, increase it.
  if (stamina[GetIndex(from)] <= (params->SP_stamina_max * params->SP_effort_dec_thr)) {
    effort[GetIndex(to)] = Max(effort[GetIndex(from)] - params->SP_effort_dec,
			       params->SP_effort_min);
    // If it's below the threshold for decreasing it, decrease it.
  } else if (stamina[GetIndex(from)] >= (params->SP_stamina_max * params->SP_effort_inc_thr)) {
    effort[GetIndex(to)] = Min(effort[GetIndex(from)] + params->SP_effort_inc,
			       params->SP_effort_max);
  } else {
    // Otherwise nothing.
    effort[GetIndex(to)] = effort[GetIndex(from)];
  }
  
  // If the new stamina is less than the threshold for
  // decreasing the recover, do it.
  // Since it never increases no check for that is needed.
  if (stamina[GetIndex(from)] <= (params->SP_stamina_max * params->SP_recover_dec_thr)) {
    recover[GetIndex(to)] = Max(recover[GetIndex(from)] - params->SP_recover_dec,
				params->SP_recover_min);
  }
  
  stamina[GetIndex(to)] = Min(params->SP_stamina_max,
			      stamina[GetIndex(from)] + recover[GetIndex(to)] * params->SP_stamina_inc_max);
  // Since stamina never should be negative it can't be less than zero
  // before or after the previous statement.
}

void CAgentObject::UpdateAuditoryInfo()
{
#ifndef _REQUIRE_PARAMETERS_IN_OBJECTS_
  if ( !have_parameters ) {
    return;
  }
#endif
    
  cycles_since_say += 1;
  hear_capacity_from_teammate = Min(hear_capacity_from_teammate + params->SP_hear_inc,
				    params->SP_hear_max);
  hear_capacity_from_opponent = Min(hear_capacity_from_opponent + params->SP_hear_inc,
				    params->SP_hear_max);
}

void CAgentObject::UpdateCatchInfo()
{
  cycles_since_catch += 1;
}

CAngleDeg CAgentObject::GetViewAngle(const int_type hist) const
{
  switch(view_width[GetIndex(hist)]) {
  case VW_Narrow:
    return CAngleDeg(45);
  case VW_Normal:
    return CAngleDeg(90);
  case VW_Wide:
    return CAngleDeg(180);
  default:
    // Should be ERROR!
    return CAngleDeg(0);
  }
}

void CAgentObject::SetLastCommand(const CCommand* const new_last_cmd)
{
  last_commands.Push(new_last_cmd);
}

void CAgentObject::SetLastExecutedCommand(const CCommand* const new_last_exec_cmd)
{
  last_executed_commands.Push(new_last_exec_cmd);
};

void CAgentObject::Print(const int_type hist) const
{

  if (hist == 9999) {
    for (int_type i = -max_hist_length; i < max_length - max_hist_length; ++i) {
      CPlayerObject::Print(i);
      std::cout << "[" << i << "]"
		<< " Stamina: " << GetStamina(i)
		<< " Effort: " << GetEffort(i)
		<< " Recover: " << GetRecover(i)
		<< " Dashes: " << GetDashes(i)
		<< " Kicks: " << GetKicks(i)
		<< " Turns: " << GetTurns(i)
		<< " TurnNecks: " << GetTurnNecks(i)
		<< " Says: " << GetSays(i)
		<< std::endl;
    }
  } else {
    CPlayerObject::Print(hist);
    std::cout << "[" << hist << "]"
	      << " Stamina: " << GetStamina(hist)
	      << " Effort: " << GetEffort(hist)
	      << " Recover: " << GetRecover(hist)
	      << " Dashes: " << GetDashes(hist)
	      << " Kicks: " << GetKicks(hist)
	      << " Turns: " << GetTurns(hist)
	      << " TurnNecks: " << GetTurnNecks(hist)
	      << " Says: " << GetSays(hist)
	      << std::endl;
  }
  
}
