#ifndef _CAGENTOBJECT_H_
#define _CAGENTOBJECT_H_

#include "types.h"
#include "CPlayerObject.h"
#include "CCommandQueue.h"
#include "CTurnNeckCommand.h"
#include "CMessages.h"

class CAgentObject : public CPlayerObject
{
public:
  CAgentObject(const CParameters* const p,
	       const int_type hist_length = DEFAULT_HIST_LENGTH);
  CAgentObject(const CAgentObject& obj);
  ~CAgentObject();

  CAgentObject& operator= (const CAgentObject& obj);

  void Copy(const int_type from, const int_type to);

  void ResetAll();
  void Reset();
  void Reset(const int_type i);
  
  void NewCycle();

  void UpdateStamina(const int_type from, const int_type to);
  void UpdateAuditoryInfo();
  void UpdateCatchInfo();
  
  inline void SetGoalie(const bool g);
  bool IsGoalie() const;
  
  void SetStamina(const float_type new_stamina, const int_type hist = NEXT);
  float_type GetStamina(const int_type hist = NOW) const;

  void SetEffort(const float_type new_effort, const int_type hist = NEXT);
  float_type GetEffort(const int_type hist = NOW) const;

  void SetRecover(const float_type new_recover, const int_type hist = NEXT);
  float_type GetRecover(const int_type hist = NOW) const;

  void SetDashes(const int_type new_dashes, const int_type hist = NEXT);
  int_type GetDashes(const int_type hist = NOW) const;

  void SetKicks(const int_type new_kicks, const int_type hist = NEXT);
  int_type GetKicks(const int_type hist = NOW) const;

  void SetSays(const int_type new_says, const int_type hist = NEXT);
  int_type GetSays(const int_type hist = NOW) const;

  void SetTurns(const int_type new_turns, const int_type hist = NEXT);
  int_type GetTurns(const int_type hist = NOW) const;

  void SetTurnNecks(const int_type new_turn_necks, const int_type hist = NEXT);
  int_type GetTurnNecks(const int_type hist = NOW) const;

  void SetHearCapacityFromTeammate(const int_type new_hear_capacity);
  int_type GetHearCapacityFromTeammate() const;

  void SetHearCapacityFromOpponent(const int_type new_hear_capacity);
  int_type GetHearCapacityFromOpponent() const;

  void SetCyclesSinceCatch(const int_type new_cycs_since_catch);
  int_type GetCyclesSinceCatch() const;

  void SetCyclesSinceSay(const int_type new_cycs_since_say);
  int_type GetCyclesSinceSay() const;

  void SetViewWidth(const t_view_width new_view_width, const int_type hist = NEXT);
  t_view_width GetViewWidth(const int_type hist = NOW) const;
  CAngleDeg GetViewAngle(const int_type hist = NOW) const;
  
  void SetViewQuality(const t_view_quality new_view_quality, const int_type hist = NEXT);
  t_view_quality GetViewQuality(const int_type hist = NOW) const;

  void SetLastSenseType(const t_sense new_last_sense_type);
  t_sense GetLastSenseType() const;
  
  void SetLastSenseTime(const CTime& new_last_sense_time);
  CTime GetLastSenseTime() const;
  
  void SetLastSightTime(const CTime& new_last_sight_time);
  CTime GetLastSightTime() const;

  void SetLastHearTime(const CTime& new_last_hear_time);
  CTime GetLastHearTime() const;
  
  void SetLastHearTeammateTime(const CTime& new_last_hear_time);
  CTime GetLastHearTeammateTime() const;
  
  void SetLastHearOpponentTime(const CTime& new_last_hear_time);
  CTime GetLastHearOpponentTime() const;
  
  void SetLastHearRefereeTime(const CTime& new_last_hear_time);
  CTime GetLastHearRefereeTime() const;
  
  void SetLastActTime(const CTime& new_last_act_time);
  CTime GetLastActTime() const;

  void SetLastCommand(const CCommand* const new_last_command);
  const CCommand* const GetLastCommand() const;
  void RemoveLastCommand();
  const CCommandQueue& GetLastCommandQueue() const;
  CCommandQueue& GetLastCommandQueue();
  
  void SetLastExecutedCommand(const CCommand* const new_last_exec_cmd);
  const CCommand* const GetLastExecutedCommand() const;

  void SetNextTurnNeck(const CTurnNeckCommand& new_turn_neck);
  CTurnNeckCommand GetNextTurnNeck() const;

  void SetDoTurnNeck(const bool);
  bool DoTurnNeck() const;

  const CMessages& SentMessages() const;
  CMessages& SentMessages();

  const CMessages& ReceivedMessages() const;
  CMessages& ReceivedMessages();

  int GetSenses() const;
  void SetSenses(int new_senses);
  
  int GetSees() const;
  void SetSees(int new_sees);
  
  void Print(const int_type hist = UNKNOWN_INT_TYPE) const;
  
protected:
  std::vector<float_type> stamina;
  std::vector<float_type> effort;
  std::vector<float_type> recover;
  std::vector<int_type> dashes;
  std::vector<int_type> kicks;
  std::vector<int_type> turns;
  std::vector<int_type> says;
  std::vector<int_type> turn_necks;
  std::vector<t_view_width> view_width;
  std::vector<t_view_quality> view_quality;

  bool goalie;
  int_type hear_capacity_from_teammate;
  int_type hear_capacity_from_opponent;
  int_type cycles_since_catch;
  int_type cycles_since_say;
  t_sense last_sense_type;
  CTime last_sense_time;
  CTime last_sight_time;
  CTime last_hear_time;
  CTime last_hear_teammate_time;
  CTime last_hear_opponent_time;
  CTime last_hear_referee_time;
  CTime last_act_time;
  CTime last_neck_movement;
  CCommandQueue last_commands;
  CCommandQueue last_executed_commands;
  CTurnNeckCommand next_turn_neck;
  bool do_turn_neck;

  CMessages sent_messages;
  CMessages received_messages;

  int senses;
  int sees;
  
  static const float_type DEFAULT_STAMINA;
  static const float_type DEFAULT_EFFORT;
  static const float_type DEFAULT_RECOVER;
  static const t_view_width DEFAULT_VIEW_WIDTH;
  static const t_view_quality DEFAULT_VIEW_QUALITY;

  static const bool DEFAULT_GOALIE;
  static const int_type DEFAULT_HEAR_CAPACITY_FROM_TEAMMATE;
  static const int_type DEFAULT_HEAR_CAPACITY_FROM_OPPONENT;
  static const int_type DEFAULT_CYCLES_SINCE_CATCH;
  static const int_type DEFAULT_CYCLES_SINCE_SAY;
  static const t_sense DEFAULT_LAST_SENSE_TYPE;
  static const CTime DEFAULT_TIME;
  /*
  static const CTime DEFAULT_LAST_SENSE_TIME;
  static const CTime DEFAULT_LAST_SIGHT_TIME;
  static const CTime DEFAULT_LAST_HEAR_TIME;
  static const CTime DEFAULT_LAST_HEAR_TEAMMATE_TIME;
  static const CTime DEFAULT_LAST_HEAR_OPPONENT_TIME;
  static const CTime DEFAULT_LAST_HEAR_REFEREE_TIME;
  static const CTime DEFAULT_LAST_ACT_TIME;
  static const CTime DEFAULT_LAST_NECK_MOVEMENT;
  */
  static const CCommandQueue DEFAULT_COMMAND_QUEUE;
  static const CTurnNeckCommand DEFAULT_NEXT_TURN_NECK;
  static const bool DEFAULT_DO_TURN_NECK;
  static const CMessages DEFAULT_MESSAGES;
  
  void CopyData(const CAgentObject&);
  
};

inline void CAgentObject::SetGoalie(const bool g)
{
  goalie = g;
}

inline bool CAgentObject::IsGoalie() const
{
  return goalie;
}

inline void CAgentObject::SetStamina(const float_type new_stamina, const int_type hist)
{
#ifndef _REQUIRE_PARAMETERS_IN_OBJECTS_
  if ( have_parameters ) {
#endif
    stamina[GetIndex(hist)] = MinMax(params->SP_stamina_min, new_stamina,
				     params->SP_stamina_max);
#ifndef _REQUIRE_PARAMETERS_IN_OBJECTS_
  } else {
    stamina[GetIndex(hist)] = new_stamina;
  }
#endif
}

inline float_type CAgentObject::GetStamina(const int_type hist) const
{
  return stamina[GetIndex(hist)];
}

inline void CAgentObject::SetEffort(const float_type new_effort, const int_type hist)
{
#ifndef _REQUIRE_PARAMETERS_IN_OBJECTS_
  if ( have_parameters ) {
#endif
    effort[GetIndex(hist)] = MinMax(params->SP_effort_min, new_effort,
				    params->SP_effort_max);
#ifndef _REQUIRE_PARAMETERS_IN_OBJECTS_
  } else {
    effort[GetIndex(hist)] = new_effort;
  }
#endif
}

inline float_type CAgentObject::GetEffort(const int_type hist) const
{
  return effort[GetIndex(hist)];
}

inline void CAgentObject::SetRecover(const float_type new_recover, const int_type hist)
{
#ifndef _REQUIRE_PARAMETERS_IN_OBJECTS_
  if ( have_parameters ) {
#endif
    recover[GetIndex(hist)] = MinMax(params->SP_recover_min, new_recover,
				     params->SP_recover_max);
#ifndef _REQUIRE_PARAMETERS_IN_OBJECTS_
  } else {
    recover[GetIndex(hist)] = new_recover;
  }
#endif
}

inline float_type CAgentObject::GetRecover(const int_type hist) const
{
  return recover[GetIndex(hist)];
}

inline void CAgentObject::SetDashes(const int_type new_dashes, const int_type hist)
{
  dashes[GetIndex(hist)] = new_dashes;
}

inline int_type CAgentObject::GetDashes(const int_type hist) const
{
  return dashes[GetIndex(hist)];
}

inline void CAgentObject::SetKicks(const int_type new_kicks, const int_type hist)
{
  kicks[GetIndex(hist)] = new_kicks;
}

inline int_type CAgentObject::GetKicks(const int_type hist) const
{
  return kicks[GetIndex(hist)];
}

inline void CAgentObject::SetTurns(const int_type new_turns, const int_type hist)
{
  turns[GetIndex(hist)] = new_turns;
}

inline int_type CAgentObject::GetTurns(const int_type hist) const
{
  return turns[GetIndex(hist)];
}

inline void CAgentObject::SetSays(const int_type new_says, const int_type hist)
{
  says[GetIndex(hist)] = new_says;
}

inline int_type CAgentObject::GetSays(const int_type hist) const
{
  return says[GetIndex(hist)];
}

inline void CAgentObject::SetTurnNecks(const int_type new_turn_necks, const int_type hist)
{
  turn_necks[GetIndex(hist)] = new_turn_necks;
}

inline int_type CAgentObject::GetTurnNecks(const int_type hist) const
{
  return turn_necks[GetIndex(hist)];
}

inline void CAgentObject::SetHearCapacityFromTeammate(const int_type new_hear_capacity) {
#ifndef _REQUIRE_PARAMETERS_IN_OBJECTS_
  if ( have_parameters ) {
#endif
    hear_capacity_from_teammate = MinMax(0, new_hear_capacity,
					 params->SP_hear_max);
#ifndef _REQUIRE_PARAMETERS_IN_OBJECTS_
  } else {
    hear_capacity_from_teammate = new_hear_capacity;
  }
#endif
}

inline int_type CAgentObject::GetHearCapacityFromTeammate() const
{
  return hear_capacity_from_teammate;
}

inline void CAgentObject::SetHearCapacityFromOpponent(const int_type new_hear_capacity) {
#ifndef _REQUIRE_PARAMETERS_IN_OBJECTS_
  if ( have_parameters ) {
#endif
    hear_capacity_from_opponent = MinMax(0, new_hear_capacity,
					 params->SP_hear_max);
#ifndef _REQUIRE_PARAMETERS_IN_OBJECTS_
  } else {
    hear_capacity_from_opponent = new_hear_capacity;
  }
#endif
}

inline int_type CAgentObject::GetHearCapacityFromOpponent() const
{
  return hear_capacity_from_opponent;
}

inline void CAgentObject::SetCyclesSinceCatch(const int_type new_cycs_since_catch)
{
  cycles_since_catch = new_cycs_since_catch;
}

inline int_type CAgentObject::GetCyclesSinceCatch() const
{
  return cycles_since_catch;
}

inline void CAgentObject::SetCyclesSinceSay(const int_type new_cycs_since_say)
{
  cycles_since_say = new_cycs_since_say;
}

inline int_type CAgentObject::GetCyclesSinceSay() const
{
  return cycles_since_say;
}

inline void CAgentObject::SetViewWidth(const t_view_width new_view_width, const int_type hist)
{
  view_width[GetIndex(hist)] = new_view_width;
}

inline t_view_width CAgentObject::GetViewWidth(const int_type hist) const
{
  return view_width[GetIndex(hist)];
}
  
inline void CAgentObject::SetViewQuality(const t_view_quality new_view_quality, const int_type hist)
{
  view_quality[GetIndex(hist)] = new_view_quality;
}

inline t_view_quality CAgentObject::GetViewQuality(const int_type hist) const
{
  return view_quality[GetIndex(hist)];
}

inline void CAgentObject::SetLastSenseType(const t_sense new_last_sense_type)
{
  last_sense_type = new_last_sense_type;
}

inline t_sense CAgentObject::GetLastSenseType() const
{
  return last_sense_type;
}

inline void CAgentObject::SetLastSenseTime(const CTime& new_last_sense_time)
{
  last_sense_time = new_last_sense_time;
}

inline CTime CAgentObject::GetLastSenseTime() const
{
  return last_sense_time;
}

inline void CAgentObject::SetLastSightTime(const CTime& new_last_sight_time)
{
  last_sight_time = new_last_sight_time;
}

inline CTime CAgentObject::GetLastSightTime() const
{
  return last_sight_time;
}

inline void CAgentObject::SetLastHearTime(const CTime& new_last_hear_time)
{
  last_hear_time = new_last_hear_time;
}

inline CTime CAgentObject::GetLastHearTime() const
{
  return last_hear_time;
}

inline void CAgentObject::SetLastHearTeammateTime(const CTime& new_last_hear_time)
{
  last_hear_teammate_time = new_last_hear_time;
}

inline CTime CAgentObject::GetLastHearTeammateTime() const
{
  return last_hear_teammate_time;
}

inline void CAgentObject::SetLastHearOpponentTime(const CTime& new_last_hear_time)
{
  last_hear_opponent_time = new_last_hear_time;
}

inline CTime CAgentObject::GetLastHearOpponentTime() const
{
  return last_hear_opponent_time;
}

inline void CAgentObject::SetLastHearRefereeTime(const CTime& new_last_hear_time)
{
  last_hear_referee_time = new_last_hear_time;
}

inline CTime CAgentObject::GetLastHearRefereeTime() const
{
  return last_hear_referee_time;
}

inline void CAgentObject::SetLastActTime(const CTime& new_last_act_time)
{
  last_act_time = new_last_act_time;
}

inline CTime CAgentObject::GetLastActTime() const
{
  return last_act_time;
}

inline const CCommand* const CAgentObject::GetLastCommand() const
{
  return last_commands.Front();
}

inline void CAgentObject::RemoveLastCommand()
{
  last_commands.PopFront();
}

inline const CCommandQueue& CAgentObject::GetLastCommandQueue() const
{
  return last_commands;
}

inline CCommandQueue& CAgentObject::GetLastCommandQueue()
{
  return last_commands;
}

inline const CCommand* const CAgentObject::GetLastExecutedCommand() const
{
  return last_executed_commands.Front();
}

inline void CAgentObject::SetNextTurnNeck(const CTurnNeckCommand& new_turn_neck)
{
  next_turn_neck = new_turn_neck;
}

inline CTurnNeckCommand CAgentObject::GetNextTurnNeck() const
{
  return next_turn_neck;
}

inline void CAgentObject::SetDoTurnNeck(const bool new_do_tn)
{
  do_turn_neck = new_do_tn;
}

inline bool CAgentObject::DoTurnNeck() const
{
  return do_turn_neck;
}

inline const CMessages& CAgentObject::SentMessages() const
{
  return sent_messages;
}

inline CMessages& CAgentObject::SentMessages()
{
  return sent_messages;
}

inline const CMessages& CAgentObject::ReceivedMessages() const
{
  return received_messages;
}

inline CMessages& CAgentObject::ReceivedMessages()
{
  return received_messages;
}

inline int CAgentObject::GetSenses() const
{
  return senses;
}

inline void CAgentObject::SetSenses(int new_senses)
{
  senses = new_senses;
}
  
inline int CAgentObject::GetSees() const
{
  return sees;
}

inline void CAgentObject::SetSees(int new_sees)
{
  sees = new_sees;
}
  
#endif
