#ifndef _CANGLEDEG_H_
#define _CANGLEDEG_H_

class CAngleDeg;

#include "types.h"
#include "MathUtils.h"
#include "CAngleRad.h"
#include <iostream>

class CAngleDeg
{
public:
  CAngleDeg(const float_type = 0.0);
  CAngleDeg(const CAngleDeg&);
  CAngleDeg(const CAngleRad&);
  
  CAngleDeg& Normalize();
  
  float_type GetRad() const;
  float_type GetDeg() const;

  // I don't think I have to define all the operators for float_type too!
  CAngleDeg operator -() const;
  CAngleDeg operator +(const CAngleDeg&) const;
  CAngleDeg operator +(const float_type) const;
  CAngleDeg operator -(const CAngleDeg&) const;
  CAngleDeg operator -(const float_type) const;
  CAngleDeg operator *(const CAngleDeg&) const;
  CAngleDeg operator *(const float_type) const;
  CAngleDeg operator /(const CAngleDeg&) const;
  CAngleDeg operator /(const float_type) const;
  
  CAngleDeg&  operator  =(const CAngleDeg&);
  CAngleDeg&  operator  =(const CAngleRad&);
  CAngleDeg&  operator  =(const float_type);
  CAngleDeg&  operator +=(const CAngleDeg&);
  CAngleDeg&  operator +=(const float_type);
  CAngleDeg&  operator -=(const CAngleDeg&);
  CAngleDeg&  operator -=(const float_type);
  CAngleDeg&  operator *=(const CAngleDeg&);
  CAngleDeg&  operator *=(const float_type);
  CAngleDeg&  operator /=(const CAngleDeg&);
  CAngleDeg&  operator /=(const float_type);
  
  bool operator ==(const CAngleDeg&) const;
  bool operator ==(const float_type) const;
  bool operator !=(const CAngleDeg&) const;
  bool operator !=(const float_type) const;
  bool operator > (const CAngleDeg&) const;
  bool operator > (const float_type) const;
  bool operator >=(const CAngleDeg&) const;
  bool operator >=(const float_type) const;
  bool operator < (const CAngleDeg&) const;
  bool operator < (const float_type) const;
  bool operator <=(const CAngleDeg&) const;
  bool operator <=(const float_type) const;
  bool operator ! () const;
  
  operator float_type() const;

  friend std::ostream& operator <<(std::ostream&, const CAngleDeg&);
  friend std::istream& operator >>(std::istream&, CAngleDeg&);
  
protected:
  void _normalize();

  float_type a;
};

#if CHECK_DIVISION_BY_ZERO == 1
static const CAngleDeg CANGLEDEG_DIVISION_BY_ZERO_RESULT(FLOAT_TYPE_DIVISION_BY_ZERO_RESULT);
#endif CHECK_DIVISION_BY_ZERO

// Doesn't work since we normalize the angle to +/-180!
static const CAngleDeg UNKNOWN_DEG(9988);


// --- public -----------------------------------------------------------------
inline CAngleDeg& CAngleDeg::Normalize()
{
  _normalize();
  return *this;
};

// --- public -----------------------------------------------------------------
inline float_type CAngleDeg::GetRad() const
{
  return a * m_deg2rad;
}

// --- public -----------------------------------------------------------------
inline float_type CAngleDeg::GetDeg() const
{
  return a;
}

// --- public -----------------------------------------------------------------
// unary minus
inline CAngleDeg CAngleDeg::operator -() const
{
  return CAngleDeg(-a);
}

// --- public -----------------------------------------------------------------
// addition
inline CAngleDeg CAngleDeg::operator +(const CAngleDeg& ang) const
{
  return CAngleDeg(a + ang.a);
}

// --- public -----------------------------------------------------------------
// addition
inline CAngleDeg CAngleDeg::operator +(const float_type ang) const
{
  return CAngleDeg(a + ang);
}

// --- public -----------------------------------------------------------------
// subtraction
inline CAngleDeg CAngleDeg::operator -(const CAngleDeg& ang) const
{
  return CAngleDeg(a - ang.a);
}

// --- public -----------------------------------------------------------------
// subtraction
inline CAngleDeg CAngleDeg::operator -(const float_type ang) const
{
  return CAngleDeg(a - ang);
}

// --- public -----------------------------------------------------------------
// multiplication
inline CAngleDeg CAngleDeg::operator *(const CAngleDeg& ang) const
{
  return CAngleDeg(a * ang.a);
}

// --- public -----------------------------------------------------------------
// multiplication
inline CAngleDeg CAngleDeg::operator *(const float_type ang) const
{
  return CAngleDeg(a * ang);
}

// --- public -----------------------------------------------------------------
// division
inline CAngleDeg CAngleDeg::operator /(const CAngleDeg& ang) const
{
#if CHECK_DIVISION_BY_ZERO == 1
  if (ang.a == 0)
    return CANGLEDEG_DIVISION_BY_ZERO_RESULT;
  else
#endif
    return CAngleDeg(a / ang.a);
}

// --- public -----------------------------------------------------------------
// division
inline CAngleDeg CAngleDeg::operator /(const float_type ang) const
{
#if CHECK_DIVISION_BY_ZERO == 1
  if (ang == 0)
    return CANGLEDEG_DIVISION_BY_ZERO_RESULT;
  else
#endif
    return CAngleDeg(a / ang);
}

// --- public -----------------------------------------------------------------
// assign, no range check
inline CAngleDeg& CAngleDeg::operator =(const CAngleDeg& ang)
{
  a = ang.a;
  return *this;
}

// --- public -----------------------------------------------------------------
// assign, range check
inline CAngleDeg& CAngleDeg::operator =(const float_type d)
{
  a = d;
  _normalize();
  return *this;
}

// --- public -----------------------------------------------------------------
inline CAngleDeg& CAngleDeg::operator +=(const CAngleDeg& ang)
{
  a += ang.a;
  _normalize();
  return *this;
}

// --- public -----------------------------------------------------------------
inline CAngleDeg& CAngleDeg::operator +=(const float_type ang)
{
  a += ang;
  _normalize();
  return *this;
}

// --- public -----------------------------------------------------------------
inline CAngleDeg& CAngleDeg::operator -=(const CAngleDeg& ang)
{
  a -= ang.a;
  _normalize();
  return *this;
}

// --- public -----------------------------------------------------------------
inline CAngleDeg& CAngleDeg::operator -=(const float_type ang)
{
  a -= ang;
  _normalize();
  return *this;
}

// --- public -----------------------------------------------------------------
inline CAngleDeg& CAngleDeg::operator *=(const CAngleDeg& ang)
{
  a *= ang.a;
  _normalize();
  return *this;
}

// --- public -----------------------------------------------------------------
inline CAngleDeg& CAngleDeg::operator *=(const float_type d)
{
  a *= d;
  _normalize();
  return *this;
}

// --- public -----------------------------------------------------------------
inline CAngleDeg& CAngleDeg::operator /=(const CAngleDeg& ang)
{
#if CHECK_DIVISION_BY_ZERO == 1
  if (ang.a == 0)
    a = FLOAT_TYPE_DIVISION_BY_ZERO_RESULT;
  else
#endif
    a /= ang.a;
  
  _normalize();
  return *this;
}

// --- public -----------------------------------------------------------------
inline CAngleDeg& CAngleDeg::operator /=(const float_type d)
{
#if CHECK_DIVISION_BY_ZERO == 1
  if (d == 0)
    a = FLOAT_TYPE_DIVISION_BY_ZERO_RESULT;
  else
#endif
    a /= d;

  _normalize();
  return *this;
}

// --- public -----------------------------------------------------------------
inline bool CAngleDeg::operator ==(const CAngleDeg& ang) const
{
  return (a == ang.a);
}

// --- public -----------------------------------------------------------------
inline bool CAngleDeg::operator ==(const float_type ang) const
{
  return (a == ang);
}

// --- public -----------------------------------------------------------------
inline bool CAngleDeg::operator !=(const CAngleDeg& ang) const
{
  return (a != ang.a);
}

// --- public -----------------------------------------------------------------
inline bool CAngleDeg::operator !=(const float_type ang) const
{
  return (a != ang);
}

// --- public -----------------------------------------------------------------
inline bool CAngleDeg::operator >(const CAngleDeg& ang) const
{
  return (a > ang.a);
}

// --- public -----------------------------------------------------------------
inline bool CAngleDeg::operator >(const float_type ang) const
{
  return (a > ang);
}

// --- public -----------------------------------------------------------------
inline bool CAngleDeg::operator >=(const CAngleDeg& ang) const
{
  return (a >= ang.a);
}

// --- public -----------------------------------------------------------------
inline bool CAngleDeg::operator >=(const float_type ang) const
{
  return (a >= ang);
}

// --- public -----------------------------------------------------------------
inline bool CAngleDeg::operator <(const CAngleDeg& ang) const
{
  return (a < ang.a);
}

// --- public -----------------------------------------------------------------
inline bool CAngleDeg::operator <(const float_type ang) const
{
  return (a < ang);
}

// --- public -----------------------------------------------------------------
inline bool CAngleDeg::operator <=(const CAngleDeg& ang) const
{
  return (a <= ang.a);
}

// --- public -----------------------------------------------------------------
inline bool CAngleDeg::operator <=(const float_type ang) const
{
  return (a <= ang);
}

// --- public -----------------------------------------------------------------
inline bool CAngleDeg::operator !() const
{
  return (a != 0);
}

// --- public -----------------------------------------------------------------
inline CAngleDeg::operator float_type() const
{
  return a;
}

// --- friend -----------------------------------------------------------------
inline std::ostream& operator <<(std::ostream& os, const CAngleDeg& ang)
{
  return os << ang.a;
}

// --- friend -----------------------------------------------------------------
inline std::istream& operator >>(std::istream& is, CAngleDeg& ang)
{
  return is >> ang.a;
}

inline float_type Sin(const CAngleDeg& x) 
{ return sin(x.GetRad()); }

inline float_type Cos(const CAngleDeg& x)
{ return cos(x.GetRad()); }

inline float_type Tan(const CAngleDeg& x)
{ return tan(x.GetRad()); }

#if TRIG_IN_DEG == 1
  // Must decied wether ASin, ACos, ATan and ATan2 should return a CAngleRad or
  // a CAngleDeg. The most efficient is to return a CAngleRad, but the most
  // used (in this context) is CAngleDeg. Therefore the current choice is
  // CAngleDeg.
  
inline CAngleDeg ASin(const float_type x)
{ return (x >= 1) ? CAngleDeg(90) : (x <= -1) ? CAngleDeg(-90) : CAngleDeg(Rad2Deg(asin(x))); }

inline CAngleDeg ACos(const float_type x)
{ return (x >= 1) ? CAngleDeg(0) : (x <= -1) ? CAngleDeg(180) : CAngleDeg(Rad2Deg(acos(x))); }

inline CAngleDeg ATan(const float_type x)
{ return CAngleDeg(Rad2Deg(atan(x))); }

inline CAngleDeg ATan2(const float_type x, const float_type y)
{  return (x == 0 && y == 0) ? CAngleDeg(0) : CAngleDeg(Rad2Deg(atan2(x,y))); }
#endif TRIG_IN_DEG

#endif _CANGLEDEG_H_
