#ifndef _CANGLERAD_H_
#define _CANGLERAD_H_

class CAngleRad;

#include "compiler_directives.h"
#include "types.h"
#include "MathUtils.h"
#include "CAngleDeg.h"
#include <iostream>

class CAngleRad
{
public:
  CAngleRad(const float_type = 0.0);
  CAngleRad(const CAngleRad&);
  CAngleRad(const CAngleDeg&);
  
  CAngleRad& Normalize();
  
  float_type GetRad() const;
  float_type GetDeg() const;

  // Don't think I have to define all the operators
  // for float_type too, since the automatic type conversion
  // will convert float_type to CAngleDeg (I hope...)
  CAngleRad operator -() const;
  CAngleRad operator +(const CAngleRad&) const;
  CAngleRad operator +(const float_type) const;
  CAngleRad operator -(const CAngleRad&) const;
  CAngleRad operator -(const float_type) const;
  CAngleRad operator *(const CAngleRad&) const;
  CAngleRad operator *(const float_type) const;
  CAngleRad operator /(const CAngleRad&) const;
  CAngleRad operator /(const float_type) const;
  
  CAngleRad& operator  =(const CAngleRad&);
  CAngleRad& operator  =(const CAngleDeg&);
  CAngleRad& operator  =(const float_type);
  CAngleRad& operator +=(const CAngleRad&);
  CAngleRad& operator +=(const float_type);
  CAngleRad& operator -=(const CAngleRad&);
  CAngleRad& operator -=(const float_type);
  CAngleRad& operator *=(const CAngleRad&);
  CAngleRad& operator *=(const float_type);
  CAngleRad& operator /=(const CAngleRad&);
  CAngleRad& operator /=(const float_type);
  
  bool operator ==(const CAngleRad&) const;
  bool operator ==(const float_type) const;
  bool operator !=(const CAngleRad&) const;
  bool operator !=(const float_type) const;
  bool operator > (const CAngleRad&) const;
  bool operator > (const float_type) const;
  bool operator >=(const CAngleRad&) const;
  bool operator >=(const float_type) const;
  bool operator < (const CAngleRad&) const;
  bool operator < (const float_type) const;
  bool operator <=(const CAngleRad&) const;
  bool operator <=(const float_type) const;
  bool operator ! () const;
  
  operator float_type() const;

  friend std::ostream& operator <<(std::ostream&, const CAngleRad&);
  friend std::istream& operator >>(std::istream&, CAngleRad&);
  
protected:
  void _normalize();
  
  float_type a;
};

#if CHECK_DIVISION_BY_ZERO == 1
static const CAngleRad CANGLERAD_DIVISION_BY_ZERO_RESULT(FLOAT_TYPE_DIVISION_BY_ZERO_RESULT);
#endif CHECK_DIVISION_BY_ZERO

// Doesn't work since we normalize the angle to +/-PI!
static const CAngleRad UNKNOWN_RAD(9989);


// --- public -----------------------------------------------------------------
inline CAngleRad& CAngleRad::Normalize()
{
  _normalize();
  return *this;
}

// --- public -----------------------------------------------------------------
inline float_type CAngleRad::GetRad() const
{
  return a;
}

// --- public -----------------------------------------------------------------
inline float_type CAngleRad::GetDeg() const
{
  return a * m_rad2deg;
}

// --- public -----------------------------------------------------------------
// unary minus
inline CAngleRad CAngleRad::operator -() const
{
  return CAngleRad(-a);
}

// --- public -----------------------------------------------------------------
// addition
inline CAngleRad CAngleRad::operator +(const CAngleRad& ang) const
{
  return CAngleRad(a + ang.a);
}

// --- public -----------------------------------------------------------------
// addition
inline CAngleRad CAngleRad::operator +(const float_type ang) const
{
  return CAngleRad(a + ang);
}

// --- public -----------------------------------------------------------------
// subtraction
inline CAngleRad CAngleRad::operator -(const CAngleRad& ang) const
{
  return CAngleRad(a - ang.a);
}

// --- public -----------------------------------------------------------------
// subtraction
inline CAngleRad CAngleRad::operator -(const float_type ang) const
{
  return CAngleRad(a - ang);
}

// --- public -----------------------------------------------------------------
// multiplication
inline CAngleRad CAngleRad::operator *(const CAngleRad& ang) const
{
  return CAngleRad(a * ang.a);
}

// --- public -----------------------------------------------------------------
// multiplication
inline CAngleRad CAngleRad::operator *(const float_type ang) const
{
  return CAngleRad(a * ang);
}

// --- public -----------------------------------------------------------------
// division
inline CAngleRad CAngleRad::operator /(const CAngleRad& ang) const
{
#if CHECK_DIVISION_BY_ZERO == 1
  if (ang.a == 0)
    return CANGLERAD_DIVISION_BY_ZERO_RESULT;
  else
#endif
    return CAngleRad(a / ang.a);
}

// --- public -----------------------------------------------------------------
// division
inline CAngleRad CAngleRad::operator /(const float_type ang) const
{
#if CHECK_DIVISION_BY_ZERO == 1
  if (ang == 0)
    return CANGLERAD_DIVISION_BY_ZERO_RESULT;
  else
#endif
    return CAngleRad(a / ang);
}

// --- public -----------------------------------------------------------------
// assign, no range check
inline CAngleRad& CAngleRad::operator =(const CAngleRad& ang)
{
  a = ang.a;
  return *this;
}

// --- public -----------------------------------------------------------------
// assign, range check
inline CAngleRad& CAngleRad::operator =(const float_type d)
{
  a = d;
  _normalize();
  return *this;
}

// --- public -----------------------------------------------------------------
inline CAngleRad& CAngleRad::operator +=(const CAngleRad& ang)
{
  a += ang.a;
  _normalize();
  return *this;
}

// --- public -----------------------------------------------------------------
inline CAngleRad& CAngleRad::operator +=(const float_type ang)
{
  a += ang;
  _normalize();
  return *this;
}

// --- public -----------------------------------------------------------------
inline CAngleRad& CAngleRad::operator -=(const CAngleRad& ang)
{
  a -= ang.a;
  _normalize();
  return *this;
}

// --- public -----------------------------------------------------------------
inline CAngleRad& CAngleRad::operator -=(const float_type ang)
{
  a -= ang;
  _normalize();
  return *this;
}

// --- public -----------------------------------------------------------------
inline CAngleRad& CAngleRad::operator *=(const CAngleRad& ang)
{
  a *= ang.a;
  _normalize();
  return *this;
}

// --- public -----------------------------------------------------------------
inline CAngleRad& CAngleRad::operator *=(const float_type d)
{
  a *= d;
  _normalize();
  return *this;
}

// --- public -----------------------------------------------------------------
inline CAngleRad& CAngleRad::operator /=(const CAngleRad& ang)
{
#if CHECK_DIVISION_BY_ZERO == 1
  if (ang.a == 0)
    a = FLOAT_TYPE_DIVISION_BY_ZERO_RESULT;
  else
#endif
    a /= ang.a;

  _normalize();
  return *this;
}

// --- public -----------------------------------------------------------------
inline CAngleRad& CAngleRad::operator /=(const float_type d)
{
#if CHECK_DIVISION_BY_ZERO == 1
  if (d == 0)
    a = FLOAT_TYPE_DIVISION_BY_ZERO_RESULT;
  else
#endif
    a /= d;

  _normalize();
  return *this;
}

// --- public -----------------------------------------------------------------
inline bool CAngleRad::operator ==(const CAngleRad& ang) const
{
  return (a == ang.a);
}

// --- public -----------------------------------------------------------------
inline bool CAngleRad::operator ==(const float_type ang) const
{
  return (a == ang);
}

// --- public -----------------------------------------------------------------
inline bool CAngleRad::operator !=(const CAngleRad& ang) const
{
  return (a != ang.a);
}

// --- public -----------------------------------------------------------------
inline bool CAngleRad::operator !=(const float_type ang) const
{
  return (a != ang);
}

// --- public -----------------------------------------------------------------
inline bool CAngleRad::operator >(const CAngleRad& ang) const
{
  return (a > ang.a);
}

// --- public -----------------------------------------------------------------
inline bool CAngleRad::operator >(const float_type ang) const
{
  return (a > ang);
}

// --- public -----------------------------------------------------------------
inline bool CAngleRad::operator >=(const CAngleRad& ang) const
{
  return (a >= ang.a);
}

// --- public -----------------------------------------------------------------
inline bool CAngleRad::operator >=(const float_type ang) const
{
  return (a >= ang);
}

// --- public -----------------------------------------------------------------
inline bool CAngleRad::operator <(const CAngleRad& ang) const
{
  return (a < ang.a);
}

// --- public -----------------------------------------------------------------
inline bool CAngleRad::operator <(const float_type ang) const
{
  return (a < ang);
}

// --- public -----------------------------------------------------------------
inline bool CAngleRad::operator <=(const CAngleRad& ang) const
{
  return (a <= ang.a);
}

// --- public -----------------------------------------------------------------
inline bool CAngleRad::operator <=(const float_type ang) const
{
  return (a <= ang);
}

// --- public -----------------------------------------------------------------
inline bool CAngleRad::operator !() const
{
  return (a != 0);
}

// --- public -----------------------------------------------------------------
inline CAngleRad::operator float_type() const
{
  return a;
}

// --- public -----------------------------------------------------------------
inline std::ostream& operator <<(std::ostream& os, const CAngleRad& ang)
{
  return os << ang.a;
}

// --- friend -----------------------------------------------------------------
inline std::istream& operator>>(std::istream& is, CAngleRad& ang)
{
  return is >> ang.a;
}

inline float_type Sin(const CAngleRad& x) 
{ return sin(x.GetRad()); }

inline float_type Cos(const CAngleRad& x)
{ return cos(x.GetRad()); }

inline float_type Tan(const CAngleRad& x)
{ return tan(x.GetRad()); }

#if TRIG_IN_RAD == 1
  // Must decied wether ASin, ACos, ATan and ATan2 should return a CAngleRad or
  // a CAngleDeg. The most efficient is to return a CAngleRad, but the most
  // used (in this context) is CAngleDeg. Therefore the current choice is
  // CAngleDeg.
  
inline CAngleRad ASin(const float_type x)
{ return (x >= 1) ? CAngleRad(90) : (x <= -1) ? CAngleRad(-90) : CAngleRad(asin(x)); }

inline CAngleRad ACos(const float_type x)
{ return (x >= 1) ? CAngleRad(0) : (x <= -1) ? CAngleRad(180) : CAngleRad(acos(x)); }

inline CAngleRad ATan(const float_type x)
{ return CAngleRad(atan(x)); }

inline CAngleRad ATan2(const float_type x, const float_type y)
{  return (x == 0 && y == 0) ? CAngleRad(0) : CAngleRad(atan2(x,y)); }
#endif TRIG_IN_RAD

#endif
