#include "CBallObject.h"
#include "compiler_directives.h"

#ifdef _REQUIRE_PARAMETERS_IN_OBJECTS_
#  include "soc_assert.h"
#endif

const CVector CBallObject::DEFAULT_RELATIVE_BALL_POS = UNKNOWN_VECTOR;
const CTime CBallObject::DEFAULT_LAST_SEEN_MOVING_TIME = UNKNOWN_TIME;

CBallObject::CBallObject(const CParameters* const p, const int_type hist_length)
  : CMobileObject(hist_length),
    params(p),
    have_parameters(false),
    relative_ball_pos(DEFAULT_RELATIVE_BALL_POS),
    last_seen_moving_time(DEFAULT_LAST_SEEN_MOVING_TIME)
{
#ifdef _REQUIRE_PARAMETERS_IN_OBJECTS_
  Assert(params != NULL,
	 "Must include a pointer to a CParameter object in CBallObject!");
  have_parameters = true;
#else
  if (params == NULL) {
    have_parameters = false;
  } else {
    have_parameters = true;
  }  
#endif  

#ifdef _REQUIRE_PARAMETERS_IN_OBJECTS_
  if ( have_parameters ) {
#endif  
    SetSpeedMax(params->SP_ball_speed_max);
    SetSpeedDecay(params->SP_ball_decay); 
#ifdef _REQUIRE_PARAMETERS_IN_OBJECTS_
  }  
#endif  
}

CBallObject::CBallObject(const CBallObject& obj)
  : CMobileObject(obj),
    relative_ball_pos(obj.relative_ball_pos),
    last_seen_moving_time(obj.last_seen_moving_time)
{
  
}

void CBallObject::ResetAll()
{
  Reset();
}

void CBallObject::Reset()
{
  CMobileObject::Reset();

  relative_ball_pos = DEFAULT_RELATIVE_BALL_POS;
  last_seen_moving_time = DEFAULT_LAST_SEEN_MOVING_TIME;
}  

void CBallObject::CopyData(const CBallObject& obj)
{
  CMobileObject::CopyData(obj);

  relative_ball_pos = obj.relative_ball_pos;
  last_seen_moving_time = obj.last_seen_moving_time;
}

CBallObject& CBallObject::operator= (const CBallObject& obj)
{

  if (this != &obj) {
    CopyData(obj);
  }

  return *this;  
  
}

// Redefined since we now have access to the player_speed_max parameter.
void CBallObject::SetAbsoluteSpeedVector(const CVector& v, const int_type hist)
{
  SetSpeed(v.GetLength(), hist),
  SetAbsoluteSpeedDirection(v.GetAngle(), hist);
}

// Redefined since we now have access to the player_speed_max parameter.
void CBallObject::SetSpeed(const t_speed& speed, const int_type hist)
{
#ifndef _REQUIRE_PARAMETERS_IN_OBJECTS_
  if ( have_parameters ) {
#endif
    CMobileObject::SetSpeed(MinMax(0.0, speed, params->SP_ball_speed_max), hist);
#ifndef _REQUIRE_PARAMETERS_IN_OBJECTS_
  } else {
    CMobileObject::SetSpeed(speed, hist);
  }
#endif    
}
