#include "CBasicIP.h"
#include "utils.h"
//#include "constants.h"

CBasicIP::CBasicIP(CKnowledge* const kb, CLogFile* const logfile)
  : k(kb),
    // This will segfault if kb is NULL!
    // That's not good, but the program would've crashed anyway
    // although the user would've gotten a proper error message.
    params(kb->GetParameters()),
    logf(logfile),
    seenPlayers(params, 22, false),
    seenTeammates(params, 22, false),
    seenOpponents(params, 22, false),
    seenLines    (4), //MAX_NUM_LINES),
    seen_lines_pos(0),
    seenMarkers(),
    seenMarkersBehind(),
    seen_markers(0),
    seen_markers_behind(0),
    last_corner_time(UNKNOWN_TIME),
    first_action_op_since_sight(false),
    last_stop_clock_time       (CTime::last_stop_clock_time),
    second_last_stop_clock_time(CTime::second_last_stop_clock_time),
    clock_stopped(false),
    stopped_clock_msec(0),
    tmp_queue(),
    showSeenPlayers(false),
    showInfo(false)
{
  /* Since k and params are constants, you can't initialize them
     here. Therefore the program will never get this far it kb is NULL.
  // Add an assertion that k is not NULL
  // If it's NULL then abort, can't continue.
  Assert(k != NULL, "Must include a pointer to a CKnowledge object in CBasicIP!");

  params = k->GetParameters();
  */
  
}

CBasicIP::CBasicIP(const CBasicIP& obj)
  : k     (obj.k),
    params(obj.params),
    logf  (obj.logf),
    seenPlayers  (obj.seenPlayers),
    seenTeammates  (obj.seenTeammates),
    seenOpponents  (obj.seenOpponents),
    seenLines    (obj.seenLines),
    seen_lines_pos(obj.seen_lines_pos),
    seenMarkers  (obj.seenMarkers),
    seenMarkersBehind  (obj.seenMarkersBehind),
    seen_markers(obj.seen_markers),
    seen_markers_behind(obj.seen_markers_behind),
    last_corner_time(obj.last_corner_time),
    first_action_op_since_sight(obj.first_action_op_since_sight),
    last_stop_clock_time       (obj.last_stop_clock_time),
    second_last_stop_clock_time(obj.second_last_stop_clock_time),
    clock_stopped(obj.clock_stopped),
    stopped_clock_msec(obj.stopped_clock_msec),
    tmp_queue(obj.tmp_queue),
    showSeenPlayers(obj.showSeenPlayers),
    showInfo       (obj.showInfo)
{

}

CBasicIP::~CBasicIP()
{
}

/*
void CBasicIP::CopyData(const CBasicIP& obj)
{
  k      = obj.k;
  params = obj.params;

  seenLines       = obj.seenLines;
  seenPlayers     = obj.seenPlayers;
  seenTeammates   = obj.seenTeammates;
  seenOpponents   = obj.seenOpponents;

  showSeenPlayers = obj.showSeenPlayers;
  showInfo        = obj.showInfo;

  last_stop_clock_time        = obj.last_stop_clock_time;
  second_last_stop_clock_time = obj.second_last_stop_clock_time;

  clock_stopped      = obj.clock_stopped;
  stopped_clock_msec = obj.stopped_clock_msec;

}

CBasicIP& CBasicIP::operator =(const CBasicIP& obj)
{

  if ( this != &obj) {
    CopyData(obj);
  }

  return *this;
  
}
*/

void CBasicIP::InitHalf()
{

  if ( k->theGame().GetOurSide() == S_Left ) {
    //std::cout << "We play on the LEFT side!\n";
    /*
      RM_LB_Flag = Flag_LT;
      RM_LC_Flag = Flag_CT;
      RM_LF_Flag = Flag_RT;
      RM_RB_Flag = Flag_LB;
      RM_RC_Flag = Flag_CB;
      RM_RF_Flag = Flag_RB;
      RM_Our_PC_Flag = Flag_PLC;    // Center of my penalty area 
      RM_Their_PC_Flag = Flag_PRC; // Center of theirs
    */
    k->ourGoal = k->goalL;
    k->theirGoal = k->goalR;
  }
  else if ( k->theGame().GetOurSide() == S_Right ) {
    //std::cout << "We play on the RIGHT side!\n";
    /*
      RM_LB_Flag = Flag_RB;
      RM_LC_Flag = Flag_CB;
      RM_LF_Flag = Flag_LB;
      RM_RB_Flag = Flag_RT;
      RM_RC_Flag = Flag_CT;
      RM_RF_Flag = Flag_LT;
      RM_Our_PC_Flag = Flag_PRC;    // Center of my penalty area 
      RM_Their_PC_Flag = Flag_PLC; // Center of theirs
    */

    k->ourGoal = k->goalL;
    k->theirGoal = k->goalR;
  } else {
    std::cout << "We play on the UNKNOWN side!\n";
    k->ourGoal = k->goalL;
    k->theirGoal = k->goalR;
  }
    
}

void CBasicIP::BeforeSight()
{

  // Remove the previous cycle's seenLines
  seen_lines_pos = 0;

  // Remove the previous cycle's seen markers
  while ( !seenMarkers.empty() )
    seenMarkers.pop();
  while ( !seenMarkersBehind.empty() )
    seenMarkersBehind.pop();
  seen_markers = 0;
  seen_markers_behind = 0;

  // Remove the previous cycle's closest_flag
  found_closest_flag = false;

  seen_ball = false;
  seen_ball_moving = false;

  HIST = NOW;
  showSeenPlayers = false;

  seenPlayers.MakeEmpty();
  // Temporary solution
  // Since we don't match the players that isn't visable we get very many
  // copies of players that we haven't seen...
  // Therefore clean the whole array right away.
  k->thePlayers().MakeEmpty();

  /*
  // Remove all players from thePlayers that I should see this turn.
  int r = k->thePlayers().RemoveAllShouldBeSeen(k->theAgent().GetNeckDir(NOW),
						k->theAgent().GetViewAngle(NOW), NOW);
  if ( showSeenPlayers ) {
    std::cout << "Removed " << r << " players that should be seen."
	      << k->thePlayers().GetNumberOfPlayers() << " players left.\n";
  }
  */
  
}

void CBasicIP::AfterSight()
{

  CAngleDeg face_direction = ComputeFaceDir();
  //  std::cout << "Computed my absolut face direction: " << face_direction << std::endl;

  if ( seen_markers + seen_markers_behind == 0 ) { // !found_closest_flag) {
    std::cout << "Didn't find any flag that I could use to calculate my position!"
	      << " Won't update anything!" << std::endl;
    // Either use some other method or use the estimated value
  } else {

    //std::cout << "I saw " << seen_markers << " markers infront and "
    //      << seen_markers_behind << " markers behind.\n";

    // Triangulation didn't help, it actually made the value worse!
    //CPoint tmp_pos(0,0);
    if ( !seenMarkers.empty() ) {
      closest_flag = seenMarkers.top();
      /*
      tmp_pos = seenMarkers.top().GetAbsolutePosition(face_direction, NOW);
      std::cout << "Without triangulation my pos would've been: " << tmp_pos
      	<< std::endl;
      seenMarkers.pop();
      int_type num_markers = 1;
      for (int i = 1; i < 2; ++i) {
	if ( seenMarkers.empty() ) {
	  if ( !seenMarkersBehind.empty() ) {
	    tmp_pos += seenMarkersBehind.top().GetAbsolutePosition(face_direction,
								   NOW);
	    seenMarkersBehind.pop();
	    num_markers++;
	  }
	} else {
	  tmp_pos += seenMarkers.top().GetAbsolutePosition(face_direction, NOW);
	  seenMarkers.pop();
	  num_markers++;	  
	}
      }
      tmp_pos /= num_markers;
      */
    } else if ( !seenMarkersBehind.empty() ) {
      closest_flag = seenMarkersBehind.top();
      /*
      tmp_pos = seenMarkersBehind.top().GetAbsolutePosition(face_direction, NOW);
      seenMarkersBehind.pop();
      */
    } else
      std::cout << "No markers seen and seen_markers + "
		<< "seen_markers_behind is greater than zero!\n";

    //CPoint position = tmp_pos;
    
    CPoint position = closest_flag.GetAbsolutePosition(face_direction, NOW);
    //ComputeAbsolutPosition(closest_flag, face_direction);

    // Compensate if I'm on the right side of the field.
    if ( k->theGame().GetOurSide() == S_Right ) {
      position = -position;
      face_direction += 180;
    }
    
    // Compare the calculated position with the estimated one based on previous
    // observations and internal state.
    //    std::cout << "Used flag " << closest_flag.GetFlagId() << " at distance "
    //	 << closest_flag.GetDistance(-HIST) << std::endl;
    //    std::cout << "To computed absolut position: " << position << std::endl;

    // How should I get a GOOD value for body- and neckdirection??
    // Maybe I have to see if I executed a turn_neck last turn
    // or I have to update the agents body_dir in turn_neck.
    // The last solution is the most elegant, in my oppinon.
    CAngleDeg body_direction = face_direction - k->theAgent().GetNeckDir(NOW);

    if ( showInfo ) {
      //std::cout << "In See!" << std::endl;
      std::cout << k->theGame().GetCurrentTime() << " My position is " << position
	   << " facing " << face_direction << " bodydir " << body_direction
	   << std::endl;
    }
  
    // Now that I have my absolut position I can calculate the other players
    // absolut positions too!
    // It's also time to find the best matching player to each of the players
    // in seenPlayers and update k->thePlayers accordingly.

    CPoint pos;
    CVector vec;

    if ( showSeenPlayers ) {
      std::cout << "Seen " << seenPlayers.GetNumberOfPlayers()
	   << " players this cycle." << std::endl;
    }
    
    if (seenPlayers.GetNumberOfPlayers() > 0) {
      //std::cout << "In  players loop!" << std::endl;
      CPlayerObject* tmp = NULL;
      for (int_type i = 0; i < seenPlayers.GetNumberOfPlayers(); ++i) {
	CPlayerObject* tmp_plr = seenPlayers[i];
	if ( tmp_plr != NULL ) {
	  if ( tmp_plr->GetDistance(NOW) != UNKNOWN_FLOAT_TYPE ) {
	    vec = tmp_plr->GetVector(NOW);
	    vec.Rotate(face_direction);
	    pos = position + vec;
	    tmp_plr->SetPosition(pos, NOW);
	    // Compensate for the neck_direction.
	    vec.Rotate(-body_direction);
	    tmp_plr->SetVector(vec, NOW);
	    if ( showSeenPlayers ) {
	      std::cout << k->theGame().GetCurrentTime()
			<< " Player vector: "
			<< tmp_plr->GetVector(NOW)
			<< " Found player at "
			<< tmp_plr->GetPosition(NOW);
	      std::cout << std::endl;
	    }
	    /*
	    tmp = k->thePlayers().GetBestMatch(*tmp_plr, NOW, NOW,
					       k->theGame().GetCurrentTime());
	    if ( tmp == NULL ) {
	      std::cout << " Couldn't find a match!\n";
	    } else {
	      tmp->CopyFrom(*tmp_plr, NOW, NOW);
	      std::cout << " Match it with: " << tmp->GetSide() << " "
			<< tmp->GetUNumber() << std::endl;
	    }
	    */
	    k->thePlayers().AddPlayer(*tmp_plr);
	  } else {
	    CAngleDeg ang = tmp_plr->GetDirection(NOW);
	    // Compensate for the neck_direction.
	    ang = ang + face_direction - body_direction;
	    tmp_plr->SetDirection(ang, NOW);
	    if ( showSeenPlayers ) {
	      std::cout << k->theGame().GetCurrentTime()
			<< " Player direction: "
			<< tmp_plr->GetDirection(NOW);
	      std::cout << std::endl;
	    }
	    /*
	    tmp = k->thePlayers().GetBestMatch(*tmp_plr, NOW, NOW,
					       k->theGame().GetCurrentTime());
	    if ( tmp == NULL ) {
	      std::cout << " Couldn't find a match!\n";
	    } else {
	      tmp->CopyFrom(*tmp_plr, NOW, NOW);
	      std::cout << " Match it with: " << tmp->GetSide() << " "
			<< tmp->GetUNumber() << std::endl;
	    }
	    */
	    k->thePlayers().AddPlayer(*tmp_plr);
	  }
	} 
      }
    }
    
    if (seen_ball) {
      // To calculate the absolute position of the ball we need
      // the absolute vector to the ball.
      vec = k->theBall().GetVector(NOW);
      vec.Rotate(face_direction);
      // Add it to my absolute position and we get it!
      pos = position + vec;
      k->theBall().SetPosition(pos, NOW);
      // Compensate for the neck_direction.
      vec.Rotate(-body_direction);
      k->theBall().SetVector(vec, NOW);
      logf->LogBallLocation(HIST);

      // If I've got dist_chng and dir_chng use them
      if ( seen_ball_moving ) {
	float_type Ex = Cos((k->theBall().GetDirection(NOW) / 180.0) * m_pi);
	float_type Ey = Sin((k->theBall().GetDirection(NOW) / 180.0) * m_pi);
	float_type Wx = (k->theBall().GetDistChanged(NOW) * Ex)
	                   - (((k->theBall().GetDirChanged(NOW) * m_pi) / 180.0)
			          * k->theBall().GetDistance(NOW) * Ey);
	float_type Wy = (k->theBall().GetDistChanged(NOW) * Ey)
	                   + (((k->theBall().GetDirChanged(NOW) * m_pi) / 180.0)
			          * k->theBall().GetDistance(NOW) * Ex);
	CVector rel_vel = CVector(CPoint(Wx, Wy));

	rel_vel.Rotate(body_direction);
	rel_vel + k->theAgent().GetSpeed(NOW);
	//rel_vel.Rotate(-body_direction);
	//std::cout << "Estimated ball speed: " << ballRelSpeed
	//  << std::endl

	/* from CMU doesn't work
	// Switched the order...
	CVector rel_vel(CPoint(k->theBall().GetDistChanged(NOW),
			       k->theBall().GetDistance(NOW)*
			       Tan(k->theBall().GetDirChanged(NOW))));
	rel_vel.Rotate(k->theBall().GetDirection(NOW));
	CVector abs_vel = k->theAgent().GetAbsoluteSpeedVector(NOW) + rel_vel.Rotate(body_direction);
	*/
	/*
	CVector abs_vel = CVector(k->theBall().GetDirChanged(NOW),
				  k->theBall().GetDistChanged(NOW))
	                   - k->theAgent().GetAbsoluteSpeedVector(NOW);
	k->theBall().SetAbsoluteSpeedVector(abs_vel, NOW);
	*/
	CVector abs_vel = rel_vel.Rotate(-body_direction);
	k->theBall().SetAbsoluteSpeedVector(abs_vel, NOW);
	if ( showInfo ) {
	  std::cout << "Calculated the ball speed to: " << abs_vel << std::endl;
	  std::cout << "rel_vel: " << rel_vel << " RelVel: "
		    << k->theBall().GetRelativeSpeedDirection(NOW)
		    << std::endl;
	  std::cout << "My speed: " << k->theAgent().GetAbsoluteSpeedVector(NOW)
		    << " body_dir: " << body_direction
		    << " ball_dir: " << k->theBall().GetDirection(NOW)
		    << std::endl;
	  logf->LogText("Updated the ball speed.");
	}
      } 

      // Estimate the speed of the ball
      // REALLY bad!

      //std::cout << "Ball rel_vel: " << rel_vel
      //	<< " abs_vel: " << abs_vel << std::endl;
      //k->theBall().SetAbsoluteSpeedVector(CVector(k->theBall().GetPosition(PREV), pos), NOW);
      // int_type t = k->theBall().GetObsTime(-HIST + 1) - k->theGame().GetCurrentTime();

      /*
	Since we don't have any predefined confidence levels, right now,
	we either need to define such or we have to rewrite or remove
	this piece of code.
      if ( t > 0 &&
	   k->theBall().GetConfidence(-HIST + 1) > CP_medium_confidence ) {
	k->theBall().SetSpeed(k->theBall().GetSpeed(-HIST) / t, HIST);
      } else {
	k->theBall().SetAbsoluteSpeedVector(CVector(0.0, 0.0), HIST);
      }
      */

      if ( showInfo ) {
	std::cout << k->theGame().GetCurrentTime() << " "
	     << k->theAgent().GetUNumber() << " Ball pos: " << pos
	     << " vector: " << k->theBall().GetVector(NOW)
	     << " speed: " << k->theBall().GetAbsoluteSpeedVector(NOW) << ")"
	     << std::endl;
      }
    
    }

    k->theAgent().SetBodyDir(body_direction, NOW);
    k->theAgent().SetAbsoluteSpeedDirection(body_direction, NOW);
    k->theAgent().SetFaceDir(face_direction, NOW);
    k->theAgent().SetPosition(position, NOW);
    k->theAgent().SetObsTime(k->theGame().GetCurrentTime(), HIST);

    if ( !seen_ball ) {
      // If I didn't see the ball I should estimate its current vector
      CVector new_ball_vec = k->theAgent().GetRelativeVectorTo(k->theBall().GetPosition(), NOW);
      // To get the relative vector subtract my body_direction.
      k->theBall().SetVector(new_ball_vec, NOW);
      if ( showInfo ) {
	std::cout << "Estimated the ball's pos to " << k->theBall().GetPosition(NOW)
	     << " vector to " << k->theBall().GetVector(NOW) << std::endl;
      }
    }

    if ( !seen_ball &&
	 k->theBall().ShouldBeSeen(k->theAgent().GetNeckDir(NOW),
				   k->theAgent().GetViewAngle(NOW), NOW) ) {
      if ( showInfo ) {
	std::cout << k->theAgent().GetUNumber() << " Should've seen the ball!!"
		  << " Ballpos = " << k->theBall().GetPosition(NOW)
		  << "  Ballvec = " << k->theBall().GetVector(NOW)
		  << std::endl;
      }
      logf->LogText("Should've seen the ball!");
      k->theBall().SetConfidence(0, NOW);
    } 

    // From seenPlayers fetch all Teammates and Opponents
    //seenTeammates.MakeEmpty();
    //seenOpponents.MakeEmpty();
    seenPlayers.GetPlayersFrom(T_Our_Team, &seenTeammates);
    seenPlayers.GetPlayersFrom(T_Their_Team, &seenOpponents);
    
    if ( showSeenPlayers ) {
      if ( seenTeammates.GetNumberOfPlayers() > 0 ) {
	std::cout << "Found " << seenTeammates.GetNumberOfPlayers()
		  << " teammates." << std::endl;
	//seenTeammates.Print();
      }
      if ( seenOpponents.GetNumberOfPlayers() > 0 ) {
	std::cout << "Found " << seenOpponents.GetNumberOfPlayers()
		  << " opponents." << std::endl;
	//seenOpponents.Print();
      }
      // As a test!
      std::cout << "Found "
		<< seenPlayers.GetPlayersFrom(T_Unknown).GetNumberOfPlayers()
		<< " unknown players." << std::endl;
    }
  }
}

void CBasicIP::SeeMarker(const t_marker marker, const float_type dist,
			 const float_type ang, const CTime& time,
			 const bool behind)
{

  CMarkerObject tmp_marker(k->GetMarker(marker));
  tmp_marker.SetMarkerId(marker);
  tmp_marker.SetDistance(dist, NOW);
  tmp_marker.SetDirection(ang, NOW);
  
  if (behind) {
    seenMarkersBehind.push(tmp_marker);
    seen_markers_behind++;
  } else {
    seenMarkers.push(tmp_marker);
    seen_markers++;
  }

  /*
  if (!found_closest_flag) {
    closest_flag = k->GetMarker(marker);
    found_closest_flag = true;
    closest_flag.SetMarkerId(marker);
    closest_flag.SetDistance(dist, HIST);
    closest_flag.SetDirection(ang, HIST);
  } else if (dist < closest_flag.GetDistance(-HIST)) {
    closest_flag = k->GetMarker(marker);
    closest_flag.SetMarkerId(marker);
    closest_flag.SetDistance(dist, HIST);
    closest_flag.SetDirection(ang, HIST);
  }
  */
  
  if ((k->theGame().GetOurSide() == S_Left && marker == Goal_R) ||
      (k->theGame().GetOurSide() == S_Right && marker == Goal_L)) {
    k->theirGoal.SetDirection(ang, HIST);
    k->theirGoal.SetDistance(dist, HIST);    
  } else {
    k->ourGoal.SetDirection(ang, HIST);
    k->ourGoal.SetDistance(dist, HIST);    
  }
  
}

void CBasicIP::SeeMarker(const t_marker marker, const float_type ang,
			 const CTime& time, const bool behind)
{

  if ((k->theGame().GetOurSide() == S_Left && marker == Goal_R) ||
      (k->theGame().GetOurSide() == S_Right && marker == Goal_L)) {
    k->theirGoal.SetDirection(ang, HIST);
  } else {
    k->ourGoal.SetDirection(ang, HIST);
  }

}

void CBasicIP::SeeLine(const t_side_line line, const float_type ang, const CTime& time)
{

  seenLines[seen_lines_pos].SetDirection(ang, HIST);
  seenLines[seen_lines_pos].SetLineId(line);
  seenLines[seen_lines_pos].SetObsTime(time, HIST);
  seen_lines_pos++;
  
}

void CBasicIP::SeeLine(const t_side_line line, const float_type dist, const float_type ang, const CTime& time)
{

  seenLines[seen_lines_pos].SetDirection(ang, HIST);
  seenLines[seen_lines_pos].SetDistance(dist, HIST);
  seenLines[seen_lines_pos].SetLineId(line);
  seenLines[seen_lines_pos].SetObsTime(time, HIST);
  seen_lines_pos++;

}

void CBasicIP::SeeBall(const float_type ang, const CTime& time)
{

  k->theBall().SetDirection(ang, HIST);
  k->theBall().SetObsTime(time, HIST);
  k->theBall().SetConfidence(1.0, HIST);
  
  seen_ball = true;

  //  std::cout << "Saw the ball, angle = " << ang << std::endl;
}

void CBasicIP::SeeBall(const float_type dist, const float_type ang, const CTime& time)
{

  k->theBall().SetDistance(dist, HIST);
  k->theBall().SetDirection(ang, HIST);
  k->theBall().SetObsTime(time, HIST);
  k->theBall().SetConfidence(1.0, HIST);
  
  seen_ball = true;
  
  //  std::cout << "Saw the ball, dist = " << dist << ", angle = " << ang << std::endl;
}

void CBasicIP::SeeBall(const float_type dist, const float_type ang, const float_type dist_chng, const float_type dir_chng, const CTime& time)
{

  k->theBall().SetDistance(dist, HIST);
  k->theBall().SetDirection(ang, HIST);
  k->theBall().SetDistChanged(dist_chng, HIST);
  k->theBall().SetDirChanged(dir_chng, HIST);
  k->theBall().SetObsTime(time, HIST);
  k->theBall().SetConfidence(1.0, HIST);
  
  seen_ball = true;
  seen_ball_moving = true;
  k->theBall().SetLastSeenMovingTime(time);

  /*
  std::cout << "Saw the ball moving, dist_chng = " << dist_chng
	    << ", dir_chng = " << dir_chng << std::endl;
	    */
}

void CBasicIP::SeePlayer(const float_type ang, const CTime& time)
{

  // How should I use this information to decide what player to update?
  // He's to far away to be "useful" anyway, therefore ignore him?
  
  CPlayerObject tmp(k->GetParameters());
  tmp.SetDirection(ang, HIST);
  tmp.SetDistance(UNKNOWN_FLOAT_TYPE, HIST);
  tmp.SetSide(S_Unknown);
  tmp.SetTeam(T_Unknown);
  tmp.SetObsTime(time, HIST);
  tmp.SetConfidence(1, HIST);

  // Add the relevant data to the seenPlayers structure.
  seenPlayers.AddPlayer(tmp);
  
}

void CBasicIP::SeePlayer(const float_type dist, const float_type ang, const CTime& time)
{

  // Find the player that is most likely the one seen by finding the player that
  // is closest to the point my_pos + vector(ang, dist)??
  
  CPlayerObject tmp(k->GetParameters());
  tmp.SetDirection(ang, HIST);
  tmp.SetDistance(dist, HIST);
  tmp.SetSide(S_Unknown);
  tmp.SetTeam(T_Unknown);
  tmp.SetObsTime(time, HIST);
  tmp.SetConfidence(1, HIST);

  // Add the relevant data to the seenPlayers structure.
  seenPlayers.AddPlayer(tmp);

}

void CBasicIP::SeePlayer(const t_side player_side, const float_type ang, const CTime& time)
{

  CPlayerObject tmp(k->GetParameters());
  tmp.SetDirection(ang, HIST);
  tmp.SetSide(player_side);
  tmp.SetObsTime(time, HIST);
  tmp.SetConfidence(1, HIST);
  tmp.SetTeam(k->theGame().GetOurSide());

  // Add the relevant data to the seenPlayers structure.
  seenPlayers.AddPlayer(tmp);

}

void CBasicIP::SeePlayer(const t_side player_side, const float_type dist, const float_type ang, const CTime& time)
{

  // Now we are getting somewhere. Find the teammate that is closest to
  // the point my_pos + vector(ang, dist).
  
  CPlayerObject tmp(k->GetParameters());
  tmp.SetDirection(ang, HIST);
  tmp.SetDistance(dist, HIST);
  tmp.SetSide(player_side);
  tmp.SetObsTime(time, HIST);
  tmp.SetConfidence(1, HIST);
  tmp.SetTeam(k->theGame().GetOurSide());
  
  // Add the relevant data to the seenPlayers structure.
  seenPlayers.AddPlayer(tmp);

}

void CBasicIP::SeePlayer(const t_side player_side, const int_type player_number, const float_type dist, const float_type ang, const CTime& time)
{

  // Since we have the number of the player we can hopefully match on that
  // otherwise find the teammate/opponent that is closest to the point my_pos + vec(a,d)
  
  CPlayerObject tmp(k->GetParameters());
  tmp.SetDirection(ang, HIST);
  tmp.SetDistance(dist, HIST);
  tmp.SetSide(player_side);
  tmp.SetUNumber(player_number);
  tmp.SetObsTime(time, HIST);
  tmp.SetConfidence(1, HIST);
  tmp.SetTeam(k->theGame().GetOurSide());

  // Add the relevant data to the seenPlayers structure.
  seenPlayers.AddPlayer(tmp);

}

void CBasicIP::SeePlayer(const t_side player_side, const int_type player_number, const float_type dist, const float_type ang, const float_type dist_chng, const float_type dir_chng, const float_type body_dir, const float_type face_dir, const CTime& time)
{

  // Since we have the number of the player we can hopefully match on that
  // otherwise find the teammate/opponent that is closest to the point my_pos + vec(a,d)
  
  CPlayerObject tmp(k->GetParameters());
  tmp.SetDirection(ang, HIST);
  tmp.SetDistance(dist, HIST);
  tmp.SetSide(player_side);
  tmp.SetUNumber(player_number);
  tmp.SetDistChanged(dist_chng, HIST);
  tmp.SetDirChanged(dir_chng, HIST);
  tmp.SetBodyDir(body_dir, HIST);
  tmp.SetNeckDir(face_dir, HIST);
  tmp.SetObsTime(time, HIST);
  tmp.SetConfidence(1, HIST);
  tmp.SetTeam(k->theGame().GetOurSide());

  // Add the relevant data to the seenPlayers structure.
  seenPlayers.AddPlayer(tmp);

}

void CBasicIP::CatchBall(const t_side side)
{
  // If I'm the goalie and I caught the ball, then save the relative distance
  // to the ball. Otherwise try to estimate the relative distance between the
  // ball and the goalie who caught the ball.
  if ( k->theAgent().IsGoalie() ) {
    CVector ball_vec = k->theBall().GetVector(CURRENT_EST);
    if ( ball_vec.GetLength() > params->SP_kickable_area )
      ball_vec.SetLength(params->SP_kickable_area * 0.75);
    k->theBall().SetRelativeBallPos(ball_vec);
  } else {
    // Nothing at the moment...
  }
}

// Function to calculate the absolut face direction. Stolen from ATHumboldt.
CAngleDeg CBasicIP::ComputeFaceDir()
{
  //CAngleDeg angle = 0;
  //int_type lineId = 0;
  // zuerst die Linie auswaehlen, deren Winkel ausgewertet wird.
  switch ( seen_lines_pos ) {
    case 0:
      std::cout << "Cannot compute face dir. Number of seen lines is 0." << std::endl;
      break;
    case 1:
      // Position ist innerhalb des Spielfeldes.
      return seenLines[0].GetAbsoluteAngle(NOW);
      //angle = seenLines[0].GetDirection(-HIST);
      //lineId = seenLines[0].GetLineId();
      break;
    default:
      // Die zweite Linie wird von innen gesehen, daher
      // kann der Winkel berechnet werden als waere man im inneren
      // des Feldes.
      int_type i;
      CAngleDeg sum = 0;
      /*
      if( seenLines[0].GetDistance(-HIST) > seenLines[1].GetDistance(-HIST) ) {
        i = 0;
      }
      else {
        i = 1;
      }
      */
      //CAngleDeg tmp;
      for (i = 0; i < seen_lines_pos; ++i) {
	//tmp = seenLines[i].GetAbsoluteAngle(NOW);
	//std::cout << "Angle " << i << " is " << tmp << std::endl;
	sum += seenLines[i].GetAbsoluteAngle(NOW);
      }
      //sum = sum / seen_lines_pos;
      //std::cout << "Saw " << seen_lines_pos << " lines, my face_dir is "
      //	<< sum << std::endl;
      return (sum / seen_lines_pos);
      //return seenLines[i].GetAbsoluteAngle(NOW);
      //angle = seenLines[i].GetDirection(-HIST);
      //lineId = seenLines[i].GetLineId();
      break;
  }
  /*
  // Jetzt abhaengig von der LinienId die absolute Blickrichtung berechnen 
  //std::cout <<  "Gewaehlte Linie: " << lineId << " Winkel: " << angle << "\n" << std::endl;
  switch ( lineId ) {
    case SL_Top:
      if ( angle > 0 ) {
        return -angle;
      }
      else {
        return 180 - angle;
      } 
      break;
    case SL_Bottom:
      if ( angle > 0 ) {
        return 180 - angle;
      }
      else {
        return -angle;
      }
      break;
    case SL_Left:
      if ( angle > 0 ) {
        return 270 - angle;
      }
      else if ( angle < 0 ) {
        return 90 - angle;
      } 
      else {
	std::cout << "Couldn't compute face dir. Special case 1 is not treated." << std::endl;
      }
      break;
    case SL_Right:
      if ( angle > 0 ) {
        return 90 - angle;
      }
      else if ( angle < 0 ) {
        return 270 - angle;
      } 
      else {
	std::cout << "Couldn't compute face dir. Special case 2 is not treated." << std::endl;
      }
      break;
    default:
      std::cout <<  "Unknown line id in computeFaceDir" << std::endl;
      break;
  }
  */
  return k->theAgent().GetFaceDir(NOW);
}

CPoint CBasicIP::ComputeAbsolutPosition(const CMarkerObject& flag, const CAngleDeg& face_dir)
{

  //std::cout << "Flag ID: " << flag.GetMarkerId() << std::endl;
  //std::cout << "Flag dist: " << flag.GetDistance(-HIST) << " dir: " << flag.GetDirection(-HIST);
  
  CVector rel_pos(flag.GetDirection(-HIST) + face_dir, flag.GetDistance(-HIST));

  //std::cout << " Flag pos: " << flag.GetPosition() << " rel pos: " << CPoint(rel_pos) << std::endl;
  
  return flag.GetPosition(-HIST) - rel_pos;
  
}

CTime CBasicIP::UpdateTime(const int_type time)
{
  // Mostly from CMUnited

  CTime current_time = k->theGame().GetCurrentTime();
  CTime last_time = current_time;
  
  // If the clock is stopped then time should be equal to current_time.t
  if ( clock_stopped ) {
    if ( false && current_time.GetT() != time ) {
      std::cerr << "Clock is stopped, but time different! new time:"
		<< time << " old time: " << current_time << std::endl;
      /*	
      if ( current_time.t == time - 1 ) // Sometimes happens in offsides mode
        current_time = CTime(time,0);
      else
        my_error("server time should be the same %d %d %d",current_time.t, current_time.s, time);
      */
    }
    else {
      // Update the server-time. Transform msec to server-cycles.
      // stopped_clock_msec keeps track of how much time (in msec) has
      // passed by since the clock stopped.
      // This variable must be updated by the server interface or info
      // processing each simulator cycle.
      current_time.SetS(stopped_clock_msec/params->SP_simulator_step);
    }
  } else if ( last_stop_clock_time.GetT() == time ) {
#ifdef DEBUG
    std::cerr << "last_stop_clock_time " << last_stop_clock_time
	      << " == time " << time << std::endl;
#endif
    current_time = last_stop_clock_time;
  } else {
    current_time = CTime(time,0);
  }
  
  k->theGame().SetCurrentTime(current_time);

  if ( last_time < current_time ) {
    //std::cout << "UPDATE NEW CYCLE!\n";
    UpdateNewCycle();
  }
  
  return current_time;
}

void CBasicIP::UpdateNewCycle()
{
  // logf->LogText("NEW CYCLE!");

  bool showT = false;
  
  if ( seenTeammates.GetNumberOfPlayers() > 0 )
    showT = false;

  if ( showT ) {
    std::cout << "I NEWCYCLE!!\n";
    seenTeammates.Print();
  }
  
  k->theAgent().NewCycle();
  k->theBall().NewCycle();
  k->thePlayers().NewCycle();
  seenTeammates.NewCycle();
  seenOpponents.NewCycle();
  k->theirGoal.NewCycle();
  k->ourGoal.NewCycle();

  if ( showT )
    seenTeammates.Print();
  
  k->theAgent().UpdateMovement(PREV, NOW);
  k->theAgent().UpdateStamina(PREV, NOW);
  k->theAgent().UpdateCatchInfo();
  k->theAgent().UpdateAuditoryInfo();

  k->theBall().UpdateMovement(PREV, NOW);
  if ( k->theGame().GetPlayMode() == PM_Our_Goalie_Got_Ball )
    k->theBall().SetPosition(k->theAgent().GetPosition(NOW)
			     + k->theBall().GetRelativeBallPos(), NOW);
  k->theBall().UpdateVector(k->theAgent().GetPosition(NOW),
			    k->theAgent().GetBodyDir(NOW), NOW);

  k->thePlayers().SetAccelerations(CVector(0,0), PREV);
  k->thePlayers().UpdateMovements(PREV, NOW);
  k->thePlayers().UpdateVectors(k->theAgent().GetPosition(NOW),
				k->theAgent().GetBodyDir(NOW), NOW);

  seenTeammates.SetAccelerations(CVector(0,0), PREV);
  seenTeammates.UpdateMovements(PREV, NOW);
  seenTeammates.UpdateVectors(k->theAgent().GetPosition(NOW),
			      k->theAgent().GetBodyDir(NOW), NOW);

  seenOpponents.SetAccelerations(CVector(0,0), PREV);
  seenOpponents.UpdateMovements(PREV, NOW);
  seenOpponents.UpdateVectors(k->theAgent().GetPosition(NOW),
			      k->theAgent().GetBodyDir(NOW), NOW);

  if ( showT )
    seenTeammates.Print();
  
  k->theirGoal.UpdateVector(k->theAgent().GetPosition(NOW),
			    k->theAgent().GetBodyDir(NOW), NOW);

  k->theirGoal.UpdateVector(k->theAgent().GetPosition(NOW),
			    k->theAgent().GetBodyDir(NOW), NOW);

  // Make a new estimation of where the players and the ball
  // will be next turn.
  UpdateEstimation();

  if ( showT )
    seenTeammates.Print();
  
}

void CBasicIP::SetPlayMode(const t_play_mode play_mode)
{

  // Mostly from CMUnited
  
#ifdef DEBUG
  std::cerr << "SetPlayMode(" << int_type(play_mode) << ")\n";
#endif
  
  // If the game is stopped, stop the clock
  if ( play_mode == PM_Before_Kick_Off ||
       play_mode == PM_Our_Offside_Kick ||
       play_mode == PM_Their_Offside_Kick ||
       play_mode == PM_Our_Goal ||
       play_mode == PM_Their_Goal ||
       play_mode == PM_Time_Over ) {
    // Reset stopped_clock_msec since UpdateTime was ran before
    // this function and UpdateTime already converted from msec to
    // server cycles.
    stopped_clock_msec = 0;
    clock_stopped = true;
  
  // Do anything special in half time or in extended time?
  } else if (play_mode == PM_Half_Time || play_mode == PM_Extended_Time) {
    if ( k->theGame().GetOurSide() == S_Left )
      k->theGame().SetOurSide(S_Right);
    else
      k->theGame().SetOurSide(S_Left);
    InitHalf();
  // Otherwise the game is resumed, start the clock
  // If clock is starting, save the old time
  } else if ( clock_stopped ) {
    clock_stopped = false;
    // Save the second last time the clock was started
    second_last_stop_clock_time = last_stop_clock_time;

    // Set the last_stop_clock_time to the time of the last action(?)
    // Is this based on the assumption that we always act when we can? Yes!
    last_stop_clock_time = k->theAgent().GetLastActTime();

    // Reset stopped_clock_msec
    stopped_clock_msec = 0; 

    // Maybe we have to check the times to see if they are valid.
    k->theGame().SetCurrentTime(SanitizeTime(k->theGame().GetCurrentTime()));
    k->theAgent().SetLastSightTime(SanitizeTime(k->theAgent().GetLastSightTime()));
    k->theAgent().SetLastHearTime(SanitizeTime(k->theAgent().GetLastHearTime()));
    k->theAgent().SetLastSenseTime(SanitizeTime(k->theAgent().GetLastSenseTime()));
  }

  /* This is already done in CatchBall()
  if ( ( k->theGame().GetPlayMode() == PM_Our_Goalie_Got_Ball ||
	 k->theGame().GetPlayMode() == PM_Their_Goalie_Got_Ball ) &&
       play_mode != PM_Our_Goalie_Got_Ball &&
       play_mode != PM_Their_Goalie_Got_Ball ) {
    k->theBall().SetRelativeBallPos(k->theBall().GetVector(HIST));
  }
  */
  
  if ( (k->theGame().GetPlayMode() == PM_Our_Goalie_Got_Ball &&
	play_mode == PM_Our_Free_Kick) ||
       (k->theGame().GetPlayMode() == PM_Their_Goalie_Got_Ball &&
	play_mode == PM_Their_Free_Kick) ) {
    return;
  }
  
  // Update the matchmode
  k->theGame().SetPlayMode(play_mode);

  // Update other things?
}

// Do we really need this? When does this occur?
CTime CBasicIP::SanitizeTime(const CTime& tm)
{
  if ( !last_stop_clock_time ) return tm;

  /* This is to take care of times that were prematurely updated
     before we knew that the clock was about to start again */
  // If the server time is equal but the stopped time is not equal
  // update the time to
  CTime new_tm = tm;
  if ( tm.GetT() == last_stop_clock_time.GetT() && tm.GetS() > last_stop_clock_time.GetS() ){
#ifdef DEBUG
    std::cerr << tm << "is sanitized to: ";
#endif
    new_tm = CTime(last_stop_clock_time.GetT() + (tm.GetS() - last_stop_clock_time.GetS()), 0);
#ifdef DEBUG
    std::cerr << new_tm << std::endl;
#endif
  }
  return new_tm;
}

void CBasicIP::SetMySensedInfo(const float_type new_stamina, const float_type new_effort,
			       const float_type new_speed, const float_type new_head_angle,
			       const int_type new_kicks, const int_type new_dashes,
			       const int_type new_turns, const int_type new_says,
			       const int_type new_turn_necks, const CTime& time)
{

#ifdef DEBUG
  std::cout << k->theAgent().GetUNumber() << " SensedInfo came at " << time << "\n";
#endif  

  bool ShowInfo = false;
  //  bool test = false;
  
  if ( false && ShowInfo ) {
    std::cout << "Time = " << k->theGame().GetCurrentTime();
    std::cout << " Last Sight = " << k->theAgent().GetLastSightTime()
	      << " Last Sense = " << k->theAgent().GetLastSenseTime()
	      << " Last Act = " << k->theAgent().GetLastActTime()
	      << std::endl;
  }

  if ( ShowInfo ) {
    std::cout << "Before update: " << std::endl;
    std::cout << "The agent: " << std::endl;
    //    k->theAgent().Print(0);
    //    k->theAgent().Print(1);
    std::cout << "[-1] pos: " << k->theAgent().GetPosition(PREV)
	 << " bodydir: " << k->theAgent().GetBodyDir(PREV)
	 << " neckdir: " << k->theAgent().GetNeckDir(PREV)
	 << std::endl;
    std::cout << "[0] pos: " << k->theAgent().GetPosition(NOW)
	 << " bodydir: " << k->theAgent().GetBodyDir(NOW)
	 << " neckdir: " << k->theAgent().GetNeckDir(NOW)
	 << std::endl;
    std::cout << "[1] pos: " << k->theAgent().GetPosition(NEXT)
	 << " bodydir: " << k->theAgent().GetBodyDir(NEXT)
	 << " neckdir: " << k->theAgent().GetNeckDir(NEXT)
	 << std::endl;
    std::cout << "The ball: " << std::endl;
    std::cout << "[0] pos: " << k->theBall().GetPosition()
	 << " vector: " << k->theBall().GetVector()
	 << " speed: " << k->theBall().GetAbsoluteSpeedVector()
	 << std::endl;
    std::cout << "[1] pos: " << k->theBall().GetPosition(NEXT)
	 << " vector: " << k->theBall().GetVector(NEXT)
	 << " speed: " << k->theBall().GetAbsoluteSpeedVector(NEXT)
	 << std::endl;
    //    k->theBall().Print(0);
    //    k->theBall().Print(1);
  }
  
  k->theAgent().SetNeckDir(new_head_angle, NOW);

  // Do Turn need the current speed or was the speed updated AFTER the turn?!
  UpdateAfterCommands(new_dashes, new_kicks, new_says, new_turns, new_turn_necks);

  //std::cout << k->theAgent().GetLastCommandQueue().Length()
  //    << " commands in the command queue.\n";
  if ( k->theAgent().GetLastCommandQueue().Length() > 1 ) {
    //std::cout << k->theAgent().GetLastCommand()->ServerString()
    //      << " " << k->theAgent().GetLastCommandQueue().Back()->ServerString()
    //      << std::endl;
    tmp_queue.MakeEmpty();
    tmp_queue.Push(k->theAgent().GetLastCommand());
    k->theAgent().GetLastCommandQueue().Remove(k->theAgent().GetLastCommandQueue().Length() - 1);
  }
  //k->theAgent().GetLastCommandQueue().MakeEmpty();
  
  k->theBall().UpdateVector(k->theAgent().GetPosition(NOW),
			    k->theAgent().GetBodyDir(NOW), NOW);
  k->thePlayers().UpdateVectors(k->theAgent().GetPosition(NOW),
				k->theAgent().GetBodyDir(NOW), NOW);
  seenTeammates.UpdateVectors(k->theAgent().GetPosition(NOW),
			      k->theAgent().GetBodyDir(NOW), NOW);
  seenOpponents.UpdateVectors(k->theAgent().GetPosition(NOW),
			      k->theAgent().GetBodyDir(NOW), NOW);
  k->theirGoal.UpdateVector(k->theAgent().GetPosition(NOW),
			    k->theAgent().GetBodyDir(NOW), NOW);
  k->ourGoal.UpdateVector(k->theAgent().GetPosition(NOW),
			  k->theAgent().GetBodyDir(NOW), NOW);  
   
  k->theAgent().SetFaceDir(k->theAgent().GetBodyDir(NOW) + new_head_angle, NOW);

  k->theAgent().SetDashes(new_dashes, NOW);
  k->theAgent().SetKicks(new_kicks, NOW);
  k->theAgent().SetSays(new_says, NOW);
  k->theAgent().SetTurns(new_turns, NOW);
  k->theAgent().SetTurnNecks(new_turn_necks, NOW);

  k->theAgent().SetStamina(new_stamina, NOW);
  k->theAgent().SetEffort(new_effort, NOW);
  k->theAgent().SetSpeed(new_speed, NOW);

  k->theAgent().SetLastSenseTime(time);

  if ( ShowInfo ) {
    std::cout << "After update: " << std::endl;
    std::cout << "The agent: " << std::endl;
    //k->theAgent().Print(0);
    std::cout << "[-1] pos: " << k->theAgent().GetPosition(PREV)
	 << " bodydir: " << k->theAgent().GetBodyDir(PREV)
	 << " neckdir: " << k->theAgent().GetNeckDir(PREV)
	 << std::endl;
    std::cout << "[0] pos: " << k->theAgent().GetPosition(NOW)
	 << " bodydir: " << k->theAgent().GetBodyDir(NOW)
	 << " neckdir: " << k->theAgent().GetNeckDir(NOW)
	 << std::endl;
    std::cout << "[1] pos: " << k->theAgent().GetPosition(NEXT)
	 << " bodydir: " << k->theAgent().GetBodyDir(NEXT)
	 << " neckdir: " << k->theAgent().GetNeckDir(NEXT)
	 << std::endl;
    std::cout << "The ball: " << std::endl;
    //k->theBall().Print(0);
    //k->theBall().Print(1);
    std::cout << "[0] pos: " << k->theBall().GetPosition(NOW)
	 << " vector: " << k->theBall().GetVector(NOW)
	 << " speed: " << k->theBall().GetAbsoluteSpeedVector(NOW)
	 << std::endl;
    std::cout << "[1] pos: " << k->theBall().GetPosition(NEXT)
	 << " vector: " << k->theBall().GetVector(NEXT)
	 << " speed: " << k->theBall().GetAbsoluteSpeedVector(NEXT)
	 << std::endl;
  }

}

void CBasicIP::UpdateAfterCommands(int_type ds, int_type ks, int_type ss,
				   int_type ts, int_type ns)
{
  // Update for the commands executed since last sense_body

  //std::cout << k->theAgent().GetTurns(PREV) << " "
  //    << k->theAgent().GetTurns(NOW) << " " << ts << " "
  //    << k->theAgent().GetTurns(NEXT) << std::endl;
  
  ds -= k->theAgent().GetDashes(NOW);
  ks -= k->theAgent().GetKicks(NOW);
  ss -= k->theAgent().GetSays(NOW);
  ts -= k->theAgent().GetTurns(NOW);
  ns -= k->theAgent().GetTurnNecks(NOW);

#ifdef DEBUG
  if ( ds < 0 ||
       ks < 0 ||
       ss < 0 ||
       ts < 0 ||
       ns < 0 )
    std::cerr << "Negative number of commands executed!!!"
	      << " ds: " << ds << " ks: " << ks << " ss: " << ss
	      << " ts: " << ts << " ns: " << ns << std::endl;
#endif

  //std::cout << "Should update " << ds+ks+ss+ts+ns << " commands.\n";

    if ( ds+ks+ss+ts /*+ns*/ > 0 && k->theAgent().GetLastCommand() == NULL ) {
      if ( tmp_queue.Front() != NULL ) {
	//std::cout << "Added " << tmp_queue.Front()->ServerString() << " to the command queue.\n";
	k->theAgent().SetLastCommand(tmp_queue.Front());
	tmp_queue.MakeEmpty();
      }
    }

  while ( ds+ks+ss+ts /*+ns*/ > 0 && k->theAgent().GetLastCommand() != NULL ) {
    const CCommand* cmd = k->theAgent().GetLastCommand();
    if (cmd != NULL) {
      logf->LogText("UpdateAfter: " + cmd->ServerString());
      switch (cmd->GetType()) {
      case CMD_Dash:
	if ( ds > 0 ) {
	  //	  if ( k->theAgent().GetSees() < 1 ) {

	  // Since I need to set the speed I always update.
	  // The precision is not that much different.
	  cmd->UpdateAfterCommand(PREV, NOW);

	  //	  } else {
	  //	    std::cerr << cmd->ServerString() << " skipped since I already saw.\n";
	  //	    logf->LogText("Didn't update since I already saw!");
	  //	  }
	  --ds;
	} else {
	  //std::cerr << cmd->ServerString() << " didn't get executed." << std::endl;
	  logf->LogText(cmd->ServerString() + " didn't get exectued.");
	  //k->theAgent().SetLastCommand(cmd);
	}
	break;
      case CMD_Kick:
	if ( ks > 0 ) {
	  //	  if ( k->theAgent().GetSees() < 1 &&
	  //	       k->theBall().GetObsTime(NOW) == k->theGame().GetCurrentTime() ) {

	  // Since I need to set the speed I always update.
	  // The precision is not that much different.
	  cmd->UpdateAfterCommand(PREV, NOW);

	  //	  } else {
	  //	    std::cerr << cmd->ServerString() << " skipped since I already saw the ball.\n";
	  //	    logf->LogText("Didn't update since I already saw the ball!");
	  //	  }
	  --ks;
	} else {
	  //std::cerr << cmd->ServerString() << " didn't get executed." << std::endl;
	  logf->LogText(cmd->ServerString() + " didn't get exectued.");
	  //k->theAgent().SetLastCommand(cmd);
	}
	break;
      case CMD_Move:
	cmd->UpdateAfterCommand(PREV, NOW);
      case CMD_Say:
	if ( ss > 0 ) {
	  cmd->UpdateAfterCommand(PREV, NOW);
	  --ss;
	} else {
	  //std::cerr << cmd->ServerString() << " didn't get executed." << std::endl;
	  logf->LogText(cmd->ServerString() + " didn't get exectued.");		  
	  //k->theAgent().SetLastCommand(cmd);
	}
	break;
      case CMD_Turn:
	if ( ts > 0 ) {
	  if ( k->theAgent().GetSees() < 1 ) {
	    cmd->UpdateAfterCommand(PREV, NOW);
	  } else {
	    //std::cerr << cmd->ServerString() << " skipped since I already saw.\n";
	    //logf->LogText("Didn't update since I already saw!");
	  }
	  --ts;
	} else {
	  //std::cerr << cmd->ServerString() << " didn't get executed." << std::endl;
	  logf->LogText(cmd->ServerString() + " didn't get exectued.");
	  //k->theAgent().SetLastCommand(cmd);
	}
	break;
      case CMD_TurnNeck:
	if ( ns > 0 ) {
	  // Is there ever a need to update after a turn_neck?
	  // Since I get the current neck_angle from sense_body I don't
	  // think so!
	  // cmd->UpdateAfterCommand(PREV, NOW);
	  //std::cerr << cmd->ServerString() << " skipped since I already sensed.\n";
	  //logf->LogText("Didn't update since I already sensed!");
	  --ns;
	} else {
	  //std::cerr << cmd->ServerString() << " didn't get executed." << std::endl;
	  logf->LogText(cmd->ServerString() + " didn't get exectued.");
	  //k->theAgent().SetLastCommand(cmd);
	}
	break;
      default:
	//std::cerr << cmd->ServerString() << " can't be updated." << std::endl;
	logf->LogText(cmd->ServerString() + " can't be updated.");
	//      k->theAgent().SetLastCommand(cmd);
      }
      k->theAgent().RemoveLastCommand();
    } else
      std::cerr << "No command in theAgent.CommandQueue!!\n";

    if ( ds+ks+ss+ts /*+ns*/ > 0 && k->theAgent().GetLastCommand() == NULL ) {
      if ( tmp_queue.Front() != NULL ) {
	//std::cout << "Added " << tmp_queue.Front()->ServerString() << " to the command queue.\n";
	k->theAgent().SetLastCommand(tmp_queue.Front());
	tmp_queue.MakeEmpty();
      }
    }

  }
  
  if ( false && ds+ks+ss+ts/*+ns*/ > 0 ) {
    std::cerr << ds+ks+ss+ts+ns
	      << " commands left to update and no command in theAgent.CommandQueue!!\n";
    std::cerr << ds << ks << ss << ts << ns << std::endl; 
    logf->LogText(ToString(ds+ks+ss+ts+ns) +
		  " commands left to update and no command in theAgent.CommandQueue!");
  }
  
}

void CBasicIP::UpdateEstimation()
{
  // Do an estimation on the next position of the ball, if nothing
  // happens to it...
  // Does this have any side-effects???
  // Is anything done to this information???

  bool ShowInfo = false;
  //  bool test = false;

  if ( ShowInfo) {
    std::cout << "Before estimation: " << std::endl;
    std::cout << "Player: "
         << "Pos: " << k->theAgent().GetPosition(NOW)
         << " NeckDir: " << k->theAgent().GetNeckDir(NOW)
         << " Bodydir: " << k->theAgent().GetBodyDir(NOW)
         << " Speed: " << k->theAgent().GetAbsoluteSpeedVector(NOW)
         << std::endl;
    std::cout << "Ball: "
         << "Pos: " << k->theBall().GetPosition(NOW)
         << " Vector: " << k->theBall().GetVector(NOW)
         << " Speed: " << k->theBall().GetAbsoluteSpeedVector(NOW)
         << std::endl;
  }
  
  k->theAgent().Copy(NOW, CLEAN_EST);
  k->theAgent().SetAcceleration(CVector(0,0), NOW);
  k->theAgent().UpdateMovement(NOW, CLEAN_EST);
  k->theAgent().Copy(CLEAN_EST, CURRENT_EST);
  
  k->theBall().Copy(NOW, CLEAN_EST);
  //  k->theBall().UpdateConfidence(NOW, CLEAN_EST);
  k->theBall().SetAcceleration(CVector(0,0), NOW);
  k->theBall().UpdateMovement(NOW, CLEAN_EST);
  if ( k->theGame().GetPlayMode() == PM_Our_Goalie_Got_Ball )
    k->theBall().SetPosition(k->theAgent().GetPosition(CLEAN_EST)
			     + k->theBall().GetRelativeBallPos(), CLEAN_EST);
  k->theBall().UpdateVector(k->theAgent().GetPosition(CLEAN_EST),
			    k->theAgent().GetBodyDir(CLEAN_EST), CLEAN_EST);
  k->theBall().Copy(CLEAN_EST, CURRENT_EST);

  k->thePlayers().Copy(NOW, CLEAN_EST);
  k->thePlayers().SetAccelerations(CVector(0,0), NOW);
  k->thePlayers().UpdateMovements(NOW, CLEAN_EST);
  k->thePlayers().UpdateVectors(k->theAgent().GetPosition(CLEAN_EST),
				k->theAgent().GetBodyDir(CLEAN_EST), CLEAN_EST);
  k->thePlayers().Copy(CLEAN_EST, CURRENT_EST);

  seenTeammates.Copy(NOW, CLEAN_EST);
  seenTeammates.SetAccelerations(CVector(0,0), NOW);
  seenTeammates.UpdateMovements(NOW, CLEAN_EST);
  seenTeammates.UpdateVectors(k->theAgent().GetPosition(CLEAN_EST),
			      k->theAgent().GetBodyDir(CLEAN_EST), CLEAN_EST);
  seenTeammates.Copy(CLEAN_EST, CURRENT_EST);
  
  seenOpponents.Copy(NOW, CLEAN_EST);
  seenOpponents.SetAccelerations(CVector(0,0), NOW);
  seenOpponents.UpdateMovements(NOW, CLEAN_EST);
  seenOpponents.UpdateVectors(k->theAgent().GetPosition(CLEAN_EST),
			      k->theAgent().GetBodyDir(CLEAN_EST), CLEAN_EST);
  seenOpponents.Copy(CLEAN_EST, CURRENT_EST);

  k->theirGoal.Copy(NOW, CLEAN_EST);
  k->theirGoal.UpdateVector(k->theAgent().GetPosition(CLEAN_EST),
			    k->theAgent().GetBodyDir(CLEAN_EST), CLEAN_EST);
  k->theirGoal.Copy(CLEAN_EST, CURRENT_EST);
  
  k->ourGoal.Copy(NOW, CLEAN_EST);
  k->ourGoal.UpdateVector(k->theAgent().GetPosition(CLEAN_EST),
			    k->theAgent().GetBodyDir(CLEAN_EST), CLEAN_EST);
  k->ourGoal.Copy(CLEAN_EST, CURRENT_EST);
  
  // Compensate for the action sent last turn that will take effect next turn.
  CCommand* tmp = k->theAgent().GetLastCommandQueue().Back();
  if ( tmp != NULL ) {
    tmp->UpdateAfterCommand(CLEAN_EST, CURRENT_EST);

    // Since I now can do TurnNeck, I should be able to look at more than
    // the last element in the queue.
    CCommand* tmp2 = k->theAgent().GetLastCommandQueue().SecondLast();
    if ( tmp2 != NULL && ( tmp->GetType() == CMD_TurnNeck ||
			   tmp2->GetType() == CMD_TurnNeck ) )
      tmp2->UpdateAfterCommand(CLEAN_EST, CURRENT_EST);
    
    if ( k->theGame().GetPlayMode() == PM_Our_Goalie_Got_Ball )
      k->theBall().SetPosition(k->theAgent().GetPosition(CURRENT_EST)
			       + k->theBall().GetRelativeBallPos(), CURRENT_EST);
    k->theBall().UpdateVector(k->theAgent().GetPosition(CURRENT_EST),
			      k->theAgent().GetBodyDir(CURRENT_EST), CURRENT_EST);
    k->thePlayers().UpdateVectors(k->theAgent().GetPosition(CURRENT_EST),
				  k->theAgent().GetBodyDir(CURRENT_EST), CURRENT_EST);
    seenTeammates.UpdateVectors(k->theAgent().GetPosition(CURRENT_EST),
				k->theAgent().GetBodyDir(CURRENT_EST), CURRENT_EST);
    seenOpponents.UpdateVectors(k->theAgent().GetPosition(CURRENT_EST),
				k->theAgent().GetBodyDir(CURRENT_EST), CURRENT_EST);
    //logf->LogText("Estimated that " + k->theAgent().GetLastCommandQueue().Back()->ServerString() + " will be executed this turn.");
  }
  
  k->theirGoal.UpdateVector(k->theAgent().GetPosition(CURRENT_EST),
			    k->theAgent().GetBodyDir(CURRENT_EST), CURRENT_EST);
  k->ourGoal.UpdateVector(k->theAgent().GetPosition(CURRENT_EST),
			    k->theAgent().GetBodyDir(CURRENT_EST), CURRENT_EST);

  // Do an approximation of the next state of the world!
  // Really don't know if it's necessary!
  k->theAgent().Copy(CURRENT_EST, NEXT_EST);
  k->theAgent().SetAcceleration(CVector(0,0), CURRENT_EST);
  k->theAgent().UpdateMovement(CURRENT_EST, NEXT_EST);
  
  k->theBall().Copy(CURRENT_EST, NEXT_EST);
  //  k->theBall().UpdateConfidence(CURRENT_EST, NEXT_EST);
  k->theBall().SetAcceleration(CVector(0,0), CURRENT_EST);
  k->theBall().UpdateMovement(CURRENT_EST, NEXT_EST);
  if ( k->theGame().GetPlayMode() == PM_Our_Goalie_Got_Ball )
    k->theBall().SetPosition(k->theAgent().GetPosition(NEXT_EST)
			     + k->theBall().GetRelativeBallPos(), NEXT_EST);
  k->theBall().UpdateVector(k->theAgent().GetPosition(NEXT_EST),
			    k->theAgent().GetBodyDir(NEXT_EST), NEXT_EST);

  k->thePlayers().Copy(CURRENT_EST, NEXT_EST);
  k->thePlayers().SetAccelerations(CVector(0,0), NOW);
  k->thePlayers().UpdateMovements(CURRENT_EST, NEXT_EST);
  k->thePlayers().UpdateVectors(k->theAgent().GetPosition(NEXT_EST),
				k->theAgent().GetBodyDir(NEXT_EST), NEXT_EST);

  seenTeammates.Copy(CURRENT_EST, NEXT_EST);
  seenTeammates.SetAccelerations(CVector(0,0), NOW);
  seenTeammates.UpdateMovements(CURRENT_EST, NEXT_EST);
  seenTeammates.UpdateVectors(k->theAgent().GetPosition(NEXT_EST),
			      k->theAgent().GetBodyDir(NEXT_EST), NEXT_EST);
  
  seenOpponents.Copy(CURRENT_EST, NEXT_EST);
  seenOpponents.SetAccelerations(CVector(0,0), NOW);
  seenOpponents.UpdateMovements(CURRENT_EST, NEXT_EST);
  seenOpponents.UpdateVectors(k->theAgent().GetPosition(NEXT_EST),
			      k->theAgent().GetBodyDir(NEXT_EST), NEXT_EST);

  k->theirGoal.Copy(CURRENT_EST, NEXT_EST);
  k->theirGoal.UpdateVector(k->theAgent().GetPosition(NEXT_EST),
			    k->theAgent().GetBodyDir(NEXT_EST), NEXT_EST);
  
  k->ourGoal.Copy(CURRENT_EST, NEXT_EST);
  k->ourGoal.UpdateVector(k->theAgent().GetPosition(NEXT_EST),
			    k->theAgent().GetBodyDir(NEXT_EST), NEXT_EST);
  
  if ( ShowInfo ) {
    std::cout << "After estimation: " << std::endl;
    std::cout << "Player: "
         << "Position: " << k->theAgent().GetPosition(CLEAN_EST)
         << " NeckDir: " << k->theAgent().GetNeckDir(CLEAN_EST)
         << " Bodydir: " << k->theAgent().GetBodyDir(CLEAN_EST)
         << " SpeedVector: " << k->theAgent().GetAbsoluteSpeedVector(CLEAN_EST)
         << std::endl;
    std::cout << "Ball: "
         << "Position: " << k->theBall().GetPosition(CLEAN_EST)
         << " Vector: " << k->theBall().GetVector(CLEAN_EST)
         << " SpeedVector: " << k->theBall().GetAbsoluteSpeedVector(CLEAN_EST)
         << std::endl;
  }

}

// Temporary until I write my own errorhandler!!
void CBasicIP::my_error(const char* msg)
{
  fprintf(stdout,"PETER's ERROR (player %d, time %d.%d): %s\n",
	  k->theAgent().GetUNumber(),k->theGame().GetCurrentTime().GetT(),
	  k->theGame().GetCurrentTime().GetS(),msg);
}

void CBasicIP::my_error(const char* msg, int_type param)
{
  char outstring[100];
  sprintf(outstring,msg,param);
  fprintf(stdout,"PETER's ERROR (player %d, time %d.%d): %s\n",
	  k->theAgent().GetUNumber(),k->theGame().GetCurrentTime().GetT(),
	  k->theGame().GetCurrentTime().GetS(),outstring);
}

// Parser adopted from CMU'98
void CBasicIP::Parse(char* sensory_info)
{

  t_sense sense_type = SENSE_Hear;
  int_type time;
  bool initmsg = false;

  //  std::cout << "In InfoProc: " << sensory_info << std::endl;
  
  switch ( sensory_info[3] ) {            
  case 'e': sense_type = SENSE_See;   break; /* see   */
  case 'n': sense_type = SENSE_Sense; break; /* sense */
  case 'a': sense_type = SENSE_Hear;  break; /* hear  */
  case 'i': initmsg = true;           break; /* init message */
  default:  my_error("Sent an illegal message"); return;
  }

  if ( initmsg ) {
    ParseInitMessage(sensory_info);
    InitHalf();
  } else {
    time = get_int_type(&sensory_info);         /* %d    */
    CTime tm = UpdateTime(time);

    //    int_type t;

    switch ( sense_type ){
    case SENSE_See:
      //if ( !k->theAgent().GetLastActTime() ) break; /* Don't parse until I've started counting time */

      /*
      if (gettimeofday(&tv, NULL) >= 0) {
	t = (tv.tv_sec-start.tv_sec)*10000 + int_type(tv.tv_usec/100) - last_see;
	//	std::cout << tm << " See: " << t << ":" << t + last_see - last_sense << "\t";
	last_see = (tv.tv_sec-start.tv_sec)*10000 +  int_type(tv.tv_usec/100);
      }
      */

      //logf->LogText("Saw at " + ToString(long(time)));
      k->theAgent().SetSees(k->theAgent().GetSees() + 1);
      //std::cout << "Start before sight ..." << std::endl;
      BeforeSight();
      ParseSight(tm, sensory_info); 
      k->theAgent().SetLastSightTime(tm);
      first_action_op_since_sight = true;
      //std::cout << "Start after sight ..." << std::endl;
      AfterSight();
      //std::cout << "Done with sight!\n" << std::endl;

      /*
      if (gettimeofday(&tv, NULL) >= 0) {
	t = ((tv.tv_sec-start.tv_sec)*10000 + int_type(tv.tv_usec/100) - last_see);
	//	std::cout << "See parsing: " << t << "\n";
      }
      */
      
      break;
    case SENSE_Sense:

      /*
      if (gettimeofday(&tv, NULL) >= 0) {
	t = (tv.tv_sec-start.tv_sec)*10000 + int_type(tv.tv_usec/100) - last_sense;
	//	std::cout << tm << " Sense: " << t << ":" << t + last_sense - last_see << "\t";
	last_sense = (tv.tv_sec-start.tv_sec)*10000 +  int_type(tv.tv_usec/100);
      }
      */

      k->theAgent().SetSenses(k->theAgent().GetSenses() + 1);
      // Doesn't work when clock is stopped!
      if ( k->theAgent().GetSenses() >= k->GetParameters()->RS_senses_per_cycle + 2 ) {	
	if ( clock_stopped ) {
	  tm.SetS(tm.GetS() + 1);
	  k->theGame().SetCurrentTime(tm);
	  //std::cout << "UPDATE NEW CYCLE!\n";
	  UpdateNewCycle();
	} else {
	  tm = UpdateTime(time + 1);
	}
	k->theAgent().SetSees(0);
	k->theAgent().SetSenses(1);
      }
      //logf->LogText("Sensed at " + ToString(long(time)));
      ParseSense(tm, sensory_info);      
      k->theAgent().SetLastSenseTime(tm);

      /*
      if (gettimeofday(&tv, NULL) >= 0) {
	t = ((tv.tv_sec-start.tv_sec)*10000 + int_type(tv.tv_usec/100) - last_sense);
	//	std::cout << "Sense parsing: " << t << "\n";
      }
      */
      
      break;
    case SENSE_Hear:  
      // if ( !k->theAgent().GetLastActTime() ) break; /* Don't parse until I've started counting time */
      //logf->LogText("Heared at " + ToString(long(time)));
      ParseSound(tm, sensory_info); 
      k->theAgent().SetLastHearTime(tm);
      break;
    }
    
    k->theAgent().SetLastSenseType(sense_type);
    UpdateEstimation();
  }
}

/*---------------------------------------------------------------------------------*/

/* Parse initialize message */
// Code adopted from CMUnited'98 
void CBasicIP::ParseInitMessage(char* init_info)
{

  k->theGame().SetCurrentTime(0);
  k->theAgent().SetLastActTime(0);

  t_side my_side;
  t_unum my_number;
  
  char mode[100];

  if ( !(strncmp(init_info,"(init",4)) ) {
    /* It's an init msg */
    sscanf(init_info,"(init %c %d %[^)]",&my_side, &my_number, mode);
  }
  else if ( !(strncmp(init_info,"(reconnect",4)) ) {
    std::cout << "reconnect!\n" << std::endl;
    /* It's a reconnect msg */
    sscanf(init_info,"(reconnect %c %[^)]",&my_side, mode);
    // Not used!
    // k->my_number = k->IP_reconnect; 
    printf("reconnecting to %d on side %c!\n",my_number,my_side);
  }

  /*
    if ( k->CP_goalie && k->FP_goalie_number != k->my_number )
    my_error("goalie number inconsistent with me being goalie");

  if ( !k->CP_goalie && k->FP_goalie_number == k->my_number )
    my_error("I should be the goalie");
  */
  
  if ( mode[0] == 'b' ){ /* Before_kick_off */ 
    SetPlayMode(PM_Before_Kick_Off);
    if ( my_side == S_Left ) {
      k->theGame().SetKickOffMode(KO_Ours);
      //      SetPlayMode(PM_Our_Kick_Off);
    } else {
      k->theGame().SetKickOffMode(KO_Theirs);
      //      SetPlayMode(PM_Their_Kick_Off);
    }
  }
  else                  /* Act as if the game's in progress */
    SetPlayMode(PM_Play_On);

  if ( my_side == S_Left )
    k->theGame().SetTheirSide(S_Right);
  else
    k->theGame().SetTheirSide(S_Left);

  k->theGame().SetOurSide(my_side);
  k->theAgent().SetUNumber(my_number);
  
  printf( "------------------------------------------------------------------------\n");
  printf( "Connected player %d on team %s\n",my_number,k->theGame().GetOurTeamName().c_str());
  printf( "He's playing on the %c side of the field.\n", my_side);
  printf( "------------------------------------------------------------------------\n");
  
}

/*---------------------------------------------------------------------------------*/

void CBasicIP::ParseSense(const CTime& time, char* sense_info)
{

  //  std::cout << sense_info << "\t";
  
  get_word(&sense_info);  
  sense_info += 10;       /* "view_mode " */

  switch ( sense_info[0] ){
  case 'h': k->theAgent().SetViewQuality(VQ_High); break;  /* high */
  case 'l': k->theAgent().SetViewQuality(VQ_Low);  break;  /* low  */
  default:  my_error("Unknown view quality");
  }

  get_next_word(&sense_info);  
  switch ( sense_info[1] ){
  case 'o': k->theAgent().SetViewWidth(VW_Normal); break;  /* normal */
  case 'a': k->theAgent().SetViewWidth(VW_Narrow); break;  /* narrow */
  case 'i': k->theAgent().SetViewWidth(VW_Wide);   break;  /* wide   */
  default:  my_error("Unknown view quality");
  }
   
  float_type stamina  = get_float_type(&sense_info);
  float_type effort   = get_float_type(&sense_info);
  float_type speed    = get_float_type(&sense_info);
  float_type head_ang = get_float_type(&sense_info);

  int_type   kicks      = get_int_type(&sense_info);
  int_type   dashes     = get_int_type(&sense_info);
  int_type   turns      = get_int_type(&sense_info);
  int_type   says       = get_int_type(&sense_info);
  int_type   turn_necks = get_int_type(&sense_info);

  SetMySensedInfo(stamina,effort,speed,head_ang,kicks,dashes,turns,says,turn_necks,time);
}

/****************************************************************************************/

void CBasicIP::ParseSight(const CTime& time, char* sight_info)
{

  //logf->LogText(sight_info);
  
  static const int_type NOCHNGINFO = -500;
  static const int_type NOFACEINFO = -501;
  static const int_type NOBODYINFO = -502;

  float_type dist = 9999;
  float_type ang = 0;
  float_type dir_chng = NOCHNGINFO; 
  float_type dist_chng = NOCHNGINFO;
  t_object object_type;  
  char player_side;
  t_unum player_number;
  float_type body_dir;
  float_type face_dir;
  t_marker marker = No_Marker;
  t_side_line line = SL_No_Line;
  t_view_quality view_qual;
  t_marker closest_marker = No_Marker;
  bool process_this_marker;
  float_type closest_marker_dist = 0.0;
  /* float_type motion_info_dist = 1000; */

  t_side my_side = k->theGame().GetOurSide();
  t_side their_side = k->theGame().GetTheirSide();
  
  while (*sight_info != ')'){

    dir_chng = NOCHNGINFO;
    face_dir = NOFACEINFO;
    body_dir = NOBODYINFO;
    player_number = player_side = 0;

    get_word(&sight_info);            /* " ((" */

    if ( *sight_info=='g' ){
      object_type = OBJ_Marker;
      sight_info+=5;          /* "goal " */
      if ( *sight_info=='r' )         marker = Goal_R; else
      if ( *sight_info=='l' )         marker = Goal_L; else
      my_error("goal ?");
    } else
    if ( *sight_info=='G' ){
      object_type = OBJ_Marker_Behind;
      marker = k->GetClosestGoal().GetMarkerId();
    } else
    if ( *sight_info=='f' ){
      object_type = OBJ_Marker;
      sight_info+=5;          /* "flag " */
      if ( *sight_info=='r' ){
	sight_info+=2;
	if ( *sight_info=='0' )       marker = Flag_R0;   else
	if ( *sight_info=='b' ){
	  sight_info+=1;
	  if ( *sight_info==')' )     marker = Flag_RB; 
	  else{
	    sight_info+=1;
	    if ( *sight_info=='1' )   marker = Flag_RB10; else
	    if ( *sight_info=='2' )   marker = Flag_RB20; else
	    if ( *sight_info=='3' )   marker = Flag_RB30; else
	    my_error("flag r b ?");
	  }  
	} else
	if ( *sight_info=='t' ){
	  sight_info+=1;
	  if ( *sight_info==')' )     marker = Flag_RT; 
	  else{
	    sight_info+=1;
	    if ( *sight_info=='1' )   marker = Flag_RT10; else
	    if ( *sight_info=='2' )   marker = Flag_RT20; else
	    if ( *sight_info=='3' )   marker = Flag_RT30; else
	    my_error("flag r t ?");
	  }  
	} else
	my_error("flag r ?");
      } else
      if ( *sight_info=='l' ){
	sight_info+=2;
	if ( *sight_info=='0' )       marker = Flag_L0;   else
	if ( *sight_info=='b' ){
	  sight_info+=1;
	  if ( *sight_info==')' )     marker = Flag_LB; 
	  else{
	    sight_info+=1;
	    if ( *sight_info=='1' )   marker = Flag_LB10; else
	    if ( *sight_info=='2' )   marker = Flag_LB20; else
	    if ( *sight_info=='3' )   marker = Flag_LB30; else
	    my_error("flag l b ?");
	  }  
	} else
	if ( *sight_info=='t' ){
	  sight_info+=1;
	  if ( *sight_info==')' )     marker = Flag_LT; 
	  else{
	    sight_info+=1;
	    if ( *sight_info=='1' )   marker = Flag_LT10; else
	    if ( *sight_info=='2' )   marker = Flag_LT20; else
	    if ( *sight_info=='3' )   marker = Flag_LT30; else
	    my_error("flag l t ?");
	  }  
	} else
	my_error("flag l ?");
      } else
      if ( *sight_info=='t' ){
	sight_info+=2;
	if ( *sight_info=='0' )       marker = Flag_T0;   else
	if ( *sight_info=='l' ){
	  sight_info+=2;
	  if ( *sight_info=='1' )     marker = Flag_TL10; else
	  if ( *sight_info=='2' )     marker = Flag_TL20; else
	  if ( *sight_info=='3' )     marker = Flag_TL30; else
	  if ( *sight_info=='4' )     marker = Flag_TL40; else
	  if ( *sight_info=='5' )     marker = Flag_TL50; else
	  my_error("flag t l ?");
	} else
	if ( *sight_info=='r' ){
	  sight_info+=2;
	  if ( *sight_info=='1' )     marker = Flag_TR10; else
	  if ( *sight_info=='2' )     marker = Flag_TR20; else
	  if ( *sight_info=='3' )     marker = Flag_TR30; else
	  if ( *sight_info=='4' )     marker = Flag_TR40; else
	  if ( *sight_info=='5' )     marker = Flag_TR50; else
	  my_error("flag t r ?");
	} else
	my_error("flag t ?");
      } else
      if ( *sight_info=='b' ){
	sight_info+=2;
	if ( *sight_info=='0' )       marker = Flag_B0;   else
	if ( *sight_info=='l' ){
	  sight_info+=2;
	  if ( *sight_info=='1' )     marker = Flag_BL10; else
	  if ( *sight_info=='2' )     marker = Flag_BL20; else
	  if ( *sight_info=='3' )     marker = Flag_BL30; else
	  if ( *sight_info=='4' )     marker = Flag_BL40; else
	  if ( *sight_info=='5' )     marker = Flag_BL50; else
	  my_error("flag b l ?");
	} else
	if ( *sight_info=='r' ){
	  sight_info+=2;
	  if ( *sight_info=='1' )     marker = Flag_BR10; else
	  if ( *sight_info=='2' )     marker = Flag_BR20; else
	  if ( *sight_info=='3' )     marker = Flag_BR30; else
	  if ( *sight_info=='4' )     marker = Flag_BR40; else
	  if ( *sight_info=='5' )     marker = Flag_BR50; else
	  my_error("flag b r ?");
	} else
	my_error("flag b ?");
      } else
      if ( *sight_info=='c' ){
	sight_info+=1;
	if ( *sight_info==')' )       marker = Flag_C;
	else{
	  sight_info+=1;
	  if ( *sight_info=='b' )     marker = Flag_CB; else
	  if ( *sight_info=='t' )     marker = Flag_CT; else
          my_error("flag c ?");
	}
      } else
      if ( *sight_info=='p' ){
	sight_info+=2;
	if ( *sight_info=='r' ){
	  sight_info+=2;
	  if ( *sight_info=='t')      marker = Flag_PRT; else
	  if ( *sight_info=='c')      marker = Flag_PRC; else
	  if ( *sight_info=='b')      marker = Flag_PRB; else	    
	  my_error("flag p r ?");
	} else
	if ( *sight_info=='l' ){
	  sight_info+=2;
	  if ( *sight_info=='t')      marker = Flag_PLT; else
	  if ( *sight_info=='c')      marker = Flag_PLC; else
	  if ( *sight_info=='b')      marker = Flag_PLB; else	    
	  my_error("flag p l ?");
	} else
	my_error("flag p ?");
      } else
      if ( *sight_info=='g' ){
	sight_info+=2;
	if ( *sight_info=='l' ){
	  sight_info+=2;
	  if ( *sight_info=='t' )     marker = Flag_GLT; else
	  if ( *sight_info=='b' )     marker = Flag_GLB; else
	  my_error("flag g l ?");
	} else
	if ( *sight_info=='r' ){
	  sight_info+=2;
	  if ( *sight_info=='t' )     marker = Flag_GRT; else
	  if ( *sight_info=='b' )     marker = Flag_GRB; else
	  my_error("flag g r ?");
	} else
	my_error("flag g ?");
      } else
      my_error("flag ?");
    } else
    if ( *sight_info=='F' ){
      object_type = OBJ_Marker_Behind;
      marker = k->GetClosestMarker().GetMarkerId(); /* could be No_Marker */
      //TAKE AWAY!!
      //std::cout << "Saw marker " << marker << " behind!" << std::endl;
    } else
    if ( *sight_info=='l' ){
      object_type = OBJ_Line;
      sight_info+=5;          /* "line " */
      if ( *sight_info=='r' )         line   = SL_Right;  else
      if ( *sight_info=='l' )         line   = SL_Left;   else
      if ( *sight_info=='t' )         line   = SL_Top;    else
      if ( *sight_info=='b' )         line   = SL_Bottom; else
      my_error("line ?");
    } else
    if ( *sight_info=='p' || *sight_info=='P' ){
      object_type = OBJ_Player;
      sight_info+=6;                          /* "player" */
      if ( *sight_info == ' ' ){              /* there's a team */ 
	sight_info++;
	// Since we compare teamnames DO NOT use Team1 and Team2
	// since this will not be able to separate them!!!
	if ( !strncmp(sight_info, k->theGame().GetOurTeamName().c_str(),
		      k->theGame().GetOurTeamNameLength()) )
	  player_side = my_side;
	else{
	  if ( k->theGame().GetTheirTeamName()[0] == '\n' ){
	    int_type a=0;
	    char* their_team_name = new char[256];
	    while ( isalpha(*sight_info) ) their_team_name[a++]=*sight_info++;
	    k->theGame().SetTheirTeamName(their_team_name);
	    delete [] their_team_name;
	  }
	  player_side = their_side;
	}
	while ( *sight_info != ' ' && *sight_info != ')' ) sight_info++; /* advance past team name */
	if ( *sight_info== ' ' ){             /* there's a number */
	  player_number = get_int_type(&sight_info);
	}
      }
    } else
      if ( *sight_info=='b' || *sight_info=='B' ) {
      object_type = OBJ_Ball;
      }
    else {
      my_error("unknown object");
      return;
    }

    advance_to(')',&sight_info);              /* advance to end of object */

    /************************************/

    ang = get_float_type(&sight_info);

    if ( *sight_info != ')' ) {                  /* 'high' quality     */
      view_qual = VQ_High;
      dist = ang;
      ang = get_float_type(&sight_info);
    }
    else {
      printf("%s",sight_info-30);
      view_qual = VQ_Low;
    }

    if ( view_qual != k->theAgent().GetViewQuality() ) my_error("View quality %d correct?",view_qual);

    if ( *sight_info != ')' ){
      dist_chng = get_float_type(&sight_info);
      dir_chng  = get_float_type(&sight_info);
    }

    if ( *sight_info != ')' ){
      if (object_type != OBJ_Player) my_error("Only players should have body_dir");
      body_dir = get_float_type(&sight_info);
    }

    if ( *sight_info != ')' ){
      if (object_type != OBJ_Player) my_error("Only players should have face_dir");
      face_dir = get_float_type(&sight_info);
    }

    if ( *sight_info != ')' ) {
      my_error("Should be done with object info here");
      return;
    }
    sight_info++;                                /* ")" */

    /************************************/

    bool behind = false;
    switch (object_type){
    case OBJ_Marker_Behind:
      behind = true;
    case OBJ_Marker:
      /* Want to save 2 closest for triangulation  */
      /* don't want marker_behind unless necessary */

      /* If it was a Marker_Behind and we don't know which one */
      if ( marker == No_Marker ){
	if ( object_type != OBJ_Marker_Behind ) my_error("Should know the marker");
	break;
      }

      //process_this_marker = false;
      // Since we want to save all markers...
      process_this_marker = true;
      if ( view_qual == VQ_Low ){               /* Low quality   */
        /* DON'T BOTHER PROCESSING ANY??? I don't think it helps ... */
	/* COULD process 2---then triangulate */
	/*if ( closest_marker_dist > 0 ){ */
	/*  closest_marker_dist = 0;  */                 /* Only process 1*/
	/*  process_this_marker = true; */
	/*}*/
      }
      else{                                          /* high quality  */
	if ( closest_marker == No_Marker || dist < closest_marker_dist){
	  closest_marker = marker;
	  closest_marker_dist = dist;
	  process_this_marker = true;
	}
	/* Don't bother with marker motion info -- get it from sense_body and my angle 
	if ( dir_chng != NOCHNGINFO && dist < motionInfoDist ){
	  motionInfoDist = dist;
	  process_this_marker = true;
	  k->ClosestMotionMarker = marker;
	}
	*/
      }
      if ( process_this_marker ){
	if ( view_qual == VQ_Low )              /* low quality   */
	  SeeMarker(marker, ang, time, behind);
	else /* if (dir_chng == NOCHNGINFO) */                  /* high quality  */
	  SeeMarker(marker, dist, ang, time, behind);   /* No motion info*/
	/* else
	  SeeMarker(marker, dist, ang, dist_chng, dir_chng, time); */
      }
      break;
    case OBJ_Line:
      //      if ( *sight_info != ')' )
	/* There's another line coming.  Assuming lines happen
	   last in the visual string and the closer line comes first */
      //	; 
      //      else
      if ( view_qual == VQ_Low )           /* low quality   */
	SeeLine(line, ang, time);
      else                                           /* high quality  */
	SeeLine(line, dist, ang, time);
      break;
    case OBJ_Ball:
      if ( view_qual == VQ_Low )                /* low quality   */
	SeeBall(ang, time);
      else if ( dir_chng == NOCHNGINFO )                   /* high quality  */
	SeeBall(dist, ang, time);
      else                                           /* know direction*/
	SeeBall(dist, ang, dist_chng, dir_chng, time);         
      break;
    case OBJ_Player:
      if ( !player_side ){                      /* Too far for team or num */
	if ( view_qual == VQ_Low )                /* low quality   */
	  SeePlayer(ang, time);
	else if ( dir_chng == NOCHNGINFO )                   /* high quality  */
	  SeePlayer(dist, ang, time);
	else {                                        /* know direction*/
	  my_error("Shouldn't know dir_chng when the player's far");
	  return;
	}
      }  

      else{
	if ( !player_number ){                  /* Too far for number     */
	  if ( view_qual == VQ_Low )                /* low quality   */
	    SeePlayer(player_side, ang, time);
	  else if ( dir_chng == NOCHNGINFO )                   /* high quality  */
	    SeePlayer(player_side, dist, ang, time);
	  else {                                           /* know direction*/
	    my_error("Shouldn't know dir_chng when the team member's far");
	    return;
	  }
	}

	else{                                   /* Know side AND number   */
	  if ( view_qual == VQ_Low )                /* low quality   */
	    SeePlayer(player_side, player_number, ang, time);
	  else if ( dir_chng == NOCHNGINFO ){                  /* high quality  */
	    printf("%s\n",sight_info-30);
	    my_error("Should know dir_chng when know number");
	    SeePlayer(player_side, player_number, dist, ang, time);
	  }
	  else                                           /* know direction*/
	    SeePlayer(player_side, player_number, dist, ang, dist_chng, dir_chng, body_dir, face_dir, time);
	}
      }
    }
  }
}


/****************************************************************************************/

void CBasicIP::ParseSound(const CTime& time, char* sound_info)
{
  // I have to set the last_hear_teammate/opponent/referee_time
  // depending on who sent the message!

  if ( sound_info[1] == 'r' ){   /* Referee or Coach message */
    sound_info += 9;             /* " referee " */
    if ( islower(sound_info[0]) )
      ParseRefereeSound(time, sound_info);  // Coach messages start with capital letters or are numbers
    else
      //      my_error("Not handling coach yet");
      ParseCoachSound(time, sound_info);
    return;
  }

  ParsePlayerSound(time, sound_info); 
  
  /* How to parse a sound from a player depends on the communication mechanism you use.
   * Our communication mechanism is described in:
   *
   * First International Workshop on Collective Robotics (CRW'98)
   * Communication in Domains with Unreliable, Single-Channel, Low-Bandwidth Communication. 
   * Peter Stone and Manuela Veloso
   * Available on-line at http://www.cs.cmu.edu/~pstone/pstone-papers.html
   */
}

/****************************************************************************************/

void CBasicIP::ParseRefereeSound(const CTime& time, char* msg)
{

  k->theAgent().SetLastHearRefereeTime(time);
  
  t_side my_side = k->theGame().GetOurSide();
  t_side their_side = k->theGame().GetTheirSide();

  //  std::cout << "In ParseRefereeSound: " << msg << "\n";
  switch( msg[0] ){
  case 'p': SetPlayMode(PM_Play_On); break;           /* play_on */
  case 'k': 
    if ( msg[5] == 'i' ){                                  /* kick_in */
      if ( msg[8] == my_side )
	SetPlayMode(PM_Our_Kick_In);
      else if ( msg[8] == their_side )
	SetPlayMode(PM_Their_Kick_In);
      else 
	my_error("kick_in_?");
    }
    else if ( msg[5] == 'o' ){                            /* kick_off */
      if ( msg[9] == my_side )
	SetPlayMode(PM_Our_Kick_Off);
      else if ( msg[9] == their_side )
	SetPlayMode(PM_Their_Kick_Off);
      else 
	my_error("kick_off_?");
    }
    else
      my_error("referee k..?");
    break;
  case 'g': 
    if ( msg[5] == 'k' ){                                 /* goal_kick */
      if ( msg[10] == my_side )
	SetPlayMode(PM_Our_Goal_Kick);
      else if ( msg[10] == their_side )
	SetPlayMode(PM_Their_Goal_Kick);
      else 
	my_error("goal_kick_?");
    }
    else if ( msg[5] == 'e' ){                           /* goalie_catch_ball */
      CatchBall(msg[18]);
      if ( msg[18] == my_side )
	SetPlayMode(PM_Our_Goalie_Got_Ball);
      else if ( msg[18] == their_side )
	SetPlayMode(PM_Their_Goalie_Got_Ball);
      else
	my_error("goalie_catch_ball_?");
    }
    else if ( msg[5] == my_side ){                    /* goal */
      k->theGame().SetOurScore(get_int_type(&msg[7]));
      k->theGame().SetKickOffMode(KO_Theirs);
      // initialized = false;
      SetPlayMode(PM_Before_Kick_Off);
    }
    else if ( msg[5] == their_side ){
      k->theGame().SetTheirScore(get_int_type(&msg[7]));
      k->theGame().SetKickOffMode(KO_Ours);
      // initialized = false;
      SetPlayMode(PM_Before_Kick_Off);
    }
    else 
      my_error("referee g..?");
    break;
  case 'c':                                               /* corner_kick */
    if ( msg[12] == my_side )
      SetPlayMode(PM_Our_Corner_Kick);
    else if ( msg[12] == their_side )
      SetPlayMode(PM_Their_Corner_Kick);
    else 
      my_error("corner_kick_?");
    SetLastCornerTime(time);
    break;
  case 'd': SetPlayMode(PM_Drop_Ball); break;        /* drop_ball */    
  case 'o':                                               /* offside */    
    if ( msg[8] == my_side )
      SetPlayMode(PM_Their_Offside_Kick);
    else if ( msg[8] == their_side )
      SetPlayMode(PM_Our_Offside_Kick);
    else 
      my_error("offside_?");
    break;
  case 'f':
    if ( msg[5] == 'k' ){                                 /* free_kick */
      if ( msg[10] == my_side )
	SetPlayMode(PM_Our_Free_Kick);
      else if ( msg[10] == their_side )
	SetPlayMode(PM_Their_Free_Kick);
      else 
	my_error("free_kick_?");
    }
    else if ( msg[5] == my_side )                     /* foul */
      ;
    else if ( msg[5] == their_side )
      ;
    else 
      my_error("referee f..?");
    break;
  case 'h':                                               /* half_time */
    //    WriteHalftimeStats("firstHalf");
    if ( my_side == 'l' )
      k->theGame().SetOurSide(S_Right);
    else if ( my_side == 'r' )
      k->theGame().SetOurSide(S_Left);
    InitHalf();
    SetPlayMode(PM_Half_Time);  /* play_mode to before_kick_off        */
    // initialized = false;
    if ( my_side == 'l' )
      k->theGame().SetKickOffMode(KO_Ours);
    else 
      k->theGame().SetKickOffMode(KO_Theirs);
    break;             
  case 'b': SetPlayMode(PM_Before_Kick_Off);
    //    initialized = false;
    break;       /* before_kick_off */
  case 't': 
    if ( msg[5] == 'u' ){                             /* time_up */
//      WriteFullGameStats();
//      WriteHalftimeStats("secondHalf");
      SetPlayMode(PM_Time_Up); 
    }
    else if ( msg[5] == 'o' )                             /* time_over */
      {
//	WriteFullGameStats();
	break;
      }
    else if ( msg[5] == 'e' ){                            /* time_extended */
//      WriteFullGameStats();
//      WriteHalftimeStats("secondHalf");
      SetPlayMode(PM_Extended_Time);
      //      initialized = false;
      if ( my_side == 'l' )
	k->theGame().SetKickOffMode(KO_Ours);
      else 
	k->theGame().SetKickOffMode(KO_Theirs);
    }
    else 
      my_error("referee t..?");
    break;
  default: my_error("Referee msg ????");
  }
}

void CBasicIP::ParseCoachSound(const CTime& time, char* coach_sound)
{

}

void CBasicIP::ParsePlayerSound(const CTime& time, char* player_sound)
{
  //  k->theAgent().SetLastHearTeammateTime(time);
  //  k->theAgent().SetLastHearOpponentTime(time);
}

/****************************************************************************/
/* These routines are to save time instead of using sscanf or atof, etc.    */
/* When passing **str_ptr, the pointer is advanced past the number          */
/* When passing  *str    , the pointer remains where it was before          */
/****************************************************************************/

float_type CBasicIP::get_float_type(char** str_ptr){

  float_type d_frac, result;
  int_type  m_flag, d_flag;

  d_frac = 1.0;
  result = 0.0;
  m_flag = d_flag = 0;

  /* Advance to the beginning of the number */
  while( !isdigit(**str_ptr) && **str_ptr!='-' && **str_ptr!='.')
    (*str_ptr)++;

  /* Process the number bit by bit */
  while((**str_ptr!=')') && (**str_ptr!=' ') && (**str_ptr!='\n') && (**str_ptr!=',')){
    if (**str_ptr=='.')
      d_flag=1;
    else if (**str_ptr=='-')
      m_flag=1;
    else if (d_flag){
      d_frac *= 10.0;
      result+=(float_type)(**str_ptr-'0')/d_frac;
    }
    else
      result=result*10.0+(float_type)(**str_ptr-'0');
    (*str_ptr)++;
  }
  if (m_flag)
    result=-result;

  //printf("%.1f\n",result);

  return result;
}

/* Get the number, but don't advance pointer */

float_type CBasicIP::get_float_type(char* str){
  char **str_ptr = &str;
  return get_float_type(str_ptr);
}

/****************************************************************************/

float CBasicIP::get_float(char** str_ptr){

  float d_frac, result;
  int_type  m_flag, d_flag;

  d_frac = 1.0;
  result = 0.0;
  m_flag = d_flag = 0;

  /* Advance to the beginning of the number */
  while( !isdigit(**str_ptr) && **str_ptr!='-' && **str_ptr!='.')
    (*str_ptr)++;

  /* Process the number bit by bit */
  while((**str_ptr!=')') && (**str_ptr!=' ') && (**str_ptr!='\n') && (**str_ptr!=',')){
    if (**str_ptr=='.')
      d_flag=1;
    else if (**str_ptr=='-')
      m_flag=1;
    else if (d_flag){
      d_frac *= 10.0;
      result+=(float)(**str_ptr-'0')/d_frac;
    }
    else
      result=result*10.0+(float)(**str_ptr-'0');
    (*str_ptr)++;
  }
  if (m_flag)
    result=-result;

  //printf("%.1f\n",result);

  return result;
}

/* Get the number, but don't advance pointer */

float CBasicIP::get_float(char* str){
  char **str_ptr = &str;
  return get_float(str_ptr);
}

/****************************************************************************/

int CBasicIP::get_int_type(char** str_ptr){

  int_type result;
  int_type m_flag;

  result = 0;
  m_flag = 0;

  /* Advance to the beginning of the number */
  while( !isdigit(**str_ptr) && **str_ptr!='-')
    (*str_ptr)++;

  /* Process the number bit by bit */
  while((**str_ptr!=')') && (**str_ptr!=' ') && (**str_ptr!='\n') && (**str_ptr!=',')){
    if (**str_ptr=='-')
      m_flag=1;
    else
      result=result*10+(int_type)(**str_ptr-'0');
    (*str_ptr)++;
  }
  if (m_flag)
    result=-result;

  return result;
}

int CBasicIP::get_int_type(char* str){
  char **str_ptr = &str;
  return get_int_type(str_ptr);
}

/****************************************************************************/

void CBasicIP::get_word(char** str_ptr){
  while ( !isalpha(**str_ptr) ) (*str_ptr)++;
}

/****************************************************************************/

void CBasicIP::get_next_word(char** str_ptr){
  while ( isalpha(**str_ptr) ) (*str_ptr)++;
  get_word(str_ptr);
}

/****************************************************************************/

void CBasicIP::advance_to(char c, char** str_ptr){
  while ( **str_ptr != c ) (*str_ptr)++;
}
