#ifndef _CBASICIP_H_
#define _CBASICIP_H_

#include "soc_types.h"
#include "CKnowledge.h"
#include "CRSParameters.h"
#include "CPlayerObject.h"
//#include "CSetofPlayersObject.h"
#include "CLineObject.h"
#include "CTime.h"
#include "soc_assert.h"
#include "CLogFile.h"
#include <vector>
#include <queue>

class CBasicIP
{
public:
  CBasicIP(CKnowledge* const k, CLogFile* const logfile);
  CBasicIP(const CBasicIP&);
  // Will I have pointers to CBasicIP but that will be used for
  // e.g. CAdvancedIP? Probably, therefor make the destructor
  // virtual.
  virtual ~CBasicIP();

  // void SetKnowledge(CKnowledge* const k);
  CKnowledge* const GetKnowledge();
  const CKnowledge* const GetKnowledge() const;

  const CLogFile* const GetLogFile() const;
  CLogFile* const GetLogFile();
  
  void Parse(char* sensory_info);

  const CSetofPlayersObject& SeenTeammates() const;
  CSetofPlayersObject& SeenTeammates();

  const CSetofPlayersObject& SeenOpponents() const;
  CSetofPlayersObject& SeenOpponents();
  
  CTime GetLastCornerTime() const;
  void SetLastCornerTime(const CTime& tm);

  bool FirstActionOpSinceSight() const;
  void SetFirstActionOpSinceSight(const bool new_val);
  
  // I'm not sure these are needed or even wanted here.
  void SetLastStopClockTime(const CTime& new_time);
  CTime GetLastStopClockTime() const;

  void SetSecondLastStopClockTime(const CTime& new_time);
  CTime GetSecondLastStopClockTime() const;

  bool IsClockStopped() const;

  void AddToStoppedClockMSec(const int_type msec);
  
protected:
  CKnowledge* const k;
  const CRSParameters* const params;
  CLogFile* const logf;

  bool seen_ball;
  bool seen_ball_moving;
  bool found_closest_flag;
  CMarkerObject closest_flag;
  
  // An object that keeps track of the players I have seen
  CSetofPlayersObject seenPlayers;
  CSetofPlayersObject seenTeammates;
  CSetofPlayersObject seenOpponents;
  
  std::vector<CLineObject> seenLines; // An array of the lines I can see
  int_type seen_lines_pos; // Keep track of how many lines there are in the memory.

  std::priority_queue<CMarkerObject> seenMarkers;
  std::priority_queue<CMarkerObject> seenMarkersBehind;
  int_type seen_markers; // Keep track of how many markers I've seen.
  int_type seen_markers_behind;
  
  CTime last_corner_time;
  bool first_action_op_since_sight;

  // These are references to the CTime class variables.
  // The reason is that I rather want
  // the user to use this class than CTime directly.
  CTime& last_stop_clock_time;
  CTime& second_last_stop_clock_time;

  bool clock_stopped;
  int_type stopped_clock_msec;  

  CCommandQueue tmp_queue;
  
  // void CopyData(const CBasicIP&);

  void BeforeSight();
  void AfterSight();
  void BeforeSense();
  void AfterSense();
  void BeforeHear();
  void AfterHear();
  
private:
  int_type HIST;
  bool showSeenPlayers;
  bool showInfo;

  // So the compiler won't generate an assignment operator.
  CBasicIP& operator =(const CBasicIP&);
  
  void InitHalf();
  //  void InitNextHalf();

  void ParseInitMessage(char* init_info);
  void ParseSight(const CTime& time, char* sight_info);
  void ParseSense(const CTime& time, char* sense_info);
  void ParseSound(const CTime& time, char* sound_info);
  void ParseRefereeSound(const CTime& time, char* referee_sound);
  void ParseCoachSound(const CTime& time, char* coach_sound);
  void ParsePlayerSound(const CTime& time, char* player_sound);

  void SeeMarker(const t_marker marker,
		 const float_type dist,
		 const float_type ang,
		 const CTime& time,
		 const bool behind);
  void SeeMarker(const t_marker marker,
		 const float_type ang,
		 const CTime& time,
		 const bool behind);
  void SeeLine(const t_side_line line,
	       const float_type ang,
	       const CTime& time);
  void SeeLine(const t_side_line line,
	       const float_type dist,
	       const float_type ang,
	       const CTime& time);
  void SeeBall(const float_type ang,
	       const CTime& time);
  void SeeBall(const float_type dist,
	       const float_type ang,
	       const CTime& time);
  void SeeBall(const float_type dist,
	       const float_type ang,
	       const float_type dist_chng,
	       const float_type dir_chng,
	       const CTime& time);
  void SeePlayer(const float_type ang,
		 const CTime& time);
  void SeePlayer(const float_type dist,
		 const float_type ang,
		 const CTime& time);
  void SeePlayer(const t_side player_side,
		 const float_type ang,
		 const CTime& time);
  void SeePlayer(const t_side player_side,
		 const float_type dist,
		 const float_type ang,
		 const CTime& time);
  void SeePlayer(const t_side player_side,
		 const int_type player_number,
		 const float_type dist,
		 const float_type ang,
		 const CTime& time);
  void SeePlayer(const t_side player_side,
		 const int_type player_number,
		 const float_type dist,
		 const float_type ang,
		 const float_type dist_chng,
		 const float_type dir_chng,
		 const float_type body_dir,
		 const float_type face_dir,
		 const CTime& time);

  void CatchBall(const t_side side);
  
  CTime UpdateTime(const int_type);
  void UpdateNewCycle();
  void SetPlayMode(const t_play_mode);
  CTime SanitizeTime(const CTime& tm);
  void SetMySensedInfo(const float_type stamina,
		       const float_type effort,
		       const float_type speed,
		       const float_type head_angle,
		       const int_type kicks,
		       const int_type dashes,
		       const int_type turns,
		       const int_type says,
		       const int_type turn_necks,
		       const CTime& time);
  void UpdateAfterCommands(int_type ds, int_type ks, int_type ss, int_type ts, int_type ns);
  void UpdateEstimation();
  
  CAngleDeg ComputeFaceDir();
  CPoint ComputeAbsolutPosition(const CMarkerObject& flag,
				const CAngleDeg& face_dir);

  void my_error(const char*);
  void my_error(const char*, int_type);

  float_type get_float_type(char** str_ptr);
  float_type get_float_type(char* str);
  float      get_float     (char** str_ptr);
  float      get_float     (char* str);
  int_type   get_int_type  (char** str_ptr);
  int_type   get_int_type  (char* str);
  void       get_word      (char** str_ptr);
  void       get_next_word (char** str_ptr);
  void       advance_to    (char c, char** str_ptr);
};

/*
inline void CBasicIP::SetKnowledge(CKnowledge* const kb)
{
  // Should I delete the old knowledge object?
  // No, BasicIP should only contain a reference to
  // an object.
  
  // Add an assertion that kb can't be NULL
  // If it's NULL then abort, can't continue.
  Assert(kb != NULL, "Must include a pointer to a CKnowledge object in CBasicIP!");
  k = kb;
  params = kb->GetParameters();
}
*/

inline const CKnowledge* const CBasicIP::GetKnowledge() const
{
  return k;
}

inline CKnowledge* const CBasicIP::GetKnowledge()
{
  return k;
}

inline const CLogFile* const CBasicIP::GetLogFile() const
{
  return logf;
}

inline CLogFile* const CBasicIP::GetLogFile()
{
  return logf;
}

inline const CSetofPlayersObject& CBasicIP::SeenTeammates() const
{
  return seenTeammates;
}

inline CSetofPlayersObject& CBasicIP::SeenTeammates()
{
  return seenTeammates;
}

inline const CSetofPlayersObject& CBasicIP::SeenOpponents() const
{
  return seenOpponents;
}

inline CSetofPlayersObject& CBasicIP::SeenOpponents()
{
  return seenOpponents;
}
  
inline CTime CBasicIP::GetLastCornerTime() const
{
  return last_corner_time;
}

inline void CBasicIP::SetLastCornerTime(const CTime& tm)
{
  last_corner_time = tm;
}

inline bool CBasicIP::FirstActionOpSinceSight() const
{
  return first_action_op_since_sight;
}

inline void CBasicIP::SetFirstActionOpSinceSight(const bool new_val)
{
  first_action_op_since_sight = new_val;
}
  
inline void CBasicIP::SetLastStopClockTime(const CTime& new_time)
{
  last_stop_clock_time = new_time;
}

inline CTime CBasicIP::GetLastStopClockTime() const
{
  return last_stop_clock_time;
}

inline void CBasicIP::SetSecondLastStopClockTime(const CTime& new_time)
{
  second_last_stop_clock_time = new_time;
}

inline CTime CBasicIP::GetSecondLastStopClockTime() const
{
  return second_last_stop_clock_time;
}

inline bool CBasicIP::IsClockStopped() const
{
  return clock_stopped;
}

inline void CBasicIP::AddToStoppedClockMSec(const int_type msec)
{
  stopped_clock_msec += msec;
}
  
#endif
