#include "CCatchCommand.h"
#include "CKnowledge.h"
#include "CRectangle.h"
#include "utils.h"

CCatchCommand::CCatchCommand(CKnowledge* const kb, const CAngleDeg& ang,
			     const t_priority prio,
			     const CTime& s_time, const CTime& f_time)
  : CCommand(kb, CMD_Catch, prio, s_time, f_time),
    catch_type(CATCH_Angle),
    angle(ang)
{
}

CCatchCommand::CCatchCommand(CKnowledge* const kb, const CPoint& dest,
			     const t_priority prio,
			     const CTime& s_time, const CTime& f_time)
  : CCommand(kb, CMD_Catch, prio, s_time, f_time),
    catch_type(CATCH_Dest),
    destination(dest)
{
}
  
CCatchCommand::CCatchCommand(CKnowledge* const kb, const CVector& obj_vec,
			     const t_priority prio,
			     const CTime& s_time, const CTime& f_time)
  : CCommand(kb, CMD_Catch, prio, s_time, f_time),
    object_vector(obj_vec)
{
}

CCatchCommand::CCatchCommand(const CCatchCommand& obj)
  : CCommand(obj),
    catch_type(obj.catch_type),
    angle(obj.angle),
    destination(obj.destination),
    object_vector(obj.object_vector)
{
}

CCatchCommand::~CCatchCommand()
{
}

void CCatchCommand::CopyCommand(const CCatchCommand& obj)
{
  CCommand::CopyCommand(obj);
  catch_type = obj.catch_type;
  angle = obj.angle;
  destination = obj.destination;
  object_vector = obj.object_vector;
}

CCatchCommand& CCatchCommand::operator =(const CCatchCommand& obj)
{
  if ( this != &obj ) {
    CopyCommand(obj);
  }

  return *this;
}

void CCatchCommand::CatchAngle(const CAngleDeg& ang)
{
  catch_type = CATCH_Angle;
  angle = ang;
}

void CCatchCommand::CatchDestination(const CPoint& dest)
{
  catch_type = CATCH_Dest;
  destination = dest;
}

void CCatchCommand::CatchObjectVector(const CVector& obj_vector)
{
  catch_type = CATCH_Object;
  object_vector = obj_vector;
}

void CCatchCommand::UpdateAfterCommand(const int_type from,
				       const int_type to) const
{
  // This function updates the agent after a catch command has been executed
  // Maybe add some flag that I've tried to catch the ball.
  // If I get a message from the referee about a free kick then I can
  // presume that I caught the ball, and that certain (somewhat unclear)
  // rules apply to what I can do. Look at the server-changes.
  // Since I have cycles_since_catch and can use that to see if it's an
  // ordinary free_kick or if the goalie caught the ball.

  CRectangle penalty_area;
  
  //  if ( k->theGame().GetOurSide() == S_Left ) {
  penalty_area = CRectangle(CPoint(k->goalL.GetPosX(), k->flagPLB.GetPosY()),
			    k->flagPLT.GetPosition());
    /*  } else if ( k->theGame().GetOurSide() == S_Right ) {
    penalty_area = CRectangle(k->flagPRB.GetPosition(),
    			      CPoint(k->goalR.GetPosX(), k->flagPRT.GetPosY()));
  } else {
    std::cout << "We're playing on an unknown side!!\n";
  }
  */

  const CRectangle catchable_area(CPoint(0, k->GetParameters()->SP_catchable_area_w),
				  CPoint(k->GetParameters()->SP_catchable_area_l,
					 -k->GetParameters()->SP_catchable_area_w));

  CVector tmp_vec = k->theBall().GetVector(from);
  tmp_vec.Rotate(/*-k->theAgent().GetBodyDir(from)*/ - angle);  
  
  if ( !k->theAgent().IsGoalie() ||
       k->theAgent().GetCyclesSinceCatch() < k->GetParameters()->SP_catch_ban_cycle ||      
       k->theGame().GetPlayMode() == PM_Before_Kick_Off ||
       k->theGame().GetPlayMode() == PM_Our_Kick_Off ||
       k->theGame().GetPlayMode() == PM_Their_Kick_Off ||
       //k->theGame().GetPlayMode() == PM_Our_Free_Kick ||
       //k->theGame().GetPlayMode() == PM_Their_Free_Kick ||
       //k->theGame().GetPlayMode() == PM_Their_Goalie_Got_Ball ||
       //k->theGame().GetPlayMode() == PM_Our_Goalie_Got_Ball ||
       k->theGame().GetPlayMode() == PM_Our_Goal ||
       k->theGame().GetPlayMode() == PM_Their_Goal ||
       k->theGame().GetPlayMode() == PM_Our_Offside_Kick ||
       k->theGame().GetPlayMode() == PM_Their_Offside_Kick ||
       !penalty_area.Inside(k->theAgent().GetPosition(from)) ||
       !catchable_area.Inside(CPoint(tmp_vec))) {
    k->theAgent().SetCyclesSinceCatch(0);
    return;
  } else {
    k->theAgent().SetCyclesSinceCatch(0);
    // Do other updates reflecting the new "rule"?
    // Maybe fix the ball position? 
  }
}

bool CCatchCommand::Validate(const int_type hist)
{

  // First calculate the angle I want to catch in
  switch(catch_type) {
  case CATCH_Angle : // Catch with angle already set
    break;
  case CATCH_Dest : // Catch with an intended destination
    CatchDestination(hist);
    break;
  case CATCH_Object : // Catch an object
    CatchObject(hist);
    break;
  default : // Error, what should I do??
    std::cout << "Unknown catch_type in CatchCommand!\n";
    return false;
  }

  // Then validate the command, so that I'll get the actual angle to catch
  
  // Should I make sure that I'm inside the penalty area? YES!
  // That the ball really is catchable? YES!
  // That it's the correct angle to the ball? NO!
  // Is it really good to change the angle? YES, if the new angle make it catchable.

  //  if ( k->theGame().GetOurSide() == S_Left ) {
  const CRectangle penalty_area = CRectangle(CPoint(k->goalL.GetPosX(),
						    k->flagPLB.GetPosY()),
					     k->flagPLT.GetPosition());
    //  } else if ( k->theGame().GetOurSide() == S_Right ) {
    //    penalty_area = CRectangle(k->flagPRB.GetPosition(),
    //    			      CPoint(k->goalR.GetPosX(), k->flagPRT.GetPosY()));
    //  } else {
    //    std::cout << "We're playing on an unknown side!!\n";
    //  }
  
  const CRectangle catchable_area(CPoint(0, k->GetParameters()->SP_catchable_area_w),
				  CPoint(k->GetParameters()->SP_catchable_area_l,
					 -k->GetParameters()->SP_catchable_area_w));

  CAngleDeg tmp_angle = MinMax(k->GetParameters()->SP_minmoment, angle,
			       k->GetParameters()->SP_maxmoment);

  CVector tmp_vec = k->theBall().GetVector(hist);
  tmp_vec.Rotate(/*-k->theAgent().GetBodyDir(hist)*/ - tmp_angle);  
  
  if ( !k->theAgent().IsGoalie() ||
       // k->theAgent().GetCyclesSinceCatch() < k->GetParameters()->SP_catch_ban_cycle ||      
       k->theGame().GetPlayMode() == PM_Before_Kick_Off ||
       k->theGame().GetPlayMode() == PM_Our_Kick_Off ||
       k->theGame().GetPlayMode() == PM_Their_Kick_Off ||
       k->theGame().GetPlayMode() == PM_Our_Free_Kick ||
       k->theGame().GetPlayMode() == PM_Their_Goalie_Got_Ball ||
       k->theGame().GetPlayMode() == PM_Our_Goalie_Got_Ball ||
       k->theGame().GetPlayMode() == PM_Their_Free_Kick ||
       k->theGame().GetPlayMode() == PM_Our_Goal ||
       k->theGame().GetPlayMode() == PM_Their_Goal ||
       k->theGame().GetPlayMode() == PM_Our_Offside_Kick ||
       k->theGame().GetPlayMode() == PM_Their_Offside_Kick ||
       !penalty_area.Inside(k->theAgent().GetPosition(hist)) ||
       !catchable_area.Inside(CPoint(tmp_vec))) {
    std::cout << !k->theAgent().IsGoalie()
	      << (k->theAgent().GetCyclesSinceCatch() <
		  k->GetParameters()->SP_catch_ban_cycle)
	      << (k->theGame().GetPlayMode() == PM_Before_Kick_Off)
	      << (k->theGame().GetPlayMode() == PM_Our_Kick_Off)
	      << (k->theGame().GetPlayMode() == PM_Their_Kick_Off)
	      << (k->theGame().GetPlayMode() == PM_Our_Free_Kick)
	      << (k->theGame().GetPlayMode() == PM_Their_Goalie_Got_Ball)
	      << (k->theGame().GetPlayMode() == PM_Our_Goalie_Got_Ball)
	      << (k->theGame().GetPlayMode() == PM_Their_Free_Kick)
	      << (k->theGame().GetPlayMode() == PM_Our_Goal)
	      << (k->theGame().GetPlayMode() == PM_Their_Goal)
	      << (k->theGame().GetPlayMode() == PM_Our_Offside_Kick)
	      << (k->theGame().GetPlayMode() == PM_Their_Offside_Kick)
	      << (!penalty_area.Inside(k->theAgent().GetPosition(hist)))
	      << (!catchable_area.Inside(CPoint(tmp_vec))) << std::endl;
    if ( !penalty_area.Inside(k->theAgent().GetPosition(hist)) ||
	 !catchable_area.Inside(CPoint(tmp_vec)) ) {
      std::cout << !penalty_area.Inside(k->theAgent().GetPosition(hist))
		<< !catchable_area.Inside(CPoint(tmp_vec))
		<< tmp_vec << std::endl;
    }
    if ( k->theAgent().GetCyclesSinceCatch() < k->GetParameters()->SP_catch_ban_cycle )
      std::cout << k->theAgent().GetCyclesSinceCatch() << " " 
		<< k->GetParameters()->SP_catch_ban_cycle << std::endl;
    return false;
  } else {
    angle = tmp_angle;
    return true;
  }
}

void CCatchCommand::CatchDestination(const int_type hist)
{
  CVector vec = destination - k->theAgent().GetPosition(hist+1);
  angle = vec.GetAngle() - k->theAgent().GetBodyDir(hist+1);
}

void CCatchCommand::CatchObject(const int_type hist)
{
  destination = k->theBall().GetPosition(hist+1);
  CatchDestination(hist);
}

const std::string CCatchCommand::ServerString() const
{
  return "(catch " + ToString(angle) + ")";
}
