#ifndef _CCATCHCOMMAND_H_
#define _CCATCHCOMMAND_H_

#include "CCommand.h"
#include "CAngleDeg.h"
#include "CPoint.h"
#include "CVector.h"

enum t_catch_type {
  CATCH_Unknown,
  CATCH_Angle,
  CATCH_Dest,
  CATCH_Object
};

class CCatchCommand : public CCommand
{
public:
  CCatchCommand(CKnowledge* const kb, const CAngleDeg& ang = 0,
		const t_priority prio = PRIO_NORMAL,
		const CTime& s_time = 0, const CTime& f_time = 0);
  CCatchCommand(CKnowledge* const kb, const CPoint& dest,
		const t_priority prio = PRIO_NORMAL,
		const CTime& s_time = 0, const CTime& f_time = 0);
  CCatchCommand(CKnowledge* const kb, const CVector& obj_vec,
		const t_priority prio = PRIO_NORMAL,
		const CTime& s_time = 0, const CTime& f_time = 0);
  CCatchCommand(const CCatchCommand& obj);
  virtual ~CCatchCommand();

  CCatchCommand& operator =(const CCatchCommand& obj);
  
  virtual const CCatchCommand* const GetCatchCommand() const;
  virtual CCatchCommand* GetCatchCommand();
  
  void CatchAngle(const CAngleDeg& angle);
  void CatchDestination(const CPoint& dest);
  void CatchObjectVector(const CVector& obj_vector);
  
  t_catch_type GetCatchType() const;
  void SetCatchType(const t_catch_type);  
  
  CAngleDeg GetAngle() const;
  void SetAngle(const CAngleDeg& angle);

  CPoint GetDestination() const;
  void SetDestination(const CPoint& destination);

  CVector GetObjectVector() const;
  void SetObjectVector(const CVector& object_vector);

  virtual void UpdateAfterCommand(const int_type from, const int_type to) const;
  virtual bool Validate(const int_type hist);

  virtual const std::string ServerString() const;
  
protected:
  t_catch_type catch_type;
  CAngleDeg angle;
  CPoint destination;
  CVector object_vector;

  void CatchDestination(const int_type hist);
  void CatchObject(const int_type hist);
  
  void CopyCommand(const CCatchCommand& obj);
};
  
inline const CCatchCommand* const CCatchCommand::GetCatchCommand() const
{
  return this;
}

inline CCatchCommand* CCatchCommand::GetCatchCommand()
{
  return this;
}
  
inline t_catch_type CCatchCommand::GetCatchType() const
{
  return catch_type;
}
  
inline CAngleDeg CCatchCommand::GetAngle() const
{
  return angle;
}

inline void CCatchCommand::SetAngle(const CAngleDeg& new_angle)
{
  angle = new_angle;
}

inline CPoint CCatchCommand::GetDestination() const
{
  return destination;
}

inline void CCatchCommand::SetDestination(const CPoint& new_destination)
{
  destination = new_destination;
}

inline CVector CCatchCommand::GetObjectVector() const
{
  return object_vector;
}

inline void CCatchCommand::SetObjectVector(const CVector& new_object_vector)
{
  object_vector = new_object_vector;
}

#endif
