#ifndef _CCOMMAND_H_
#define _CCOMMAND_H_

#include "types.h"
#include "CTime.h"
#include <string>
#include <iostream>

class CKnowledge;

class CCatchCommand;
class CDashCommand;
class CKickCommand;
class CMoveCommand;
class CSayCommand;
class CTurnCommand;
class CTurnNeckCommand;
class CCustomCommand;

typedef enum t_priority {
  PRIO_LOW    = -1,
  PRIO_NORMAL =  0,
  PRIO_HIGH   =  1
};

enum t_cmd_type {
  CMD_Unknown, CMD_Catch, CMD_Dash,
  CMD_Kick, CMD_Move, CMD_Say, 
  CMD_Turn, CMD_TurnNeck, CMD_Custom
};

class CCommand
{
public:
  CCommand(CKnowledge* const kb, const t_cmd_type = CMD_Unknown,
	   const t_priority = PRIO_NORMAL,
	   const CTime& s_time = 0, const CTime& f_time = 0);
  CCommand(const CCommand& obj);
  virtual ~CCommand();

  CCommand& operator =(const CCommand& obj);
  
  virtual const CCatchCommand*    const GetCatchCommand()    const;
  virtual       CCatchCommand*          GetCatchCommand();
  virtual const CDashCommand*     const GetDashCommand()     const;
  virtual       CDashCommand*           GetDashCommand();
  virtual const CKickCommand*     const GetKickCommand()     const;
  virtual       CKickCommand*           GetKickCommand();
  virtual const CMoveCommand*     const GetMoveCommand()     const;
  virtual       CMoveCommand*           GetMoveCommand(); 
  virtual const CSayCommand*      const GetSayCommand()      const;
  virtual       CSayCommand*            GetSayCommand();
  virtual const CTurnCommand*     const GetTurnCommand()     const;
  virtual       CTurnCommand*           GetTurnCommand();
  virtual const CTurnNeckCommand* const GetTurnNeckCommand() const;
  virtual       CTurnNeckCommand*       GetTurnNeckCommand();
  virtual const CCustomCommand*   const GetCustomCommand()   const;
  virtual       CCustomCommand*         GetCustomCommand();

  t_cmd_type GetType() const;

  t_priority GetPriority() const;
  void SetPriority(const t_priority prio);

  CTime GetStartTime() const;
  void SetStartTime(const CTime& new_start_time);

  CTime GetFinishTime() const;
  void SetFinishTime(const CTime& new_finish_time);
  
  virtual const std::string ServerString() const = 0;

  virtual void UpdateAfterCommand(const int_type from, const int_type to) const = 0;
  virtual bool Validate(const int_type hist) = 0; 

  // CPoint GetPositionAfterCommand(const int_type hist) const = 0;
  // CVector GetSpeedAfterCommand(const int_type hist) const = 0;
  
  void print() const;
  friend std::ostream& operator<<(std::ostream&, const CCommand*);
  
protected:
  CKnowledge* k;
  t_cmd_type cmd_type;
  t_priority cmd_priority;
  CTime start_time;
  CTime finish_time;

  void CopyCommand(const CCommand& obj);
  
private:
  void Error() const;
};

inline const CCatchCommand* const CCommand::GetCatchCommand() const
{
  Error();
  return NULL;
}

inline CCatchCommand* CCommand::GetCatchCommand()
{
  Error();
  return NULL;
}

inline const CDashCommand* const CCommand::GetDashCommand() const
{
  Error();
  return NULL;
}

inline CDashCommand* CCommand::GetDashCommand()
{
  Error();
  return NULL;
}

inline const CKickCommand* const CCommand::GetKickCommand() const
{
  Error();
  return NULL;
}

inline CKickCommand* CCommand::GetKickCommand()
{
  Error();
  return NULL;
}

inline const CMoveCommand* const CCommand::GetMoveCommand() const
{
  Error();
  return NULL;
}

inline CMoveCommand* CCommand::GetMoveCommand()
{
  Error();
  return NULL;
} 

inline const CSayCommand* const CCommand::GetSayCommand() const
{
  Error();
  return NULL;
}

inline CSayCommand* CCommand::GetSayCommand()
{
  Error();
  return NULL;
}

inline const CTurnCommand* const CCommand::GetTurnCommand() const
{
  Error();
  return NULL;
}

inline CTurnCommand* CCommand::GetTurnCommand()
{
  Error();
  return NULL;
}

inline const CTurnNeckCommand* const CCommand::GetTurnNeckCommand() const
{
  Error();
  return NULL;
}

inline CTurnNeckCommand* CCommand::GetTurnNeckCommand()
{
  Error();
  return NULL;
}

inline const CCustomCommand* const CCommand::GetCustomCommand() const
{
  Error();
  return NULL;
}

inline CCustomCommand* CCommand::GetCustomCommand()
{
  Error();
  return NULL;
}

inline t_cmd_type CCommand::GetType() const
{
  return cmd_type;
}

inline t_priority CCommand::GetPriority() const
{
  return cmd_priority;
}

inline void CCommand::SetPriority(const t_priority new_priority)
{
  cmd_priority = new_priority;
}

inline CTime CCommand::GetStartTime() const
{
  return start_time;
}

inline void CCommand::SetStartTime(const CTime& new_start_time)
{
  start_time = new_start_time;
}

inline CTime CCommand::GetFinishTime() const
{
  return finish_time;
}

inline void CCommand::SetFinishTime(const CTime& new_finish_time)
{
  finish_time = new_finish_time;
}

inline void CCommand::print() const
{
  std::cout << ServerString();
}

inline std::ostream& operator<<(std::ostream& os, const CCommand* cmd)
{
  return os << cmd->ServerString();
}

#endif

