#ifndef _CCOMMAND_QUEUE_H_
#define _CCOMMAND_QUEUE_H_

#include <deque>
#include "CCommand.h"
#include "CPlan.h"

class CCommandQueue
{
public:
  CCommandQueue();
  CCommandQueue(const CPlan&);
  CCommandQueue(const CCommandQueue&);
  ~CCommandQueue();

  CCommandQueue& operator =(const CCommandQueue&);
  CCommandQueue& operator =(const CPlan&);

  CCommandQueue  operator+ (const CCommandQueue&) const;
  CCommandQueue  operator+ (const CPlan&)         const;
  CCommandQueue  operator+ (const CCommand* const)      const;
  CCommandQueue& operator+=(const CCommandQueue&);
  CCommandQueue& operator+=(const CPlan&);
  CCommandQueue& operator+=(const CCommand* const);

  void Push(const CCommandQueue&);
  void Push(const CPlan&);
  void Push(const CCommand* const);
  
  CCommand* const Front();
  const CCommand* const Front() const;
  CCommand* const Back();
  const CCommand* const Back() const;
  void  Remove(const int);
  void  PopFront();
  void  PopBack();

  CCommand* const SecondLast();
  
  void MakeEmpty();
  bool Empty() const;
  int  Length() const;

private:
  std::deque<CCommand*> commands;

};

inline CCommandQueue& CCommandQueue::operator+=(const CCommandQueue& q)
{
  Push(q);
  return *this;
}

inline CCommandQueue& CCommandQueue::operator+=(const CPlan& p)
{
  Push(p);
  return *this;
}

inline CCommandQueue& CCommandQueue::operator+=(const CCommand* const cmd)
{
  Push(cmd);
  return *this;
}

inline bool CCommandQueue::Empty() const
{
  return commands.empty();
}

inline int CCommandQueue::Length() const
{
  return commands.size();
}

#endif
