#include "CCommandQueueSafe.h"
#include "CCatchCommand.h"
#include "CCustomCommand.h"
#include "CDashCommand.h"
#include "CKickCommand.h"
#include "CMoveCommand.h"
#include "CSayCommand.h"
#include "CTurnCommand.h"
#include "CTurnNeckCommand.h"

CCommandQueue::CCommandQueue()
  : commands()
{
}

CCommandQueue::CCommandQueue(const CPlan&)
  : commands()
{
}

CCommandQueue::CCommandQueue(const CCommandQueue& obj)
  : commands(obj.commands)
{
}

CCommandQueue::~CCommandQueue()
{
  MakeEmpty();
}

CCommandQueue& CCommandQueue::operator =(const CCommandQueue& obj)
{
  if ( this != &obj ) {
    MakeEmpty();
    commands = obj.commands;
  }

  return *this;
}

CCommandQueue& CCommandQueue::operator =(const CPlan&)
{
  return *this;
}

CCommandQueue  CCommandQueue::operator+ (const CCommandQueue& q2) const
{
  CCommandQueue q1(*this);
  q1.Push(q2);
  return q1;
}

CCommandQueue  CCommandQueue::operator+ (const CPlan& p) const
{
  CCommandQueue q(*this);
  q.Push(p);
  return q;
}

CCommandQueue  CCommandQueue::operator+ (const CCommand* const cmd) const
{
  CCommandQueue q(*this);
  q.Push(cmd);
  return q;
}

void CCommandQueue::Push(const CCommandQueue& q)
{
  std::queue<CCommand*> tmp_q(q.commands);
  while (!tmp_q.empty()) {
    Push(tmp_q.front());
    tmp_q.pop();
  }
}

void CCommandQueue::Push(const CPlan& p)
{
}

void CCommandQueue::Push(const CCommand* const cmd)
{
  CCommand* tmp;
  switch ( cmd->GetType() ) {
  case CMD_Catch:
    tmp = new CCatchCommand(*(cmd->GetCatchCommand()));
    break;
  case CMD_Dash:
    tmp = new CDashCommand(*(cmd->GetDashCommand()));
    break;
  case CMD_Kick:
    tmp = new CKickCommand(*(cmd->GetKickCommand()));
    break;
  case CMD_Move:
    tmp = new CMoveCommand(*(cmd->GetMoveCommand()));
    break;
  case CMD_Say:
    tmp = new CSayCommand(*(cmd->GetSayCommand()));
    break;
  case CMD_Turn:
    tmp = new CTurnCommand(*(cmd->GetTurnCommand()));
    break;
  case CMD_TurnNeck:
    tmp = new CTurnNeckCommand(*(cmd->GetTurnNeckCommand()));
    break;
  case CMD_Custom:
    tmp = new CCustomCommand(*(cmd->GetCustomCommand()));
    break;
  default:
    // Error!
    std::cout << "Error in CCommandQueue::Push() not a correct command type!" << std::endl;
    tmp = NULL;
  }
  commands.push(tmp);
}
  
CCommand* const CCommandQueue::Front()
{
  // What about if the queue is empty?
  // Currently just return NULL.
  if ( commands.empty() )
    return NULL;
  else
    return commands.front();
}

const CCommand* const CCommandQueue::Front() const
{
  // What about if the queue is empty?
  // Currently just return NULL.
  if ( commands.empty() )
    return NULL;
  else
    return commands.front();
}

void CCommandQueue::Remove(const int elts)
{
  int i = 0;
  while ( i < elts && !commands.empty() ) {
    Pop();
    ++i;
  }
}

void CCommandQueue::Pop()
{
  if ( !commands.empty() ) {
    CCommand* cmd = commands.front();
    delete cmd;
    commands.pop();
  }
}

void CCommandQueue::MakeEmpty()
{
  CCommand* tmp;
  while ( !commands.empty() ) {
    tmp = commands.front();
    delete tmp;
    commands.pop();
  }
}
