#include "CCommandQueueUnsafe.h"

CCommandQueue::CCommandQueue()
  : commands()
{
}

CCommandQueue::CCommandQueue(const CPlan&)
  : commands()
{
}

CCommandQueue::CCommandQueue(const CCommandQueue& obj)
  : commands(obj.commands)
{
}

CCommandQueue::~CCommandQueue()
{
  MakeEmpty();
}

CCommandQueue& CCommandQueue::operator =(const CCommandQueue& obj)
{
  if ( this != &obj ) {
    MakeEmpty();
    commands = obj.commands;
  }

  return *this;
}

CCommandQueue& CCommandQueue::operator =(const CPlan&)
{
  return *this;
}

CCommandQueue  CCommandQueue::operator+ (const CCommandQueue& q2) const
{
  CCommandQueue q1(*this);
  q1 += q2;
  return q1;
}

CCommandQueue  CCommandQueue::operator+ (const CPlan& p) const
{
  CCommandQueue q(*this);
  q += p;
  return q;
}

CCommandQueue  CCommandQueue::operator+ (CCommand* const cmd) const
{
  CCommandQueue q(*this);
  q += cmd;
  return q;
}

CCommandQueue& CCommandQueue::operator+=(const CCommandQueue& q)
{
  std::queue<CCommand*> tmp_q(q.commands);
  while (!tmp_q.empty()) {
    Push(tmp_q.front());
    tmp_q.pop();
  }
  
  return *this;
}

CCommandQueue& CCommandQueue::operator+=(const CPlan&)
{
  return *this;
}

CCommandQueue& CCommandQueue::operator+=(CCommand* const cmd)
{
  commands.push(cmd);
  return *this;
}

void CCommandQueue::Push(const CCommandQueue& q)
{
  (*this) += q;
}

void CCommandQueue::Push(const CPlan& p)
{
  (*this) += p;
}

void CCommandQueue::Push(CCommand* const cmd)
{
  (*this) += cmd;
}
  
const CCommand* const CCommandQueue::Top()
{
  // What about if the queue is empty?
  // Currently just return NULL.
  if ( commands.empty() )
    return NULL;
  else
    return commands.front();
}

const CCommand* const CCommandQueue::Top() const
{
  // What about if the queue is empty?
  // Currently just return NULL.
  if ( commands.empty() )
    return NULL;
  else
    return commands.front();
}

void CCommandQueue::Remove(const int elts)
{
  int i = 0;
  while ( i < elts && !commands.empty() ) {
    Pop();
    ++i;
  }
}

void CCommandQueue::Pop()
{
  if ( !commands.empty() ) {
    //    CCommand* cmd = commands.front();
    //    delete cmd;
    commands.pop();
  }
}

void CCommandQueue::MakeEmpty()
{
  CCommand* tmp;
  while ( !commands.empty() ) {
    //    tmp = commands.front();
    //    delete tmp;
    commands.pop();
  }
}
