#ifndef _CCUSTOMCOMMAND_H_
#define _CCUSTOMCOMMAND_H_

#include "CCommand.h"
#include "CVector.h"
#include "CPoint.h"

enum t_custom_command_type {
  CCMD_FindBall, CCMD_Score,
  CCMD_Dribble, CCMD_MoveTo,
  CCMD_Catch, CCMD_Rush, CCMD_Pass,
  CCMD_ClearBall, CCMD_GetBehindBall,
  CCMD_InterceptBall, CCMD_MoveAlong,
  CCMD_TrackBall, CCMD_Custom, CCMD_TurnBall,
  CCMD_GoalKick, CCMD_MakePassable,
  CCMD_CutOffPassAngle, CCMD_CutOffGoalAngle,
  CCMD_GoalieMoveTo, CCMD_CornerClearBall,
  CCMD_GetBehindCorner
};

class CCustomCommand : public CCommand
{
public:
  CCustomCommand(CKnowledge* const kb, const t_custom_command_type type,
		 const int m = 0, const t_priority prio = PRIO_NORMAL,
		 const CTime& s_time = 0, const CTime& f_time = 0);
  CCustomCommand(const CCustomCommand& obj);
  virtual ~CCustomCommand();

  CCustomCommand& operator =(const CCustomCommand& obj);

  virtual const CCustomCommand* const GetCustomCommand() const;
  virtual CCustomCommand* GetCustomCommand();

  t_custom_command_type GetCustomCommandType() const;
  void SetCustomCommandType(const t_custom_command_type new_type);
  
  int GetMode() const;
  void SetMode(const int m);

  CVector GetVector() const;
  void SetVector(const CVector& v);

  CPoint GetPoint() const;
  void SetPoint(const CPoint& p);

  virtual void UpdateAfterCommand(const int_type from, const int_type to) const;
  virtual bool Validate(const int_type hist);
  
  virtual const std::string ServerString() const;  

private:
  t_custom_command_type custom_command_type;
  int mode;
  CVector vec;
  CPoint pt;

  void CopyCommand(const CCustomCommand& obj);
};

inline const CCustomCommand* const CCustomCommand::GetCustomCommand() const
{
  return this;
}

inline CCustomCommand* CCustomCommand::GetCustomCommand()
{
  return this;
}
  
inline t_custom_command_type CCustomCommand::GetCustomCommandType() const
{
  return custom_command_type;
}

inline void CCustomCommand::SetCustomCommandType(const t_custom_command_type new_type)
{
  custom_command_type = new_type;
}
  
inline int CCustomCommand::GetMode() const
{
  return mode;
}

inline void CCustomCommand::SetMode(const int m)
{
  mode = m;
}

inline CVector CCustomCommand::GetVector() const
{
  return vec;
}

inline void CCustomCommand::SetVector(const CVector& v)
{
  vec = v;
}

inline CPoint CCustomCommand::GetPoint() const
{
  return pt;
}

inline void CCustomCommand::SetPoint(const CPoint& p)
{
  pt = p;
}

#endif
