#ifndef _CDASHCOMMAND_H_
#define _CDASHCOMMAND_H_

#include "soc_types.h"
#include "CCommand.h"
#include "CPoint.h"

enum t_dash_type {
  DASH_Unknown,
  DASH_Power,
  DASH_Speed,
  DASH_Dest,
  DASH_Dir,
  DASH_Dist
};

// Since we have max_pow = 100 as default we no longer have a general
// class since the max_power isn't fixed (even though it's not likely
// to change). Is there a better solution to this problem? The easiest
// is to let the user supply the max_pow always, they can then supply
// k->SP_max_power or something. But I think the user shouldn't have
// to worry about that parameter unless really necessary.

static const t_power DEFAULT_MAX_POW = 100.0;

class CDashCommand : public CCommand
{
public:
  CDashCommand(CKnowledge* const kb, const t_power pow = 0.0,
	       const t_power max_pow = DEFAULT_MAX_POW,
	       const bool backp = false, const t_priority prio = PRIO_NORMAL,
	       const CTime& s_time = 0, const CTime& f_time = 0);
  CDashCommand(CKnowledge* const kb, const CPoint& dest,
	       const t_power max_pow = DEFAULT_MAX_POW,
	       const bool backp = false, const t_priority prio = PRIO_NORMAL,
	       const CTime& s_time = 0, const CTime& f_time = 0);
  CDashCommand(CKnowledge* const kb, const CVector& dir,
	       const t_power max_pow = DEFAULT_MAX_POW,
	       const bool backp = false, const t_priority prio = PRIO_NORMAL,
	       const CTime& s_time = 0, const CTime& f_time = 0);
  CDashCommand(CKnowledge* const kb, const float_type,
	       const t_dash_type, const t_power max_pow = DEFAULT_MAX_POW,
	       const bool backp = false, const t_priority prio = PRIO_NORMAL,
	       const CTime& s_time = 0, const CTime& f_time = 0);
  CDashCommand(const CDashCommand& obj);
  virtual ~CDashCommand();

  CDashCommand& operator =(const CDashCommand& obj);
  
  virtual const CDashCommand* const GetDashCommand() const;
  virtual CDashCommand* GetDashCommand();

  void DashPower(const t_power power);
  void DashSpeed(const float_type speed);
  void DashDestination(const CPoint& dest);
  void DashDirection(const CVector& dir);
  void DashDistance(const float_type dist);
  
  t_dash_type GetDashType() const;
  void SetDashType(const t_dash_type);  
  
  t_power GetPower() const;
  void SetPower(const t_power new_power);

  t_power GetMaxPower() const;
  void SetMaxPower(const t_power new_max_power);

  bool BackPeddel() const;
  void SetBackPeddel(const bool new_back_peddel);
  
  float_type GetSpeed() const;
  void SetSpeed(const float_type new_speed);

  CPoint GetDestination() const;
  void SetDestination(const CPoint& new_destination);

  CPoint GetDirection() const;
  void SetDirection(const CVector& new_direction);

  float_type GetDistance() const;
  void SetDistance(const float_type new_distance);

  virtual void UpdateAfterCommand(const int_type from, const int_type to) const;
  virtual bool Validate(const int_type hist);

  virtual const std::string ServerString() const;
  
private:
  t_dash_type dash_type;
  t_power power;
  t_power max_power;
  bool back_peddel;
  float_type speed;
  CPoint destination;
  CVector direction;
  float_type distance;

  void DashSpeed(const int_type hist);
  void DashDestination(const int_type hist);
  void DashDirection(const int_type hist);
  void DashDistance(const int_type hist);

  void CopyCommand(const CDashCommand& obj);
};
  
inline const CDashCommand* const CDashCommand::GetDashCommand() const
{
  return this;
}

inline CDashCommand* CDashCommand::GetDashCommand()
{
  return this;
}
  
inline t_dash_type CDashCommand::GetDashType() const
{
  return dash_type;
}
  
inline t_power CDashCommand::GetPower() const
{
  return power;
}

inline void CDashCommand::SetPower(const t_power new_power)
{
  power = new_power;
}
  
inline t_power CDashCommand::GetMaxPower() const
{
  return max_power;
}

inline void CDashCommand::SetMaxPower(const t_power new_max_power)
{
  max_power = new_max_power;
}

inline bool CDashCommand::BackPeddel() const
{
  return back_peddel;
}
  
inline void CDashCommand::SetBackPeddel(const bool new_back_peddel)
{
  back_peddel = new_back_peddel;
}

inline float_type CDashCommand::GetSpeed() const
{
  return speed;
}

inline void CDashCommand::SetSpeed(const float_type new_speed)
{
  speed = new_speed;
}

inline CPoint CDashCommand::GetDestination() const
{
  return destination;
}

inline void CDashCommand::SetDestination(const CPoint& new_destination)
{
  destination = new_destination;
}

inline CPoint CDashCommand::GetDirection() const
{
  return direction;
}

inline void CDashCommand::SetDirection(const CVector& new_direction)
{
  direction = new_direction;
}

inline float_type CDashCommand::GetDistance() const
{
  return distance;
}

inline void CDashCommand::SetDistance(const float_type new_distance)
{
  distance = new_distance;
}

#endif
