#include "CFCFooIP.h"

CFCFooIP::CFCFooIP(CSoccerPlayer*& role, CKnowledge* const k,
		   CLogFile* const logfile)
  : CAdvancedIP(k, logfile),
    current_role(role),
    current_dt(DT_Normal),
    current_mode(MODE_Neutral)
{
}

CFCFooIP::CFCFooIP(const CFCFooIP& obj)
  : CAdvancedIP(obj),
    current_role(obj.current_role),
    current_dt(obj.current_dt),
    current_mode(obj.current_mode)
{
}

/*
CFCFooIP::~CFCFooIP()
{
}
*/

CPoint CFCFooIP::GoalieCatchPoint(const int_type hist) const
{
  switch (current_mode) {
  case MODE_Defensive:
    return CPoint(-49,0);
  case MODE_Neutral:
    if ( Sparse(CRectangle(CPoint(-40, 15), CPoint(-15, 30)), hist) )
      return CPoint(-45, -5);
    else if ( Sparse(CRectangle(CPoint(-40, -30), CPoint(-15, -15)), hist) )
      return CPoint(-45, 5);
    else
      return CPoint(-45, 0);
  case MODE_Offensive:
    if ( Sparse(CRectangle(CPoint(-40, 15), CPoint(-15, 30)), hist) )
      return CPoint(-38, -15);
    else if ( Sparse(CRectangle(CPoint(-40, -30), CPoint(-15, -15)), hist) )
      return CPoint(-38, 15);
    else
      return CPoint(-38, 0);
  default:
    return CPoint(-49,0);    
  }    
}

CPoint CFCFooIP::GoaliePos(const int_type hist) const
{

  /*
  CVector vec(k->goalL.GetPosition(), k->theBall().GetPosition(hist));
  if ( vec.GetLength() > 30 )
    vec.SetLength(10);
  else if ( vec.GetLength() > 20 )
    vec.SetLength(15);
  else
    vec.SetLength(vec.GetLength() * 0.75);

  //std::cout << "Goalie pos: " << k->goalL.GetPosition() + vec << std::endl;
  
  return k->goalL.GetPosition() + vec;
  */

  CPoint dest;

  if ( !BallInOurPenaltyArea(hist) ) {
    // The ball is outside the penalty area
    CVector abs_ball_vec(k->goalL.GetPosition(), k->theBall().GetPosition(hist));
    abs_ball_vec.SetLength(2.0);
    dest = k->goalL.GetPosition() + abs_ball_vec;
    float_type dy = k->theBall().GetPosY(hist) / 10;
    dest += CPoint(0, dy);
    //std::cout << "New goalie pos: " << dest << std::endl;
  } else if ( k->theBall().GetLastSeenMovingTime() >=
	      k->theGame().GetCurrentTime() -1 ) {
    // The ball is in the penalty area and I know the direction
    // of the ball.
    CAngleDeg ball_dir = k->theBall().GetAbsoluteSpeedDirection(hist);
    float_type dx = k->theBall().GetPosX(hist) + 51.5;
    float_type dy = Tan(ball_dir)*dx;
    if ( ball_dir < 0 )
      dy = -dy;
    dest = CPoint(-51.5, k->theBall().GetPosY(hist) + dy);
    //std::cout << "Ball dir: " << ball_dir << " dx: " << dx
    //      << " dy: " << dy << " dest: " << dest << std::endl;
  } else {
    // The ball is in the penalty area but I don't know the direction
    // of the ball.

    // Here I have two options.
    // 1. Estimated direction
    // 2. Do the as when ball is outside the penalty area
    CVector abs_ball_vec(k->goalL.GetPosition(), k->theBall().GetPosition(hist));
    abs_ball_vec.SetLength(1.0);
    dest = k->goalL.GetPosition() + abs_ball_vec;
    float_type dy = k->theBall().GetPosY(hist) / 10;
    dest += CPoint(0, dy);
    //std::cout << "New goalie pos: " << dest << std::endl;
  }

  if ( Abs(dest.GetY()) > params->SP_goal_width ) {
    //std::cout << "Goalie pos outside goal!\n";
    if ( dest.GetY() > 0 )
      dest.SetY(params->SP_goal_width);
    else
      dest.SetY(-params->SP_goal_width);
  }

  return dest;

}

float_type CFCFooIP::DistToHomePos(const int_type hist) const
{

  return k->theAgent().GetDistanceTo(current_role->GetHomePosition(), hist);

}
  
bool CFCFooIP::BallInMyHomeArea(const int_type hist) const
{

  return current_role->GetHomeArea().Inside(k->theBall().GetPosition(hist));
  
}

bool CFCFooIP::OpponentInMyHomeArea(const int_type hist) const
{

  if ( seenOpponents.GetNumberOfPlayers() == 0 )
    return false;
  else {
    CRectangle home_area = current_role->GetHomeArea();
    bool res = false;
    for (int i = 0; i < seenOpponents.GetNumberOfPlayers(); ++i) {
      res = home_area.Inside(seenOpponents[i]->GetPosition(hist));
      if (res) break;
    }
    return res;
  }
  
}

bool CFCFooIP::BallOnMySide(const int_type hist) const
{

  return (( PlayOnLeftSide() &&
	    BallOnLeftSide(hist) ) ||
	  ( PlayOnRightSide() &&
	    BallOnRightSide(hist) ));

}

bool CFCFooIP::OuterPlayer() const
{

  t_field_pos f_pos = current_role->GetFieldPosition();

  return (f_pos == PP_OuterLeft ||
	  f_pos == PP_OuterRight);
  
}

bool CFCFooIP::InnerPlayer() const
{

  t_field_pos f_pos = current_role->GetFieldPosition();

  return (f_pos == PP_InnerLeft ||
	  f_pos == PP_InnerRight);

}

bool CFCFooIP::PlayOnLeftSide() const
{

  t_field_pos f_pos = current_role->GetFieldPosition();

  return (f_pos == PP_Left ||
	  f_pos == PP_OuterLeft ||
	  f_pos == PP_InnerLeft);
  
}

bool CFCFooIP::PlayOnRightSide() const
{

  t_field_pos f_pos = current_role->GetFieldPosition();

  return (f_pos == PP_Right ||
	  f_pos == PP_OuterRight ||
	  f_pos == PP_InnerRight);
  
}

bool CFCFooIP::PlayInCenter() const
{

  return (current_role->GetFieldPosition() == PP_Center);
  
}

bool CFCFooIP::SparseMySide(const int_type hist) const
{

  if ( seenOpponents.GetNumberOfPlayers() == 0 )
    return true;
  else {
    if ( PlayOnLeftSide() )
      return Sparse(CRectangle(CPoint(0, -10), CPoint(20, -34)), hist);
    else if ( PlayOnRightSide() )
      return Sparse(CRectangle(CPoint(0, 34), CPoint(20, 10)), hist);
    else
      return false;
  }
}

bool CFCFooIP::GoalKeeper() const
{

  return (current_role->GetType() == ROLE_GoalKeeper);
  
}

bool CFCFooIP::Defender() const
{
    
  t_role_type r_type = current_role->GetType();
    
  return (r_type == ROLE_Defender ||
	  r_type == ROLE_InnerDefender ||
	  r_type == ROLE_OuterDefender ||
	  r_type == ROLE_CenterDefender);
  
}

bool CFCFooIP::Midfielder() const
{

  t_role_type r_type = current_role->GetType();

  return (r_type == ROLE_Midfielder ||
	  r_type == ROLE_InnerMidfielder ||
	  r_type == ROLE_OuterMidfielder ||
	  r_type == ROLE_CenterMidfielder);
  
}

bool CFCFooIP::OuterMidfielder() const
{

  return (current_role->GetType() == ROLE_OuterMidfielder);

  /*
    t_field_pos f_pos = current_role->GetFieldPosition();
  
  return (f_pos == PP_OuterRight ||
	  f_pos == PP_OuterLeft);
	  */
  
}

bool CFCFooIP::InnerMidfielder() const
{

  return (current_role->GetType() == ROLE_InnerMidfielder);

  /*
  t_field_pos f_pos = current_role->GetFieldPosition();
  
  return (f_pos == PP_InnerRight ||
	  f_pos == PP_InnerLeft);
	  */
  
}

bool CFCFooIP::CenterMidfielder() const
{

  return (current_role->GetType() == ROLE_CenterMidfielder);

}

bool CFCFooIP::Attacker() const
{

  t_role_type r_type = current_role->GetType();

  return (r_type == ROLE_Attacker ||
	  r_type == ROLE_LeftAttacker ||
	  r_type == ROLE_RightAttacker);
  
}
