#include "CGameObject.h"

const CPoint CGameObject::DEFAULT_POSITION = UNKNOWN_POINT;
const CVector CGameObject::DEFAULT_VECTOR = UNKNOWN_VECTOR;
const CTime CGameObject::DEFAULT_OBS_TIME = UNKNOWN_TIME;
//const CConfidence CGameObject::DEFAULT_CONFIDENCE = UNKNOWN_CONFIDENCE;
const float_type CGameObject::DEFAULT_CONFIDENCE_DECAY = UNKNOWN_FLOAT_TYPE;

CGameObject::CGameObject(const int_type hist_length)
  : max_hist_length(hist_length),
    max_length(hist_length + 5),
    current_obs(0),
    positions(max_length),
    vectors(max_length),
    obs_times(max_length),
    confidences(max_length),
    confidence_decay(DEFAULT_CONFIDENCE_DECAY)
{

  for (int_type i=0; i < max_length; ++i) {
    Reset(i);
  }

}

CGameObject::CGameObject(const CGameObject& obj)
  : max_hist_length(obj.max_hist_length),
    max_length(obj.max_length),
    current_obs(obj.current_obs),
    positions(obj.positions),
    vectors(obj.vectors),
    obs_times(obj.obs_times),
    confidences(obj.confidences),
    confidence_decay(obj.confidence_decay)
{

}

/*
CGameObject::~CGameObject()
{
  
}
*/

void CGameObject::CopyData(const CGameObject& obj)
{
  max_hist_length = obj.max_hist_length;
  max_length      = obj.max_length;
  current_obs     = obj.current_obs;

  confidences = obj.confidences;
  positions   = obj.positions;
  vectors     = obj.vectors;
  obs_times   = obj.obs_times;
  confidence_decay = obj.confidence_decay;
}

CGameObject& CGameObject::operator= (const CGameObject& obj)
{

  if (this != &obj) {
    CopyData(obj);
  }

  return *this;
  
}

void CGameObject::Copy(const int_type from, const int_type to)
{
  CopyPosition(from, to);
  CopyVector(from, to);
  CopyObsTime(from, to);
}

void CGameObject::CopyFrom(const CGameObject& obj, const int_type from,
			   const int_type to)
{
  positions[GetIndex(to)] = obj.positions[GetIndex(from)];
  vectors[GetIndex(to)] = obj.vectors[GetIndex(from)];
  obs_times[GetIndex(to)] = obj.obs_times[GetIndex(from)];
}

void CGameObject::ResetAll()
{
  Reset();

  for (int_type i=0; i < max_length; ++i)
    Reset(i);
}

void CGameObject::Reset()
{
  current_obs = 0;
  confidence_decay = DEFAULT_CONFIDENCE_DECAY;
}

void CGameObject::Reset(const int_type i)
{
  
  confidences[GetIndex(i)].SetMinConfidence();
  positions[GetIndex(i)]   = DEFAULT_POSITION;
  vectors[GetIndex(i)]     = DEFAULT_VECTOR;
  obs_times[GetIndex(i)]   = DEFAULT_OBS_TIME;

}

void CGameObject::NewCycle()
{
  /*
    std::cout << "max_length = " << max_length 
	    << " cur_obs = " << current_obs
	    << " GetIndex(PREV) = " << GetIndex(PREV)
	    << " GetIndex(NOW) = " << GetIndex(NOW)
	    << " GetIndex(NEXT) = " << GetIndex(NEXT) << std::endl;
	    */
  current_obs = GetIndex(NEXT);
  Copy(PREV, NOW);
  UpdateConfidence(PREV, NOW);
  /*
  std::cout << "cur_obs = " << current_obs
	    << " GetIndex(PREV) = " << GetIndex(PREV)
	    << " GetIndex(NOW) = " << GetIndex(NOW)
	    << " GetIndex(NEXT) = " << GetIndex(NEXT) << std::endl;
  std::cout << "length of positions = " << positions.size() << std::endl;
  */
}

void CGameObject::UpdateVector(const CPoint& agent_pos,
			       const CAngleDeg& agent_body_dir, const int_type hist)
{
  CVector obj_vec(agent_pos, positions[GetIndex(hist)]);
  obj_vec.Rotate(-agent_body_dir);
  vectors[GetIndex(hist)] = obj_vec;
}

CVector CGameObject::GetAbsoluteVectorTo(const CPoint& pt, const int_type hist) const
{
  
  if (positions[GetIndex(hist)] == UNKNOWN_POINT)
    return UNKNOWN_VECTOR;
  else
    return positions[GetIndex(hist)].VectorTo(pt);

}

CAngleDeg CGameObject::GetAbsoluteDirectionTo(const CPoint& pt, const int_type hist) const
{

  if (positions[GetIndex(hist)] == UNKNOWN_POINT)
    return UNKNOWN_DEG;
  else
    return positions[GetIndex(hist)].DirectionTo(pt);

}

float_type CGameObject::GetDistanceTo(const CPoint& pt, const int_type hist) const
{

  if (positions[GetIndex(hist)] == UNKNOWN_POINT)
    return UNKNOWN_FLOAT_TYPE;
  else
    return positions[GetIndex(hist)].DistanceTo(pt);

}

void CGameObject::Print(const int_type hist) const
{

  if (hist == UNKNOWN_INT_TYPE) {
    for (int_type i = -max_hist_length; i < max_length - max_hist_length; ++i) {
      std::cout << "[" << i << "]"
		<< " Pos: " << GetPosition(i)
		<< " Vec: " << GetVector(i)
		<< " Conf: " << GetConfidence(i)
		<< std::endl;
    }
  } else {
    std::cout << "[" << hist << "]"
	      << " Pos: " << GetPosition(hist)
	      << " Vec: " << GetVector(hist)
	      << " Conf: " << GetConfidence(hist)
	      << std::endl;
  }
  
}

