#include "CKickCommand.h"
#include "CKnowledge.h"
#include "utils.h"

CKickCommand::CKickCommand(CKnowledge* const kb, const t_power pow,
			   const CAngleDeg& ang, const t_priority prio,
			   const CTime& s_time, const CTime& f_time)
  : CCommand(kb, CMD_Kick, prio, s_time, f_time),
    kick_type(KICK_PowerAngle),
    power(pow),
    angle(ang)
{
}

CKickCommand::CKickCommand(CKnowledge* const kb, const CPoint& dest,
			   const t_priority prio,
			   const CTime& s_time, const CTime& f_time)
  : CCommand(kb, CMD_Kick, prio, s_time, f_time),
    kick_type(KICK_Dest),
    destination(dest)
{
}

CKickCommand::CKickCommand(CKnowledge* const kb, const CVector& dir,
			   const t_priority prio,
			   const CTime& s_time, const CTime& f_time)
  : CCommand(kb, CMD_Kick, prio, s_time, f_time),
    kick_type(KICK_Dir),
    direction(dir)
{
}

CKickCommand::CKickCommand(const CKickCommand& obj)
  : CCommand(obj),
    kick_type(obj.kick_type),
    power(obj.power),
    angle(obj.angle),
    destination(obj.destination),
    direction(obj.direction)
{
}

CKickCommand::~CKickCommand()
{
}

void CKickCommand::CopyCommand(const CKickCommand& obj)
{
  CCommand::CopyCommand(obj);
  kick_type = obj.kick_type;
  power = obj.power;
  angle = obj.angle;
  destination = obj.destination;
  direction = obj.direction;
}

CKickCommand& CKickCommand::operator =(const CKickCommand& obj)
{
  if ( this != &obj ) {
    CopyCommand(obj);
  }

  return *this;
}

void CKickCommand::KickPowerAngle(const t_power pow, const CAngleDeg& ang)
{
  kick_type = KICK_PowerAngle;
  power = pow;
  angle = ang;
}

void CKickCommand::KickDestination(const CPoint& dest)
{
  kick_type = KICK_Dest;
  destination = dest;
}

void CKickCommand::KickDirection(const CVector& dir)
{
  kick_type = KICK_Dir;
  direction = dir;
}
  
void CKickCommand::UpdateAfterCommand(const int_type from,
				      const int_type to) const
{
  if ( k->theGame().GetPlayMode() == PM_Before_Kick_Off ||
       k->theGame().GetPlayMode() == PM_Our_Goal ||
       k->theGame().GetPlayMode() == PM_Their_Goal ||
       k->theGame().GetPlayMode() == PM_Our_Offside_Kick ||
       k->theGame().GetPlayMode() == PM_Their_Offside_Kick )
    return ;

  if ( k->theBall().GetDistance(from) > k->GetParameters()->SP_kickable_area ) {
    //    cout << "Didn't update after kick! distance: " << k->theBall().GetDistance(from)
    //	 << std::endl;
    return ;
  }
  
 const float_type dir_diff = Abs(k->theBall().GetDirection(from));
 const float_type dist_diff = k->theBall().GetDistance(from)
                               - k->GetParameters()->SP_player_size
			       - k->GetParameters()->SP_ball_size;
 t_power kick_power = k->GetParameters()->SP_kick_power_rate * power
                       * (1.0 - 0.25*dir_diff/180.0
			      - 0.25*dist_diff/k->GetParameters()->SP_kickable_margin);
  
 float_type ball_dir = k->theAgent().GetBodyDir(from) + angle;
 k->theBall().SetAcceleration(CVector(ball_dir, kick_power), from);
 k->theBall().UpdateMovement(from, to);
 // What on earth does this mean?
 // I interpret it as the "facing direction" of the ball.
 // If that's it, where is it used, for what?
 // I didn't find it out from the server code, yet.
 // ball->angle = dir+angle ;
 
}

bool CKickCommand::Validate(const int_type hist)
{
  switch(kick_type) {
  case KICK_PowerAngle : // Kick with poewr and angle already set
    break;
  case KICK_Dest : // Kick the ball with an intended destination
    KickDestination(hist);
    break;
  case KICK_Dir : // Kick the ball with an intended direction
    KickDirection(hist);
    break;
  default : // Error, what should I do??
    std::cerr << "Unknown kick_type in KickCommand!\n";
    return false;
    break;
  }

  if ( k->theGame().GetPlayMode() == PM_Before_Kick_Off ||
       k->theGame().GetPlayMode() == PM_Our_Goal ||
       k->theGame().GetPlayMode() == PM_Their_Goal ||
       k->theGame().GetPlayMode() == PM_Our_Offside_Kick ||
       k->theGame().GetPlayMode() == PM_Their_Offside_Kick ) {
    //#ifdef DEBUG
    std::cerr << "Can't kick since play's off!" << std::endl;
    //#endif
    return false;
  }

  if ( k->theBall().GetDistance(hist) > k->GetParameters()->SP_kickable_area ) {
#ifdef DEBUG
    std::cerr << "Can't kick since ball is too far away!" << std::endl;
#endif
    return false;
  }
  
  power = MinMax(k->GetParameters()->SP_minpower, power, k->GetParameters()->SP_maxpower);

  /* If this is wanted, put it in CPrimAct
     if (Abs(power) < k->GetParameters()->RS_min_kick_power)
       power = 0;
   */

  angle = MinMax(k->GetParameters()->SP_minmoment, angle, k->GetParameters()->SP_maxmoment);

  /* If this is wanted, put it in CPrimAct
     if (Abs(angle) < k->GetParameters()->RS_min_kick_angle)
       angle = 0;
   */
  
  return true;
}

void CKickCommand::KickDestination(const int_type hist)
{
  direction = CVector(k->theBall().GetPosition(hist), destination);
  KickDirection(hist);
}

void CKickCommand::KickDirection(const int_type hist)
{
  CVector dir = direction - k->theBall().GetAbsoluteSpeedVector(hist);
  const float_type dir_diff = Abs(k->theBall().GetDirection(hist));
  const float_type dist_diff = k->theBall().GetDistance(hist)
                               - k->GetParameters()->SP_player_size
			       - k->GetParameters()->SP_ball_size;
  t_power kick_power = k->GetParameters()->SP_kick_power_rate
                        * (1.0 - 0.25*dir_diff/180.0
			   - 0.25*dist_diff/k->GetParameters()->SP_kickable_margin);

  if ( kick_power == 0 ) {
    std::cerr << "Kick_power is ZERO!\n";
    kick_power = 1;
  }
  
  angle = dir.GetAngle() - k->theAgent().GetBodyDir(hist);
  power = (dir.GetLength() / k->GetParameters()->SP_ball_decay) / kick_power;
  if ( power < 0 && dir.GetLength() > 0 )
    power = k->GetParameters()->SP_maxpower;

  /*
  if ( power < 100 )
    std::cout << "dir_diff = " << dir_diff << " dist_diff = " << dist_diff
	      << std::endl;
  
  std::cout << "Kick angle = " << angle << " Kick power = " << power << std::endl;
  */
  
}

const std::string CKickCommand::ServerString() const
{
  return "(kick " + ToString(power) + " " + ToString(angle) + ")";
}
