#ifndef _CKICKCOMMAND_H_
#define _CKICKCOMMAND_H_

#include "soc_types.h"
#include "CCommand.h"
#include "CAngleDeg.h"
#include "CPoint.h"
#include "CVector.h"

enum t_kick_type {
  KICK_Unknown,
  KICK_PowerAngle,
  KICK_Dest,
  KICK_Dir
};

class CKickCommand : public CCommand
{
public:
  CKickCommand(CKnowledge* const kb, const t_power pow = 0.0,
	       const CAngleDeg& ang = 0, const t_priority prio = PRIO_NORMAL,
	       const CTime& s_time = 0, const CTime& f_time = 0);
  CKickCommand(CKnowledge* const kb, const CPoint& dest,
	       const t_priority prio = PRIO_NORMAL,
	       const CTime& s_time = 0, const CTime& f_time = 0);
  CKickCommand(CKnowledge* const kb, const CVector& dir,
	       const t_priority prio = PRIO_NORMAL,
	       const CTime& s_time = 0, const CTime& f_time = 0);
  CKickCommand(const CKickCommand& obj);
  virtual ~CKickCommand();

  CKickCommand& operator =(const CKickCommand& obj);
  
  virtual const CKickCommand* const GetKickCommand() const;
  virtual CKickCommand* GetKickCommand();
  
  void KickPowerAngle(const t_power power, const CAngleDeg& angle);
  void KickDestination(const CPoint& dest);
  void KickDirection(const CVector& dir);
  
  t_kick_type GetKickType() const;
  void SetKickType(const t_kick_type);  
  
  t_power GetPower() const;
  void SetPower(const t_power);

  CAngleDeg GetAngle() const;
  void SetAngle(const CAngleDeg&);

  CPoint GetDestination() const;
  void SetDestination(const CPoint&);

  CVector GetDirection() const;
  void SetDirection(const CVector&);

  virtual void UpdateAfterCommand(const int_type from, const int_type to) const;
  virtual bool Validate(const int_type hist);

  virtual const std::string ServerString() const;

private:
  t_kick_type kick_type;
  t_power power;
  CAngleDeg angle;
  CPoint destination;
  CVector direction;

  void KickDestination(const int_type hist);
  void KickDirection(const int_type hist);

  void CopyCommand(const CKickCommand& obj);
};

inline const CKickCommand* const CKickCommand::GetKickCommand() const
{
  return this;
}

inline CKickCommand* CKickCommand::GetKickCommand()
{
  return this;
}
  
inline t_kick_type CKickCommand::GetKickType() const
{
  return kick_type;
}

inline t_power CKickCommand::GetPower() const
{ 
  return power;
}

inline void CKickCommand::SetPower(const t_power new_power)
{
  power = new_power;
}

inline CAngleDeg CKickCommand::GetAngle() const
{
  return angle;
}

inline void CKickCommand::SetAngle(const CAngleDeg& new_angle)
{
  angle = new_angle;
}

inline CPoint CKickCommand::GetDestination() const
{
  return destination;
}

inline void CKickCommand::SetDestination(const CPoint& dest)
{
  destination = dest;
}

inline CVector CKickCommand::GetDirection() const
{
  return direction;
}

inline void CKickCommand::SetDirection(const CVector& dir)
{
  direction = dir;
}

#endif
