#include "CKnowledge.h"

/*
CKnowledge::CKnowledge()
  : theGame(), theAgent(), theBall(), thePlayers(MAX_NUM_PLAYERS)
{
  parameters = NULL;
}
*/

CKnowledge::CKnowledge(CRSParameters* params,
		       const CTheGameObject& game,
		       const CAgentObject& agent,
		       const CBallObject& ball,
		       const CSetofPlayersObject& players)
  : parameters(NULL),
    TheGame(game),
    TheAgent(agent),
    TheBall(ball),
    ThePlayers(players),

    // What should I init the relative goals to?
    // Right and Left or NoMarker or what???
    theirGoal(No_Marker),
    ourGoal(No_Marker),
    
    goalL(Goal_L, CPoint( -params->SP_pitch_length/2.0, 0.0 )),
    goalR(Goal_R, CPoint( params->SP_pitch_length/2.0, 0.0 )),

    flagC(Flag_C, CPoint( 0.0, 0.0 )),
    flagCT(Flag_CT, CPoint( 0.0, -params->SP_pitch_width/2.0 )),
    flagCB(Flag_CB, CPoint( 0.0, params->SP_pitch_width/2.0 )),

    flagLT(Flag_LT, CPoint( -params->SP_pitch_length/2.0, -params->SP_pitch_width/2.0 )),
    flagLB(Flag_LB, CPoint( -params->SP_pitch_length/2.0, params->SP_pitch_width/2.0 )),
    flagRT(Flag_RT, CPoint( params->SP_pitch_length/2.0, -params->SP_pitch_width/2.0 )),
    flagRB(Flag_RB, CPoint( params->SP_pitch_length/2.0, params->SP_pitch_width/2.0 )),

    flagPLT(Flag_PLT, CPoint( -params->SP_pitch_length/2.0+params->SP_penalty_area_length,
			      -params->SP_penalty_area_width/2.0 )),
    flagPLC(Flag_PLC, CPoint( -params->SP_pitch_length/2.0+params->SP_penalty_area_length, 0 )),
    flagPLB(Flag_PLB, CPoint( -params->SP_pitch_length/2.0+params->SP_penalty_area_length,
			      params->SP_penalty_area_width/2.0 )),

    flagPRT(Flag_PRT, CPoint( params->SP_pitch_length/2.0-params->SP_penalty_area_length,
			      -params->SP_penalty_area_width/2.0 )),
    flagPRC(Flag_PRC, CPoint( params->SP_pitch_length/2.0-params->SP_penalty_area_length, 0 )),
    flagPRB(Flag_PRB, CPoint( params->SP_pitch_length/2.0-params->SP_penalty_area_length,
			      params->SP_penalty_area_width/2.0 )),

    flagGLT(Flag_GLT, CPoint( -params->SP_pitch_length/2.0, -params->SP_goal_width/2.0 )),
    flagGLB(Flag_GLB, CPoint( -params->SP_pitch_length/2.0, params->SP_goal_width/2.0 )),
    flagGRT(Flag_GRT, CPoint( params->SP_pitch_length/2.0, -params->SP_goal_width/2.0 )),
    flagGRB(Flag_GRB, CPoint( params->SP_pitch_length/2.0, params->SP_goal_width/2.0 )),

    flagTL50(Flag_TL50, CPoint( -50.0, -params->SP_pitch_width/2.0-params->SP_pitch_margin )),
    flagTL40(Flag_TL40, CPoint( -40.0, -params->SP_pitch_width/2.0-params->SP_pitch_margin )),
    flagTL30(Flag_TL30, CPoint( -30.0, -params->SP_pitch_width/2.0-params->SP_pitch_margin )),
    flagTL20(Flag_TL20, CPoint( -20.0, -params->SP_pitch_width/2.0-params->SP_pitch_margin )),
    flagTL10(Flag_TL10, CPoint( -10.0, -params->SP_pitch_width/2.0-params->SP_pitch_margin )),
    flagT0(Flag_T0, CPoint( 0.0, -params->SP_pitch_width/2.0-params->SP_pitch_margin )),
    flagTR10(Flag_TR10, CPoint( 10.0, -params->SP_pitch_width/2.0-params->SP_pitch_margin )),
    flagTR20(Flag_TR20, CPoint( 20.0, -params->SP_pitch_width/2.0-params->SP_pitch_margin )),
    flagTR30(Flag_TR30, CPoint( 30.0, -params->SP_pitch_width/2.0-params->SP_pitch_margin )),
    flagTR40(Flag_TR40, CPoint( 40.0, -params->SP_pitch_width/2.0-params->SP_pitch_margin )),
    flagTR50(Flag_TR50, CPoint( 50.0, -params->SP_pitch_width/2.0-params->SP_pitch_margin )),

    flagBL50(Flag_BL50, CPoint( -50.0, params->SP_pitch_width/2.0+params->SP_pitch_margin )),
    flagBL40(Flag_BL40, CPoint( -40.0, params->SP_pitch_width/2.0+params->SP_pitch_margin )),
    flagBL30(Flag_BL30, CPoint( -30.0, params->SP_pitch_width/2.0+params->SP_pitch_margin )),
    flagBL20(Flag_BL20, CPoint( -20.0, params->SP_pitch_width/2.0+params->SP_pitch_margin )),
    flagBL10(Flag_BL10, CPoint( -10.0, params->SP_pitch_width/2.0+params->SP_pitch_margin )),
    flagB0(Flag_B0, CPoint( 0.0, params->SP_pitch_width/2.0+params->SP_pitch_margin )),
    flagBR10(Flag_BR10, CPoint( 10.0, params->SP_pitch_width/2.0+params->SP_pitch_margin )),
    flagBR20(Flag_BR20, CPoint( 20.0, params->SP_pitch_width/2.0+params->SP_pitch_margin )),
    flagBR30(Flag_BR30, CPoint( 30.0, params->SP_pitch_width/2.0+params->SP_pitch_margin )),
    flagBR40(Flag_BR40, CPoint( 40.0, params->SP_pitch_width/2.0+params->SP_pitch_margin )),
    flagBR50(Flag_BR50, CPoint( 50.0, params->SP_pitch_width/2.0+params->SP_pitch_margin )),

    flagLT30(Flag_LT30, CPoint( -params->SP_pitch_length/2.0-params->SP_pitch_margin, -30 )),
    flagLT20(Flag_LT20, CPoint( -params->SP_pitch_length/2.0-params->SP_pitch_margin, -20 )),
    flagLT10(Flag_LT10, CPoint( -params->SP_pitch_length/2.0-params->SP_pitch_margin, -10 )),
    flagL0(Flag_L0, CPoint( -params->SP_pitch_length/2.0-params->SP_pitch_margin, 0 )),
    flagLB10(Flag_LB10, CPoint( -params->SP_pitch_length/2.0-params->SP_pitch_margin, 10 )),
    flagLB20(Flag_LB20, CPoint( -params->SP_pitch_length/2.0-params->SP_pitch_margin, 20 )),
    flagLB30(Flag_LB30, CPoint( -params->SP_pitch_length/2.0-params->SP_pitch_margin, 30 )),

    flagRT30(Flag_RT30, CPoint( params->SP_pitch_length/2.0+params->SP_pitch_margin, -30 )),
    flagRT20(Flag_RT20, CPoint( params->SP_pitch_length/2.0+params->SP_pitch_margin, -20 )),
    flagRT10(Flag_RT10, CPoint( params->SP_pitch_length/2.0+params->SP_pitch_margin, -10 )),
    flagR0(Flag_R0, CPoint( params->SP_pitch_length/2.0+params->SP_pitch_margin, 0 )),
    flagRB10(Flag_RB10, CPoint( params->SP_pitch_length/2.0+params->SP_pitch_margin, 10 )),
    flagRB20(Flag_RB20, CPoint( params->SP_pitch_length/2.0+params->SP_pitch_margin, 20 )),
    flagRB30(Flag_RB30, CPoint( params->SP_pitch_length/2.0+params->SP_pitch_margin, 30 )),

    noMarker(No_Marker)
{
  // Add an assertion that params can't be NULL
  // If it's NULL then abort, can't continue.
  Assert(params != NULL, "Must include a pointer to a CRSParameters object in CKnowledge!");
  parameters = params;
}

CKnowledge::CKnowledge(const CKnowledge& k)
  : parameters(k.parameters),
    TheGame(k.TheGame),
    TheAgent(k.TheAgent),
    TheBall(k.TheBall),
    ThePlayers(k.ThePlayers),

    theirGoal(k.theirGoal),
    ourGoal(k.ourGoal),
    
    goalL(k.goalL),
    goalR(k.goalR),

    flagC(k.flagC),
    flagCT(k.flagCT),
    flagCB(k.flagCB),

    flagLT(k.flagLT),
    flagLB(k.flagLB),
    flagRT(k.flagRT),
    flagRB(k.flagRB),

    flagPLT(k.flagPLT),			
    flagPLC(k.flagPLC),
    flagPLB(k.flagPLB),
	    
    flagPRT(k.flagPRT),
    flagPRC(k.flagPRC),
    flagPRB(k.flagPRB),			

    flagGLT(k.flagGLT),
    flagGLB(k.flagGLB),
    flagGRT(k.flagGRT),
    flagGRB(k.flagGRB),

    flagTL50(k.flagTL50),
    flagTL40(k.flagTL40),
    flagTL30(k.flagTL30),
    flagTL20(k.flagTL20),
    flagTL10(k.flagTL10),
    flagT0(k.flagT0),
    flagTR10(k.flagTR10),
    flagTR20(k.flagTR20),
    flagTR30(k.flagTR30),
    flagTR40(k.flagTR40),
    flagTR50(k.flagTR50),

    flagBL50(k.flagBL50),
    flagBL40(k.flagBL40),
    flagBL30(k.flagBL30),
    flagBL20(k.flagBL20),
    flagBL10(k.flagBL10),
    flagB0(k.flagB0), 
    flagBR10(k.flagBR10),
    flagBR20(k.flagBR20),
    flagBR30(k.flagBR30),
    flagBR40(k.flagBR40),
    flagBR50(k.flagBR50),

    flagLT30(k.flagLT30),
    flagLT20(k.flagLT20),
    flagLT10(k.flagLT10),
    flagL0(k.flagL0),
    flagLB10(k.flagLB10),
    flagLB20(k.flagLB20),
    flagLB30(k.flagLB30),

    flagRT30(k.flagRT30),
    flagRT20(k.flagRT20),
    flagRT10(k.flagRT10),
    flagR0(k.flagR0), 
    flagRB10(k.flagRB10),
    flagRB20(k.flagRB20),
    flagRB30(k.flagRB30),

    noMarker(k.noMarker)
{

}

void CKnowledge::CopyData(const CKnowledge& k)
{
    parameters = k.parameters;
    TheGame    = k.TheGame;
    TheAgent   = k.TheAgent;
    TheBall    = k.TheBall;
    ThePlayers = k.ThePlayers;

    theirGoal = k.theirGoal;
    ourGoal   = k.ourGoal;
    
    goalL = k.goalL;
    goalR = k.goalR;

    flagC  = k.flagC;
    flagCT = k.flagCT;
    flagCB = k.flagCB;

    flagLT = k.flagLT;
    flagLB = k.flagLB;
    flagRT = k.flagRT;
    flagRB = k.flagRB;

    flagPLT = k.flagPLT;			
    flagPLC = k.flagPLC;
    flagPLB = k.flagPLB;
	    
    flagPRT = k.flagPRT;
    flagPRC = k.flagPRC;
    flagPRB = k.flagPRB;			

    flagGLT = k.flagGLT;
    flagGLB = k.flagGLB;
    flagGRT = k.flagGRT;
    flagGRB = k.flagGRB;

    flagTL50 = k.flagTL50;
    flagTL40 = k.flagTL40;
    flagTL30 = k.flagTL30;
    flagTL20 = k.flagTL20;
    flagTL10 = k.flagTL10;
    flagT0   = k.flagT0;
    flagTR10 = k.flagTR10;
    flagTR20 = k.flagTR20;
    flagTR30 = k.flagTR30;
    flagTR40 = k.flagTR40;
    flagTR50 = k.flagTR50;

    flagBL50 = k.flagBL50;
    flagBL40 = k.flagBL40;
    flagBL30 = k.flagBL30;
    flagBL20 = k.flagBL20;
    flagBL10 = k.flagBL10;
    flagB0   = k.flagB0; 
    flagBR10 = k.flagBR10;
    flagBR20 = k.flagBR20;
    flagBR30 = k.flagBR30;
    flagBR40 = k.flagBR40;
    flagBR50 = k.flagBR50;

    flagLT30 = k.flagLT30;
    flagLT20 = k.flagLT20;
    flagLT10 = k.flagLT10;
    flagL0   = k.flagL0;
    flagLB10 = k.flagLB10;
    flagLB20 = k.flagLB20;
    flagLB30 = k.flagLB30;

    flagRT30 = k.flagRT30;
    flagRT20 = k.flagRT20;
    flagRT10 = k.flagRT10;
    flagR0   = k.flagR0; 
    flagRB10 = k.flagRB10;
    flagRB20 = k.flagRB20;
    flagRB30 = k.flagRB30;

    noMarker = k.noMarker;
}  

CKnowledge& CKnowledge::operator =(const CKnowledge& k)
{

  if ( this != &k ) {
    CopyData(k);
  }

  return *this;
}

/*
  goalL.SetMarkerId(Goal_L);
  goalR.SetMarkerId(Goal_R);
  flagC.SetMarkerId(Flag_C);
  flagCT.SetMarkerId(Flag_CT);
  flagCB.SetMarkerId(Flag_CB);
  flagLT.SetMarkerId(Flag_LT);
  flagLB.SetMarkerId(Flag_LB);
  flagRT.SetMarkerId(Flag_RT);
  flagRB.SetMarkerId(Flag_RB);
  flagPLT.SetMarkerId(Flag_PLT);
  flagPLC.SetMarkerId(Flag_PLC);
  flagPLB.SetMarkerId(Flag_PLB);
  flagPRT.SetMarkerId(Flag_PRT);
  flagPRC.SetMarkerId(Flag_PRC);
  flagPRB.SetMarkerId(Flag_PRB);
  flagGLT.SetMarkerId(Flag_GLT);
  flagGLB.SetMarkerId(Flag_GLB);
  flagGRT.SetMarkerId(Flag_GRT);
  flagGRB.SetMarkerId(Flag_GRB);
  flagTL50.SetMarkerId(Flag_TL50);
  flagTL40.SetMarkerId(Flag_TL40);
  flagTL30.SetMarkerId(Flag_TL30);
  flagTL20.SetMarkerId(Flag_TL20);
  flagTL10.SetMarkerId(Flag_TL10);
  flagT0.SetMarkerId(Flag_T0);
  flagTR10.SetMarkerId(Flag_TR10);
  flagTR20.SetMarkerId(Flag_TR20);
  flagTR30.SetMarkerId(Flag_TR30);
  flagTR40.SetMarkerId(Flag_TR40);
  flagTR50.SetMarkerId(Flag_TR50);
  flagBL50.SetMarkerId(Flag_BL50);
  flagBL40.SetMarkerId(Flag_BL40);
  flagBL30.SetMarkerId(Flag_BL30);
  flagBL20.SetMarkerId(Flag_BL20);
  flagBL10.SetMarkerId(Flag_BL10);
  flagB0.SetMarkerId(Flag_B0);
  flagBR10.SetMarkerId(Flag_BR10);
  flagBR20.SetMarkerId(Flag_BR20);
  flagBR30.SetMarkerId(Flag_BR30);
  flagBR40.SetMarkerId(Flag_BR40);
  flagBR50.SetMarkerId(Flag_BR50);
  flagLT30.SetMarkerId(Flag_LT30);
  flagLT20.SetMarkerId(Flag_LT20);
  flagLT10.SetMarkerId(Flag_LT10);
  flagL0.SetMarkerId(Flag_L0);
  flagLB10.SetMarkerId(Flag_LB10);
  flagLB20.SetMarkerId(Flag_LB20);
  flagLB30.SetMarkerId(Flag_LB30);
  flagRT30.SetMarkerId(Flag_RT30);
  flagRT20.SetMarkerId(Flag_RT20);
  flagRT10.SetMarkerId(Flag_RT10);
  flagR0.SetMarkerId(Flag_R0);
  flagRB10.SetMarkerId(Flag_RB10);
  flagRB20.SetMarkerId(Flag_RB20);
  flagRB30.SetMarkerId(Flag_RB30);
  noMarker.SetMarkerId(No_Marker);

  goalL.SetPosition(CPoint( -SP_pitch_length/2.0, 0.0 )); // Goal_L 
  goalR.SetPosition(CPoint( SP_pitch_length/2.0, 0.0 )); // Goal_R 
  
  flagC.SetPosition(CPoint( 0.0, 0.0 )); // Flag_C 
  flagCT.SetPosition(CPoint( 0.0, -SP_pitch_width/2.0 )); // Flag_CT 
  flagCB.SetPosition(CPoint( 0.0, SP_pitch_width/2.0 )); // Flag_CB 

  flagLT.SetPosition(CPoint( -SP_pitch_length/2.0, -SP_pitch_width/2.0 )); // Flag_LT 
  flagLB.SetPosition(CPoint( -SP_pitch_length/2.0, SP_pitch_width/2.0 )); // Flag_LB 
  flagRT.SetPosition(CPoint( SP_pitch_length/2.0, -SP_pitch_width/2.0 )); // Flag_RT 
  flagRB.SetPosition(CPoint( SP_pitch_length/2.0, SP_pitch_width/2.0 )); // Flag_RB 

  flagPLT.SetPosition(CPoint( -SP_pitch_length/2.0+SP_penalty_area_length,
			      -SP_penalty_area_width/2.0 )); // Flag_PLT 
  flagPLC.SetPosition(CPoint( -SP_pitch_length/2.0+SP_penalty_area_length, 0 )); // Flag_PLC 
  flagPLB.SetPosition(CPoint( -SP_pitch_length/2.0+SP_penalty_area_length,
			      SP_penalty_area_width/2.0 )); // Flag_PLB
  flagPRT.SetPosition(CPoint( SP_pitch_length/2.0-SP_penalty_area_length,
			      -SP_penalty_area_width/2.0 )); // Flag_PRT 
  flagPRC.SetPosition(CPoint( SP_pitch_length/2.0-SP_penalty_area_length, 0 )); // Flag_PRC 
  flagPRB.SetPosition(CPoint( SP_pitch_length/2.0-SP_penalty_area_length,
			      SP_penalty_area_width/2.0 )); // Flag_PRB 

  flagGLT.SetPosition(CPoint( -SP_pitch_length/2.0, -SP_goal_width/2.0 )); // Flag_GLT 
  flagGLB.SetPosition(CPoint( -SP_pitch_length/2.0, SP_goal_width/2.0 )); // Flag_GLB 
  flagGRT.SetPosition(CPoint( SP_pitch_length/2.0, -SP_goal_width/2.0 )); // Flag_GRT 
  flagGRB.SetPosition(CPoint( SP_pitch_length/2.0, SP_goal_width/2.0 )); // Flag_GRB 

  flagTL50.SetPosition(CPoint( -50.0, -SP_pitch_width/2.0-SP_pitch_margin )); // Flag_TL50 
  flagTL40.SetPosition(CPoint( -40.0, -SP_pitch_width/2.0-SP_pitch_margin )); // Flag_TL40 
  flagTL30.SetPosition(CPoint( -30.0, -SP_pitch_width/2.0-SP_pitch_margin )); // Flag_TL30 
  flagTL20.SetPosition(CPoint( -20.0, -SP_pitch_width/2.0-SP_pitch_margin )); // Flag_TL20 
  flagTL10.SetPosition(CPoint( -10.0, -SP_pitch_width/2.0-SP_pitch_margin )); // Flag_TL10 
  flagT0.SetPosition(CPoint( 0.0, -SP_pitch_width/2.0-SP_pitch_margin )); // Flag_T0 
  flagTR10.SetPosition(CPoint( 10.0, -SP_pitch_width/2.0-SP_pitch_margin )); // Flag_TR10 
  flagTR20.SetPosition(CPoint( 20.0, -SP_pitch_width/2.0-SP_pitch_margin )); // Flag_TR20 
  flagTR30.SetPosition(CPoint( 30.0, -SP_pitch_width/2.0-SP_pitch_margin )); // Flag_TR30 
  flagTR40.SetPosition(CPoint( 40.0, -SP_pitch_width/2.0-SP_pitch_margin )); // Flag_TR40 
  flagTR50.SetPosition(CPoint( 50.0, -SP_pitch_width/2.0-SP_pitch_margin )); // Flag_TR50 

  flagBL50.SetPosition(CPoint( -50.0, SP_pitch_width/2.0+SP_pitch_margin )); // Flag_BL50 
  flagBL40.SetPosition(CPoint( -40.0, SP_pitch_width/2.0+SP_pitch_margin )); // Flag_BL40 
  flagBL30.SetPosition(CPoint( -30.0, SP_pitch_width/2.0+SP_pitch_margin )); // Flag_BL30 
  flagBL20.SetPosition(CPoint( -20.0, SP_pitch_width/2.0+SP_pitch_margin )); // Flag_BL20 
  flagBL10.SetPosition(CPoint( -10.0, SP_pitch_width/2.0+SP_pitch_margin )); // Flag_BL10 
  flagB0.SetPosition(CPoint( 0.0, SP_pitch_width/2.0+SP_pitch_margin )); // Flag_B0 
  flagBR10.SetPosition(CPoint( 10.0, SP_pitch_width/2.0+SP_pitch_margin )); // Flag_BR10 
  flagBR20.SetPosition(CPoint( 20.0, SP_pitch_width/2.0+SP_pitch_margin )); // Flag_BR20 
  flagBR30.SetPosition(CPoint( 30.0, SP_pitch_width/2.0+SP_pitch_margin )); // Flag_BR30 
  flagBR40.SetPosition(CPoint( 40.0, SP_pitch_width/2.0+SP_pitch_margin )); // Flag_BR40 
  flagBR50.SetPosition(CPoint( 50.0, SP_pitch_width/2.0+SP_pitch_margin )); // Flag_BR50 

  flagLT30.SetPosition(CPoint( -SP_pitch_length/2.0-SP_pitch_margin, -30 )); // Flag_LT30 
  flagLT20.SetPosition(CPoint( -SP_pitch_length/2.0-SP_pitch_margin, -20 )); // Flag_LT20 
  flagLT10.SetPosition(CPoint( -SP_pitch_length/2.0-SP_pitch_margin, -10 )); // Flag_LT10 
  flagL0.SetPosition(CPoint( -SP_pitch_length/2.0-SP_pitch_margin, 0 )); // Flag_L0 
  flagLB10.SetPosition(CPoint( -SP_pitch_length/2.0-SP_pitch_margin, 10 )); // Flag_LB10 
  flagLB20.SetPosition(CPoint( -SP_pitch_length/2.0-SP_pitch_margin, 20 )); // Flag_LB20 
  flagLB30.SetPosition(CPoint( -SP_pitch_length/2.0-SP_pitch_margin, 30 )); // Flag_LB30 

  flagRT30.SetPosition(CPoint( SP_pitch_length/2.0+SP_pitch_margin, -30 )); // Flag_RT30 
  flagRT20.SetPosition(CPoint( SP_pitch_length/2.0+SP_pitch_margin, -20 )); // Flag_RT20 
  flagRT10.SetPosition(CPoint( SP_pitch_length/2.0+SP_pitch_margin, -10 )); // Flag_RT10 
  flagR0.SetPosition(CPoint( SP_pitch_length/2.0+SP_pitch_margin, 0 )); // Flag_R0 
  flagRB10.SetPosition(CPoint( SP_pitch_length/2.0+SP_pitch_margin, 10 )); // Flag_RB10 
  flagRB20.SetPosition(CPoint( SP_pitch_length/2.0+SP_pitch_margin, 20 )); // Flag_RB20 
  flagRB30.SetPosition(CPoint( SP_pitch_length/2.0+SP_pitch_margin, 30 )); // Flag_RB30 
*/

CMarkerObject CKnowledge::GetMarker(const t_marker marker) const
{

  switch (marker) {
    case Goal_L    : return goalL;
    case Goal_R    : return goalR;

    case Flag_C    : return flagC;
    case Flag_CT   : return flagCT;
    case Flag_CB   : return flagCB;
    case Flag_LT   : return flagLT;
    case Flag_LB   : return flagLB;
    case Flag_RT   : return flagRT;
    case Flag_RB   : return flagRB;

    case Flag_PLT  : return flagPLT;
    case Flag_PLC  : return flagPLC;
    case Flag_PLB  : return flagPLB;
    case Flag_PRT  : return flagPRT;
    case Flag_PRC  : return flagPRC;
    case Flag_PRB  : return flagPRB;

    case Flag_GLT  : return flagGLT;
    case Flag_GLB  : return flagGLB;
    case Flag_GRT  : return flagGRT;
    case Flag_GRB  : return flagGRB;

    case Flag_TL50 : return flagTL50;
    case Flag_TL40 : return flagTL40;
    case Flag_TL30 : return flagTL30;
    case Flag_TL20 : return flagTL20;
    case Flag_TL10 : return flagTL10;
    case Flag_T0   : return flagT0;
    case Flag_TR10 : return flagTR10;
    case Flag_TR20 : return flagTR20;
    case Flag_TR30 : return flagTR30;
    case Flag_TR40 : return flagTR40;
    case Flag_TR50 : return flagTR50;

    case Flag_BL50 : return flagBL50;
    case Flag_BL40 : return flagBL40;
    case Flag_BL30 : return flagBL30;
    case Flag_BL20 : return flagBL20;
    case Flag_BL10 : return flagBL10;
    case Flag_B0   : return flagB0;
    case Flag_BR10 : return flagBR10;
    case Flag_BR20 : return flagBR20;
    case Flag_BR30 : return flagBR30;
    case Flag_BR40 : return flagBR40;
    case Flag_BR50 : return flagBR50;

    case Flag_LT30 : return flagLT30;
    case Flag_LT20 : return flagLT20;
    case Flag_LT10 : return flagLT10;
    case Flag_L0   : return flagL0;
    case Flag_LB10 : return flagLB10;
    case Flag_LB20 : return flagLB20;
    case Flag_LB30 : return flagLB30;

    case Flag_RT30 : return flagRT30;
    case Flag_RT20 : return flagRT20;
    case Flag_RT10 : return flagRT10;
    case Flag_R0   : return flagR0;
    case Flag_RB10 : return flagRB10;
    case Flag_RB20 : return flagRB20;
    case Flag_RB30 : return flagRB30;

    default        : return noMarker;
  }
  
}

CMarkerObject CKnowledge::GetMarker(const int_type marker) const
{
  if ( marker >= 0 && marker < parameters->SP_num_markers )
    return GetMarker(t_marker(marker));
  else
    return No_Marker;
}

CMarkerObject CKnowledge::GetClosestMarker() const
{

  //  return No_Marker;
  
  // I think this method should be improved.
  // It should take a position as input and use it instead.
  
  CMarkerObject marker;
  float_type dist = 9999;
  float_type tmp;
  for (int i = 0; i < NUM_OF_MARKERS - 1; ++i) {
    if ( TheGame.GetOurSide() == S_Right )
      tmp = TheAgent.GetDistanceTo(-GetMarker(t_marker(i)).GetPosition(), CURRENT_EST);
    else
      tmp = TheAgent.GetDistanceTo(GetMarker(t_marker(i)).GetPosition(), CURRENT_EST);
    if ( tmp < dist ) {
      marker = GetMarker(t_marker(i));
      dist = tmp;
    }
  }

  //std::cout << "Closest marker: " << marker.GetMarkerId() << std::endl;
  
  return marker;

}

CMarkerObject CKnowledge::GetClosestGoal() const
{

  // Same improvements as above

  CMarkerObject res;
  
  if (TheAgent.GetPosX() > 0)
    res = (TheGame.GetOurSide() == S_Left) ? goalR : goalL;
  else
    res = (TheGame.GetOurSide() == S_Left) ? goalL : goalR;

  //std::cout << "Closest goal: " << res.GetMarkerId() << std::endl;

  return res;
  
}

void CKnowledge::RotateAllMarkers()
{
  for (int_type i = 0; i < parameters->SP_num_markers; ++i)
    GetMarker(i).Rotate();
}
