#ifndef _CKNOWLEDGE_H_
#define _CKNOWLEDGE_H_

#include "soc_types.h"
#include "CRSParameters.h"
#include "CTheGameObject.h"
#include "CAgentObject.h"
#include "CBallObject.h"
#include "CMarkerObject.h"
#include "CPoint.h"
#include "CSetofPlayersObject.h"
#include "soc_assert.h"

// This class is used to store the information about
// the agent, the ball, the other players and the markers
// on the field.

class CKnowledge
{
protected:
  CRSParameters*      parameters;
  CTheGameObject      TheGame;
  CAgentObject        TheAgent;
  CBallObject         TheBall;
  CSetofPlayersObject ThePlayers;

  void CopyData(const CKnowledge&);
  
public:
  // Since we don't want NULL pointers, do not allow
  // the construction of default objects of this type.
  // Do I need to make the default constructor private??
  //  CKnowledge();
  CKnowledge(CRSParameters*, const CTheGameObject&,
	     const CAgentObject&, const CBallObject&,
	     const CSetofPlayersObject&);
  CKnowledge(const CKnowledge&);

  CKnowledge& operator =(const CKnowledge&);

  void SetParameters(CRSParameters*);
  const CRSParameters* const GetParameters() const;
  
  // Made them public so that I don't have to add
  // Get and Set methods for each one of them.
  CTheGameObject& theGame();
  const CTheGameObject& theGame() const;
  CTheGameObject* GetGame();
  void           SetGame(const CTheGameObject&);
  
  CAgentObject&  theAgent();
  const CAgentObject&  theAgent() const;
  CAgentObject*  GetAgent();
  void           SetAgent(const CAgentObject&);

  CBallObject&   theBall();
  const CBallObject&   theBall() const;
  CBallObject*   GetBall();
  void           SetBall(const CBallObject&);

  CSetofPlayersObject& thePlayers();
  const CSetofPlayersObject& thePlayers() const;
  CSetofPlayersObject* GetPlayers();
  void                 SetPlayers(const CSetofPlayersObject&);

  CMarkerObject GetMarker(const t_marker marker) const;
  CMarkerObject GetMarker(const int_type marker) const;
  CMarkerObject GetClosestMarker() const;
  CMarkerObject GetClosestGoal() const;

  void RotateAllMarkers();
  
  // Objects for all the markers on the field
  CMarkerObject theirGoal; // The opponents goal
  CMarkerObject ourGoal;   // Our own goal

  CMarkerObject goalL;
  CMarkerObject goalR;
  CMarkerObject flagC;
  CMarkerObject flagCT;
  CMarkerObject flagCB;
  CMarkerObject flagLT;
  CMarkerObject flagLB;
  CMarkerObject flagRT;
  CMarkerObject flagRB;
  CMarkerObject flagPLT;
  CMarkerObject flagPLC;
  CMarkerObject flagPLB;
  CMarkerObject flagPRT;
  CMarkerObject flagPRC;
  CMarkerObject flagPRB;
  CMarkerObject flagGLT;
  CMarkerObject flagGLB;
  CMarkerObject flagGRT;
  CMarkerObject flagGRB;
  CMarkerObject flagTL50;
  CMarkerObject flagTL40;
  CMarkerObject flagTL30;
  CMarkerObject flagTL20;
  CMarkerObject flagTL10;
  CMarkerObject flagT0;
  CMarkerObject flagTR10;
  CMarkerObject flagTR20;
  CMarkerObject flagTR30;
  CMarkerObject flagTR40;
  CMarkerObject flagTR50;
  CMarkerObject flagBL50;
  CMarkerObject flagBL40;
  CMarkerObject flagBL30;
  CMarkerObject flagBL20;
  CMarkerObject flagBL10;
  CMarkerObject flagB0;
  CMarkerObject flagBR10;
  CMarkerObject flagBR20;
  CMarkerObject flagBR30;
  CMarkerObject flagBR40;
  CMarkerObject flagBR50;
  CMarkerObject flagLT30;
  CMarkerObject flagLT20;
  CMarkerObject flagLT10;
  CMarkerObject flagL0;
  CMarkerObject flagLB10;
  CMarkerObject flagLB20;
  CMarkerObject flagLB30;
  CMarkerObject flagRT30;
  CMarkerObject flagRT20;
  CMarkerObject flagRT10;
  CMarkerObject flagR0;
  CMarkerObject flagRB10;
  CMarkerObject flagRB20;
  CMarkerObject flagRB30;
  CMarkerObject noMarker;

  /* Include these?
  // Relative markers
  t_marker RM_LB_Flag;
  t_marker RM_LC_Flag;
  t_marker RM_LF_Flag;
  t_marker RM_RB_Flag;
  t_marker RM_RC_Flag;
  t_marker RM_RF_Flag;
  t_marker RM_Our_PC_Flag;   // Center of our penalty area
  t_marker RM_Their_PC_Flag; // Center of theirs
  */

};
  
inline void CKnowledge::SetParameters(CRSParameters* p)
{
  // Should I delete the old parameters object?
  // No, BasicIP should only contain a reference to
  // an object.

  // Add an assertion that p can't be NULL
  // If it's NULL then abort, can't continue.
  Assert(p != NULL, "Must include a pointer to a CRSParameters object in CKnowledge!");
  parameters = p;
}

inline   const CRSParameters* const CKnowledge::GetParameters() const
{
  return parameters;
}

inline CTheGameObject& CKnowledge::theGame()
{
  return TheGame;
}

inline const CTheGameObject& CKnowledge::theGame() const
{
  return TheGame;
}

inline CTheGameObject* CKnowledge::GetGame()
{
  return &TheGame;
}

inline void CKnowledge::SetGame(const CTheGameObject& game)
{
  TheGame = game;
}
  
inline CAgentObject& CKnowledge::theAgent()
{
  return TheAgent;
}

inline const CAgentObject& CKnowledge::theAgent() const
{
  return TheAgent;
}

inline CAgentObject* CKnowledge::GetAgent()
{
  return &TheAgent;
}

inline void CKnowledge::SetAgent(const CAgentObject& agent)
{
  TheAgent = agent;
}

inline CBallObject& CKnowledge::theBall()
{
  return TheBall;
}

inline const CBallObject& CKnowledge::theBall() const
{
  return TheBall;
}

inline CBallObject* CKnowledge::GetBall()
{
  return &TheBall;
}

inline void CKnowledge::SetBall(const CBallObject& ball)
{
  TheBall = ball;
}

inline CSetofPlayersObject& CKnowledge::thePlayers()
{
  return ThePlayers;
}

inline const CSetofPlayersObject& CKnowledge::thePlayers() const
{
  return ThePlayers;
}

inline CSetofPlayersObject* CKnowledge::GetPlayers()
{
  return &ThePlayers;
}

inline void CKnowledge::SetPlayers(const CSetofPlayersObject& players)
{
  ThePlayers = players;
}

#endif
