#ifndef _CLINE_H_
#define _CLINE_H_

#include "types.h"
#include "CVector.h"
#include "CPoint.h"
#include "CAngleDeg.h"
#include <iostream>

class CLine {
public:
  CLine();
  CLine(const CLine& l);
  CLine(const CPoint& pt1, const CPoint& pt2);
  CLine(const CPoint& pt, const CVector& v);
  CLine(const CVector& v);

  CLine& operator =(const CLine& l);
  
  CLine& Rotate(const CAngleDeg& ang);
  CLine& Scale(const float_type factor);
  CLine& MoveTo(const CPoint& pt);

  CLine& SetStart(const CPoint& pt);
  CPoint GetStart() const;
  CLine& SetVector(const CVector& v);
  CVector GetVector() const;
  float_type GetLength() const;
  CAngleDeg GetDirection() const;

  bool OnLine(const CPoint& pt) const;
  bool Crosses(const CLine& l) const;
  bool LeftOf(const CLine& l) const;
  bool LeftOf(const CPoint& pt) const;
  bool RightOf(const CLine& l) const;
  bool RightOf(const CPoint& pt) const;

  friend std::ostream& operator<<(std::ostream& os, const CLine& l);

private:
  CPoint start;
  CVector line;
  
};

inline CLine& CLine::operator =(const CLine& l)
{
  start = l.start;
  line = l.line;
  return *this;
}

inline CLine& CLine::Rotate(const CAngleDeg& ang)
{
  line.Rotate(ang);
  return *this;
}

inline CLine& CLine::Scale(const float_type factor)
{
  line.Scale(factor);
  return *this;
}

inline CLine& CLine::SetStart(const CPoint& pt)
{
  start = pt;
  return *this;
}

inline CPoint CLine::GetStart() const
{
  return start;
}

inline CLine& CLine::SetVector(const CVector& v)
{
  line = v;
  return *this;
}

inline CVector CLine::GetVector() const
{
  return line;
}

inline float_type CLine::GetLength() const
{
  return line.GetLength();
}

inline std::ostream& operator<<(std::ostream&  os, const CLine& l)
{
  return os << "<" << l.start << ", " << l.start + l.line << ">";
}

#endif
