#include "CLineObject.h"

const t_side_line CLineObject::DEFAULT_LINE = SL_Unknown;

CLineObject::CLineObject(const t_side_line line, const int_type hist_len)
  : CStationaryObject(hist_len),
    line_id(line)  
{
  
}

CLineObject::CLineObject(const CLineObject& obj)
  : CStationaryObject(obj),
    line_id(obj.line_id)
{

}

void CLineObject::CopyData(const CLineObject& obj)
{
  CStationaryObject::CopyData(obj);
  line_id = obj.line_id;
}

CLineObject& CLineObject::operator= (const CLineObject& obj)
{

  if (this != &obj) {
    CopyData(obj);
  }

  return *this;
  
}

void CLineObject::ResetAll()
{
  Reset();
}

void CLineObject::Reset()
{
  CStationaryObject::Reset();

  line_id = DEFAULT_LINE;
}

CAngleDeg CLineObject::GetAbsoluteAngle(const int_type hist) const
{
  CAngleDeg angle = GetDirection(hist);

  switch ( line_id ) {
    case SL_Top:
      if ( angle > 0 ) {
        return -angle;
      }
      else {
        return 180 - angle;
      } 
      break;
    case SL_Bottom:
      if ( angle > 0 ) {
        return 180 - angle;
      }
      else {
        return -angle;
      }
      break;
    case SL_Left:
      if ( angle > 0 ) {
        return 270 - angle;
      }
      else if ( angle < 0 ) {
        return 90 - angle;
      } 
      else {
#ifdef DEBUG
	std::cout << "Couldn't compute face dir. Special case 1 is not treated.\n";
#endif
	return 270;
      }
      break;
    case SL_Right:
      if ( angle > 0 ) {
        return 90 - angle;
      }
      else if ( angle < 0 ) {
        return 270 - angle;
      } 
      else {
#ifdef DEBUG
	std::cout << "Couldn't compute face dir. Special case 2 is not treated.\n";
#endif
	return 90;
      }
      break;
    default:
      std::cout <<  "Unknown line id in computeFaceDir.\n";
      return UNKNOWN_DEG;
  }

}
