#include "CLogFile.h"

CLogFile::CLogFile(CKnowledge* kb, const std::string& fname,
		   const bool app)
  : k(kb),
    filename(fname),
    append(app),
    open(false),
    log()
{
}

/*
CLogFile::~CLogFile()
{
}
*/

bool CLogFile::Open()
{
  if (append)
    log.open(filename.c_str(), std::ios::out | std::ios::app);
  else
    log.open(filename.c_str(), std::ios::out | std::ios::trunc);
      
  if (!log) {
    open = false;
    return false;
  } else {
    open = true;
    LogNewUnformattedEntry();
    log << "Logfile opened" << std::endl;
    return true;
  }
}

bool CLogFile::Close()
{
  LogNewUnformattedEntry();
  log << "Logfile closed" << std::endl;
  log.close();

  if (!log) {
    return false;
  } else {
    open = false;
    return true;
  }
}

void CLogFile::LogText(const std::string& txt)
{
  LogNewUnformattedEntry();
  log << txt << std::endl;
}

void CLogFile::LogGameStatus()
{
  LogNewFormattedEntry();
  log << int(k->theGame().GetPlayMode()) << " "
      << int(k->theGame().GetKickOffMode()) << " "
      << k->theGame().GetOurScore() << " - "
      << k->theGame().GetTheirScore()
      << std::endl;
}

void CLogFile::LogAgentStatus(const int_type hist)
{
  LogNewFormattedEntry();
  log << "hist: " << hist
      << " stamina: " << k->theAgent().GetStamina(hist)
      << " effort: " << k->theAgent().GetEffort(hist)
      << " recover: " << k->theAgent().GetRecover(hist)
      << " dashes: " << k->theAgent().GetDashes(hist)
      << " kicks: " << k->theAgent().GetKicks(hist)
      << " turns: " << k->theAgent().GetTurns(hist)
      << " says: " << k->theAgent().GetSays(hist)
      << " turn_necks: " << k->theAgent().GetTurnNecks(hist)
      << " view_width: " << int(k->theAgent().GetViewWidth(hist))
      << " view_quality: " << int(k->theAgent().GetViewQuality(hist))
      << std::endl;
}

void CLogFile::LogAgentLocation(const int_type hist)
{
  LogNewFormattedEntry();
  log << "AGENT " // << hist
      << " position: " << k->theAgent().GetPosition(hist)
      << " speed: " << k->theAgent().GetAbsoluteSpeedVector(hist)
      << " face_dir: " << k->theAgent().GetFaceDir(hist)
      << " body_dir: " << k->theAgent().GetBodyDir(hist)
    //      << " neck_dir: " << k->theAgent().GetNeckDir(hist)
      << std::endl;
}

void CLogFile::LogBallLocation(const int_type hist)
{
  LogNewFormattedEntry();
  log << "BALL " // << hist
      << " position: " << k->theBall().GetPosition(hist)
      << " vector: " << k->theBall().GetVector(hist)
      << " speed: " << k->theBall().GetAbsoluteSpeedVector(hist)
      << std::endl;
}

void CLogFile::LogSentCommand(const CCommand* const cmd)
{
  LogNewFormattedEntry();
  log << "Sent command " << cmd->ServerString() << std::endl;
}
