#include "CMarkerObject.h"

const t_marker CMarkerObject::DEFAULT_MARKER = No_Marker;

CMarkerObject::CMarkerObject(const t_marker marker,
			     const CPoint& pos,
			     const int_type hist_len)
  : CStationaryObject(hist_len),
    marker_id(marker)
{

  for (int_type i=0; i < max_length; ++i) {
    positions[i] = pos;
  }
  
}

CMarkerObject::CMarkerObject(const CMarkerObject& obj)
  : CStationaryObject(obj),
    marker_id(obj.marker_id)
{

}

void CMarkerObject::CopyData(const CMarkerObject& obj)
{
  CStationaryObject::CopyData(obj);
  marker_id = obj.marker_id;
}    

CMarkerObject& CMarkerObject::operator= (const CMarkerObject& obj)
{

  if (this != &obj) {
    CopyData(obj);
  }

  return *this;
  
}

void CMarkerObject::ResetAll()
{
  Reset();
}

void CMarkerObject::Reset()
{
  CStationaryObject::Reset();

  marker_id = DEFAULT_MARKER;
}

CPoint CMarkerObject::GetAbsolutePosition(const CAngleDeg& obj_ang,
					  const int_type hist) const
{
  CAngleDeg abs_ang = GetDirection(hist) + obj_ang;
  CVector rel_vec(abs_ang, GetDistance(hist));
  
  return GetPosition(hist) - rel_vec;
}

bool operator <(const CMarkerObject& obj1, const CMarkerObject& obj2)
{
  return (obj1.GetDistance(NOW) > obj2.GetDistance(NOW));
}
