#include "CMobileObject.h"

const float_type CMobileObject::DEFAULT_DIST_CHANGED = UNKNOWN_FLOAT_TYPE;
const CAngleDeg CMobileObject::DEFAULT_DIR_CHANGED = UNKNOWN_DEG;
const CVector CMobileObject::DEFAULT_SPEED = CVector(0,0); //UNKNOWN_VECTOR;
const CVector CMobileObject::DEFAULT_ACCELERATION = UNKNOWN_VECTOR;
const float_type CMobileObject::DEFAULT_SPEED_DECAY = UNKNOWN_FLOAT_TYPE;
const float_type CMobileObject::DEFAULT_SPEED_MAX = UNKNOWN_FLOAT_TYPE;

CMobileObject::CMobileObject(const int_type hist_length)
  : CGameObject(hist_length),
    dist_changed(max_length),
    dir_changed(max_length),
    speeds(max_length),
    accelerations(max_length),
    speed_decay(DEFAULT_SPEED_DECAY),
    speed_max(DEFAULT_SPEED_MAX)
{

  for (int_type i=0; i < max_length; ++i) {
    Reset(i);
  }
  
}

CMobileObject::CMobileObject(const CMobileObject& obj)
  : CGameObject(obj),
    dist_changed(obj.dist_changed),
    dir_changed(obj.dir_changed),
    speeds(obj.speeds),
    accelerations(obj.accelerations),
    speed_decay(obj.speed_decay),
    speed_max(obj.speed_max)
{

}

/*
CMobileObject::~CMobileObject()
{
  
}
*/

void CMobileObject::CopyData(const CMobileObject& obj)
{
  CGameObject::CopyData(obj);
  dist_changed  = obj.dist_changed;
  dir_changed   = obj.dir_changed;
  speeds        = obj.speeds;
  accelerations = obj.accelerations;
  speed_decay   = obj.speed_decay;
  speed_max     = obj.speed_max;
}

CMobileObject& CMobileObject::operator= (const CMobileObject& obj)
{

  if (this != &obj) {
    CopyData(obj);
  }

  return *this;  
  
}

void CMobileObject::Copy(const int_type from, const int_type to)
{
  CGameObject::Copy(from, to);

  CopyDistChanged(from, to);
  CopyDirChanged(from, to);
  CopySpeedVector(from, to);
  CopyAcceleration(from, to);
}

void CMobileObject::CopyFrom(const CMobileObject& obj, const int_type from,
			     const int_type to)
{
  CGameObject::CopyFrom(obj, from, to);

  dist_changed[GetIndex(to)] = obj.dist_changed[GetIndex(from)];
  dir_changed[GetIndex(to)] = obj.dir_changed[GetIndex(from)];
  speeds[GetIndex(to)] = obj.speeds[GetIndex(from)];
  accelerations[GetIndex(to)] = obj.accelerations[GetIndex(from)];
}

void CMobileObject::ResetAll()
{
  Reset();

  for (int_type i=0; i < max_length; ++i) {
    Reset(i);
  }
}  

void CMobileObject::Reset()
{
  CGameObject::Reset();

  speed_decay = DEFAULT_SPEED_DECAY;
  speed_max   = DEFAULT_SPEED_MAX;
}

void CMobileObject::Reset(const int_type i)
{

  CGameObject::Reset(i);

  dist_changed[GetIndex(i)]  = DEFAULT_DIST_CHANGED;
  dir_changed[GetIndex(i)]   = DEFAULT_DIR_CHANGED;
  speeds[GetIndex(i)]        = DEFAULT_SPEED;
  accelerations[GetIndex(i)] = DEFAULT_ACCELERATION;
  
}

void CMobileObject::NewCycle()
{

  CGameObject::NewCycle();
  
  dist_changed[GetIndex(NOW)] = DEFAULT_DIST_CHANGED;
  dir_changed[GetIndex(NOW)]  = DEFAULT_DIR_CHANGED;
  CopySpeedVector(PREV, NOW);
  CopyAcceleration(PREV, NOW);
  
}

void CMobileObject::UpdateMovement(const int_type from, const int_type to)
{
  //	      U = speed + acceleration
  CVector U = speeds[GetIndex(from)] + accelerations[GetIndex(from)];

  //position: new_position = position + U
  positions[GetIndex(to)] = positions[GetIndex(from)] + U;
  
  //speed:    new_speed = speed_decay x U
  speeds[GetIndex(to)] = speed_decay * U;
  if ( GetSpeed(to) > speed_max )
    SetSpeed(speed_max, to);

  //acc:      new_acceleration = (0,0)
  accelerations[GetIndex(to)] = CVector(0,0);
}

CVector CMobileObject::GetRelativeSpeedVector(const CAngleDeg& obj_ang,
					      const int_type hist) const
{
  CVector rel_vel = speeds[GetIndex(hist)];
  rel_vel.Rotate(-obj_ang);
  return rel_vel;
}

/*
CPoint CMobileObject::GetFuturePosition(int_type steps) const
{

  return CPoint();
  
}

bool CMobileObject::OnCollisionCourse(const CVector& v) const
{

  return false;
  
}

CPoint CMobileObject::GetPositionAfterCommand(const t_cmd_type cmd, const float_type arg1, const float_type arg2) const
{

  // if dash then power = arg1 else power = arg2

  return CPoint();
  
}

CVector CMobileObject::GetSpeedAfterCommand(const t_cmd_type cmd, const float_type arg1, const float_type arg2) const
{

  return CVector();
  
}
*/

void CMobileObject::Print(const int_type hist) const
{

  if (hist == UNKNOWN_INT_TYPE) {
    for (int_type i = -max_hist_length; i < max_length - max_hist_length; ++i) {
      CGameObject::Print(i);
      std::cout << "[" << i << "]"
		<< " DistChng: " << GetDistChanged(i)
		<< " DirChng: " << GetDirChanged(i)
		<< " SpeedVec: " << GetAbsoluteSpeedVector(i)
		<< std::endl;
    }
  } else {
    CGameObject::Print(hist);
    std::cout << "[" << hist << "]"
	      << " DistChng: " << GetDistChanged(hist)
	      << " DirChng: " << GetDirChanged(hist)
	      << " SpeedVec: " << GetAbsoluteSpeedVector(hist)
	      << std::endl;
  }
  
}
