#ifndef _CMOBILEOBJECT_H_
#define _CMOBILEOBJECT_H_

#include "CGameObject.h"

class CMobileObject : public CGameObject
{
public:
  CMobileObject(const int_type hist_length = DEFAULT_HIST_LENGTH);
  CMobileObject(const CMobileObject& obj);
  //  ~CMobileObject();

  CMobileObject& operator= (const CMobileObject& obj);

  void Copy(const int_type from, const int_type to);
  void CopyFrom(const CMobileObject& obj, const int_type from, const int_type to);

  void ResetAll();
  void Reset();
  void Reset(const int_type i);
  
  void NewCycle();

  void UpdateMovement(const int_type from = NOW, const int_type to = NEXT);
  
  void SetDistChanged(const float_type dist, const int_type hist = NEXT);
  float_type GetDistChanged(const int_type hist = NOW) const;
  void CopyDistChanged(const int_type from, const int_type to);
  void SetDirChanged(const CAngleDeg& dir, const int_type hist = NEXT);
  CAngleDeg GetDirChanged(const int_type hist = NOW) const;
  void CopyDirChanged(const int_type from, const int_type to);

  void SetAbsoluteSpeedVector(const CVector& v, const int_type hist = NEXT);
  // Absolute speed and direction
  CVector GetAbsoluteSpeedVector(const int_type hist = NOW) const;
  // This is actually relative direction and absolute speed
  CVector GetRelativeSpeedVector(const CAngleDeg& obj_ang,
				 const int_type hist = NOW) const;
  void CopySpeedVector(const int_type from, const int_type to);
  void SetSpeed(const t_speed& speed, const int_type hist = NEXT);
  t_speed GetSpeed(const int_type hist = NOW) const;
  void SetAbsoluteSpeedDirection(const CAngleDeg& ang, const int_type hist = NOW);
  CAngleDeg GetAbsoluteSpeedDirection(const int_type hist = NOW) const;
  CAngleDeg GetRelativeSpeedDirection(const CAngleDeg& obj_ang,
				      const int_type hist = NOW) const;
  void SetAcceleration(const CVector& v, const int_type hist = NEXT);
  CVector GetAcceleration(const int_type hist = NOW) const;
  void CopyAcceleration(const int_type from, const int_type to);

  void SetSpeedDecay(const float_type new_speed_decay);
  float_type GetSpeedDecay() const;
  void SetSpeedMax(const float_type new_speed_max);
  float_type GetSpeedMax() const;
  
  // CPoint GetFuturePosition(int_type steps) const;
  // bool OnCollisionCourse(const CVector& v) const;

  void Print(const int_type hist = UNKNOWN_INT_TYPE) const;
  
protected:
  std::vector<float_type> dist_changed;
  std::vector<CAngleDeg> dir_changed;
  std::vector<CVector> speeds;
  std::vector<CVector> accelerations;

  float_type speed_decay;
  float_type speed_max;

  static const float_type DEFAULT_DIST_CHANGED;
  static const CAngleDeg DEFAULT_DIR_CHANGED;
  static const CVector DEFAULT_SPEED;
  static const CVector DEFAULT_ACCELERATION;
  static const float_type DEFAULT_SPEED_DECAY;
  static const float_type DEFAULT_SPEED_MAX;
  
  void CopyData(const CMobileObject&);
  
};

inline void CMobileObject::SetDistChanged(const float_type dist, const int_type hist)
{
  dist_changed[GetIndex(hist)] = dist;
}

inline float_type CMobileObject::GetDistChanged(const int_type hist) const
{
  return dist_changed[GetIndex(hist)];
}

inline void CMobileObject::CopyDistChanged(const int_type from, const int_type to)
{
  dist_changed[GetIndex(to)] = dist_changed[GetIndex(from)];
}

inline void CMobileObject::SetDirChanged(const CAngleDeg& dir, const int_type hist)
{
  dir_changed[GetIndex(hist)] = dir;
}

inline CAngleDeg CMobileObject::GetDirChanged(const int_type hist) const
{
  return dir_changed[GetIndex(hist)];
}

inline void CMobileObject::CopyDirChanged(const int_type from, const int_type to)
{
  dir_changed[GetIndex(to)] = dir_changed[GetIndex(from)];
}
  
inline void CMobileObject::SetAbsoluteSpeedVector(const CVector& v,
						  const int_type hist)
{
  speeds[GetIndex(hist)] = v;
};

inline CVector CMobileObject::GetAbsoluteSpeedVector(const int_type hist) const
{
  return speeds[GetIndex(hist)];
}

inline void CMobileObject::CopySpeedVector(const int_type from, const int_type to)
{
  speeds[GetIndex(to)] = speeds[GetIndex(from)];
}

inline void CMobileObject::SetSpeed(const t_speed& speed, const int_type hist)
{
  speeds[GetIndex(hist)].SetLength(speed);
}

inline t_speed CMobileObject::GetSpeed(const int_type hist) const
{
  return speeds[GetIndex(hist)].GetLength();
}

inline void CMobileObject::SetAbsoluteSpeedDirection(const CAngleDeg& ang,
						     const int_type hist)
{
  speeds[GetIndex(hist)].SetAngle(ang);
}

inline CAngleDeg CMobileObject::GetAbsoluteSpeedDirection(const int_type hist) const
{
  return speeds[GetIndex(hist)].GetAngle();
}

inline CAngleDeg CMobileObject::GetRelativeSpeedDirection(const CAngleDeg& obj_ang,
							  const int_type hist) const
{
  return speeds[GetIndex(hist)].GetAngle() - obj_ang;
}

inline void CMobileObject::SetAcceleration(const CVector& a, const int_type hist)
{
  accelerations[GetIndex(hist)] = a;
};

inline CVector CMobileObject::GetAcceleration(const int_type hist) const
{
  return accelerations[GetIndex(hist)];
}

inline void CMobileObject::CopyAcceleration(const int_type from, const int_type to)
{
  accelerations[GetIndex(to)] = accelerations[GetIndex(from)];
}

inline void CMobileObject::SetSpeedDecay(const float_type new_speed_decay)
{
  speed_decay = new_speed_decay;
}

inline float_type CMobileObject::GetSpeedDecay() const
{
  return speed_decay;
}
  
inline void CMobileObject::SetSpeedMax(const float_type new_speed_max)
{
  speed_max = new_speed_max;
}

inline float_type CMobileObject::GetSpeedMax() const
{
  return speed_max;
}
  
#endif
