#include "CMoveCommand.h"
#include "CKnowledge.h"
#include "compiler_directives.h"
#include "utils.h"

CMoveCommand::CMoveCommand(CKnowledge* const kb, const CPoint& dest,
			   const t_priority prio,
			   const CTime& s_time, const CTime& f_time)
  : CCommand(kb, CMD_Move, prio, s_time, f_time),
    move_type(MOVE_Dest),
    destination(dest)
{
}

CMoveCommand::CMoveCommand(const CMoveCommand& obj)
  : CCommand(obj),
    move_type(obj.move_type),
    destination(obj.destination)
{
}

CMoveCommand::~CMoveCommand()
{
}

void CMoveCommand::CopyCommand(const CMoveCommand& obj)
{
  CCommand::CopyCommand(obj);
  move_type = obj.move_type;
  destination = obj.destination;
}

CMoveCommand& CMoveCommand::operator =(const CMoveCommand& obj)
{
  if ( this != &obj ) {
    CopyCommand(obj);
  }

  return *this;
}

void CMoveCommand::MoveDestination(const CPoint& dest)
{
  move_type = MOVE_Dest;
  destination = dest;
}

void CMoveCommand::UpdateAfterCommand(const int_type from,
				      const int_type to) const
{
  if ( k->theGame().GetPlayMode() == PM_Before_Kick_Off ||
       k->theGame().GetPlayMode() == PM_Our_Goal ||	
       k->theGame().GetPlayMode() == PM_Their_Goal ||
#ifndef _FREEKICK_MOVE_ALL_
       ( k->theGame().GetPlayMode() == PM_Our_Free_Kick &&
	 k->theAgent().GetCyclesSinceCatch() == 0 &&
	 k->theAgent().IsGoalie() )
#else
       k->theGame().GetPlayMode() == PM_Our_Free_Kick ||
       k->theGame().GetPlayMode() == PM_Their_Free_Kick
#endif
       ) {
    k->theAgent().SetPosition(destination, to);
  }
}

bool CMoveCommand::Validate(const int_type hist)
{
  switch(move_type) {
  case MOVE_Dest : // Move to a destination
    break;
  default : // Error, what should I do??
    std::cout << "Unknown move_type in MoveCommand!" << std::endl;
    return false;
    break;
  }

  if ( k->theGame().GetPlayMode() == PM_Before_Kick_Off ||
       k->theGame().GetPlayMode() == PM_Our_Goal ||	
       k->theGame().GetPlayMode() == PM_Their_Goal ||
#ifndef _FREEKICK_MOVE_ALL_
       ( k->theGame().GetPlayMode() == PM_Our_Goalie_Got_Ball &&
	 k->theAgent().IsGoalie() )
#else
       k->theGame().GetPlayMode() == PM_Our_Free_Kick ||
       k->theGame().GetPlayMode() == PM_Their_Free_Kick
#endif
       ) {
    return true;
  } else {
    return false;
  }
}

const std::string CMoveCommand::ServerString() const
{
  return "(move " + ToString(destination.GetX()) + " "
    + ToString(destination.GetY()) + ")";
}
