#include "CParameters.h"
#include <cstdio>

CParameters::CParameters()
{
  Init();
  InitOptions();
}

CParameters::CParameters(int argc, char** argv, int verbose)
{
  Init();
  InitOptions();
  ReadCommandLine(argc, argv, verbose);
}

CParameters::CParameters(const CParameters& obj)
{
  CopyData(obj);
  // Since options refer to the this objects'
  // parameters it shouldn't be copied but
  // rather initialized from 'scratch'.
  InitOptions();
}

CParameters::~CParameters()
{
}

CParameters& CParameters::operator =(const CParameters& obj)
{
  if ( this != &obj) {
    CopyData(obj);
  }

  return *this;
}

void CParameters::Init()
{
  // no option flags for these
  SP_pitch_length    		= 105.0;
  SP_pitch_width 	 	= 68.0;
  SP_pitch_margin	 	= 5.0;
  SP_penalty_area_length 	= 16.5;
  SP_penalty_area_width		= 40.32;
  SP_goal_area_length		= 5.5;
  SP_goal_area_width		= 18.32;
  SP_penalty_spot_dist		= 11.0;
  SP_corner_arc_r		= 1.0;
  SP_free_kick_buffer		= 9.15;
  SP_after_goal_wait  		= 50;
  SP_feel_distance 		= 3.0;
  SP_num_lines     		= 4;
  SP_num_markers   		= 55;
  SP_unum_far_length		= 20.0;
  SP_unum_too_far_length	= 40.0;
  SP_team_far_length		= 40.0;
  SP_team_too_far_length	= 60.0;
  SP_version	 		= 5.06;
  SP_team_size     		= 11;
  SP_half	    		= 1;        
  SP_host                       = "localhost";

  // server.conf starts here        
  SP_goal_width    		= 14.02;       
  SP_player_size   		= 0.3;
  SP_player_decay  		= 0.4;
  SP_player_rand 		= 0.1;
  SP_player_weight		= 60.0;
  SP_player_speed_max		= 1.0;
  SP_stamina_min                = 0.0;
  SP_stamina_max                = 3500.0;
  SP_stamina_inc_max		= 20.0;
  SP_recover_dec_thr		= 0.3;
  SP_recover_dec   		= 0.002;
  SP_recover_min   		= 0.5;
  SP_recover_max   		= 1.0;
  SP_effort_dec_thr		= 0.3;
  SP_effort_dec    		= 0.005;	
  SP_effort_inc_thr		= 0.6;	
  SP_effort_inc    		= 0.01;
  SP_effort_min    		= 0.6;
  SP_effort_max    		= 1.0;
  SP_hear_max                   = 2;
  SP_hear_inc                   = 1;
  SP_hear_decay    		= 2;
  SP_inertia_moment		= 5.0;
  SP_sense_body_step            = 100;
  SP_catchable_area_l  		= 2.0;	
  SP_catchable_area_w  		= 1.0;	
  SP_catch_probability		= 1.0;
  SP_catch_ban_cycle		= 5;        
  SP_ball_size     		= 0.085;		
  SP_ball_decay    		= 0.94;
  SP_ball_rand     		= 0.05;		
  SP_ball_weight   		= 0.2;
  SP_ball_speed_max		= 2.7;
  SP_wind_dir		        = 0.0;	
  SP_wind_force    		= 0.0;
  SP_wind_rand     		= 0.0;	
  SP_wind_none     		= false;
  SP_wind_random   		= false;
  SP_kickable_margin		= 0.7;	
  SP_control_radius             = 2.0;
  SP_ckick_margin               = 1.0;
  SP_dash_power_rate		= 0.006;        
  SP_kick_power_rate		= 0.016;
  SP_visible_angle 		= 90.0;
  SP_visible_distance           = 3.0;
  SP_audio_cut_dist		= 50.0;	
  SP_quantize_step    		= 0.1;
  SP_quantize_step_l    	= 0.01;
  SP_maxpower     		= 100;
  SP_minpower     		= -100;
  SP_maxmoment    		= 180;
  SP_minmoment    		= -180;
  SP_maxneckang                 = 90;
  SP_minneckang                 = -90;
  SP_maxneckmoment              = 180;
  SP_minneckmoment              = -180;
  SP_port	    		= 6000;
  SP_coach_port    		= 6001;
  SP_olcoach_port               = 6002;
  SP_say_coach_cnt_max          = 128;
  SP_say_coach_msg_size         = 128;
  SP_send_vi_step               = 100;
  SP_simulator_step		= 100;
  SP_send_step     		= 150;
  SP_recv_step     		= 10;
  SP_half_time     		= 300;
  SP_say_msg_size  		= 512;
  SP_use_offside   		= true;
  SP_offside_active_area_size   = 9.15;
  SP_forbid_kick_off_offside    = true;
  SP_verbose	 		= false;
  SP_record_version             = 2;
  SP_record_log                 = false;
  SP_send_log                   = true;
  SP_log_file                   = "";
  SP_record                     = "";
  SP_replay                     = "";
  SP_coach                      = false;
  SP_coach_w_referee            = false;
  SP_old_coach_hear             = false;

  SP_kickable_area = SP_kickable_margin + SP_ball_size + SP_player_size;

}

void CParameters::InitOptions()
{
    // variable types and names
  option_t opt[] =
  {{ "pitch_length",              (void *) &SP_pitch_length,               V_FLOAT_TYPE },
   { "pitch_width",               (void *) &SP_pitch_width,                V_FLOAT_TYPE },
   { "pitch_margin",              (void *) &SP_pitch_margin,               V_FLOAT_TYPE },
   { "penalty_area_length",       (void *) &SP_penalty_area_length,        V_FLOAT_TYPE },
   { "penalty_area_width",        (void *) &SP_penalty_area_width,         V_FLOAT_TYPE },
   { "goal_area_length",          (void *) &SP_goal_area_length,           V_FLOAT_TYPE },
   { "goal_area_width",           (void *) &SP_goal_area_width,            V_FLOAT_TYPE },
   { "penalty_spot_dist",         (void *) &SP_penalty_spot_dist,          V_FLOAT_TYPE },
   { "corner_arc_r",              (void *) &SP_corner_arc_r,               V_FLOAT_TYPE },
   { "free_kick_buffer",          (void *) &SP_free_kick_buffer,           V_FLOAT_TYPE },
   { "after_goal_wait",           (void *) &SP_after_goal_wait,            V_INT_TYPE },
   { "feel_distance",             (void *) &SP_feel_distance,              V_FLOAT_TYPE },
   { "num_lines",                 (void *) &SP_num_lines,                  V_INT_TYPE },
   { "num_markers",               (void *) &SP_num_markers,                V_INT_TYPE },
   { "unum_far_length",           (void *) &SP_unum_far_length,            V_FLOAT_TYPE },
   { "unum_too_far_length",       (void *) &SP_unum_too_far_length,        V_FLOAT_TYPE },
   { "team_far_length",           (void *) &SP_team_far_length,            V_FLOAT_TYPE },
   { "team_too_far_length",       (void *) &SP_team_too_far_length,        V_FLOAT_TYPE },
   { "version",                   (void *) &SP_version,                    V_FLOAT_TYPE },
   { "team_size",                 (void *) &SP_team_size,                  V_INT_TYPE },
   { "half",                      (void *) &SP_half,                       V_INT_TYPE },
   { "host",                      (void *) &SP_host,                       V_STRING },  
   { "goal_width",                (void *) &SP_goal_width,                 V_FLOAT_TYPE },
   { "player_size",               (void *) &SP_player_size,                V_FLOAT_TYPE },
   { "player_decay",              (void *) &SP_player_decay,               V_FLOAT_TYPE },
   { "player_rand",               (void *) &SP_player_rand,                V_FLOAT_TYPE },
   { "player_weight",             (void *) &SP_player_weight,              V_FLOAT_TYPE },
   { "player_speed_max",          (void *) &SP_player_speed_max,           V_FLOAT_TYPE },
   { "stamina_min",               (void *) &SP_stamina_min,                V_FLOAT_TYPE },
   { "stamina_max",               (void *) &SP_stamina_max,                V_FLOAT_TYPE },
   { "stamina_inc_max",           (void *) &SP_stamina_inc_max,            V_FLOAT_TYPE },
   { "recover_dec_thr",           (void *) &SP_recover_dec_thr,            V_FLOAT_TYPE },
   { "recover_dec",               (void *) &SP_recover_dec,                V_FLOAT_TYPE },
   { "recover_min",               (void *) &SP_recover_min,                V_FLOAT_TYPE },
   { "recover_max",               (void *) &SP_recover_max,                V_FLOAT_TYPE },
   { "effort_dec_thr",            (void *) &SP_effort_dec_thr,             V_FLOAT_TYPE },
   { "effort_dec",                (void *) &SP_effort_dec,                 V_FLOAT_TYPE },
   { "effort_inc_thr",            (void *) &SP_effort_inc_thr,             V_FLOAT_TYPE },
   { "effort_inc",                (void *) &SP_effort_inc,                 V_FLOAT_TYPE },
   { "effort_min",                (void *) &SP_effort_min,                 V_FLOAT_TYPE },
   { "effort_max",                (void *) &SP_effort_max,                 V_FLOAT_TYPE },
   { "hear_max",                  (void *) &SP_hear_max,                   V_INT_TYPE },
   { "hear_inc",                  (void *) &SP_hear_inc,                   V_INT_TYPE },
   { "hear_decay",                (void *) &SP_hear_decay,                 V_INT_TYPE },
   { "inertia_moment",            (void *) &SP_inertia_moment,             V_FLOAT_TYPE },
   { "sense_body_step",           (void *) &SP_sense_body_step,            V_INT_TYPE },
 
   { "catchable_area_l",          (void *) &SP_catchable_area_l,           V_FLOAT_TYPE },
   { "catchable_area_w",          (void *) &SP_catchable_area_w,           V_FLOAT_TYPE },
   { "catch_probability",         (void *) &SP_catch_probability,          V_FLOAT_TYPE },
   { "catch_ban_cycle",           (void *) &SP_catch_ban_cycle,            V_INT_TYPE },
 
   { "ball_size",                 (void *) &SP_ball_size,                  V_FLOAT_TYPE },
   { "ball_decay",                (void *) &SP_ball_decay,                 V_FLOAT_TYPE },
   { "ball_rand",                 (void *) &SP_ball_rand,                  V_FLOAT_TYPE },
   { "ball_weight",               (void *) &SP_ball_weight,                V_FLOAT_TYPE },
   { "ball_speed_max",            (void *) &SP_ball_speed_max,             V_FLOAT_TYPE },
 
   { "wind_force",                (void *) &SP_wind_force,                 V_FLOAT_TYPE },
   { "wind_dir",                  (void *) &SP_wind_dir,                   V_FLOAT_TYPE },
   { "wind_rand",                 (void *) &SP_wind_rand,                  V_FLOAT_TYPE },
   { "wind_none",                 (void *) &SP_wind_none,                  V_BOOL },
   { "wind_random",               (void *) &SP_wind_random,                V_BOOL },
   { "kickable_margin",           (void *) &SP_kickable_margin,            V_FLOAT_TYPE },
   { "control_radius",            (void *) &SP_control_radius,             V_FLOAT_TYPE },
   { "ckick_margin",              (void *) &SP_ckick_margin,               V_FLOAT_TYPE },
   { "dash_power_rate",           (void *) &SP_dash_power_rate,            V_FLOAT_TYPE },
   { "kick_power_rate",           (void *) &SP_kick_power_rate,            V_FLOAT_TYPE },
   { "visible_angle",             (void *) &SP_visible_angle,              V_FLOAT_TYPE },
   { "visible_distance",          (void *) &SP_visible_distance,           V_FLOAT_TYPE },
   { "audio_cut_dist",            (void *) &SP_audio_cut_dist,             V_FLOAT_TYPE },
   { "quantize_step",             (void *) &SP_quantize_step,              V_FLOAT_TYPE },
   { "quantize_step_l",           (void *) &SP_quantize_step_l,            V_FLOAT_TYPE },
   { "maxpower",                  (void *) &SP_maxpower,                   V_FLOAT_TYPE },
   { "minpower",                  (void *) &SP_minpower,                   V_FLOAT_TYPE },
   { "maxmoment",                 (void *) &SP_maxmoment,                  V_FLOAT_TYPE },
   { "minmoment",                 (void *) &SP_minmoment,                  V_FLOAT_TYPE },
   { "maxneckang",                (void *) &SP_maxneckang,                 V_FLOAT_TYPE },
   { "minneckang",                (void *) &SP_minneckang,                 V_FLOAT_TYPE },
   { "maxneckmoment",             (void *) &SP_maxneckmoment,              V_FLOAT_TYPE },
   { "minneckmoment",             (void *) &SP_minneckmoment,              V_FLOAT_TYPE },
   { "port",                      (void *) &SP_port,                       V_INT_TYPE },
   { "coach_port",                (void *) &SP_coach_port,                 V_INT_TYPE },
   { "olcoach_port",              (void *) &SP_olcoach_port,               V_INT_TYPE },
   { "say_coach_cnt_max",         (void *) &SP_say_coach_cnt_max,          V_INT_TYPE },
   { "say_coach_msg_size",        (void *) &SP_say_coach_msg_size,         V_INT_TYPE },
   { "send_vi_step",              (void *) &SP_send_vi_step,               V_INT_TYPE },
   { "simulator_step",            (void *) &SP_simulator_step,             V_INT_TYPE },
   { "send_step",                 (void *) &SP_send_step,                  V_INT_TYPE },
   { "recv_step",                 (void *) &SP_recv_step,                  V_INT_TYPE },
   { "half_time",                 (void *) &SP_half_time,                  V_INT_TYPE },
   { "say_msg_size",              (void *) &SP_say_msg_size,               V_INT_TYPE },
   { "use_offside",               (void *) &SP_use_offside,                V_ONOFF },
   { "offside_active_area_size",  (void *) &SP_offside_active_area_size,   V_FLOAT_TYPE },
   { "forbid_kick_off_offside",   (void *) &SP_forbid_kick_off_offside,    V_ONOFF },
   { "verbose",                   (void *) &SP_verbose,                    V_ONOFF },
   { "record_version",            (void *) &SP_record_version,             V_INT_TYPE },
   { "record_log",                (void *) &SP_record_log,                 V_BOOL },
   { "send_log",                  (void *) &SP_send_log,                   V_BOOL },
   { "log_file",                  (void *) &SP_log_file,                   V_STRING },
   { "record",                    (void *) &SP_record,                     V_STRING },
   { "replay",                    (void *) &SP_replay,                     V_STRING },
 
   { "coach",                     (void *) &SP_coach,                      V_BOOL },
   { "coach_w_referee",           (void *) &SP_coach_w_referee,            V_BOOL },
   { "old_coach_hear",            (void *) &SP_old_coach_hear,             V_BOOL },
   // server.conf ends here

   { "kickable_area",             (void *) &SP_kickable_area,              V_FLOAT_TYPE },
   { "\000", 0, V_NONE }
  };

  int i = 0;
  while (opt[i].vptr != 0 && i < optsize) {
    strcpy(options[i].name, opt[i].name);
    options[i].vptr = opt[i].vptr;
    options[i].vsize = opt[i].vsize;
    ++i;
  }

  /*
  RASSERT1(opt[i].vptr == 0, ROBOBUG_ERROR, PARAMETER_CLASS,
	   "Option array too small (%u)", optsize);
  */
  
  i = (i < optsize - 1) ? i : optsize - 1;
  options[i].vptr = 0;
}

void CParameters::CopyData(const CParameters& obj)
{

  SP_pitch_length    		= obj.SP_pitch_length;
  SP_pitch_width 	 	= obj.SP_pitch_width;
  SP_pitch_margin	 	= obj.SP_pitch_margin;
  SP_penalty_area_length 	= obj.SP_penalty_area_length;
  SP_penalty_area_width		= obj.SP_penalty_area_width;
  SP_goal_area_length		= obj.SP_goal_area_length;
  SP_goal_area_width		= obj.SP_goal_area_width;
  SP_penalty_spot_dist		= obj.SP_penalty_spot_dist;
  SP_corner_arc_r		= obj.SP_corner_arc_r;
  SP_free_kick_buffer		= obj.SP_free_kick_buffer;
  SP_after_goal_wait  		= obj.SP_after_goal_wait;
  SP_feel_distance 		= obj.SP_feel_distance;
  SP_num_lines     		= obj.SP_num_lines;
  SP_num_markers   		= obj.SP_num_markers;
  SP_unum_far_length		= obj.SP_unum_far_length;
  SP_unum_too_far_length	= obj.SP_unum_too_far_length;
  SP_team_far_length		= obj.SP_team_far_length;
  SP_team_too_far_length	= obj.SP_team_too_far_length;
  SP_version	 		= obj.SP_version;
  SP_team_size     		= obj.SP_team_size;
  SP_half	    		= obj.SP_half;
  SP_host                       = obj.SP_host;

  // server.conf starts here        
  SP_goal_width    		= obj.SP_goal_width;
  SP_player_size   		= obj.SP_player_size;
  SP_player_decay  		= obj.SP_player_decay;
  SP_player_rand 		= obj.SP_player_rand;
  SP_player_weight		= obj.SP_player_weight;
  SP_player_speed_max		= obj.SP_player_speed_max;
  SP_stamina_min                = obj.SP_stamina_min;
  SP_stamina_max                = obj.SP_stamina_max;
  SP_stamina_inc_max		= obj.SP_stamina_inc_max;
  SP_recover_dec_thr		= obj.SP_recover_dec_thr;
  SP_recover_dec   		= obj.SP_recover_dec;
  SP_recover_min   		= obj.SP_recover_min;
  SP_recover_max   		= obj.SP_recover_max;
  SP_effort_dec_thr		= obj.SP_effort_dec_thr;
  SP_effort_dec    		= obj.SP_effort_dec;	
  SP_effort_inc_thr		= obj.SP_effort_inc_thr;	
  SP_effort_inc    		= obj.SP_effort_inc;
  SP_effort_min    		= obj.SP_effort_min;
  SP_effort_max    		= obj.SP_effort_max;
  SP_hear_max                   = obj.SP_hear_max;
  SP_hear_inc                   = obj.SP_hear_inc;
  SP_hear_decay    		= obj.SP_hear_decay;
  SP_inertia_moment		= obj.SP_inertia_moment;
  SP_sense_body_step            = obj.SP_sense_body_step;
  SP_catchable_area_l  		= obj.SP_catchable_area_l;	
  SP_catchable_area_w  		= obj.SP_catchable_area_w;	
  SP_catch_probability		= obj.SP_catch_probability;
  SP_catch_ban_cycle		= obj.SP_catch_ban_cycle;
  SP_ball_size     		= obj.SP_ball_size;
  SP_ball_decay    		= obj.SP_ball_decay;
  SP_ball_rand     		= obj.SP_ball_rand;
  SP_ball_weight   		= obj.SP_ball_weight;
  SP_ball_speed_max		= obj.SP_ball_speed_max;
  SP_wind_dir		        = obj.SP_wind_dir;	
  SP_wind_force    		= obj.SP_wind_force;
  SP_wind_rand     		= obj.SP_wind_rand;	
  SP_wind_none     		= obj.SP_wind_none;
  SP_wind_random   		= obj.SP_wind_random;
  SP_kickable_margin		= obj.SP_kickable_margin;
  SP_control_radius             = obj.SP_control_radius;
  SP_ckick_margin               = obj.SP_ckick_margin;
  SP_dash_power_rate		= obj.SP_dash_power_rate;
  SP_kick_power_rate		= obj.SP_kick_power_rate;
  SP_visible_angle 		= obj.SP_visible_angle;
  SP_visible_distance           = obj.SP_visible_distance;
  SP_audio_cut_dist		= obj.SP_audio_cut_dist;
  SP_quantize_step    		= obj.SP_quantize_step;
  SP_quantize_step_l    	= obj.SP_quantize_step_l;
  SP_maxpower     		= obj.SP_maxpower;
  SP_minpower     		= obj.SP_minpower;
  SP_maxmoment    		= obj.SP_maxmoment;
  SP_minmoment    		= obj.SP_minmoment;
  SP_maxneckang                 = obj.SP_maxneckang;
  SP_minneckang                 = obj.SP_minneckang;
  SP_maxneckmoment              = obj.SP_maxneckmoment;
  SP_minneckmoment              = obj.SP_minneckmoment;
  SP_port	    		= obj.SP_port;
  SP_coach_port    		= obj.SP_coach_port;
  SP_olcoach_port               = obj.SP_olcoach_port;
  SP_say_coach_cnt_max          = obj.SP_say_coach_cnt_max;
  SP_say_coach_msg_size         = obj.SP_say_coach_msg_size;
  SP_send_vi_step               = obj.SP_send_vi_step;
  SP_simulator_step		= obj.SP_simulator_step;
  SP_send_step     		= obj.SP_send_step;
  SP_recv_step     		= obj.SP_recv_step;
  SP_half_time     		= obj.SP_half_time;
  SP_say_msg_size  		= obj.SP_say_msg_size;
  SP_use_offside   		= obj.SP_use_offside;
  SP_offside_active_area_size   = obj.SP_offside_active_area_size;
  SP_forbid_kick_off_offside    = obj.SP_forbid_kick_off_offside;
  SP_verbose	 		= obj.SP_verbose;
  SP_record_version             = obj.SP_record_version;
  SP_record_log                 = obj.SP_record_log;
  SP_send_log                   = obj.SP_send_log;
  SP_log_file                   = obj.SP_log_file;
  SP_record                     = obj.SP_record;
  SP_replay                     = obj.SP_replay;
  SP_coach                      = obj.SP_coach;
  SP_coach_w_referee            = obj.SP_coach_w_referee;
  SP_old_coach_hear             = obj.SP_old_coach_hear;
  // server.conf ends here

  SP_kickable_area              = obj.SP_kickable_area;

  // Since this assignment isn't safe and
  // options already is initialized if CopyData is called
  // from operator = don't do anything about it.
  // This means that the copy constructor needs to
  // initialize options.
  // options = obj.options;
  
}

void CParameters::ReadCommandLine(int argc, char** argv, int verbose)
{
  // skip command name
  argv++ ; argc-- ;

  // first, search for option '-f' that's read a configuration file
  for(int i = 0 ; i < argc ; ++i) {
    if (!strcmp(*(argv + i),"-f")) {
      ReadOptions(*(argv+i+1), verbose);
    }
  }

  // next, analyze command line option
  while (argc) {
    if (!strcmp(*argv, "-f")) {
      argv += 2 ;
      argc -= 2 ;
      continue ;
    }

    if ( ReadOption(*argv + 1, *(argv + 1)) ) {
      argv += 2 ;
      argc -= 2 ;
    } else {
      argv++ ;
      argc-- ;
    }
  }
}

// --- public -----------------------------------------------------------------
void CParameters::ReadOptions(const std::string& filename, int verbose)
{
  FILE *file;

  file = fopen(filename.c_str(), "r");

  if (file == NULL) {
    if (verbose > 0) {
      fprintf(stderr, "can't open (%s)\n",
	      filename.c_str());
      /*
	} else {
	ROBOBUG1(ROBOBUG_WARN, PARAMETER_CLASS,
	"can't open (%s)", filename);
	*/
    }
	  
    return;
  }

  char linebuf[256];
  char *t;
  int i = 0;
        
  while (fgets(linebuf, 256, file) != 0) {
    ++i;
    if (verbose == 1 && i % 10 == 0)
      fprintf(stderr, ".");
                
    if (verbose > 2) {
      fprintf(stderr,linebuf);
      fflush(stderr);
    }

    if (linebuf[0] == '#'
	|| linebuf[0] == '\n'
	|| linebuf[0] == '\0')
      continue;

    t = linebuf;
    // replace the ':' by space
    while (*t != '\0') {
      if (*t == ':') *t = ' ';
      t++;
    }
    ReadOption(linebuf);
		
  }
}

bool CParameters::ReadOption(const char* linebuf, int verbose)
{
  return ReadOption(linebuf, verbose, options);
}

bool CParameters::ReadOption(const char* option, const char* value)
{
  return ReadOption(option, value, options);
}

bool CParameters::ReadOption(const char* linebuf, int verbose, const option_t* opts)
{
  int n;
  char com[64];
  char val[64] ;

  n = sscanf(linebuf,"%s %s", com, val) ;

  /*
    if (n < 1) {
    ROBOBUG1(ROBOBUG_WARN, PARAMETER_CLASS,
    "Read illegal line: %s", linebuf);
    continue;
    }
    */

  return ReadOption(com, val, opts);

  /*
  for (int p = 0 ; opts[p].vptr != 0; p++) {
    if (strcmp(com, opts[p].name))
      continue;
    
    // match
    switch(opts[p].vsize) {
    case V_INT_TYPE:
      n = sscanf(linebuf, "%s %d",
		 com, (int_type *) opts[p].vptr);
      break;
    case V_STRING:
      n = sscanf(linebuf, "%s %s",
		 com, (char *) opts[p].vptr);
      break;
    case V_FLOAT_TYPE:
      n = sscanf(linebuf, "%s %lf", com,
		 (float_type *) opts[p].vptr);
      break;
    case V_BOOL:
      n = 2;
      *((bool *) opts[p].vptr) = true;
      break;
    case V_ONOFF:
      n = sscanf(linebuf, "%s %s", com, onoff);
      *((bool *) opts[p].vptr) =
	(!strcmp(onoff, "on")) ? true : false;
      break;
    case V_NONE:
      n = 0;
    }

    if (n < 2) {
      if (verbose > 0) {
	fprintf(stderr, "Illegal line (%s)\n",
		com);
	fflush(stderr);
      // } else {
      // ROBOBUG1(ROBOBUG_WARN, PARAMETER_CLASS,
      //	 "Read illegal line (%s)",
      //	 com);
      }
      break;
    }
  }
  if (opts[p].vptr == 0) {
    if (verbose > 0) {
      fprintf(stderr, "Unknown option (%s)\n", com);
      fflush(stderr);
	// } else {
	//ROBOBUG1(ROBOBUG_WARN, PARAMETER_CLASS,
	//"Unknown option (%s)", com);
    }
  }
  */
}

bool CParameters::ReadOption(const char* option, const char* value, const option_t* opts)
{
  int p;
  for (p = 0 ; opts[p].vptr != NULL ; p++) {
    if (strcmp(option, opts[p].name))
      continue ;

    switch(opts[p].vsize) {
    case V_INT_TYPE:
      *((int_type *)opts[p].vptr) = atoi(value) ;
      break ;

    case V_STRING:
      *((std::string *)opts[p].vptr) = value ;
      break ;

    case V_FLOAT_TYPE:
      *((float_type *)opts[p].vptr) = atof(value) ;
      break ;

    case V_BOOL:
      *((bool *)opts[p].vptr) = (!strcmp(value, "false")) ? false : true ;
      break ;

    case V_ONOFF:
      *((bool *)opts[p].vptr) = (!strcmp(value, "on")) ? true : false ;
      break ;
    case V_NONE:
      break ;
    }
    break ;
  }

  if (opts[p].vptr == NULL) {
    // cerr << "Unrecognized Option : " << option << endl ;
    return false;
  }

  return true;
}
