#ifndef _CPARAMETERS_H
#define _CPARAMETERS_H

// This class is based on the Parameter class in RoboLog

#include <string>
#include "types.h"

class CParameters
{
public:
  CParameters();
  CParameters(int argc, char** argv, int verbose = 0);
  CParameters(const CParameters&);
  virtual ~CParameters();

  CParameters& operator =(const CParameters&);
  
  // I make these virtual so that the user can define its own
  // subclass and still be able to send pointers to it to the
  // classes that use CParameters and still call the "right"
  // functions even though it's a pointer to a super class.
  virtual void ReadCommandLine(int argc, char** argv, int verbose = 0);
  virtual void ReadOptions(const std::string& filename, int verbose = 0);
  virtual bool ReadOption(const char* linebuf, int verbose = 0);
  virtual bool ReadOption(const char* option, const char* value);

  /*
  // These are included so that one can have pointers to this
  // class and get values from subclasses.
  virtual Get(const std::string&, float_type*);
  virtual Get(const std::string&, int_type*);
  virtual Get(const std::string&, std::string*);
  */
  
  // Server params
  float_type  SP_pitch_length;
  float_type  SP_pitch_width;
  float_type  SP_pitch_margin;
  float_type  SP_penalty_area_length;
  float_type  SP_penalty_area_width;
  float_type  SP_goal_area_length;
  float_type  SP_goal_area_width;
  float_type  SP_penalty_spot_dist;
  float_type  SP_corner_arc_r;
  float_type  SP_free_kick_buffer;
  int_type    SP_after_goal_wait;
  float_type  SP_feel_distance;
  int_type    SP_num_lines;
  int_type    SP_num_markers;
  float_type  SP_unum_far_length;
  float_type  SP_unum_too_far_length;
  float_type  SP_team_far_length;
  float_type  SP_team_too_far_length;
  
  float_type  SP_version;
  int_type    SP_team_size;
  int_type    SP_half;
  std::string SP_host;
  
  // server.conf starts here
  float_type  SP_goal_width;
  
  float_type  SP_player_size;	
  float_type  SP_player_decay;    	
  float_type  SP_player_rand;	
  float_type  SP_player_weight;	
  float_type  SP_player_speed_max;

  float_type  SP_stamina_min;
  float_type  SP_stamina_max;
  float_type  SP_stamina_inc_max;
  float_type  SP_recover_dec_thr;
  float_type  SP_recover_dec;
  float_type  SP_recover_min;
  float_type  SP_recover_max;
        
  float_type  SP_effort_dec_thr;
  float_type  SP_effort_dec;
  float_type  SP_effort_inc_thr;
  float_type  SP_effort_inc;
  float_type  SP_effort_min;
  float_type  SP_effort_max;

  int_type    SP_hear_max;
  int_type    SP_hear_inc;
  int_type    SP_hear_decay;        

  float_type  SP_inertia_moment;
  
  int_type    SP_sense_body_step;
  
  float_type  SP_catchable_area_l;
  float_type  SP_catchable_area_w;
  float_type  SP_catch_probability;
  int_type    SP_catch_ban_cycle;
        
  float_type  SP_ball_size;		
  float_type  SP_ball_decay;	
  float_type  SP_ball_rand;		
  float_type  SP_ball_weight;
  float_type  SP_ball_speed_max;

  float_type  SP_wind_force;
  float_type  SP_wind_dir;        
  float_type  SP_wind_rand;
  bool        SP_wind_none;
  bool        SP_wind_random;

  float_type  SP_kickable_margin;
  float_type  SP_control_radius;
  float_type  SP_ckick_margin;
  float_type  SP_dash_power_rate;
  float_type  SP_kick_power_rate;	

  float_type  SP_visible_angle;
  float_type  SP_visible_distance;
  float_type  SP_audio_cut_dist;

  float_type  SP_quantize_step;
  float_type  SP_quantize_step_l;        

  float_type  SP_maxpower;		
  float_type  SP_minpower;
  float_type  SP_maxmoment;		
  float_type  SP_minmoment;
  float_type  SP_maxneckang;
  float_type  SP_minneckang;
  float_type  SP_maxneckmoment;
  float_type  SP_minneckmoment;

  int_type    SP_port;
  int_type    SP_coach_port;
  int_type    SP_olcoach_port;
        
  int_type    SP_say_coach_cnt_max;
  int_type    SP_say_coach_msg_size;
  int_type    SP_send_vi_step;
  
  int_type    SP_simulator_step;
  int_type    SP_send_step;
  int_type    SP_recv_step;
  int_type    SP_half_time;        
  int_type    SP_say_msg_size;
  
  bool	      SP_use_offside;
  float_type  SP_offside_active_area_size;
  bool	      SP_forbid_kick_off_offside;
  
  bool	      SP_verbose;
  int_type    SP_record_version;
  bool        SP_record_log;
  bool        SP_send_log;
  std::string SP_log_file;
  std::string SP_record;
  std::string SP_replay;
  
  bool	      SP_coach;
  bool	      SP_coach_w_referee;
  bool        SP_old_coach_hear;
  // server.conf ends here
  
  float_type  SP_kickable_area;

  // Why can't this be protected?? (According to CC, not g++)
  enum t_input_type { V_INT_TYPE, V_FLOAT_TYPE, V_BOOL, V_STRING, V_ONOFF, V_NONE };

protected:
  struct option_t 
  {
    char name[32];
    void *vptr;
    t_input_type vsize;
  };
  
  void CopyData(const CParameters&);
  bool ReadOption(const char* linebuf, int verbose, const option_t*);
  bool ReadOption(const char* option, const char* value, const option_t*);
  
private:
  static const int_type optsize = 110; // The current number is 103
  option_t options[optsize];
        
  void Init();
  void InitOptions();  
};

#endif

