#ifndef _CPLAYEROBJECT_H_
#define _CPLAYEROBJECT_H_

#include "CMobileObject.h"
#include "CParameters.h"

class CPlayerObject : public CMobileObject
{
public:
  CPlayerObject(const CParameters* const p, const int_type,
		const t_team = DEFAULT_TEAM, const t_side = DEFAULT_SIDE,
		const t_unum = DEFAULT_U_NUMBER);
  CPlayerObject(const CParameters* const p, const t_team = DEFAULT_TEAM,
		const t_side = DEFAULT_SIDE, const t_unum = DEFAULT_U_NUMBER,
		const int_type = DEFAULT_HIST_LENGTH);
  CPlayerObject(const CPlayerObject& obj);
  //  ~CPlayerObject();

  CPlayerObject& operator= (const CPlayerObject& obj);

  void Copy(const int_type from, const int_type to);
  void CopyFrom(const CPlayerObject& plr, const int_type from, const int_type to);
  
  void ResetAll();
  void Reset();
  void Reset(const int_type i);

  void NewCycle();
  
  t_team GetTeam() const;
  void SetTeam(const t_team tm);
  void SetTeam(const t_side our_side); 

  t_side GetSide() const;
  void SetSide(const t_side new_side);
  
  t_unum GetUNumber() const;
  void SetUNumber(const t_unum u_num);

  CAngleDeg GetBodyDir(const int_type hist = NOW) const;
  void SetBodyDir(const CAngleDeg& ang, const int_type hist = NEXT);
  void CopyBodyDir(const int_type from, const int_type to);

  CAngleDeg GetNeckDir(const int_type hist = NOW) const;
  void SetNeckDir(const CAngleDeg& ang, const int_type hist = NEXT);
  void CopyNeckDir(const int_type from, const int_type to);

  CAngleDeg GetFaceDir(const int_type hist = NOW) const;
  void SetFaceDir(const CAngleDeg& ang, const int_type hist = NEXT);
  void CopyFaceDir(const int_type from, const int_type to);

  // Redefined since we now have access to the player_speed_max parameter.
  void SetAbsoluteSpeedVector(const CVector& v, const int_type hist = NEXT);
  void SetSpeed(const t_speed& speed, const int_type hist = NEXT);

  CVector GetRelativeVectorTo(const CPoint& pt, const int_type hist = NOW) const;
  CAngleDeg GetRelativeDirectionTo(const CPoint& pt, const int_type hist = NOW) const;

  void Print(const int_type hist = UNKNOWN_INT_TYPE) const;
  
protected:
  const CParameters* params;
  bool have_parameters;
  
  t_team team;
  t_side side;
  t_unum u_number;
  // One of these should be calculated from the other two! Probably body_dir
  std::vector<CAngleDeg> body_dirs;
  std::vector<CAngleDeg> neck_dirs;
  std::vector<CAngleDeg> face_dirs;

  static const t_team DEFAULT_TEAM;
  static const t_side DEFAULT_SIDE;
  static const t_unum DEFAULT_U_NUMBER;
  static const CAngleDeg DEFAULT_BODY_DIR;
  static const CAngleDeg DEFAULT_NECK_DIR;
  static const CAngleDeg DEFAULT_FACE_DIR;
  
  void CopyData(const CPlayerObject&);
  
};

inline t_team CPlayerObject::GetTeam() const
{
  return team;
}

inline void CPlayerObject::SetTeam(const t_team tm)
{
  team = tm;
}

inline t_side CPlayerObject::GetSide() const
{
  return side;
}

inline void CPlayerObject::SetSide(const t_side new_side)
{
  side = new_side;
}
  
inline t_unum CPlayerObject::GetUNumber() const
{
  return u_number;
}

inline void CPlayerObject::SetUNumber(const t_unum u_num)
{
  u_number = u_num;
}

inline CAngleDeg CPlayerObject::GetBodyDir(const int_type hist) const
{
  return body_dirs[GetIndex(hist)];
}

inline void CPlayerObject::SetBodyDir(const CAngleDeg& ang, const int_type hist)
{
  body_dirs[GetIndex(hist)] = ang;
}

inline void CPlayerObject::CopyBodyDir(const int_type from, const int_type to)
{
  body_dirs[GetIndex(to)] = body_dirs[GetIndex(from)];
}

inline CAngleDeg CPlayerObject::GetNeckDir(const int_type hist) const
{
  return neck_dirs[GetIndex(hist)];
}

inline void CPlayerObject::SetNeckDir(const CAngleDeg& ang, const int_type hist)
{
#ifndef _REQUIRE_PARAMETERS_IN_OBJECTS_
  if ( have_parameters ) {
#endif
    neck_dirs[GetIndex(hist)] = MinMax(params->SP_minneckang, ang, params->SP_maxneckang);
#ifndef _REQUIRE_PARAMETERS_IN_OBJECTS_
  } else {
    neck_dirs[GetIndex(hist)] = ang;
  }
#endif
}

inline void CPlayerObject::CopyNeckDir(const int_type from, const int_type to)
{
  neck_dirs[GetIndex(to)] = neck_dirs[GetIndex(from)];
}

inline CAngleDeg CPlayerObject::GetFaceDir(const int_type hist) const
{
  return face_dirs[GetIndex(hist)];
}

inline void CPlayerObject::SetFaceDir(const CAngleDeg& ang, const int_type hist)
{
  face_dirs[GetIndex(hist)] = ang;
}

inline void CPlayerObject::CopyFaceDir(const int_type from, const int_type to)
{
  face_dirs[GetIndex(to)] = face_dirs[GetIndex(from)];
}

#endif
