#ifndef _CPOINT_H_
#define _CPOINT_H_

class CPoint;

#include "compiler_directives.h"
#include "types.h"
#include "CAngleDeg.h"
#include "MathUtils.h"
#include "CVector.h"
#include <iostream>

class CPoint {
public:

  CPoint(const float_type = 0, const float_type = 0);
  CPoint(const CPoint&);
  CPoint(const CVector&, const CPoint&);
  CPoint(const CPoint&, const CVector&);
  CPoint(const CVector&);

  CPoint  operator - ()                 const;
  CPoint  operator - (const float_type) const;
  CPoint  operator - (const CPoint&)    const;
  CPoint  operator - (const CVector&)   const;
  CPoint  operator * (const float_type) const;
  CPoint  operator * (const CPoint&)    const;
  CPoint  operator / (const float_type) const;
  CPoint  operator / (const CPoint&)    const;
  CPoint  operator + (const float_type) const;
  CPoint  operator + (const CPoint&)    const;
  CPoint  operator + (const CVector&)   const;
  CPoint& operator -=(const float_type);
  CPoint& operator -=(const CPoint&);
  CPoint& operator +=(const float_type);
  CPoint& operator +=(const CPoint&);
  CPoint& operator *=(const float_type);
  CPoint& operator *=(const CPoint&);
  CPoint& operator /=(const float_type);
  CPoint& operator /=(const CPoint&);
  CPoint& operator  =(const CPoint&);
  bool    operator ==(const CPoint&)    const;
  bool    operator !=(const CPoint&)    const;
  bool    operator ! ()                 const;

  /*
  // These use the coordinate system of the soccer field
  bool InFront(const CPoint&) const;  // Equal to LeftOf
  bool Behind(const CPoint&)  const;   // Equal to RightOf
  // These use standar coordinate system
  bool LeftOf(const CPoint&)  const;
  bool RightOf(const CPoint&) const;
  boolbove(const CPoint&)     const;
  bool Below(const CPoint&)   const;
  */
  
  CPoint& AddX (const float_type);
  CPoint& AddY (const float_type);
  float_type GetX() const;
  float_type GetY() const;
  CPoint& SetX(const float_type);
  CPoint& SetY(const float_type);
  CPoint& Set (const float_type, const float_type);
  
  CVector VectorTo(const CPoint&) const;
  CAngleDeg DirectionTo(const CPoint&) const;
  float_type DistanceTo(const CPoint&) const;
  
  friend std::ostream& operator<<(std::ostream&, const CPoint&);
  friend std::istream& operator>>(std::istream&, CVector&);
  
protected:
  float_type x;
  float_type y;
  
};

#if CHECK_DIVISION_BY_ZERO == 1
static const CPoint CPOINT_DIVISION_BY_ZERO_RESULT(FLOAT_TYPE_DIVISION_BY_ZERO_RESULT,
						   FLOAT_TYPE_DIVISION_BY_ZERO_RESULT);
#endif CHECK_DIVISION_BY_ZERO

static const CPoint UNKNOWN_POINT(UNKNOWN_FLOAT_TYPE, UNKNOWN_FLOAT_TYPE);


inline CPoint CPoint::operator -() const
{
  return CPoint(-x,-y);
}

inline CPoint CPoint::operator -(const float_type a) const
{
  return CPoint(x - a, y - a);
}

inline CPoint CPoint::operator -(const CPoint& pt) const
{
  return CPoint(x - pt.x, y - pt.y);
}

inline CPoint CPoint::operator *(const float_type a) const
{
  return CPoint(x * a, y * a);
}

inline CPoint CPoint::operator *(const CPoint& pt) const
{
  return CPoint(x * pt.x, y * pt.y);
}

inline CPoint CPoint::operator /(const float_type a) const
{
#if CHECK_DIVISION_BY_ZERO == 1
  if (a == 0)
    return CPOINT_DIVISION_BY_ZERO_RESULT;
  else
#endif
    return CPoint(x / a, y / a);
}

inline CPoint CPoint::operator /(const CPoint& pt) const
{
#if CHECK_DIVISION_BY_ZERO == 1
  if ( pt.x == 0 ) {
    if ( pt.y == 0 ) {
      return CPOINT_DIVISION_BY_ZERO_RESULT;
    } else {
      return CPoint(FLOAT_TYPE_DIVISION_BY_ZERO_RESULT, y / pt.y);
    }
  } else {
    if ( pt.y == 0 ) {
      return CPoint(x / pt.x, FLOAT_TYPE_DIVISION_BY_ZERO_RESULT);
    } else {
#endif
      return CPoint(x / pt.x, y / pt.y);
#if CHECK_DIVISION_BY_ZERO == 1      
    }
  }
#endif
}

inline CPoint CPoint::operator +(const float_type a) const
{
  return CPoint(x + a, y + a);
}

inline CPoint CPoint::operator +(const CPoint& pt) const
{
  return CPoint(x + pt.x, y + pt.y);
}

inline CPoint& CPoint::operator-=(const CPoint& pt)
{
  x -= pt.x;
  y -= pt.y;
  return *this;
}

inline CPoint& CPoint::operator-=(const float_type a)
{
  x -= a;
  y -= a;
  return *this;
}

inline CPoint& CPoint::operator+=(const CPoint& pt)
{
  x += pt.x;
  y += pt.y;
  return *this;
}

inline CPoint& CPoint::operator+=(const float_type a)
{
  x += a;
  y += a;
  return *this;
}

inline CPoint& CPoint::operator*=(const float_type a)
{
  x *= a;
  y *= a;
  return *this;
}

inline CPoint& CPoint::operator*=(const CPoint& pt)
{
  x *= pt.x;
  y *= pt.y;
  return *this;
}

inline CPoint& CPoint::operator/=(const float_type a)
{
#if CHECK_DIVISION_BY_ZERO == 1
  if ( a == 0) {
    x = FLOAT_TYPE_DIVISION_BY_ZERO_RESULT;
    y = FLOAT_TYPE_DIVISION_BY_ZERO_RESULT;
  } else {
#endif
    x /= a;
    y /= a;
#if CHECK_DIVISION_BY_ZERO == 1
  }
#endif
  return *this;
}

inline CPoint& CPoint::operator/=(const CPoint& pt)
{
#if CHECK_DIVISION_BY_ZERO == 1
  if ( pt.x == 0 )
    x = FLOAT_TYPE_DIVISION_BY_ZERO_RESULT;
  else
#endif
    x /= pt.x;
  
#if CHECK_DIVISION_BY_ZERO == 1
  if ( pt.y == 0 )
    y = FLOAT_TYPE_DIVISION_BY_ZERO_RESULT;
  else
#endif
    y /= pt.y;

  return *this;
}

inline CPoint& CPoint::operator =(const CPoint& pt)
{
  y = pt.y;
  x = pt.x;
  return *this;
}

inline bool CPoint::operator ==(const CPoint& pt) const
{
  return (y == pt.y) && (x == pt.x);
}

inline bool CPoint::operator !=(const CPoint& pt) const
{
  return (y != pt.y) || (x != pt.x);
}

inline bool CPoint::operator !() const
{
  return (y != 0) && (x != 0);
}

/*
inline bool CPoint::InFront(const CPoint& pt) const
{
  return (x > pt.x);
}

inline bool CPoint::Behind(const CPoint& pt) const
{
  return (x < pt.x);
}
*/

/*
inline bool CPoint::LeftOf(const CPoint& pt) const
{
  return (x > pt.x);
}

inline bool CPoint::RightOf(const CPoint& pt) const
{
  return (x < pt.x);
}

inline bool CPoint::Above(const CPoint& pt) const
{
  return (y < pt.y);
}

inline bool CPoint::Below(const CPoint& pt) const
{
  return (y > pt.y);
}
*/

inline CPoint& CPoint::AddX (const float_type pt_x)
{
  x += pt_x;
  return *this;
}

inline CPoint& CPoint::AddY (const float_type pt_y)
{
  y += pt_y;
  return *this;
}

inline float_type CPoint::GetX() const
{
  return x;
}

inline float_type CPoint::GetY() const
{
  return y;
}

inline CPoint& CPoint::SetX(const float_type pt_x)
{
  x = pt_x;
  return *this;
}

inline CPoint& CPoint::SetY(const float_type pt_y)
{
  y = pt_y;
  return *this;
}

inline CPoint& CPoint::Set(const float_type pt_x, const float_type pt_y)
{
  x = pt_x;
  y = pt_y;
  return *this;
}

inline CAngleDeg CPoint::DirectionTo(const CPoint& pt) const
{
  return ATan2((y-pt.y), (x-pt.x)) - 180;
}

inline float_type CPoint::DistanceTo(const CPoint& pt) const
{
  return sqrt(Sqr(pt.x - x) + Sqr(pt.y - y));
}

inline std::ostream& operator<<(std::ostream& os, const CPoint& pt)
{
  return os << "(" << pt.x << ", " << pt.y << ")";
}

// Extend this to be the point on the line between pt1 and pt2 
// with a certain ratio 1:1 is the middle point?
// CPoint MiddlePoint(const CPoint&, const CPoint&);
inline CPoint MiddlePoint(const CPoint& pt1, const CPoint& pt2)
{
  return pt1 + CPoint((pt2.GetX() - pt1.GetX())/2, (pt2.GetY() - pt1.GetY())/2);
}

#endif 

