#ifndef _CPointWUnknown_H_
#define _CPointWUnknown_H_

#include "types.h"
#include "MathUtils.h"
#include "CAngleDeg.h"
#include <iostream>

class CVector;

class CPointWUnknown {
public:

  CPointWUnknown(const float_type pt_x, const float_type pt_y);
  CPointWUnknown(const CPointWUnknown& pt);
  CPointWUnknown(const CVector& v, const CPointWUnknown& pt);
  CPointWUnknown(const CPointWUnknown& pt, const CVector& v);
  CPointWUnknown(const CVector& v);

  inline CPointWUnknown  operator - ()                 const;
  inline CPointWUnknown  operator - (const float_type  a) const;
  inline CPointWUnknown  operator - (const CPointWUnknown& pt) const;
  inline CPointWUnknown  operator - (const CVector& v) const;
  inline CPointWUnknown  operator * (const float_type  a) const;
  inline CPointWUnknown  operator / (const float_type  a) const;
  inline CPointWUnknown  operator + (const float_type  a) const;
  inline CPointWUnknown  operator + (const CPointWUnknown& pt) const;
  inline CPointWUnknown  operator + (const CVector& v) const;
  inline CPointWUnknown& operator -=(const CPointWUnknown& pt);
  inline CPointWUnknown& operator -=(const float_type a);
  inline CPointWUnknown& operator +=(const CPointWUnknown& pt);
  inline CPointWUnknown& operator +=(const float_type a);
  inline CPointWUnknown& operator *=(const float_type  a);
  inline CPointWUnknown& operator /=(const float_type  a);
  inline CPointWUnknown& operator  =(const CPointWUnknown& pt);
  inline bool    operator ==(const CPointWUnknown& pt) const;
  inline bool    operator !=(const CPointWUnknown& pt) const;
  inline bool    operator ! ()                 const;

  // These use the coordinate system of the soccer field
  inline bool InFront(const CPointWUnknown& pt) const;  // Equal to LeftOf
  inline bool Behind(const CPointWUnknown& pt) const;   // Equal to RightOf
  // These use a standar coordinate system
  inline bool LeftOf(const CPointWUnknown& pt) const;
  inline bool RightOf(const CPointWUnknown& pt) const;
  inline bool Above(const CPointWUnknown& pt) const;
  inline bool Below(const CPointWUnknown& pt) const;
  
  inline CPointWUnknown& AddX (const float_type pt_x);
  inline CPointWUnknown& AddY (const float_type pt_y);
  inline float_type GetX() const;
  inline float_type GetY() const;
  inline CPointWUnknown& SetX(const float_type pt_x);
  inline CPointWUnknown& SetY(const float_type pt_y);
  inline CPointWUnknown& Set(const float_type pt_x, const float_type pt_y);
  
  inline CVector VectorTo(const CPointWUnknown& pt) const;
  inline CAngleDeg DirectionTo(const CPointWUnknown& pt) const;
  inline float_type DistanceTo(const CPointWUnknown& pt) const;
  
  inline friend ostream& operator<<(ostream& os, const CPointWUnknown& pt);
         friend istream& operator>>(istream& is, CVector& v);
  
private:
  float_type x;
  float_type y;
  bool unknown;
  
};

// Extend this to be the PointWUnknown on the line between pt1 and pt2 
// with a certain ratio 1:1 is the middle PointWUnknown?
inline CPointWUnknown MiddlePointWUnknown(const CPointWUnknown& pt1, const CPointWUnknown& pt2);

#endif 
