#include "CRSParameters.h"
#include <cstdio>

CRSParameters::CRSParameters()
  : CParameters()
{
  Init();
  InitOptions();
}

CRSParameters::CRSParameters(int argc, char** argv)
  : CParameters()
{
  Init();
  InitOptions();
  ReadCommandLine(argc, argv);
}

CRSParameters::CRSParameters(const CRSParameters& obj)
  : CParameters(obj)
{
  CopyData(obj);
}

CRSParameters::~CRSParameters()
{
}

CRSParameters& CRSParameters::operator =(const CRSParameters& obj)
{
  if ( this != &obj) {
    CParameters::CopyData(obj);
    CopyData(obj);
  }

  return *this;
}

void CRSParameters::Init()
{
  RS_team_name                 = "";
  RS_unumber                   = 0;
  RS_goalie                    = false;
  RS_senses_per_cycle          = 0;

  RS_low_confidence            = 0.5;
  RS_medium_confidence         = 0.75;
  RS_high_confidence           = 0.9;
  RS_confidence_decay_ball     = 0.9;
  RS_confidence_decay_player   = 0.95;
  
  RS_min_score_dist            = 25;
  RS_max_turn_angle_percentage = 90;
  
  RS_safe_kickable_area        = 0.95;
  
  RS_max_dev_ball_dir          = 4.0;
  RS_close_to_ball             = 1.2;
  RS_dash_to_ball              = 80.0;
  RS_max_dev_point_dir         = 4.0;
  RS_close_to_point            = 1.2;
  RS_dash_to_point             = 30.0;
  RS_close_to_goal             = 20.0;
  RS_dribble_kick_power        = 30.0;
  RS_dribble_dash_power        = 100.0;
  RS_turnball_max_dir_dev      = 4.0;
  RS_turnball_kick_power       = 10.0;
  RS_turnball_max_moment       = 100.0;
  RS_min_player_dist           = 5.0;
  RS_dribble_min_player_dist   = 5.0;
  RS_max_pass_dist             = 20.0;
  RS_goalie_min_rush_dist      = 15.0;
  RS_goalie_max_catch_dir      = 3.0;

  RS_timer_interval            = int_type(SP_simulator_step / (RS_senses_per_cycle + 1));
}

void CRSParameters::InitOptions()
{
  // variable types and names
  option_t opt[] =
  {{"n",                         (void *)&RS_team_name,                 V_STRING},
   {"team_name",                 (void *)&RS_team_name,                 V_STRING},
   {"p",                         (void *)&RS_unumber,                   V_INT_TYPE},
   {"goalie",                    (void *)&RS_goalie,                    V_BOOL},
   {"senses_per_cycle",          (void *)&RS_senses_per_cycle,          V_INT_TYPE},
   {"low_confidence",            (void *)&RS_low_confidence,            V_FLOAT_TYPE},
   {"medium_confidence",         (void *)&RS_medium_confidence,         V_FLOAT_TYPE},
   {"high_confidence",           (void *)&RS_high_confidence,           V_FLOAT_TYPE},
   {"confidence_decay_ball",     (void *)&RS_confidence_decay_ball,     V_FLOAT_TYPE},
   {"confidence_decay_player",   (void *)&RS_confidence_decay_player,   V_FLOAT_TYPE},
   {"min_score_dist",            (void *)&RS_min_score_dist,            V_FLOAT_TYPE},
   {"max_turn_angle_percentage", (void *)&RS_max_turn_angle_percentage, V_FLOAT_TYPE},
   {"close_to_ball",             (void *)&RS_close_to_ball,             V_FLOAT_TYPE},
   {"dash_to_ball",              (void *)&RS_dash_to_ball,              V_FLOAT_TYPE},
   {"max_dev_point_dir",         (void *)&RS_max_dev_point_dir,         V_FLOAT_TYPE},
   {"max_dev_ball_dir",          (void *)&RS_max_dev_ball_dir,          V_FLOAT_TYPE},
   {"close_to_point",            (void *)&RS_close_to_point,            V_FLOAT_TYPE},
   {"dash_to_point",             (void *)&RS_dash_to_point,             V_FLOAT_TYPE},
   {"close_to_goal",             (void *)&RS_close_to_goal,             V_FLOAT_TYPE},
   {"dribble_kick_power",        (void *)&RS_dribble_kick_power,        V_FLOAT_TYPE},
   {"dribble_dash_power",        (void *)&RS_dribble_dash_power,        V_FLOAT_TYPE},
   {"turnball_max_dir_dev",      (void *)&RS_turnball_max_dir_dev,      V_FLOAT_TYPE},
   {"turnball_kick_power",       (void *)&RS_turnball_kick_power,       V_FLOAT_TYPE},
   {"turnball_max_moment",       (void *)&RS_turnball_max_moment,       V_FLOAT_TYPE},
   {"min_player_dist",           (void *)&RS_min_player_dist,           V_FLOAT_TYPE},
   {"dribble_min_player_dist",   (void *)&RS_dribble_min_player_dist,   V_FLOAT_TYPE},
   {"max_pass_dist",             (void *)&RS_max_pass_dist,             V_FLOAT_TYPE},
   {"goalie_min_rush_dist",      (void *)&RS_goalie_min_rush_dist,      V_FLOAT_TYPE},
   {"goalie_max_catch_dir",      (void *)&RS_goalie_max_catch_dir,      V_FLOAT_TYPE},
   {"safe_kickable_area",        (void *)&RS_safe_kickable_area,        V_FLOAT_TYPE},
   {"timer_interval",            (void *)&RS_timer_interval,            V_INT_TYPE},
   { "\000", 0, V_NONE }
  };

  int i = 0;
  while (opt[i].vptr != 0 && i < optsize) {
    strcpy(options[i].name, opt[i].name);
    options[i].vptr = opt[i].vptr;
    options[i].vsize = opt[i].vsize;
    ++i;
  }

  /*
  RASSERT1(opt[i].vptr == 0, ROBOBUG_ERROR, PARAMETER_CLASS,
	   "Option array too small (%u)", optsize);
  */
  
  i = (i < optsize - 1) ? i : optsize - 1;
  options[i].vptr = 0;
}

void CRSParameters::CopyData(const CRSParameters& obj)
{
  RS_senses_per_cycle          = obj.RS_senses_per_cycle;

  RS_low_confidence            = obj.RS_low_confidence;
  RS_medium_confidence         = obj.RS_medium_confidence;
  RS_high_confidence           = obj.RS_high_confidence;
  RS_confidence_decay_ball     = obj.RS_confidence_decay_ball;
  RS_confidence_decay_player   = obj.RS_confidence_decay_player;
  
  RS_min_score_dist            = obj.RS_min_score_dist;
  RS_max_turn_angle_percentage = obj.RS_max_turn_angle_percentage;
  
  RS_safe_kickable_area        = obj.RS_safe_kickable_area;
  
  RS_max_dev_ball_dir          = obj.RS_max_dev_ball_dir;
  RS_close_to_ball             = obj.RS_close_to_ball;
  RS_dash_to_ball              = obj.RS_dash_to_ball;
  RS_max_dev_point_dir         = obj.RS_max_dev_point_dir;
  RS_close_to_point            = obj.RS_close_to_point;
  RS_dash_to_point             = obj.RS_dash_to_point;
  RS_close_to_goal             = obj.RS_close_to_goal;
  RS_dribble_kick_power        = obj.RS_dribble_kick_power;
  RS_dribble_dash_power        = obj.RS_dribble_dash_power;
  RS_turnball_max_dir_dev      = obj.RS_turnball_max_dir_dev;
  RS_turnball_kick_power       = obj.RS_turnball_kick_power;
  RS_turnball_max_moment       = obj.RS_turnball_max_moment;
  RS_min_player_dist           = obj.RS_min_player_dist;
  RS_dribble_min_player_dist   = obj.RS_dribble_min_player_dist;
  RS_max_pass_dist             = obj.RS_max_pass_dist;
  RS_goalie_min_rush_dist      = obj.RS_goalie_min_rush_dist;
  RS_goalie_max_catch_dir      = obj.RS_goalie_max_catch_dir;

  RS_timer_interval            = obj.RS_timer_interval;
}

bool CRSParameters::ReadOption(const char* linebuf, int verbose)
{
  return ( CParameters::ReadOption(linebuf, verbose)
	   || CParameters::ReadOption(linebuf, verbose, options) );
}

bool CRSParameters::ReadOption(const char* option, const char* value)
{
  return ( CParameters::ReadOption(option, value)
	   || CParameters::ReadOption(option, value, options) );
}

