#ifndef _CRSPARAMETERS_H_
#define _CRSPARAMETERS_H_

#include "CParameters.h"

class CRSParameters : public CParameters
{
public:
  CRSParameters();
  CRSParameters(int argc, char** argv);
  CRSParameters(const CRSParameters&);
  virtual ~CRSParameters();

  CRSParameters& operator =(const CRSParameters&);
  
  virtual bool ReadOption(const char* linebuf, int verbose = 0);
  virtual bool ReadOption(const char* option, const char* value);

  /*
  // These are included so that one can have pointers to this
  // class and get values from subclasses.
  virtual Get(const string&, float_type*);
  virtual Get(const string&, int_type*);
  virtual Get(const string&, string*);
  */  

  // RS = RoboSoc parameters
  // They will be loaded from the RoboSoc.conf file
  std::string RS_team_name;
  int_type    RS_unumber;
  bool        RS_goalie;
  
  int_type    RS_senses_per_cycle;

  float_type  RS_low_confidence;
  float_type  RS_medium_confidence;
  float_type  RS_high_confidence;
  float_type  RS_confidence_decay_ball;
  float_type  RS_confidence_decay_player;

  float_type  RS_min_score_dist;
  float_type  RS_max_turn_angle_percentage;

  float_type  RS_safe_kickable_area;
  
  float_type  RS_max_dev_ball_dir;
  float_type  RS_close_to_ball;
  float_type  RS_dash_to_ball;
  float_type  RS_max_dev_point_dir;
  float_type  RS_close_to_point;
  float_type  RS_dash_to_point;
  float_type  RS_close_to_goal;
  float_type  RS_dribble_kick_power;
  float_type  RS_dribble_dash_power;
  float_type  RS_turnball_max_dir_dev;
  float_type  RS_turnball_kick_power;
  float_type  RS_turnball_max_moment;
  float_type  RS_min_player_dist;
  float_type  RS_dribble_min_player_dist;
  float_type  RS_max_pass_dist;
  float_type  RS_goalie_min_rush_dist;
  float_type  RS_goalie_max_catch_dir;
  int_type    RS_timer_interval;
  
protected:
  void CopyData(const CRSParameters&);

private:
  static const int_type optsize = 35; // The current number is 31
  option_t options[optsize];
        
  void Init();
  void InitOptions();
};

#endif
