#ifndef _CRECTANGLE_H_
#define _CRECTANGLE_H_

#include "types.h"
#include "CVector.h"
#include "CPoint.h"
#include "CLine.h"
#include <iostream>

class CRectangle {
public:
  CRectangle();
  CRectangle(const float_type x1, const float_type y1, const float_type x2, const float_type y2);
  CRectangle(const CRectangle& r);
  CRectangle(const CPoint& pt1, const CPoint& pt2);
  CRectangle(const CVector& v);
  CRectangle(const CVector& v, const CPoint& pt);
  CRectangle(const CPoint& pt, const CVector& v);

  CRectangle& operator =(const CRectangle& r);

  float_type Width()  const;
  float_type Heigth() const;
  
  CPoint GetUpperLeft()  const;
  CPoint GetLowerLeft()  const;
  CPoint GetUpperRight() const;
  CPoint GetLowerRight() const;
  CPoint GetCenter()     const;
  CRectangle& SetUpperLeft(const CPoint& pt);
  CRectangle& SetLowerLeft(const CPoint& pt);
  CRectangle& SetUpperRight(const CPoint& pt);
  CRectangle& SetLowerRight(const CPoint& pt);
  CRectangle& SetCenter(const CPoint& pt);

  CRectangle& MoveCenter(const CPoint& pt);
  CRectangle& MoveLeft(const float_type d);
  CRectangle& MoveRight(const float_type d);
  CRectangle& MoveUp(const float_type d);
  CRectangle& MoveDown(const float_type d);
    
  CRectangle& ExtendLeft(const float_type d);
  CRectangle& ExtendRight(const float_type d);
  CRectangle& ExtendUp(const float_type d);
  CRectangle& ExtendDown(const float_type d);
  
  bool Inside(const CPoint& pt)    const;
  bool Inside(const CLine& l)      const;
  bool Inside(const CRectangle& r) const;

  bool LeftOf(const CPoint& pt)     const;
  bool LeftOf(const CLine& l)       const;
  bool LeftOf(const CRectangle& r)  const;
  bool RightOf(const CPoint& pt)    const;
  bool RightOf(const CLine& l)      const;
  bool RightOf(const CRectangle& r) const;
  bool Above(const CPoint& pt)      const;
  bool Above(const CLine& l)        const;
  bool Above(const CRectangle& r)   const;
  bool Below(const CPoint& pt)      const;
  bool Below(const CLine& l)        const;
  bool Below(const CRectangle& r)   const;  
  
  friend std::ostream& operator<<(std::ostream& os, const CRectangle& r);

private:
  CPoint upper_left;
  CPoint lower_right;
};

//   Things to incorporate!
//         inline float_type TopY()     { return top_y; }
//         inline float_type BottomY()  { return bottom_y; }
//         inline float_type RightX()   { return right_x; }
//         inline float_type LeftX()    { return left_x; }

//         inline Cvector TopLeftCorner()
//                 { return Cvector(left_x, top_y); }
//         inline Cvector TopRightCorner()
//                 { return Cvector(right_x, top_y); }
//         inline Cvector BottomLeftCorner()
//                 { return Cvector(left_x, bottom_y); }
//         inline Cvector BottomRightCorner()
//                 { return Cvector(right_x, bottom_y); }

//         inline Line LeftEdge()
//                 { return LineFromTwoPoints(TopLeftCorner(),
//                                            BottomLeftCorner()); }
//         inline Line RightEdge()
//                 { return LineFromTwoPoints(TopRightCorner(),
//                                            BottomRightCorner()); }
//         inline Line TopEdge()
//                 { return LineFromTwoPoints(TopLeftCorner(),
//                                            TopRightCorner()); }
//         inline Line BottomEdge()
//                 { return LineFromTwoPoints(BottomLeftCorner(),
//                                            BottomRightCorner()); }

//         Line GetEdge(int n);  //order: top, right, bot, left; starting at 0
//         Cvector GetPoint(int n); //order: TL, TR, BR, BL; starting at 0
  
//         inline float_type Width()    { return right_x - left_x; }   
//         inline float_type Height()   { return bottom_y - top_y; }  

//         inline Cvector Center()
//                 { return Cvector( LeftX() + Width()/2, TopY() + Height()/2 ); }

//         /**
//          * Check if the point is in the rectangle.
//          * @param p             the point to check
//          * @return              true, if point is inside
//          */
//         bool   IsWithin(const Cvector& p);
//         Cvector nearestHEdge(const Cvector& p);
//         Cvector nearestVEdge(const Cvector& p);
//         Cvector nearestEdge(const Cvector& p);

//         /**
//          * Returns the distance of a point to the nearest edge.
//          * @param p             the point to check
//          * @return              the distance (negative, if point outside)
//          */
//         float_type  DistanceToEdge(const Cvector& p);

//         /**
//          * Returns the nearest point within the rectangular area
//          * @param p             the point to check
//          * @return              if p is inside the rectangle,
//          *                      p will be returned; else a point
//          *                      on an edge of the rectangle is returned.
//          */
//         Cvector AdjustToWithin(const Cvector& p);

//         Line nearestHEdgeLine(const Cvector& p);
//         Line nearestVEdgeLine(const Cvector& p);
//         Line nearestEdgeLine(const Cvector& p);
  
//         Cvector random() ;

inline float_type CRectangle::Width() const
{
  return lower_right.GetX() - upper_left.GetX();
}

inline float_type CRectangle::Heigth() const
{
  return lower_right.GetY() - upper_left.GetY();
}
  
inline CPoint CRectangle::GetUpperLeft() const
{
  return upper_left;
}

inline CPoint CRectangle::GetLowerLeft() const
{
  return CPoint(upper_left.GetX(), lower_right.GetY());
}

inline CPoint CRectangle::GetUpperRight() const
{
  return CPoint(lower_right.GetX(), upper_left.GetY());
}

inline CPoint CRectangle::GetLowerRight() const
{
  return lower_right;
}

inline CPoint CRectangle::GetCenter() const
{
  return MiddlePoint(upper_left, lower_right);
}

inline CRectangle& CRectangle::SetUpperLeft(const CPoint& pt)
{
  upper_left = pt;
  return *this;
}

inline CRectangle& CRectangle::SetLowerLeft(const CPoint& pt)
{
  upper_left.SetX(pt.GetX());
  lower_right.SetY(pt.GetY());
  return *this;
}

inline CRectangle& CRectangle::SetUpperRight(const CPoint& pt)
{
  upper_left.SetY(pt.GetY());
  lower_right.SetX(pt.GetX());
  return *this;
}
  
inline CRectangle& CRectangle::SetLowerRight(const CPoint& pt)
{
  lower_right = pt;
  return *this;
}
inline CRectangle& CRectangle::ExtendLeft(const float_type d)
{
  upper_left.AddX(-d);
  return *this;
}

inline CRectangle& CRectangle::ExtendRight(const float_type d)
{
  lower_right.AddX(d);
  return *this;
}

inline CRectangle& CRectangle::ExtendUp(const float_type d)
{
  upper_left.AddY(-d);
  return *this;
}

inline CRectangle& CRectangle::ExtendDown(const float_type d)
{
  lower_right.AddY(d);
  return *this;
}

inline bool CRectangle::Inside(const CPoint& pt) const
{
  return (upper_left.GetX() < pt.GetX() &&
	  upper_left.GetY() > pt.GetY() &&
	  lower_right.GetX() > pt.GetX() &&
	  lower_right.GetY() < pt.GetY());
}

/*
inline bool CRectangle::Inside(const CLine& l) const
{
  return false;
}
*/

inline bool CRectangle::Inside(const CRectangle& r) const
{
  return (upper_left.GetX() < r.upper_left.GetX() &&
	  upper_left.GetY() > r.upper_left.GetY() &&
	  lower_right.GetX() > r.lower_right.GetX() &&
	  lower_right.GetY() < r.lower_right.GetY());
}
/*
inline bool CRectangle::LeftOf(const CPoint& pt) const
{
  return upper_left.LeftOf(pt);
}

//inline bool CRectangle::LeftOf(const CLine& l) const

inline bool CRectangle::LeftOf(const CRectangle& r) const
{
  return upper_left.LeftOf(r.lower_right);
}

inline bool CRectangle::RightOf(const CPoint& pt) const
{
  return lower_right.RightOf(pt);
}

//inline bool CRectangle::RightOf(const CLine& l) const

inline bool CRectangle::RightOf(const CRectangle& r) const
{
  return lower_right.RightOf(r.upper_left);
}

inline bool CRectangle::Above(const CPoint& pt) const
{
  return upper_left.Above(pt);
}

//inline bool CRectangle::Above(const CLine& l) const

inline bool CRectangle::Above(const CRectangle& r) const
{
  return upper_left.Above(r.lower_right);
}

inline bool CRectangle::Below(const CPoint& pt) const
{
  return lower_right.Below(pt);
}

//inline bool CRectangle::Below(const CLine& l) const

inline bool CRectangle::Below(const CRectangle& r) const
{
  return lower_right.Below(r.upper_left);
}
*/

inline std::ostream& operator<<(std::ostream& os, const CRectangle& r)
{
  return os << "Rectangle(" << r.upper_left << ", " << r.lower_right << ")";
}

#endif
