#include "CServerIFace.h"
#include "utils.h"

CServerIFace* CServerIFace::current_server_object = NULL;
const bool CServerIFace::showsig = false;
const bool CServerIFace::showCommand = false;
const int_type CServerIFace::MAX_SERVERSTRING_LENGTH = 4096;
char CServerIFace::sensor_data[MAX_SERVERSTRING_LENGTH];
const int_type CServerIFace::SERVER_TIMEOUT_CYCLES = 200;

CServerIFace::CServerIFace(CBasicIP* b_ip, CCommandQueue* cmd_queue, bool init,
			   const std::string& host, const int_type port)
  : ip(b_ip),
    command_queue(cmd_queue),
    sock(MAX_SERVERSTRING_LENGTH, SOCK_TYPE_UDP),
    k(ip->GetKnowledge()),
    wait_new_cycle(false),
    wait_new_info(false),
    senses(0),
    server_alive(false),
    alrsigs_since_iosig(0)
{
  // Always make the latest CServerIFace object the current one.
  current_server_object = this;

  if (init)
    Init(host, port);
}

Sigfunc* CServerIFace::Signal(int_type signo, Sigfunc *func, sigset_t *mask)
{
  //Adapted from Stevens "Network programming"

  struct sigaction act, oact;

  act.sa_handler = func;
  sigemptyset(&act.sa_mask);
  if (mask != NULL)
    act.sa_mask = *mask;
  act.sa_flags = 0;

  if (signo == SIGALRM) {
#ifdef SA_INTERRUPT
    act.sa_flags |= SA_INTERRUPT;   // SunOS 4.x
#endif
  } else {
#ifdef SA_RESTART
    act.sa_flags |= SA_RESTART;     // SVR4, 4.4BSD
#endif
  }

  if (sigaction(signo, &act, &oact) < 0)
    return (SIG_ERR);

  return (oact.sa_handler);

}

// SIGIO handler: since it must be static just call the
// current serverobject's SigIOHandler
void CServerIFace::StaticSigIOHandler(int_type signo)
{
  // Since current_server_object shouldn't be NULL
  // don't check to see if it's NULL.
  current_server_object->SigIOHandler(signo);
}

// SIGIO handler: receive and parse messages from server
void CServerIFace::SigIOHandler(int_type signo) {
  // Adopted from CMUnited'98 by Peter Stone et al.
  
  //sigprocmask(SIG_BLOCK, &all_sigs_mask, NULL);
  sigprocmask(SIG_BLOCK, &sigalrmask, NULL);  

  if ( showsig )
    std::cout << k->theGame().GetCurrentTime() << " "
         << k->theAgent().GetUNumber() << " in sigio!" << std::endl;

  int counter = 0;

  CTime start_time = k->theGame().GetCurrentTime();
  CTime last_time = start_time;
  CTime new_cycle_time = last_time;

  while ( ReceiveInput(sensor_data) == 1) {
    if ( false )
      std::cout << "NEW INFO! " << k->theGame().GetCurrentTime() << ": "
		<< sensor_data << std::endl;;
    ip->Parse(sensor_data);
    if ( new_cycle_time < k->theGame().GetCurrentTime() ) {
      //k->theAgent().GetLastSenseType() == SENSE_Sense ) {
      new_cycle_time = k->theGame().GetCurrentTime();
      wait_new_cycle = true;
    }
    counter++;
    if ( k->theAgent().GetLastSenseType() == SENSE_See )
      wait_new_info = true;
  }

  if ( k->theGame().GetCurrentTime().GetT() - start_time.GetT() > 1 )
    std::cout << "Received several steps at once -- missing action ops!!! "
	 << start_time << " to " << k->theGame().GetCurrentTime()
	 << " last_act_time = " << k->theAgent().GetLastActTime() << std::endl;
  
  alrsigs_since_iosig = 0;

  if ( showsig )
    std::cout << k->theGame().GetCurrentTime() << " "
         << k->theAgent().GetUNumber() << " leaving sigio!" << std::endl;
  
  //  sigprocmask(SIG_UNBLOCK, &all_sigs_mask, NULL);
  sigprocmask(SIG_UNBLOCK, &sigalrmask, NULL);
}

// SIGALRM handler: since it must be static just call the
// current serverobject's SigAlrmHandler
void CServerIFace::StaticSigAlrmHandler(int_type signo)
{
  // Since current_server_object shouldn't be NULL
  // don't check to see if it's NULL.
  current_server_object->SigAlrmHandler(signo);
}

// SIGALRM handler: send something to the server
void CServerIFace::SigAlrmHandler(int_type signo) {
  // Adopted from CMUnited'98 by Peter Stone et al.

  //sigprocmask(SIG_BLOCK, &all_sigs_mask, NULL);
  sigprocmask(SIG_BLOCK, &sigiomask, NULL);  

  if ( showsig )
    std::cout << k->theGame().GetCurrentTime() << " "
         << k->theAgent().GetUNumber() << " in sigalrm!" << std::endl;

  if (ip->IsClockStopped()) {
    ip->AddToStoppedClockMSec(k->GetParameters()->RS_timer_interval);
  }
  
  if (alrsigs_since_iosig++ > SERVER_TIMEOUT_CYCLES)
    server_alive = false;
  
  //  std::cout << "Time: " << k->theGame().GetCurrentTime() << "\tLAtime: "
  //       << k->theAgent().GetLastActTime() << std::endl;

  if ( k->theGame().GetCurrentTime() > k->theAgent().GetLastActTime() ) {

    // Static because we don't want it to be initialized each turn;
    static CTime new_la_time;

    /*
    if ( k->theGame().GetCurrentTime() == k->theAgent().GetLastActTime() ) {
      // Clock hasn't been update yet but it's time to act.
      new_la_time = k->theGame().GetCurrentTime();
      if ( ip->IsClockStopped() )
	new_la_time.SetS(new_la_time.GetS() + 1);
      else
	new_la_time += 1;
    } else {
    */

    if ( k->theGame().GetCurrentTime() - 1 != k->theAgent().GetLastActTime() ) {
      std::cout << k->theGame().GetCurrentTime() << ": Missed a cycle!! LATime = "
		<< k->theAgent().GetLastActTime() << std::endl;
    }

    new_la_time = k->theGame().GetCurrentTime();

    //}    

    ip->GetLogFile()->LogAgentLocation(CURRENT_EST);

    // I would like to test adding the Decide() stuff here.
    // One major benefit is that I might get two chances to
    // Decide on an action.
    
    // Get the first command in the command queue.
    CCommand* nxt_cmd = command_queue->Front();

    if ( nxt_cmd != NULL ) {
      if ( nxt_cmd->Validate(CURRENT_EST) ) {
	nxt_cmd->UpdateAfterCommand(CURRENT_EST, NEXT_EST);
	SendCommand(nxt_cmd->ServerString());
	ip->GetLogFile()->LogSentCommand(nxt_cmd);
	// Save the last command so that the agent can update itself when it get executed
	k->theAgent().SetLastActTime(new_la_time);
	if ( nxt_cmd->GetType() != CMD_Catch )
	  k->theAgent().SetLastCommand(nxt_cmd);
      } else {
	std::cout << k->theGame().GetCurrentTime() << ": "
		  << nxt_cmd->ServerString() << " wasn't validated!" << std::endl;
	ip->GetLogFile()->LogText(nxt_cmd->ServerString() + " wasn't validated!");
      }
      command_queue->PopFront();
    } else {
      std::cout << k->theGame().GetCurrentTime() << ": Nothing to send to the server!" << std::endl;
      ip->GetLogFile()->LogText("Nothing to send to the server!");
    }

    // Reset the sense count.
    senses = 0;

  } else if ( k->theAgent().DoTurnNeck() ) {
    k->theAgent().SetDoTurnNeck(false);
    CTurnNeckCommand nxt_cmd = k->theAgent().GetNextTurnNeck();
    SendCommand(nxt_cmd.ServerString());
    ip->GetLogFile()->LogSentCommand(&nxt_cmd);
    senses++;
  } else {
    //std::cout << k->theGame().GetCurrentTime() << ": Tried to do more than one action this cycle!" << std::endl;
  }
  
  if ( showsig )
    std::cout << k->theGame().GetCurrentTime() << " "
         << k->theAgent().GetUNumber() << " leaving sigalrm!" << std::endl;
  
  //sigprocmask(SIG_UNBLOCK, &all_sigs_mask, NULL);
  sigprocmask(SIG_UNBLOCK, &sigiomask, NULL);
}

sigset_t CServerIFace::InitHandlers() { 
  // Adopted from CMUnited'98 by Peter Stone et al.
  sigemptyset(&sigalrmask);
  sigaddset(&sigalrmask, SIGALRM);
  sigemptyset(&sigiomask);
  sigaddset(&sigiomask, SIGIO);

  Signal(SIGALRM, CServerIFace::StaticSigAlrmHandler, &sigiomask);
  Signal(SIGIO, CServerIFace::StaticSigIOHandler, &sigalrmask);

  sigprocmask(SIG_UNBLOCK, &sigalrmask, NULL);
  sigprocmask(SIG_UNBLOCK, &sigiomask, NULL);

  sigset_t sigsetmask;
  sigprocmask(SIG_BLOCK, NULL, &sigsetmask);   // Get's the currently unblocked signals
  return sigsetmask;   
}

// suspend the process until one of the signals comes through
// could check for situations to kill the client, return false
// i.e. too many actions with no sensory input coming in
bool CServerIFace::WaitForSignals(sigset_t *mask){
  sigsuspend(mask);
  return true;
}

t_server_result CServerIFace::Init(const std::string& host, const int_type port)
{

  all_sigs_mask = InitHandlers();

  // Open the UDP connection
  Connect( (host == "") ? k->GetParameters()->SP_host : host,
	   (port ==  0) ? k->GetParameters()->SP_port : port );

  // Does this work with our threads???
  sock.SetOwner(getpid());

  const int_type on = 1;
  int_type flags = sock.GetFlags();
  sock.SetFlags(flags |= O_NONBLOCK);
  ioctl(sock.GetFD(), FIOASYNC, &on);
  ioctl(sock.GetFD(), FIONBIO, &on); // Same as O_NONBLOCK

  // The code for initialising the current player
  std::string init_str =  "(init " + k->theGame().GetOurTeamName()
    + " (version " + ToString(k->GetParameters()->SP_version) + ")";
  if (k->theAgent().IsGoalie()) {
    init_str += "(goalie))";
  } else {
    init_str += ")";
  }
  SendCommand(init_str);

  wait_new_info = false;
  while( !wait_new_info );
  
  return SERVER_OK ;
  
}

t_server_result CServerIFace::Start()
{
  sigprocmask(SIG_BLOCK, &sigalrmask, NULL);  

  // Now that the player is ready start the timers
  // This call will initialize the handling of the timer and the io from the server
  // Everything will be taken care of by the handlers installed
  // This should be done after a iomsg!
  //  while (alrsigs_since_iosig != 0);
  // Does the value matter? I'm NOT sure!
  //WaitForNewInformation();
  wait_new_cycle = false;
  while ( !wait_new_cycle );
  SetTimer(60000);

  sigprocmask(SIG_UNBLOCK, &sigalrmask, NULL);

  return SERVER_OK;
}

bool CServerIFace::WaitForNewCycle()
{
  while ( !wait_new_cycle ) {
    sigsuspend(&sigiomask);
    if (errno != EINTR) {
      std::cerr << "Error in sigsuspend! errno = " << errno
	//		<< " EINTR = " << EINTR << " EFAULT = " << EFAULT
		<< std::endl;
      return false;
    }
  }
  
  wait_new_cycle = false;
  return true;

  /*
    if ( !wait_new_cycle )
      std::cerr << "Something wront, wait_new_cycle = false..." << std::endl;
    int res = sem_wait(&new_cycle);
    wait_new_cycle = true;
    if ( res != 0 ) {
      std::cerr << "Error in WaitForNewCycle, errno = " << res << " EINTR = "
      << EINTR << std::endl;EINVAL ENOSYS (EAGAIN) EDEADLK
      return false;
    }
    return true;
  */

}

bool CServerIFace::WaitForNewInformation()
{
  while ( !wait_new_info ) {
    sigsuspend(&sigiomask);
    if (errno != EINTR) {
      std::cerr << "Error in sigsuspend! errno = " << errno
	//		<< " EINTR = " << EINTR << " EFAULT = " << EFAULT
		<< std::endl;
      return false;
    }
  }
  
  wait_new_info = false;
  return true;

  /*
  if ( !wait_new_info )
    std::cerr << "Something wrong, wait_new_info = false..." << std::endl;

  int res = sem_wait(&new_info);
  if ( errno != EINTR ) {
    std::cerr << "Error in WaitForNewInformation, errno = " << errno
	      << " EINTR = " << EINTR << " EAGAIN = " << EAGAIN
	      << std::endl;
    return false;
  }
  wait_new_info = true;
  return true;
  */
}

bool CServerIFace::NewCycle()
{
  if ( wait_new_cycle ) {
    wait_new_cycle = false;
    return true;
  } else
    return false;

  /*
  int res = sem_trywait(&new_cycle);
  if ( res == 0 ) {
    wait_new_cycle = true;
    return true;
  } else
    return false;
  */
}

bool CServerIFace::NewInformation()
{
  if ( wait_new_info ) {
    wait_new_info = false;
    return true;
  } else
    return false;

  /*
  int res = sem_trywait(&new_info);
  if ( res == 0 ) {
    wait_new_cycle = true;
    return true;
  } else
    return false;
  */  
}

t_server_result CServerIFace::Connect(const std::string& host, const int_type port)
{
  sock.Open(host, port);
  server_alive = true;
  return SERVER_OK;
};

t_server_result CServerIFace::ReConnect()
{
  return SERVER_OK;
}

t_server_result CServerIFace::Disconnect()
{
  sock.Close();
  server_alive = false;
  return SERVER_OK;
};
  
int CServerIFace::SendCommand(const std::string& buf)
{
  int_type n;

  if (buf == "") return 0;
  n = buf.length() + 1;

  if ( showCommand ) {
    std::cout << k->theGame().GetCurrentTime() <<  ":" << alrsigs_since_iosig << " "
	 << k->theAgent().GetUNumber() << " " << buf << std::endl;
  }
  
  //  if (k->CP_save_log && k->Initialized)
  //    fprintf(k->SaveLogFile, ">> %s\n\n",buf); 

  /* t_sock_result res = sock.WriteData(buf, n);
     switch (res) {
       case SOCK_OK: return 1; break;
       default:      return -1;
     }
  */

  bool res = sock.WriteData(buf, n);

  if ( res != SOCK_OK )
    std::cout << buf << " didn't get through socket!" << std::endl;

  return res;
  
}

void CServerIFace::SendSenseBody()
{
  SendCommand("(sense_body)");
}

void CServerIFace::SendTurnNeck(const CAngleDeg& ang)
{
  SendCommand("(turn_neck " + ToString(ang) + ")");
}

void CServerIFace::SendChangeView(const t_view_width view_width, const t_view_quality view_quality)
{
  std::string str = "(change_view ";
  switch ( view_width ){
  case VW_Normal:
    str += "normal ";
    break;
  case VW_Wide:
    str += "wide ";
    break;
  case VW_Narrow:
    str += "narrow ";
    break;
  }
  
  switch ( view_quality ) {
  case VQ_High:
    str += "high)";
    break;
  case VQ_Low:
    str += "low)";
    break;
  }
  
  SendCommand(str);
}

int CServerIFace::ReceiveInput(char* buf)
{
/*
    if (k->CP_save_log && k->Initialized){ 
      fprintf(k->SaveLogFile, "%s\n", buf);
      if (k->SaveLogCounter++ % k->CP_save_freq == 0){
        fclose(k->SaveLogFile);
        k->SaveLogFile = fopen(k->SaveLogFileName,"a");
      }
    }
    if (k->CP_save_sound_log && k->Initialized && buf[1] == 'h'){ // Hear message
      fprintf(k->SaveSoundLogFile, "%s\n", buf);
      if (k->SaveSoundLogCounter++ % k->CP_save_sound_freq == 0){
        fclose(k->SaveSoundLogFile);
        k->SaveSoundLogFile = fopen(k->SaveSoundLogFileName,"a");
      }
    }
*/
  
  int_type len;

  t_sock_result res = sock.ReadData(buf, &len);

  if ( res == SOCK_ERROR ) {
    std::cout << "[Socket] Error!" << std::endl;
    //res = sock.ReadData(buf, len);
  }
  
  //  else if ( res == SOCK_NODATA )
  //    std::cout << " Nodata on socket!" << std::endl;

  switch (res) {
    case SOCK_ERROR:  return -1; break;
    case SOCK_NODATA: return  0; break;
    case SOCK_OK:     return  1; break;
    default:          return -1;
  }
}
