#ifndef _CSERVERIFACE_H_
#define _CSERVERIFACE_H_

#include "CCommandQueue.h"
#include <string>
#include <signal.h>
#include <sys/filio.h> // For FIOASYNC!! Why doesn't it work with sys/ioctl.h as it should?
#include "types.h"
#include "CSocket.h"
#include "CBasicIP.h"
#include <semaphore.h>

enum t_server_result{ SERVER_ERROR = 0, SERVER_OK = 1 };
typedef void Sigfunc(int_type);

class CServerIFace
{
public:
  CServerIFace(CBasicIP*, CCommandQueue*, bool = false,
	       const std::string& = "", const int_type = 0);

  t_server_result Init(const std::string& = "", const int_type = 0);
  t_server_result Start();
  t_server_result Disconnect();
  t_server_result ReConnect();

  bool ServerAlive() const;
  
  // Special constructs for these four since
  // they can be sent any time.
  void SendChangeView(const t_view_width, const t_view_quality);
  void SendSay(const std::string&);
  void SendSenseBody();
  void SendTurnNeck(const CAngleDeg&);

  bool WaitForNewCycle();
  bool WaitForNewInformation();
  bool NewCycle();
  bool NewInformation();
  
  void SetCurrentServerObject(CServerIFace*);
  CServerIFace* GetCurrentServerObject() const;
  
protected:
  CBasicIP* ip;
  CCommandQueue* command_queue;
  CSocket sock;

  //  sem_t new_cycle;
  //  sem_t new_info;
  
  t_server_result Connect(const std::string&, const int_type);
  int_type SendCommand(const std::string&);
  int_type ReceiveInput(char*);

  bool WaitForSignals(sigset_t*);
  
  void SetTimer(int_type = 0);
  Sigfunc* Signal(int_type signo, Sigfunc *func, sigset_t *mask = NULL);
  
  sigset_t sigalrmask;
  sigset_t sigiomask;
  sigset_t all_sigs_mask;

private:
  CKnowledge* k;

  bool wait_new_cycle;
  bool wait_new_info;
  
  sigset_t InitHandlers();
  static CServerIFace* current_server_object;
  static void StaticSigAlrmHandler(int_type);
  static void StaticSigIOHandler(int_type);
  void SigAlrmHandler(int_type);
  void SigIOHandler(int_type);

  // For efficency reasons keep this as a char*
  // until the rest of the library (parser) is
  // rewritten with std::string instead of char*.
  static const int_type MAX_SERVERSTRING_LENGTH; // = 4096;
  static char sensor_data[4096]; //[MAX_SERVERSTRING_LENGTH];

  // These are used in sigalrm and sigio
  int_type senses;
  bool server_alive;
  int_type alrsigs_since_iosig;
  static const int_type SERVER_TIMEOUT_CYCLES;

  // For debugging, should be replaced!
  static const bool showsig;
  static const bool showCommand;

  // Declare these two private so that the compiler
  // don't generate default versions of them.
  CServerIFace(const CServerIFace&);
  CServerIFace& operator=(const CServerIFace&);
};

inline void CServerIFace::SetCurrentServerObject(CServerIFace* new_obj)
{
  if ( new_obj != NULL )
    current_server_object = new_obj;
}

inline bool CServerIFace::ServerAlive() const
{
  return server_alive;
}

inline CServerIFace* CServerIFace::GetCurrentServerObject() const
{
  return current_server_object;
}
  
// Set time interval between command sending
// The parameter is the time before the first
// sigalrm is generated.
// From CMUnited'98 by Peter Stone et al.
inline void CServerIFace::SetTimer(int_type usec)
{
  /*
  struct itimerval itv;
  itv.it_interval.tv_sec = 0;
  itv.it_interval.tv_usec = k->GetParameters()->RS_timer_interval * 1000;
  itv.it_value.tv_sec = 0;
  itv.it_value.tv_usec = usec;
  setitimer(ITIMER_REAL, &itv, NULL);
  */
  ualarm(usec, k->GetParameters()->RS_timer_interval * 1000);
}

#endif
