#include "CSetofPlayersObject.h"
#include "MathUtils.h"

CSetofPlayersObject::CSetofPlayersObject(const CParameters* const p,
					 const int_type max_no_plrs,
					 const bool fxed_size)
  : fixed_size(fxed_size),
    params(p),
    max_no_players(max_no_plrs),
    no_players(0),
    players(max_no_plrs, CPlayerObject(p))
{
  if ( fixed_size )
    no_players = max_no_players;
}

CSetofPlayersObject::CSetofPlayersObject(const CSetofPlayersObject& obj)
  : fixed_size(obj.fixed_size),
    params(obj.params),
    max_no_players(obj.max_no_players),
    no_players(obj.no_players),
    players(obj.players)
{
  
}  

/*
CSetofPlayersObject::~CSetofPlayersObject()
{

}
*/

void CSetofPlayersObject::CopyData(const CSetofPlayersObject& obj)
{
  fixed_size = obj.fixed_size;
  params = obj.params;
  max_no_players = obj.max_no_players;
  no_players = obj.no_players;
  players = obj.players;
}

CSetofPlayersObject& CSetofPlayersObject::operator= (const CSetofPlayersObject& obj)
{

  if (this != &obj) {
    CopyData(obj);
  }

  return *this;
  
}

CPlayerObject* CSetofPlayersObject::operator[] (const int_type elt)
{

  // This operator returns a pointer to a CPlayerObject
  // If the index isn't valid the method returns NULL.

  if ( elt >= no_players || elt < 0 ) // Works even when there are no players!
    return NULL;
  else {
    return &players[elt];
  }
  
}

const CPlayerObject* const CSetofPlayersObject::operator[] (const int_type elt) const
{

  // This operator returns a pointer to a CPlayerObject
  // If the index isn't valid the method returns NULL.

  if ( elt >= no_players || elt < 0 ) // Works even when there are no players!
    return NULL;
  else {
    return &players[elt];
  }

}

void CSetofPlayersObject::Copy(const int_type from, const int_type to)
{
  for (int_type i = 0; i < no_players; ++i) {
    players[i].Copy(from, to);
  }
}

void CSetofPlayersObject::NewCycle()
{

  for (int_type i = 0; i < no_players; ++i) {
    players[i].NewCycle();
  }
  
}

void CSetofPlayersObject::UpdateVectors(const CPoint& agent_pos,
					const CAngleDeg& agent_body_dir,
					const int_type hist)
{

  for (int_type i = 0; i < no_players; ++i) {
    players[i].UpdateVector(agent_pos, agent_body_dir, hist);
  }

}

void CSetofPlayersObject::UpdateConfidences(const int_type from, const int_type to)
{

  for (int_type i = 0; i < no_players; ++i) {
    players[i].UpdateConfidence(from, to);
  }
  
}

void CSetofPlayersObject::UpdateMovements(const int_type from, const int_type to)
{

  for (int_type i = 0; i < no_players; ++i) {
    players[i].UpdateMovement(from, to);
  }

}

bool CSetofPlayersObject::ExistPlayer(const t_unum unum, const t_team team)
{
  
  for (int_type i = 0; i < no_players; ++i) {
    if ( players[i].GetUNumber() == unum &&
	 ( team == T_Unknown || players[i].GetTeam() == team ) )
      return true;
  }

  return false;

}

CPlayerObject* CSetofPlayersObject::GetPlayer(const t_unum unum, const t_team team)
{

  // This method returns the wanted player or NULL if he isn't found.

  for (int_type i = 0; i < no_players; ++i) {
    if ( players[i].GetUNumber() == unum &&
	 ( team == T_Unknown || players[i].GetTeam() == team ) )
      return &players[i];
  }
 
  return NULL;
  
}

const CPlayerObject* const CSetofPlayersObject::GetPlayer(const t_unum unum,
							  const t_team team) const
{

  // This method returns the wanted player or NULL if he isn't found.

  for (int_type i = 0; i < no_players; ++i) {
    if ( players[i].GetUNumber() == unum &&
	 ( team == T_Unknown || players[i].GetTeam() == team ) )
      return &players[i];
  }
 
  return NULL;
  
}

CPlayerObject* CSetofPlayersObject::GetBestMatch(const CPlayerObject& plr,
						 const int_type from,
						 const int_type to,
						 const CTime tm)
{
  if ( no_players == 0 )
    return NULL;
  else {
    if ( plr.GetUNumber() != UNKNOWN_UNUM ) {
      return GetPlayer(plr.GetUNumber(), plr.GetTeam());
    } else if (plr.GetPosition(from) != UNKNOWN_POINT ) {
      float_type dist = UNKNOWN_FLOAT_TYPE;
      float_type plr_dist = UNKNOWN_FLOAT_TYPE;
      float_type tmp_dist;
      CPlayerObject* tmp = NULL;
      t_team team = plr.GetTeam();
      CPoint pt = plr.GetPosition(from);
      for (int i = 0; i < no_players; ++i) {
	if ( (team == T_Unknown || players[i].GetTeam() == team) &&
	     (tm == UNKNOWN_TIME || players[i].GetObsTime(to) == UNKNOWN_TIME ||
	      players[i].GetObsTime(to) < tm) ) {
	  tmp_dist = players[i].GetDistanceTo(pt, to);
	  // Find the closest player that is the closest to that point
	  // and haven't been seen after tm.
	  if ( tmp_dist < dist ||
	       (tmp_dist == dist && players[i].GetDistance(to) <= plr_dist) ) {
	    dist = tmp_dist;
	    plr_dist = players[i].GetDistance(to);
	    tmp = &players[i];
	  }
	}
      }
      return tmp;
    } else {
      float dirdiff = 9999;
      float plr_dist = 9999;
      float tmp_dirdiff;
      CPlayerObject* tmp = NULL;
      CAngleDeg dir = plr.GetDirection(from);
      for (int i = 0; i < no_players; ++i) {
	if ( tm == UNKNOWN_TIME || players[i].GetObsTime(to) < tm ) {
	  tmp_dirdiff = Abs(players[i].GetDirection(to) - dir);
	  // Find the closest player that is the closest to that direction
	  // and haven't been seen after tm.
	  if ( tmp_dirdiff < dirdiff ||
	       (tmp_dirdiff == dirdiff && players[i].GetDistance(to) < plr_dist) ) {
	    dirdiff = tmp_dirdiff;
	    plr_dist = players[i].GetDistance(to);
	    tmp = &players[i];
	  }
	}
      }      
      return tmp;
    }
  }
}

bool CSetofPlayersObject::UpdatePlayerWith(const CPlayerObject& plr,
					   const int_type from, const int_type to,
					   const t_unum unum, const t_team team)
{

  CPlayerObject* tmp = GetPlayer(unum, team);
  if ( tmp == NULL )
    return false;
  else {
    tmp->CopyFrom(plr, from, to);
    return true;
  }
  
}

CSetofPlayersObject CSetofPlayersObject::GetPlayersFrom(const t_team team)
{

  CSetofPlayersObject res(params, max_no_players, false);
  GetPlayersFrom(team, &res);
  return res;
    
}

void CSetofPlayersObject::GetPlayersFrom(const t_team team,
					 CSetofPlayersObject* const res)
{
  res->MakeEmpty();
  CPlayerObject* tmp;
  
  for (int_type i = 0; i < no_players; ++i) {
    tmp = &players[i]; 
    if ( tmp->GetTeam() == team ) {
      res->AddPlayer(*tmp);
    }
  }

}

void CSetofPlayersObject::GetPlayersFrom(const t_side side,
					 CSetofPlayersObject* const res)
{
  res->MakeEmpty();
  CPlayerObject* tmp;
  
  for (int_type i = 0; i < no_players; ++i) {
    tmp = &players[i]; 
    if ( tmp->GetSide() == side ) {
      res->AddPlayer(*tmp);
    }
  }

}

void CSetofPlayersObject::MakeEmpty()
{
  //  for (int_type i = 0; i < no_players; ++i)
  // players[i].ResetAll();
  
  no_players = 0;
}

bool CSetofPlayersObject::AddPlayer(const CPlayerObject& player)
{

  // Should I check whether that player already exist or should I just add it??
  // This implementation just add the player

  if (no_players >= max_no_players)
    return false;

  players[no_players] = player;
  no_players++;
  return true;
  
}

bool CSetofPlayersObject::DeletePlayer(const t_unum unum, const t_team team)
{

  if ( unum == UNKNOWN_UNUM ) {
    // This method removes the last player in the set and return
    // true if there is a player to remove otherwise false.
    
    if ( no_players == 0 )
      return false;
    else {
      no_players--;
      return true;
    }
  } else {
    // If I find a player with the uniform number unum
    // then I remove that player and return true.
    // Otherwise return false.

    bool result = false;
    int_type pos = 0;
    
    while ( !result && pos < no_players ) {
      if ( players[pos].GetUNumber() == unum &&
	   ( team == T_Unknown || players[pos].GetTeam() == team ) ) {
	result = true;
      }
      pos++;
    }

    if ( result ) {
      while ( pos < no_players ) {
	players[pos-1] = players[pos];
	pos++;
      }
      no_players--;
    }
  
    return result;
  }
}

int_type CSetofPlayersObject::RemoveAllShouldBeSeen(const CAngleDeg& face_dir,
						    const CAngleDeg& view_width,
						    const int_type hist)
{

  // This method should remove all players that should be seen at hist.
  // The number of players removed should be returned.

  int_type no_removed = 0;

  if ( no_players > 0 ) {
    CPlayerObject* tmp = NULL;
    int_type j = 0;
    for (int_type i = 0; i < no_players; ++i) {
      tmp = &players[i];
      if ( tmp->ShouldBeSeen(face_dir, view_width, hist) ) {
	// Remove tmp from players
	for (j = i+1; j < no_players; ++j) {
	  players[j-1] = players[j];
	}
	tmp = NULL;
	no_players--;
 	no_removed++;
      }
    }    
  }
  
  return no_removed;
  
}

void CSetofPlayersObject::CopyPositions(const int_type from, const int_type to)
{

  for (int_type i = 0; i < no_players; ++i)
    players[i].CopyPosition(from, to);
  
}

void CSetofPlayersObject::CopyVectors(const int_type from, const int_type to)
{

  for (int_type i = 0; i < no_players; ++i)
    players[i].CopyVector(from, to);
  
}

void CSetofPlayersObject::CopyObsTimes(const int_type from, const int_type to)
{

  for (int_type i = 0; i < no_players; ++i)
    players[i].CopyObsTime(from, to);
  
}

void CSetofPlayersObject::CopyConfidences(const int_type from, const int_type to)
{

  for (int_type i = 0; i < no_players; ++i)
    players[i].CopyConfidence(from, to);
  
}

void CSetofPlayersObject::CopyDistChanged(const int_type from, const int_type to)
{

  for (int_type i = 0; i < no_players; ++i)
    players[i].CopyDistChanged(from, to);
  
}

void CSetofPlayersObject::CopyDirChanged(const int_type from, const int_type to)
{

  for (int_type i = 0; i < no_players; ++i)
    players[i].CopyDirChanged(from, to);
  
}

void CSetofPlayersObject::CopySpeedVectors(const int_type from, const int_type to)
{

  for (int_type i = 0; i < no_players; ++i)
    players[i].CopySpeedVector(from, to);
  
}

void CSetofPlayersObject::CopyAccelerations(const int_type from, const int_type to)
{

  for (int_type i = 0; i < no_players; ++i)
    players[i].CopyAcceleration(from, to);
  
}

void CSetofPlayersObject::CopyBodyDirs(const int_type from, const int_type to)
{

  for (int_type i = 0; i < no_players; ++i)
    players[i].CopyBodyDir(from, to);
  
}

void CSetofPlayersObject::CopyNeckDirs(const int_type from, const int_type to)
{

  for (int_type i = 0; i < no_players; ++i)
    players[i].CopyNeckDir(from, to);
  
}

void CSetofPlayersObject::CopyFaceDirs(const int_type from, const int_type to)
{

  for (int_type i = 0; i < no_players; ++i)
    players[i].CopyFaceDir(from, to);
  
}

void CSetofPlayersObject::SetAccelerations(const CVector& acc, const int_type hist)
{

  for (int_type i = 0; i < no_players; ++i)
    players[i].SetAcceleration(acc, hist);

}

CPlayerObject* CSetofPlayersObject::GetClosestPlayerTo(const CPoint& pt,
						       const int_type hist)
{

  // This method return the player that has the shortest distance to a point.
  // If more than one player is the closest then return the last one.
  // If there are no players return NULL.

  if (no_players == 0)
    return NULL;
  else {
    CPlayerObject* tmp = NULL;
    float_type dist = UNKNOWN_FLOAT_TYPE;
    float_type tmp_dist = dist;
    for (int_type i = 0; i < no_players; ++i) {
      tmp_dist = players[i].GetDistanceTo(pt, hist);
      if (tmp_dist <= dist) {
	tmp = &players[i];
	dist = tmp_dist;
      }
    }
    return tmp;
  }
  
}
 
float_type CSetofPlayersObject::GetClosestDistTo(const CPoint& pt,
						 const int_type hist) const
{

  // This method return the distance from the player that has the shortest
  // distance to a point. The default value is UNKNOWN = 9999.

  if (no_players == 0)
    return UNKNOWN_FLOAT_TYPE;
  else {
    float dist = UNKNOWN_FLOAT_TYPE;
    float tmp_dist = dist;
    for (int_type i = 0; i < no_players; ++i) {
      tmp_dist = players[i].GetDistanceTo(pt, hist);
      if (tmp_dist <= dist) {
	dist = tmp_dist;
      }
    }
    return dist;
  }
}

void CSetofPlayersObject::Print(const int_type hist) const
{

  std::cout << "Max number of players = " << max_no_players << std::endl;
  std::cout << "Current number of players = " << no_players << std::endl;

  for (int_type i = 0; i < no_players; ++i) {
    std::cout << "Player[" << i << "]:" << std::endl;
    players[i].Print(hist);
  }

}
