#ifndef _CSETOFPLAYERSOBJECT_H_
#define _CSETOFPLAYERSOBJECT_H_

#include "soc_types.h"
#include "CPlayerObject.h"
#include <vector>

class CSetofPlayersObject
{

public:
  CSetofPlayersObject(const CParameters* const p, const int_type max_no_plrs,
		      const bool fixed_size = true);
  CSetofPlayersObject(const CSetofPlayersObject& obj);
  //~CSetofPlayersObject();

  CSetofPlayersObject& operator= (const CSetofPlayersObject& obj);
  // Checked access
  CPlayerObject* operator[] (const int_type elt);
  const CPlayerObject* const operator[] (const int_type elt) const;

  void Copy(const int_type from, const int_type to);
  
  void NewCycle();
  void UpdateVectors(const CPoint& agent_pos, const CAngleDeg& agent_body_dir,
		    const int_type = NEXT);  
  void UpdateConfidences(const int_type from = NOW, const int_type to = NEXT);
  void UpdateMovements(const int_type from = NOW, const int_type to = NEXT);
  
  int_type GetNumberOfPlayers() const;

  bool ExistPlayer(const t_unum, const t_team = T_Unknown);
  CPlayerObject* GetPlayer(const t_unum, const t_team = T_Unknown);
  const CPlayerObject* const GetPlayer(const t_unum, const t_team = T_Unknown) const;
  CPlayerObject* GetBestMatch(const CPlayerObject&, const int_type from,
			      const int_type to, const CTime = UNKNOWN_TIME);
  bool UpdatePlayerWith(const CPlayerObject&, const int_type from, const int_type to,
			const t_unum, const t_team = T_Unknown);
  CSetofPlayersObject GetPlayersFrom(const t_team);
  void GetPlayersFrom(const t_side, CSetofPlayersObject* const res);
  void GetPlayersFrom(const t_team, CSetofPlayersObject* const res);
  
  // Not needed for static sets
  // Delete all players
  void MakeEmpty();
  // Add a player at the end of the vector
  bool AddPlayer(const CPlayerObject&);
  // If no UNumber or team is supplied delete the last player
  bool DeletePlayer(const t_unum = UNKNOWN_UNUM, const t_team = T_Unknown);
  int_type RemoveAllShouldBeSeen(const CAngleDeg& face_dir,
				 const CAngleDeg& view_width,
				 const int_type hist = NOW);

  void CopyPositions(const int_type from, const int_type to);
  void CopyVectors(const int_type from, const int_type to);
  void CopyObsTimes(const int_type from, const int_type to);
  void CopyConfidences(const int_type from, const int_type to);
  void CopyDistChanged(const int_type from, const int_type to);
  void CopyDirChanged(const int_type from, const int_type to);
  void CopySpeedVectors(const int_type from, const int_type to);
  void CopyAccelerations(const int_type from, const int_type to);
  void CopyBodyDirs(const int_type from, const int_type to);
  void CopyNeckDirs(const int_type from, const int_type to);
  void CopyFaceDirs(const int_type from, const int_type to);

  void SetAccelerations(const CVector& acc, const int_type hist);
  
  CPlayerObject* GetClosestPlayerTo(const CPoint&, const int_type hist = NOW);
  float_type GetClosestDistTo(const CPoint&, const int_type hist = NOW) const;

  void Print(const int_type hist = UNKNOWN_INT_TYPE) const;
  
protected:
  bool fixed_size;
  const CParameters* params;
  int_type max_no_players;
  int_type no_players;
  std::vector<CPlayerObject> players;

  void CopyData(const CSetofPlayersObject&);
  
};

inline int_type CSetofPlayersObject::GetNumberOfPlayers() const
{
  return no_players;
}

#endif
