#include "CSkills.h"

#include "CCustomCommand.h"
#include "CTurnCommand.h"
#include "CKickCommand.h"
#include "CDashCommand.h"
#include "CMoveCommand.h"
#include "CCatchCommand.h"

CSkills::CSkills(CAdvancedIP* a_ip)
  : ip(a_ip),
    k(ip->GetKnowledge()),
    p(k->GetParameters()),
    showSkill(false), //false),
    test(false) //false)
{
}

CSkills::CSkills(const CSkills& obj)
  : ip(obj.ip),
    k(obj.k),
    p(obj.p),
    showSkill(obj.showSkill),
    test(obj.test)
{
}

/*
CSkills::~CSkills()
{
}
*/

CCommand* CSkills::Decied(CCommand* cmd)
{

  if (cmd == NULL) {
    // If nothing new from above we either continue with the
    // last task or do nothing.
    //std::cout << k->theGame().GetCurrentTime()
    //      << " [skills] No orders from Decision-layer!!"
    //      << std::endl;
    return NULL;
  } else if ( cmd->GetType() == CMD_Custom ) {
    CCustomCommand* nxt_cmd = cmd->GetCustomCommand();
    CCommand* next_cmd;
    switch (nxt_cmd->GetCustomCommandType())
      {
      case CCMD_FindBall        : next_cmd = FindBall(nxt_cmd);        break;
      case CCMD_Score           : next_cmd = Score(nxt_cmd);           break;
      case CCMD_Pass            : next_cmd = Pass(nxt_cmd);            break;
      case CCMD_Dribble         : next_cmd = Dribble(nxt_cmd);         break;
      case CCMD_MoveTo          : next_cmd = MoveTo(nxt_cmd);          break;
      case CCMD_MoveAlong       : next_cmd = MoveAlong(nxt_cmd);       break;
      case CCMD_TurnBall        : next_cmd = TurnBall(nxt_cmd);        break;
      case CCMD_Catch           : next_cmd = Catch(nxt_cmd);           break;
      case CCMD_Rush            : next_cmd = Rush(nxt_cmd);            break;
      case CCMD_TrackBall       : next_cmd = TrackBall(nxt_cmd);       break;
      case CCMD_GoalieMoveTo    : next_cmd = GoalieMoveTo(nxt_cmd);    break;
      case CCMD_GoalKick        : next_cmd = GoalKick(nxt_cmd);        break;
      case CCMD_MakePassable    : next_cmd = MakePassable(nxt_cmd);    break;
      case CCMD_GetBehindBall   : next_cmd = GetBehindBall(nxt_cmd);   break;
      case CCMD_GetBehindCorner : next_cmd = GetBehindCorner(nxt_cmd); break;
      case CCMD_ClearBall       : next_cmd = ClearBall(nxt_cmd);       break;
      case CCMD_CornerClearBall : next_cmd = CornerClearBall(nxt_cmd); break;
      case CCMD_InterceptBall   : next_cmd = InterceptBall(nxt_cmd);   break;
      case CCMD_CutOffPassAngle : next_cmd = CutOffPassAngle(nxt_cmd); break;
      case CCMD_CutOffGoalAngle : next_cmd = CutOffGoalAngle(nxt_cmd); break;
      default :
	//std::cout << k->theGame().GetCurrentTime()
	//  << " [skills] Unknown command! command type = "
	//  << nxt_cmd->GetCustomCommandType() << std::endl;
	return cmd;
      }
    // Since I got one command in and I created a new one I
    // should delete the first one.
    // The principle is: one command in, one command out!
    delete cmd;
    return next_cmd;
  } else {
    // Since all skills are based on CCustomCommand just pass the
    // command along
    return cmd;
  }
  
}

/*
CCommand* CSkills::TurnNeck(CCustomCommand* cmd)
{

 if ( showSkill )
    std::cout << k->theGame().GetCurrentTime() << " " << k->theAgent().GetUNumber() << " In TurnNeck" << std::endl;
  
  // Turn neck so that I look at the ball the next turn
  // This has to take movement and actions into account.

  // k->theBall().GetDirection(-1) estimates the next position of the ball
  // if no action is made upon it.
  // Therefore look in the actions_queue and see if the command
  // makes any difference to the turn_neck.

  t_angle_deg ang;
  
  if ( k->theBall().GetConfidence() < p->RS_low_confidence ) {
    // If I don't know where the ball is turn my head straight forward
    ang = -k->theAgent().GetNeckDir();
  } else {
    // Otherwise look at the ball
    CPoint agent_pos;
    CPoint ball_pos;
    CVector ball_vec;

    CCommand* cmd = ip->skills_queue->First();
    
    if ( cmd != NULL ) {
      if ( cmd->GetType() == CMD_Turn ) {
	CTurnCommand* tmp_cmd = cmd->GetTurnCommand();

	t_angle_deg act_ang = tmp_cmd->GetAngle() / (1.0 + p->SP_inertia_moment *
						     k->theAgent().GetSpeed());

	agent_pos = k->theAgent().GetPosition() + k->theAgent().GetAbsoluteSpeedVector();
	ball_pos = k->theBall().GetPosition() + k->theBall().GetAbsoluteSpeedVector();
	ball_vec = CVector(agent_pos, ball_pos);
	ball_vec.Rotate(-k->theAgent().GetBodyDir() - act_ang);
	
      } else if ( cmd->GetType() == CMD_Dash ) {
	CDashCommand* tmp_cmd = cmd->GetDashCommand();

	float effort = k->theAgent().GetEffort();
	CVector a(k->theAgent().GetBodyDir(),
		  tmp_cmd->GetPower() * effort * p->SP_dash_power_rate);
	CVector u = k->theAgent().GetAbsoluteSpeedVector() + a;
	agent_pos = k->theAgent().GetPosition() + u;
      
	ball_pos = k->theBall().GetPosition() + k->theBall().GetAbsoluteSpeedVector();
	ball_vec = CVector(agent_pos, ball_pos);
	ball_vec.Rotate(-k->theAgent().GetBodyDir());

      } else if ( cmd->GetType() == CMD_Kick ) {
	CKickCommand* tmp_cmd = cmd->GetKickCommand();
    
	t_power kick_power = p->SP_kick_power_rate * tmp_cmd->GetPower() *
	  (1.0 - 0.25*Abs(k->theBall().GetDirection())/180.0 -
	   0.25*(k->theBall().GetDistance() - p->SP_player_size -
		 p->SP_ball_size)/p->SP_kickable_area);
  
	float ball_dir = k->theBall().GetAbsoluteSpeedDirection() +
	  k->theAgent().GetBodyDir() + tmp_cmd->GetAngle();
	CVector u = k->theBall().GetAbsoluteSpeedVector() + CVector(ball_dir, kick_power);
	if ( u.GetLength() > p->SP_ball_speed_max )
	  u.SetLength(p->SP_ball_speed_max);
	
	agent_pos = k->theAgent().GetPosition() + k->theAgent().GetAbsoluteSpeedVector();
	ball_pos = k->theBall().GetPosition() + u;
	ball_vec = CVector(agent_pos, ball_pos);
	ball_vec.Rotate(-k->theAgent().GetBodyDir());
      
      } else {

	agent_pos = k->theAgent().GetPosition() + k->theAgent().GetAbsoluteSpeedVector();
	ball_pos = k->theBall().GetPosition() + k->theBall().GetAbsoluteSpeedVector();
	ball_vec = CVector(agent_pos, ball_pos);
	ball_vec.Rotate(-k->theAgent().GetBodyDir());
      
      }
    } else {

      ball_pos = k->theBall().GetPosition() + k->theBall().GetAbsoluteSpeedVector();
      ball_vec = CVector(k->theAgent().GetPosition(), ball_pos);
      ball_vec.Rotate(-k->theAgent().GetBodyDir());
      
    }

    ang = ball_vec.GetAngle();
    ang -= k->theAgent().GetNeckDir();
    NormalizeAngleDeg(&ang);
    ang = min(p->SP_max_turn_neck_moment, ang);
    ang = max(p->SP_min_turn_neck_moment, ang);

    if ( test ) {
      std::cout << "In CalcTurnNeck. Ballpos = " << ball_pos
	   << " Ballvec = " << ball_vec
	   << " Angle = " << ang << std::endl;
    }
  }

  if ( Abs(ang) > 3.0 ) {
    CTurnNeckCommand* new_cmd = new CTurnNeckCommand(k, ang);
    k->theAgent().SetNextTurnNeck(new_cmd);
    k->theAgent().SetDoTurnNeck(true);
  } else {
    k->theAgent().SetDoTurnNeck(false);
    k->theAgent().SetNextTurnNeck(NULL);
  }
  
}
*/

CCommand* CSkills::MoveAlong(CCustomCommand* cmd)
{

  const int_type hist = CURRENT_EST;
  
  if ( showSkill )
    std::cout << k->theGame().GetCurrentTime() << " " << k->theAgent().GetUNumber()
	 << " In MoveAlong" << std::endl;

  CCommand* nxt_cmd = NULL;
  
  CVector v = cmd->GetVector();

  if ( test ) {
    std::cout << k->theGame().GetCurrentTime() << " "
	 << k->theAgent().GetUNumber()
	 << " MoveAlong vector = " << v
	 << std::endl;
  }
  
  v.Rotate(-k->theAgent().GetBodyDir(hist));
  
  if ( Abs(v.GetAngle()) > p->RS_max_dev_point_dir ) {
    nxt_cmd = new CTurnCommand(k, v);
  } else { //if ( ip->DistToLastOpponent(hist) > 0 ) {
    if ( cmd->GetMode() == 1 )
      nxt_cmd = new CDashCommand(k, 60);
    else if ( cmd->GetMode() == 2 )
      nxt_cmd = new CDashCommand(k, 15);
    else
      nxt_cmd = new CDashCommand(k, 30);
  }
  /*
    else {
    tmp_cmd = new CCustomCommand(k, CMD_MoveTo, 2);
    tmp_cmd->SetPoint(k->theAgent().GetRole()->GetHomePosition());
    nxt_cmd = MoveTo(tmp_cmd);
    delete tmp_cmd;
  }
  */

  return nxt_cmd;

}

CCommand* CSkills::GoalKick(CCustomCommand* cmd)
{

  const int_type hist = CURRENT_EST;
  
  if ( showSkill )
    std::cout << k->theGame().GetCurrentTime() << " "
	      << k->theAgent().GetUNumber() << " In GoalKick" << std::endl;

  CCommand* nxt_cmd = NULL;

  if (k->theAgent().GetBodyDir(hist) > 0)
    nxt_cmd = new CKickCommand(k, 100, 80 - k->theAgent().GetBodyDir(hist) );
  else
    nxt_cmd = new CKickCommand(k, 100, -80 - k->theAgent().GetBodyDir(hist) );

  return nxt_cmd;
  
}

CCommand* CSkills::Pass(CCustomCommand* cmd)
{

  const int_type hist = CURRENT_EST;
  
  if ( showSkill )
    std::cout << k->theGame().GetCurrentTime() << " "
	      << k->theAgent().GetUNumber()
	      << " In Pass" << std::endl;

  CCommand* nxt_cmd = NULL;

  if ( false )
    std::cout << "I see " << ip->SeenTeammates().GetNumberOfPlayers()
	      << " teammates.\n";

  if ( ip->SeenTeammates().GetNumberOfPlayers() < 1 ) {
    //std::cout << "No-one to pass! Don't see any teammates!" << std::endl;
    return NULL;
  }    

  int i = 0;
  float my_x = k->theAgent().GetPosX(hist);
  float my_y = k->theAgent().GetPosY(hist);
  const CPlayerObject* clst_plr = NULL;

  if ( cmd->GetMode() == 1 ) {
    //Goalie pass
    for (int i = 0; i < ip->SeenTeammates().GetNumberOfPlayers(); ++i) {
      clst_plr = ip->SeenTeammates()[i];
      if ( clst_plr != NULL &&
	   20 < Abs(clst_plr->GetPosY(hist)) &&
	   clst_plr->GetDistance(hist) < p->RS_max_pass_dist &&
	   Abs(clst_plr->GetDirection(hist)) < 80.0 ) {
	if ( test )
	  std::cout << "Passable player at vector " << clst_plr->GetVector(hist)
		    << std::endl;
	break;
      }
    }
  } else if ( cmd->GetMode() == 2 ) {
    //Defender pass
    for (int i = 0; i < ip->SeenTeammates().GetNumberOfPlayers(); ++i) {
      clst_plr = ip->SeenTeammates()[i];
      if ( clst_plr != NULL &&
	   Abs(my_y) < Abs(clst_plr->GetPosY(hist)) &&
	   clst_plr->GetDistance(hist) < p->RS_max_pass_dist &&
	   Abs(clst_plr->GetDirection(hist)) < 80.0 ) {
	if ( test )
	  std::cout << "Passable player at vector " << clst_plr->GetVector(hist)
		    << std::endl;
	break;
      }
    }
  } else {
    //Other player pass
    for (i = 0; i < ip->SeenTeammates().GetNumberOfPlayers(); i++) {
      clst_plr = ip->SeenTeammates()[i];
      if ( clst_plr != NULL &&
	   my_x < clst_plr->GetPosX(hist) &&
	   clst_plr->GetDistance(hist) < p->RS_max_pass_dist &&
	   Abs(clst_plr->GetDirection(hist)) < 80.0 ) {
	if ( test )
	  std::cout << "Passable player at vector " << clst_plr->GetVector(hist)
		    << std::endl;
	break;
      }
    }
  
    if ( clst_plr == NULL ) {
      for (i = 0; i < ip->SeenTeammates().GetNumberOfPlayers(); i++) {
	clst_plr = ip->SeenTeammates()[i];
	if ( clst_plr != NULL &&
	     clst_plr->GetDistance(hist) < p->RS_max_pass_dist*2 &&
	     Abs(clst_plr->GetDirection(hist)) < 90.0 ) {
	  if ( test )
	    std::cout << "Less passable player at vector "
		      << clst_plr->GetVector(hist) << std::endl;
	  break;
	}
      }
    }
  }  

  if ( clst_plr != NULL ) {

    if ( test ) {
      std::cout <<  k->theGame().GetCurrentTime()
		<< " Pass vector = " << clst_plr->GetVector(hist)
		<< std::endl;
    }
  
    //std::cout << "Closest teammate = " << clst_plr->GetSide() << "-"
    // << clst_plr->GetUNumber() << std::endl;
    //std::cout << "Vector = " << clst_plr->GetVector(hist) << std::endl;

    CAngleDeg dir = clst_plr->GetDirection(hist);
    if ( clst_plr->GetPosY(hist) > 0 )
      dir -= 5;
    else
      dir += 5;

    if ( clst_plr->GetDistance(hist) > 15 )
      nxt_cmd = new CKickCommand(k, 100, dir);
    else if ( clst_plr->GetDistance(hist) > 10 )
      nxt_cmd = new CKickCommand(k, 80, dir);
    else
      nxt_cmd = new CKickCommand(k, 60, dir);
  } else {
    //std::cout << "No-one to pass!" << std::endl;
    nxt_cmd = NULL;
  }
    
  return nxt_cmd;
  
}

CCommand* CSkills::Score(CCustomCommand* cmd)
{

  const int_type hist = CURRENT_EST;
  
  if ( showSkill )
    std::cout << k->theGame().GetCurrentTime() << " " << k->theAgent().GetUNumber()
	      << " In Score" << std::endl;

  CCommand* nxt_cmd = NULL;

  CPoint pt1 = k->flagGRT.GetPosition() + CPoint(0, 2);
  CPoint pt2 = k->flagGRB.GetPosition() + CPoint(0, -2);

  //std::cout << " pt1: " << pt1 << " pt2: " << pt2 << std::endl;
  
  CAngleDeg min_ang = k->theAgent().GetAbsoluteVectorTo(pt1, hist).GetAngle();
  CAngleDeg max_ang = k->theAgent().GetAbsoluteVectorTo(pt2, hist).GetAngle();

  //std::cout << "min_ang = " << min_ang << " max_ang = " << max_ang << std::endl;

  if ( min_ang > max_ang) {
    CAngleDeg tmp = max_ang;
    max_ang = min_ang;
    min_ang = tmp;
    //std::cout << "[SWAP] min_ang = " << min_ang << " max_ang = " << max_ang << std::endl;
  }

  //  CAngleDeg step = (max_ang - min_ang) / 15;
  CAngleDeg shoot_angle = min_ang;

  /*
  for (CAngleDeg ang = min_ang; ang < max_ang; ang += step) {
    // Decide if something is in the way of the ball.
    // If not this is a good angle
  }
  */
  
  if ( k->theAgent().GetPosY(hist) > 0 )
    shoot_angle = max_ang;
  
  // shoot_angle -= k->theAgent().GetBodyDir(hist);
  // std::cout << "Score angle: " << shoot_angle << std::endl;
  // nxt_cmd = new CKickCommand(k, 100, shoot_angle);

  shoot_angle -= k->theAgent().GetBodyDir(hist);
  
  if ( test ) {
    std::cout << k->theGame().GetCurrentTime()
	 << " Score direction = " << shoot_angle
	 << std::endl;
  }
  
  //nxt_cmd = new CKickCommand(k, CVector(shoot_angle, 100));
  nxt_cmd = new CKickCommand(k, 100, shoot_angle);

  return nxt_cmd;
  
}

CCommand* CSkills::FindBall(CCustomCommand* cmd)
{

  const int_type hist = CURRENT_EST;
  
  if ( showSkill )
    std::cout << k->theGame().GetCurrentTime() << " " << k->theAgent().GetUNumber()
	      << " In FindBall" << std::endl;

  CCommand* nxt_cmd = NULL;

  CAngleDeg ang = p->SP_visible_angle * p->RS_max_turn_angle_percentage;
  if ( test ) {
    std::cout << "Ball pos: " << k->theBall().GetPosition(hist)
	      << " Ball relative vector: " << k->theBall().GetVector(hist)
	      << " Conf = " << k->theBall().GetConfidence(hist)
	      << std::endl;
    std::cout <<  k->theGame().GetCurrentTime()
	      << " FindBall dir = " << ang
	      << std::endl;
  }
    
  //if ( k->theAgent().GetLastSightTime() == k->theGame().GetCurrentTime() )
  /*
  if ( ip->FirstActionOpSinceSight() )
    nxt_cmd = new CTurnCommand(k, ang);
  else
    nxt_cmd = NULL;
  */
  if ( k->theAgent().GetPosY(hist) > 0 )
    nxt_cmd = new CTurnCommand(k, CAngleDeg(60));
  else
    nxt_cmd = new CTurnCommand(k, CAngleDeg(-60));
  
  return nxt_cmd;
}

CCommand* CSkills::GoalieMoveTo(CCustomCommand* cmd)
{
  const int_type hist = NEXT_EST;
  
  CCommand* nxt_cmd = NULL;

  CVector vec = k->theAgent().GetRelativeVectorTo(cmd->GetPoint(), hist);
  float_type max_dash = 100;
  if ( k->theBall().GetDistance(hist) > 30 )
    max_dash = 40;

  if ( showSkill )
    std::cout << k->theGame().GetCurrentTime() << " " << k->theAgent().GetUNumber()
	      << " In GoalieMoveTo"
	      << " Vector to point: " << vec
	      << std::endl;

  if ( Abs(vec.GetAngle()) > p->RS_max_dev_point_dir )
    nxt_cmd = new CTurnCommand(k, vec.GetAngle());
  else if ( vec.GetLength() < 2 )
    nxt_cmd = new CDashCommand(k, vec.GetLength(), DASH_Dist, max_dash);
  else
    nxt_cmd = new CDashCommand(k, max_dash);
  
  /*
  if ( Abs(vec.GetAngle()) < 90 &&
       Abs(vec.GetAngle()) < p->RS_max_dev_point_dir ) {
    if ( k->theBall().GetDistance(hist) > 20 )
      nxt_cmd = new CDashCommand(k, vec.GetLength(), DASH_Dist, 25);
    else
      nxt_cmd = new CDashCommand(k, vec.GetLength(), DASH_Dist, 40);
  } else if ( vec.GetLength() > 3.0 &&
	      Abs(vec.GetAngle()) < 90 ) {
    nxt_cmd = new CTurnCommand(k, vec.GetAngle());
  } else if ( k->theBall().GetDirection(hist) > p->RS_max_dev_ball_dir )
    nxt_cmd = new CTurnCommand(k, k->theBall().GetDirection(hist));
  else {
    if ( k->theBall().GetDistance(hist) > 20 )
      nxt_cmd = new CDashCommand(k, -20);
    else if ( k->theBall().GetDistance(hist) < 5.0 )
      nxt_cmd = new CDashCommand(k, -100);
    else
      nxt_cmd = new CDashCommand(k, -30);
  }
  */

  /*
  if ( ip->BallInOurPenaltyArea(hist) ) {
    // The ball is outside the penalty area
    CVector abs_ball_vec(k->goalL.GetPosition(), k->theBall().GetPosition(hist));
    abs_ball_vec.SetLength(2.0);
    CCustomCommand* tmp_cmd = new CCustomCommand(k, CCMD_MoveTo, 2);
    CPoint dest = k->goalL.GetPosition() + abs_ball_vec;
    tmp_cmd->SetPoint(dest);
    nxt_cmd = MoveTo(tmp_cmd);
    delete tmp_cmd;
  } else if ( k->theBall().GetLastSeenMovingTime() >=
	      k->theGame().GetCurrentTime() -1 ) {
    // The ball is in the penalty area and I know the direction
    // of the ball.
    CAngleDeg ball_dir = k->theBall().GetAbsoluteSpeedDirection(hist);
    float_type dx = k->theBall().GetPosX(hist) - 51.5;
    float_type dy = Tan(ball_dir)*dx;
    if ( ball_dir < 0 )
      dy = -dy;
    CPoint dest = CPoint(-51.5, k->theBall().GetPosY(hist) + dy);
    std::cout << "Ball dir: " << ball_dir << " dx: " << dx
	      << " dy: " << dy << " dest: " << dest << std::endl;
    CCustomCommand* tmp_cmd = new CCustomCommand(k, CCMD_MoveTo, 2);
    tmp_cmd->SetPoint(dest);
    nxt_cmd = MoveTo(tmp_cmd);
    delete tmp_cmd;
  } else {
    // The ball is in the penalty area but I don't know the direction
    // of the ball.

    // Here I have two options.
    // 1. Estimated direction
    // 2. Do the as when ball is outside the penalty area
    CVector abs_ball_vec(k->goalL.GetPosition(), k->theBall().GetPosition(hist));
    abs_ball_vec.SetLength(2.0);
    CCustomCommand* tmp_cmd = new CCustomCommand(k, CCMD_MoveTo, 2);
    CPoint dest = k->goalL.GetPosition() + abs_ball_vec;
    tmp_cmd->SetPoint(dest);
    nxt_cmd = MoveTo(tmp_cmd);
    delete tmp_cmd;    
  }
  */  
  
  return nxt_cmd;
  
}

CCommand* CSkills::MoveTo(CCustomCommand* cmd)
{
 
  const int_type hist = CURRENT_EST;
  static bool avoid_obstacle = false;
  static CTime avoid_obstacle_time;
  static CPoint tmp_pos;
  
  CCommand* nxt_cmd = NULL;

  // At the present time MoveTo doesn't take obstacles into account
  // const CPlayerObject* clst_plr = ip->GetClosestPlayer(hist);

  if (cmd->GetMode() == 1) {  // Mode 1 == MoveToBall

    if ( showSkill )
      std::cout << k->theGame().GetCurrentTime() << " "
		<< k->theAgent().GetUNumber()
		<< " In MoveToBall" << std::endl;

    CVector ball_vec = k->theBall().GetVector(hist);
    
    avoid_obstacle = false;
    if ( test ) {
      std::cout << k->theGame().GetCurrentTime()
	   << " MoveToBall vector = " << ball_vec //k->theBall().GetVector(hist)
	   << std::endl;
    }

    /* NOT UPDATED!!!
    if (  avoid_turns > 0 || (clst_plr != NULL && !dribble &&
	  clst_plr->GetDistance() < p->RS_min_player_dist) ) {
      float dir = clst_plr->GetDirection();
      //      std::cout << "MT1" << std::endl;
      //      std::cout << "Found obstacle in direction: " << dir << std::endl;
      if (  !have_turned && avoid_turns < 1 &&
	    Abs(dir) < 40.0 ) {
	if (dir > 0)
	  dir = dir - 90;
	else
	  dir = 90 + dir;
	//	if ( Abs(dir) > 70 )
	have_turned = true;
	no_findball = true;
	avoid_turns = 2;
	cmd = new CTurnCommand(k, dir);
      } else {
	avoid_turns--;
	if (avoid_turns == 0)
	  no_findball = false;	  
	avoided_last_turn = true;
	have_turned = false;
	cmd = new CDashCommand(k, p->RS_dash_to_point);
      }
    } else*/
    // std::cout << "Balldir = " << k->theBall().GetDirection(hist) << std::endl;
    //    if ( Abs(k->theBall().GetDirection(hist)) > p->RS_max_dev_ball_dir ) {
    if ( Abs(ball_vec.GetAngle()) > p->RS_max_dev_ball_dir ) {
      nxt_cmd = new CTurnCommand(k, ball_vec.GetAngle());
    } else if ( ball_vec.GetLength() > p->SP_kickable_area ) {
      nxt_cmd = new CDashCommand(k, ball_vec.GetLength(),
				 DASH_Dist, p->RS_dash_to_ball);
    }
#ifdef DEBUG_SKILL
    else {
      std::cout << "Did a MoveToBall when already at the ball!" << std::endl;	
    }
#endif
  } else if (cmd->GetMode() == 2) { // Mode 2 == MoveToPoint(cmd->point)
    if ( showSkill )
      std::cout << k->theGame().GetCurrentTime() << " " << k->theAgent().GetUNumber()
		<< " In MoveToPoint" << std::endl;

    CPoint dest;
    /*
    if ( avoid_obstacle &&
	 (k->theGame().GetCurrentTime() - avoid_obstacle_time) < 10 )
      dest = tmp_pos;
    else
    */
      dest = cmd->GetPoint();

    CVector vec = k->theAgent().GetRelativeVectorTo(dest, hist);

    if ( test ) {
      std::cout << k->theGame().GetCurrentTime()
		<< " MoveToPoint point = " << dest
		<< " Vector to that point = " << vec
		<< std::endl;
    }

    if ( false && !avoid_obstacle && ip->ObstacleOnVector(vec, hist) ) {
      avoid_obstacle = true;
      avoid_obstacle_time = k->theGame().GetCurrentTime();
      tmp_pos = ip->GetAvoidObstaclePoint(hist);
      //std::cout << "Avoiding the ball! New destination = " << tmp_pos << std::endl;
      nxt_cmd = MoveTo(cmd);
    } else if ( Abs(vec.GetAngle()) > 90 &&
		Abs(k->theBall().GetDirection(hist)) < 45 ) {
      nxt_cmd = new CDashCommand(k, -p->RS_dash_to_point/2);
    } else if ( Abs(vec.GetAngle()) > p->RS_max_dev_point_dir ) {
      nxt_cmd = new CTurnCommand(k, vec.GetAngle());
    } else if ( vec.GetLength() > p->SP_kickable_area*p->RS_safe_kickable_area ) {
      nxt_cmd = new CDashCommand(k, vec.GetLength(), DASH_Dist, p->RS_dash_to_point);
    } else if ( avoid_obstacle ) {
      avoid_obstacle = false;
      nxt_cmd = MoveTo(cmd);
    }
#ifdef DEBUG_SKILL
    else {
      std::cout << "Did a MoveToPoint " << cmd->GetPoint()
		<< " when already at the point!" << std::endl;
    }
#endif
  } else if (cmd->GetMode() == 3) { // Should be backpeddel!!

    if ( showSkill )
      std::cout << k->theGame().GetCurrentTime() << " " << k->theAgent().GetUNumber()
		<< " In BackpeddelToPoint" << std::endl;

    CVector vec = k->theAgent().GetRelativeVectorTo(cmd->GetPoint(), hist);
    vec = -vec;

    if ( test ) {
      std::cout << k->theGame().GetCurrentTime()
		<< " MoveToPointBackpeddel point = " << cmd->GetPoint()
		<< " vector = " << vec 
		<< std::endl;
    }
    
    if ( Abs(vec.GetAngle()) > p->RS_max_dev_point_dir ) {
      nxt_cmd = new CTurnCommand(k, vec.GetAngle());
    } else if ( vec.GetLength() > p->RS_close_to_point ) {
      //      cmd = new CDashCommand(k, p->RS_dash_to_point);
      //CDashCommand* tmp_cmd = new CDashCommand(k);
      //tmp_cmd->DashDistance(vec.GetLength());
      //cmd = tmp_cmd;
      nxt_cmd = new CDashCommand(k, vec.GetLength(), DASH_Dist, p->RS_dash_to_point, true);
    }
#ifdef DEBUG_SKILL
    else {
      std::cout << "Did a BackpeddelToPoint " << cmd->GetPoint()
		<< " when already at the point!" << std::endl;
    }
#endif    
  } else {
    //std::cout << "Unknown mode for MoveTo!" << std::endl;
    nxt_cmd = NULL;
  }

  return nxt_cmd;
}

CCommand* CSkills::TurnBall(CCustomCommand* cmd)
{

  const int_type hist = CURRENT_EST;
  
  if ( showSkill )
    std::cout << k->theGame().GetCurrentTime() << " " << k->theAgent().GetUNumber()
	      << " In TurnBall" << std::endl;

  CCommand* nxt_cmd = NULL;
  
  CAngleDeg ang = Abs(cmd->GetVector().GetAngle() - k->theAgent().GetBodyDir(hist));

  if ( test ) {
    std::cout << k->theGame().GetCurrentTime()
	      << " TurnBall vector = " << cmd->GetVector()
	      << std::endl;
  }
  
  if ( k->theBall().GetDistance(hist) > p->SP_kickable_area*p->RS_safe_kickable_area ) {
    //    nxt_cmd = new CDashCommand(k, p->RS_dribble_dash_power);
    //CDashCommand* nxt_cmd = new CDashCommand(k);
    //nxt_cmd->DashDistance(k->theBall().GetDistance(hist));
    nxt_cmd = new CDashCommand(k, k->theBall().GetDistance(hist) - p->SP_player_size,
			       DASH_Dist);
  } else if ( ang > p->RS_max_dev_point_dir ) {
    if ( Abs(k->theBall().GetDirection(hist)) > p->RS_turnball_max_dir_dev ) {
      nxt_cmd = new CTurnCommand(k, k->theBall().GetDirection(hist));
    } else {
      
      if (ang > p->RS_turnball_max_moment)
	ang = p->RS_turnball_max_moment;
      else if (ang < -p->RS_turnball_max_moment)
	ang = -p->RS_turnball_max_moment;

      if ( k->theAgent().GetPosY(hist) < 0 )
	ang = -ang;
      
      if ( Abs(k->theAgent().GetPosY(hist)) > 25 )
	ang = -ang;
      
      nxt_cmd = new CKickCommand(k, p->RS_turnball_kick_power, ang);
    }
  }
#ifdef DEBUG_SKILL
    else {
      std::cout << "Did a TurnBall when already in the desired direction!" << std::endl;
    }
#endif      
  
  return nxt_cmd;
  
}

CCommand* CSkills::FindPlayer(CCustomCommand* cmd)
{
  
  const int_type hist = CURRENT_EST;

  CCommand* nxt_cmd = NULL;
  
  if ( showSkill )
    std::cout << k->theGame().GetCurrentTime() << " " << k->theAgent().GetUNumber()
	      << " In FindPlayer.  NOT IMPLEMENTED YET!" << std::endl;

  return nxt_cmd;
}

CCommand* CSkills::Dribble(CCustomCommand* cmd)
{

  const int_type hist = CURRENT_EST;
  static bool dribble = true;
  
  if ( showSkill )
    std::cout << k->theGame().GetCurrentTime() << " " << k->theAgent().GetUNumber()
	      << " In Dribble" << std::endl;

  CCommand* nxt_cmd = NULL;

  // Extremely lousy :-)
  CVector goal_vec = k->theAgent().GetAbsoluteVectorTo(k->theirGoal.GetPosition(hist), hist);
  CAngleDeg shoot_angle = goal_vec.GetAngle();
  if ( k->theAgent().GetPosX(hist) < 20 ) {
    if ( k->theAgent().GetPosY(hist) > 20 )
      shoot_angle = -10;
    else if ( k->theAgent().GetPosY(hist) < -20 )
      shoot_angle = 10;
  }


  CPlayerObject* clst_plr = ip->GetClosestOpponent(hist);
  dribble = true;
  if (cmd->GetMode() == 1) {  // Dribble towards goal!!
    /*
    if ( k->theAgent().GetPosX(hist) + 0.5 > k->theBall().GetPosX(hist) &&
	 Abs(k->theAgent().GetBodyDir(hist)) > 45 ) {
      CCustomCommand* tmp_cmd = new CCustomCommand(k, CCMD_TurnBall);
      tmp_cmd->SetVector(CVector(0,10));			      
      nxt_cmd = TurnBall(tmp_cmd);
      delete tmp_cmd;
    } else */
    if ( Abs(k->theBall().GetDirection(hist)) > p->RS_max_dev_ball_dir ) {
      nxt_cmd = new CTurnCommand(k, k->theBall().GetDirection(hist));
    } else if ( k->theBall().GetDistance(hist) > p->SP_kickable_area ) {
      nxt_cmd = new CDashCommand(k, k->theBall().GetDistance(hist)
				 - p->SP_player_size
				 - p->SP_ball_size, DASH_Dist);
    } else if (  (clst_plr != NULL &&
		  clst_plr->GetDistance(hist) < p->RS_dribble_min_player_dist) ) {
      CAngleDeg dir = clst_plr->GetDirection(hist) - shoot_angle;
      //      std::cout << "Dribble1" << std::endl;
      //std::cout << "[Dribble] Found obstacle in direction: " << dir << std::endl;
      if (Abs(dir) < 30) {
	// Opponent is in the way. Kick in the other direction than he is.
	shoot_angle -= dir;
      }
      nxt_cmd = new CKickCommand(k, 60, dir);
      dribble = false;
    } else if ( goal_vec.GetLength() > p->RS_close_to_goal ) {
      //nxt_cmd = new CKickCommand(k, GetKickPower(goal_vec.GetLength()),
      //			 GetKickDirection(goal_vec.GetAngle()));
      float_type kick_dist = k->theAgent().GetSpeed(hist);
      if ( kick_dist < 0.3 )
	kick_dist = 0.3;
      if ( ip->DistToClosestOpponentInFront(hist) > 10.0 &&
	   k->theAgent().GetPosX(hist) < 30 )
	nxt_cmd = new CKickCommand(k, CVector(shoot_angle, kick_dist*2));
      else
	nxt_cmd = new CKickCommand(k, CVector(shoot_angle, kick_dist));
      //nxt_cmd = new CKickCommand(k, p->RS_dribble_kick_power * (p->SP_kickable_area - k->theBall().GetDistance(hist) + 0.1), shoot_angle);
      //dribble = false;
    } else {
      nxt_cmd = Score(cmd);
    }
  } else {
    //std::cout << "Unknown mode for Dribble!" << std::endl;
    nxt_cmd = NULL;
  }
  
  return nxt_cmd;

}

CCommand* CSkills::TrackBall(CCustomCommand* cmd)
{

  const int_type hist = CURRENT_EST;
  
  if ( showSkill )
    std::cout << k->theGame().GetCurrentTime() << " " << k->theAgent().GetUNumber()
	      << " In TrackBall" << std::endl;

  CCommand* nxt_cmd = NULL;

  if ( test ) {
    std::cout << k->theGame().GetCurrentTime()
	      << " TrackBall ball vector = " << k->theBall().GetVector(hist)
	      << std::endl;
  }

  //  if ( k->theBall().GetConfidence(hist) < p->RS_low_confidence )
  //  nxt_cmd = FindBall(cmd);
  //else
  nxt_cmd = new CTurnCommand(k, k->theBall().GetDirection(hist));

  return nxt_cmd;

}

CCommand* CSkills::InterceptBall(CCustomCommand* cmd)
{

  const int_type hist = CURRENT_EST;
  
  if ( showSkill )
    std::cout << k->theGame().GetCurrentTime() << " " << k->theAgent().GetUNumber()
	      << " In InterceptBall"
	      << " Ballspeed: " << k->theBall().GetAbsoluteSpeedVector(NOW)
	      << std::endl;

  CCommand* nxt_cmd = NULL;

  if ( k->theBall().GetLastSeenMovingTime() >=
       k->theGame().GetCurrentTime() -1 &&
       k->theBall().GetDistance(hist) > 2 &&
       k->theBall().GetSpeed(hist) > 0.3 ) {
    CVector vec = k->theBall().GetAbsoluteSpeedVector(hist);
    vec.SetLength(k->theBall().GetDistance(hist) / 2);
    CCustomCommand* tmp_cmd = new CCustomCommand(k, CCMD_MoveTo, 2);
    CPoint pt = k->theBall().GetPosition(hist) + vec;
    //std::cout << " New dest: " << pt << std::endl;
    tmp_cmd->SetPoint(pt);
    nxt_cmd = MoveTo(tmp_cmd);
    delete tmp_cmd;
  } else {
    CCustomCommand* tmp_cmd = new CCustomCommand(k, CCMD_MoveTo, 1);
    nxt_cmd = MoveTo(tmp_cmd);
    delete tmp_cmd;
  }
  
  return nxt_cmd;
  
};

CCommand* CSkills::MakePassable(CCustomCommand* cmd)
{

  const int_type hist = CURRENT_EST;
  
  if ( showSkill )
    std::cout << k->theGame().GetCurrentTime() << " "
	      << k->theAgent().GetUNumber()
	      << " In MakePassable" << std::endl;

  CCommand* nxt_cmd = NULL;

  if ( k->theBall().GetDistance(hist) > p->RS_max_pass_dist ) {
    CCustomCommand* tmp_cmd = new CCustomCommand(k, CCMD_MoveTo, 1);
    nxt_cmd = MoveTo(tmp_cmd);
    delete tmp_cmd;
  } else {
    nxt_cmd = TrackBall(cmd);
  }
  
  return nxt_cmd;

}

CCommand* CSkills::ClearBall(CCustomCommand* cmd)
{

  const int_type hist = CURRENT_EST;
  
  if ( showSkill )
    std::cout << k->theGame().GetCurrentTime() << " "
	      << k->theAgent().GetUNumber()
	      << " In ClearBall" << std::endl;

  CCommand* nxt_cmd = NULL;

  CAngleDeg ang = k->theBall().GetAbsoluteVectorTo(k->theirGoal.GetPosition(hist),
						   hist).GetAngle();
  
  if ( k->theAgent().GetPosX(hist) < 0 )
    ang = 60;

  if ( Abs(k->theAgent().GetPosY(hist)) > 15 )
    ang = 30;
  
  if ( Abs(k->theAgent().GetPosY(hist)) > 25 )
    ang = -5;

  if ( Abs(k->theAgent().GetPosY(hist)) > 30 )
    ang = -15;

  if (k->theAgent().GetPosY(hist) < 0)
    nxt_cmd = new CKickCommand(k, 100, -ang - k->theAgent().GetBodyDir(hist));
  else
    nxt_cmd = new CKickCommand(k, 100, ang - k->theAgent().GetBodyDir(hist));
  
  return nxt_cmd;

}

CCommand* CSkills::CornerClearBall(CCustomCommand* cmd)
{

  const int_type hist = CURRENT_EST;
  
  if ( showSkill )
    std::cout << k->theGame().GetCurrentTime() << " "
	      << k->theAgent().GetUNumber()
	      << " In CornerClearBall" << std::endl;

  CCommand* nxt_cmd = NULL;

  CAngleDeg ang;
  if ( ip->LeftCornerKick(hist) )
    ang = k->theBall().GetAbsoluteVectorTo(k->flagPRT.GetPosition(hist), hist).GetAngle();
  else
    ang = k->theBall().GetAbsoluteVectorTo(k->flagPRB.GetPosition(hist), hist).GetAngle();
  
  nxt_cmd = new CKickCommand(k, 100, ang - k->theAgent().GetBodyDir(hist));
  
  return nxt_cmd;

}

CCommand* CSkills::CutOffPassAngle(CCustomCommand* cmd)
{

  const int_type hist = CURRENT_EST;
  
  if ( showSkill )
    std::cout << k->theGame().GetCurrentTime() << " " << k->theAgent().GetUNumber()
	      << " In CutOffPassAngle" << std::endl;

  CCommand* nxt_cmd = NULL;

  CCustomCommand* tmp_cmd = new CCustomCommand(k, CCMD_GoalieMoveTo);
  CVector vec(k->theBall().GetPosition(), k->goalL.GetPosition());
  if ( ip->BehindBall(hist) )
    vec.SetLength(5);
  else
    vec.SetLength(10);
  tmp_cmd->SetPoint(k->theBall().GetPosition() + vec);
  nxt_cmd = TrackBall(cmd);
  delete tmp_cmd;
  
  return nxt_cmd;

}

CCommand* CSkills::CutOffGoalAngle(CCustomCommand* cmd)
{

  const int_type hist = CURRENT_EST;
  
  if ( showSkill )
    std::cout << k->theGame().GetCurrentTime() << " " << k->theAgent().GetUNumber()
	      << " In CutOffGoalAngle" << std::endl;

  CCommand* nxt_cmd = NULL;

  CPoint dest(k->theBall().GetPosX(hist) - 3,
	      k->theAgent().GetPosY(hist));
  
  float_type my_y =  k->theAgent().GetPosY(hist);
  float_type ball_y =  k->theBall().GetPosY(hist);
  
  if ( my_y > 0 && ball_y > 0 && ball_y-1 < my_y )
    dest.SetY(ball_y - 2);
  else if ( my_y < 0 && ball_y < 0 && ball_y+1 > my_y )
    dest.SetY(ball_y + 2);

  /*
  CCustomCommand* tmp_cmd = new CCustomCommand(k, CCMD_MoveTo);
  tmp_cmd->SetPoint(dest);
  nxt_cmd = MoveTo(tmp_cmd);
  delete tmp_cmd;
  */

  CVector vec = k->theAgent().GetRelativeVectorTo(dest, hist);

  if ( Abs(vec.GetAngle()) > p->RS_max_dev_point_dir )
    nxt_cmd = new CTurnCommand(k, vec.GetAngle());
  else
    nxt_cmd = new CDashCommand(k, 90);

  /*
  if ( Abs(k->theAgent().GetBodyDir(hist)) < 170 ) {
    CAngleDeg ang = 180 - Abs(k->theAgent().GetBodyDir(hist));
    if ( k->theAgent().GetBodyDir(hist) > 0 )
      nxt_cmd = new CTurnCommand(k, ang);
    else
      nxt_cmd = new CTurnCommand(k, -ang);
    //nxt_cmd = new CTurnCommand(k, CVector(-180, k->theBall().GetPosX(hist)));
  } else if ( k->theBall().GetPosX(hist) - k->theAgent().GetPosX(hist) < 3 )
    nxt_cmd = new CDashCommand(k, 80);
  else
    nxt_cmd = TrackBall(cmd);
  */
  
  return nxt_cmd;
  
}

CCommand* CSkills::GetBehindBall(CCustomCommand* cmd)
{

  const int_type hist = CURRENT_EST;
  
  if ( showSkill )
    std::cout << k->theGame().GetCurrentTime() << " " << k->theAgent().GetUNumber()
	      << " In GetBehindBall" << std::endl;

  CCommand* nxt_cmd = NULL;
  
  if ( k->theBall().GetPosX(hist) > k->theAgent().GetPosX(hist) &&
       Abs(k->theBall().GetDirection(hist)) < 85 ) {
    CCustomCommand* tmp_cmd = new CCustomCommand(k, CCMD_MoveTo, 1);

    if ( test ) {
      std::cout << k->theGame().GetCurrentTime()
	   << " GetBehindBall ball vector = " << k->theBall().GetVector(hist)
	   << " Next point = " << k->theBall().GetPosition(hist)
	   << std::endl;
    }
    
    nxt_cmd = MoveTo(tmp_cmd);
    delete tmp_cmd;
  } else {
    CCustomCommand* tmp_cmd = new CCustomCommand(k, CCMD_MoveTo, 2);

    CPoint pt = k->theBall().GetPosition(hist);
    pt += CPoint(-5, 0);
    if ( Abs(k->theBall().GetPosY(hist) - k->theAgent().GetPosY(hist)) < 5 ) {
      if ( k->theBall().GetPosY(hist) > k->theAgent().GetPosY(hist) )
	pt += CPoint(0, -5);
      else
	pt += CPoint(0, 5);
    }
    tmp_cmd->SetPoint(pt);
    
    if ( test ) {
      std::cout << k->theGame().GetCurrentTime()
	   << " GetBehindBall ball vector = " << k->theBall().GetVector(hist)
	   << " Next point = " << tmp_cmd->GetPoint()
	   << std::endl;
    }
    
    nxt_cmd = MoveTo(tmp_cmd);
    delete tmp_cmd;
  }

  return nxt_cmd;

}

CCommand* CSkills::GetBehindCorner(CCustomCommand* cmd)
{

  const int_type hist = CURRENT_EST;
  
  if ( showSkill )
    std::cout << k->theGame().GetCurrentTime() << " "
	      << k->theAgent().GetUNumber()
	      << " In GetBehindCorner" << std::endl;

  CCommand* nxt_cmd = NULL;

  if ( k->theBall().GetPosX(hist) < k->theAgent().GetPosX(hist) &&
       Abs(k->theBall().GetDirection(hist)) < 85 ) {
    // If I'm behind the ball then move to the ball
    CCustomCommand* tmp_cmd = new CCustomCommand(k, CCMD_MoveTo, 1);
    nxt_cmd = MoveTo(tmp_cmd);
    delete tmp_cmd;
  } else {
    // Move to the point behind the corner flag.
    CCustomCommand* tmp_cmd = new CCustomCommand(k, CCMD_MoveTo, 2);

    CPoint dest;
    if ( ip->LeftCornerKick(hist) ) {
      dest = k->flagRT.GetPosition();
      dest += CPoint(2, -3);
    } else {
      dest = k->flagRB.GetPosition();
      dest += CPoint(2, 3);
    }
    tmp_cmd->SetPoint(dest);
    
    if ( test ) {
      std::cout << k->theGame().GetCurrentTime()
		<< " GetBehindCorner next point = " << tmp_cmd->GetPoint()
		<< std::endl;
    }
    
    nxt_cmd = MoveTo(tmp_cmd);
    delete tmp_cmd;
  }  

  return nxt_cmd;

}

CCommand* CSkills::MarkPlayer()
{

  const int_type hist = CURRENT_EST;
  
  if ( showSkill )
    std::cout << k->theGame().GetCurrentTime() << " " << k->theAgent().GetUNumber()
	      << " In MarkPlayer" << std::endl;

  CCommand* nxt_cmd = NULL;
  return nxt_cmd;

}

CCommand* CSkills::HoldPosition(CPoint pt, CTime = 0)
{

  const int_type hist = CURRENT_EST;
  
  if ( showSkill )
    std::cout << k->theGame().GetCurrentTime() << " " << k->theAgent().GetUNumber()
	      << " In HoldPosition" << std::endl;

  CCommand* nxt_cmd = NULL;
  return nxt_cmd;

}

// Goalkeeper skills
CCommand* CSkills::Catch(CCustomCommand* cmd)
{

  const int_type hist = CURRENT_EST;
  
  if ( showSkill )
    std::cout << k->theGame().GetCurrentTime() << " " << k->theAgent().GetUNumber()
	      << " In Catch" << std::endl;

  CCommand* nxt_cmd = NULL;

  if ( test ) {
    std::cout << k->theGame().GetCurrentTime()
	      << " Catch ball vector = " << k->theBall().GetVector(hist)
	      << std::endl;
  }
  
  if ( k->theBall().GetLastSeenMovingTime() >=
       k->theGame().GetCurrentTime() -1 &&
       k->theBall().GetDistance(hist) > 2 &&
       k->theBall().GetSpeed(hist) > 0.3 ) {
    CPoint new_ball_pos = k->theBall().GetPosition(hist);
    new_ball_pos += k->theBall().GetAbsoluteSpeedVector(hist);
    CAngleDeg catch_dir = k->theAgent().GetRelativeDirectionTo(new_ball_pos,
							       NEXT_EST);
    nxt_cmd = new CCatchCommand(k, catch_dir);
  } else {
    nxt_cmd = new CCatchCommand(k, k->theBall().GetDirection(hist));
  }
  
  return nxt_cmd;
}

CCommand* CSkills::Rush(CCustomCommand* cmd)
{

  const int_type hist = CURRENT_EST;
  
  if ( showSkill )
    std::cout << k->theGame().GetCurrentTime() << " " << k->theAgent().GetUNumber()
	      << " In Rush" << std::endl;

  CCommand* nxt_cmd = NULL;

  if ( test ) {
    std::cout << k->theGame().GetCurrentTime()
	      << " Rush ball vector = " << k->theBall().GetVector(hist)
	      << std::endl;
  }
  
  //std::cout << "Rushing in direction: " << k->theBall().GetDirection(hist)
  //       << " Distance to ball: " << k->theBall().GetDistance(hist) << std::endl;

  // What if I don't see the ball?
  if ( ip->BallCatchable(hist) ) {
    nxt_cmd = Catch(cmd);
  } else {
    CCustomCommand* tmp_cmd = new CCustomCommand(k, CCMD_MoveTo, 1);
    nxt_cmd = MoveTo(tmp_cmd);
    delete tmp_cmd;
  }
  /*
    if ( Abs(k->theBall().GetDirection(hist)) > p->RS_goalie_max_catch_dir ) {
    nxt_cmd = new CTurnCommand(k, k->theBall().GetDirection(hist));
  } else {
    nxt_cmd = new CDashCommand(k, k->theBall().GetDistance(hist), DASH_Dist);
  }
  */
  return nxt_cmd;

}

/*
  // These should be put in CDecision and/or CStrategy since
  // they are not skills but "plays".
// Fasta situationer
CCommand* CSkills::GoalKick()
{

  const int_type hist = CURRENT_EST;
  
  if ( showSkill )
    std::cout << k->theGame().GetCurrentTime() << " " << k->theAgent().GetUNumber() << " In GoalKick" << std::endl;

  return nxt_cmd;
}

CCommand* CSkills::CornerKick()
{

  const int_type hist = CURRENT_EST;
  
  if ( showSkill )
    std::cout << k->theGame().GetCurrentTime() << " " << k->theAgent().GetUNumber() << " In CornerKick" << std::endl;

  return nxt_cmd;
}

CCommand* CSkills::FreeKick()
{

  const int_type hist = CURRENT_EST;
  
  if ( showSkill )
    std::cout << k->theGame().GetCurrentTime() << " " << k->theAgent().GetUNumber() << " In FreeKick" << std::endl;

  return nxt_cmd;
}
*/
