#ifndef _CSOCCERPLAYER_H_
#define _CSOCCERPLAYER_H_

#include "types.h"
#include "CVector.h"
#include "CPoint.h"
#include "CRectangle.h"

class CGoalKeeper;
class CDefender;
class CMidfielder;
class CAttacker;

enum t_role_type {
  ROLE_GoalKeeper, ROLE_Attacker,
  ROLE_LeftAttacker, ROLE_RightAttacker,
  ROLE_Defender, ROLE_InnerDefender,
  ROLE_OuterDefender, ROLE_CenterDefender,
  ROLE_Midfielder, ROLE_InnerMidfielder,
  ROLE_OuterMidfielder, ROLE_CenterMidfielder,
  ROLE_Unknown
};

enum t_field_pos {
  PP_Left,
  PP_OuterLeft,
  PP_InnerLeft,
  PP_Center,
  PP_InnerRight,
  PP_OuterRight,
  PP_Right
};

class CSoccerPlayer
{
public:
  CSoccerPlayer(const t_role_type role_t = ROLE_Unknown);
  virtual ~CSoccerPlayer();

  virtual const CGoalKeeper* const GetGoalKeeper() const;
  virtual CGoalKeeper* GetGoalKeeper();
  virtual const CDefender* const GetDefender() const;
  virtual CDefender* GetDefender();
  virtual const CMidfielder* const GetMidfielder() const;
  virtual CMidfielder* GetMidfielder();
  virtual const CAttacker* const GetAttacker() const;
  virtual CAttacker* GetAttacker();
  
  t_role_type GetType() const;
  void SetType(const t_role_type new_role_type);

  CPoint GetInitPosition() const;
  void SetInitPosition(const CPoint& new_init_position);
  virtual CPoint GetHomePosition() const;

  virtual CVector GetMoveAlongVector() const;
  void SetMoveAlongVector(const CVector& new_move_along_vector);
  
  virtual CPoint GetHomePosDeviation() const;
  void SetHomePosDeviation(const CPoint& new_home_pos_dev);
  virtual CRectangle GetHomeArea() const;

  virtual CRectangle GetFreeKickArea() const;
  void SetFreeKickArea(const CRectangle& new_free_kick_area);

  virtual t_field_pos GetFieldPosition() const;
  void SetFieldPosition(const t_field_pos new_field_position);

  void SetDoCornerKickLeft(const bool);
  void SetDoCornerKickRight(const bool);
  void SetDoGoalKick(const bool);
  void SetDoKickOff(const bool);

  virtual bool DoCornerKick(const CPoint& ball_pos) const;
  virtual bool DoCornerKickLeft() const;
  virtual bool DoCornerKickRight() const;
  virtual bool DoGoalKick() const;
  virtual bool DoFreeKick(CPoint) const;
  virtual bool DoKickOff() const;
  
protected:
  t_role_type role_type;

  CPoint init_position;
  // The X and Y values of the point describes the deviation from the home_pos.
  CPoint home_pos_deviation; 
  CVector move_along_vector;
  t_field_pos field_position;
  CRectangle free_kick_area;

  bool do_corner_kick_left;
  bool do_corner_kick_right;
  bool do_goal_kick;
  bool do_kick_off;
  
  void Error() const;
};

inline const CGoalKeeper* const CSoccerPlayer::GetGoalKeeper() const
{
  Error();
  return NULL;
}

inline CGoalKeeper* CSoccerPlayer::GetGoalKeeper()
{
  Error();
  return NULL;
}

inline const CDefender* const CSoccerPlayer::GetDefender() const
{
  Error();
  return NULL;
}

inline CDefender* CSoccerPlayer::GetDefender()
{
  Error();
  return NULL;
}

inline const CMidfielder* const CSoccerPlayer::GetMidfielder() const
{
  Error();
  return NULL;
}

inline CMidfielder* CSoccerPlayer::GetMidfielder()
{
  Error();
  return NULL;
}

inline const CAttacker* const CSoccerPlayer::GetAttacker() const
{
  Error();
  return NULL;
}

inline CAttacker* CSoccerPlayer::GetAttacker()
{
  Error();
  return NULL;
}

inline t_role_type CSoccerPlayer::GetType() const
{
  return role_type;
}

inline void CSoccerPlayer::SetType(const t_role_type new_role_type)
{
  role_type = new_role_type;
}

inline CPoint CSoccerPlayer::GetInitPosition() const
{
  return init_position;
}

inline void CSoccerPlayer::SetInitPosition(const CPoint& pt)
{
  init_position = pt;
}

inline CPoint CSoccerPlayer::GetHomePosition() const
{
  return init_position;
}

inline CPoint CSoccerPlayer::GetHomePosDeviation() const
{
  return home_pos_deviation;
}

inline void CSoccerPlayer::SetHomePosDeviation(const CPoint& new_home_pos_deviation)
{
  home_pos_deviation = new_home_pos_deviation;
}

inline CVector CSoccerPlayer::GetMoveAlongVector() const
{
  return CVector();
}

inline void CSoccerPlayer::SetMoveAlongVector(const CVector& new_move_along_vector)
{
  move_along_vector = new_move_along_vector;
}

inline CRectangle CSoccerPlayer::GetFreeKickArea() const
{
  return free_kick_area;
}

inline void CSoccerPlayer::SetFreeKickArea(const CRectangle& new_free_kick_area)
{
  free_kick_area = new_free_kick_area;
}

inline t_field_pos CSoccerPlayer::GetFieldPosition() const
{
  return field_position;
}

inline void CSoccerPlayer::SetFieldPosition(const t_field_pos new_field_position)
{
  field_position = new_field_position;
}

inline void CSoccerPlayer::SetDoCornerKickLeft(const bool new_do_corner_kick_left)
{
  do_corner_kick_left = new_do_corner_kick_left;
}

inline void CSoccerPlayer::SetDoCornerKickRight(const bool new_do_corner_kick_right)
{
  do_corner_kick_right = new_do_corner_kick_right;
}

inline void CSoccerPlayer::SetDoGoalKick(const bool new_do_goal_kick)
{
  do_goal_kick = new_do_goal_kick;
}

inline void CSoccerPlayer::SetDoKickOff(const bool new_do_kick_off)
{
  do_kick_off = new_do_kick_off;
}
  
inline bool CSoccerPlayer::DoCornerKickLeft() const
{
  return do_corner_kick_left;
}

inline bool CSoccerPlayer::DoCornerKickRight() const
{
  return do_corner_kick_right;
}

inline bool CSoccerPlayer::DoGoalKick() const
{
  return do_goal_kick;
}

inline bool CSoccerPlayer::DoFreeKick(CPoint pt) const
{
  return GetFreeKickArea().Inside(pt);
}

inline bool CSoccerPlayer::DoKickOff() const
{
  return do_kick_off;
}

#endif
