#include "CSocket.h"

CSocket::CSocket(const int_type maxd, const t_sock_type s_type)
  : addr()
{
  sockfd = -1;
  port = -1;
  sock_type = s_type;
  maxdata = maxd;
};

CSocket::CSocket(const std::string& address, const int_type maxd,
		 const t_sock_type s_type)
  : addr(address)
{
  sockfd = -1;
  port = -1;
  sock_type = s_type;
  maxdata = maxd;
};

CSocket::CSocket(const std::string& address, const int_type p,
		 const int_type maxd, const t_sock_type s_type)
  : addr(address)
{
  port = p;
  sockfd = -1;
  sock_type = s_type;
  maxdata = maxd;
};

CSocket::~CSocket()
{
  if (sockfd != -1)
    Close();
}

t_sock_type CSocket::SetType(const t_sock_type new_type)
{
  t_sock_type old_type = sock_type;
  sock_type = new_type;
  return old_type;
}

int_type CSocket::SetFD(const int_type fd)
{
  int_type oldfd = sockfd;
  sockfd = fd;
  return oldfd;
};

int_type CSocket::SetFlags(const int_type flags)
{
  int_type fl = fcntl(sockfd, F_GETFL, 0);
  if (fcntl(sockfd, F_SETFL, flags) < 0)
    return SOCK_ERROR;
  else
    return fl;
}

int_type CSocket::SetOwner(const int_type owner)
{
  int_type old_owner = fcntl(sockfd, F_GETOWN, 0);
  if (fcntl(sockfd, F_SETOWN, owner) < 0)
    return SOCK_ERROR;
  else
    return old_owner;
}

int_type CSocket::SetMaxData(const int_type mdata)
{
  int_type oldmd = maxdata;
  maxdata = mdata;
  return oldmd;
};

t_sock_result CSocket::Open()
{

  if (addr == "" || port == -1)
    return SOCK_ERROR;
  else
    return Open(addr, port);
  
}

t_sock_result CSocket::Open(const int_type p)
{

  port = p;
  if (addr == "")
    return SOCK_ERROR;
  else
    return Open(addr, port);
  
}

t_sock_result CSocket::Open(const std::string& host, const int_type new_port)
{

  addr = host;
  port = new_port;
  
  if ((host_ent = (struct Fix::hostent *)Fix::gethostbyname(addr.c_str())) == NULL) {
    /* Check if numeric address */
    if ((int_type)Fix::inet_addr(addr.c_str()) == -1) {
      perror("CSocket [open]");
      return SOCK_ERROR;
    }
  } else {
    addr_ptr = (struct Fix::in_addr *)*host_ent->h_addr_list;
    addr = Fix::inet_ntoa(*addr_ptr);
  }

  switch (sock_type) {
  case SOCK_TYPE_TCP :
    if ((sockfd = Fix::socket(AF_INET, SOCK_STREAM, 0)) < 0) {
      perror("CSocket [open] TCP");
      return SOCK_ERROR;
    };
    break;
  case SOCK_TYPE_UDP :
    if ((sockfd = Fix::socket(AF_INET, SOCK_DGRAM, 0)) < 0) {
      perror("CSocket [open] UDP");
      return SOCK_ERROR;
    };
    break;
  default:
    perror("CSocket [open] socket type not set!");
    return SOCK_ERROR;
  };

  /*
   *  Bind any local address.
   */
  memset((char *) &client_addr, 0, sizeof(client_addr));
  client_addr.sin_family      = AF_INET;
  // since INADDR_ANY is an ugly macro I can't just add Fix::
  // since then the macro will expand into something unknown
  // for the compiler, since that's in Fix!
  using namespace Fix;
  client_addr.sin_addr.s_addr = htonl(INADDR_ANY);
  client_addr.sin_port	      = htons(0);

  if (Fix::bind(sockfd, (struct Fix::sockaddr *) &client_addr,
	   sizeof(client_addr)) < 0) {
    perror("CSocket [open] bind");
    return SOCK_ERROR;  /* Can't bind local address */
  }

  /*
   *  Fill in the address of the server.
   */
  memset((char *) &server_addr, 0, sizeof(server_addr));
  server_addr.sin_family      = AF_INET;
  server_addr.sin_addr.s_addr = Fix::inet_addr(addr.c_str());
  server_addr.sin_port	      = htons(port);
  
  return SOCK_OK;
};  

t_sock_result CSocket::Close()
{
  if (close(sockfd) < 0)
    {
      perror("CSocket [close]");
      return SOCK_ERROR;
    };

  sockfd = -1;
  
  return SOCK_OK;
}

t_sock_result CSocket::WriteData(const std::string& data, const int_type size)
{

  if (data == "")
    return SOCK_NODATA;

  //  b = write(sockfd, data, size);  // Old code, don't really know the difference!
 
  int_type b = Fix::sendto(sockfd, data.c_str(), size, 0,
		 (struct Fix::sockaddr *)&server_addr, sizeof(server_addr));

  if (b < 0) {
    perror("CSocket [write]");
    return SOCK_ERROR;
  } else if ( b != size ) {
    perror("CSocket [write] not all of the data came through!");
    return SOCK_ERROR;
  };
  
  return SOCK_OK;
}

t_sock_result CSocket::ReadData(char* data, int_type* size)
{

  /* OLD CODE 
  
  if ((*size = read(sockfd, data, maxdata)) < 1)
    {
      if ((errno == EWOULDBLOCK) || (*size==0))
	{
	  *size=0;
	  return SOCK_NODATA;
	}
      else
	{
	  perror("CSocket [read]");
	  return SOCK_ERROR;
	};
    };
  */

  int_type servlen;
  struct Fix::sockaddr_in serv_addr;
  
  servlen = sizeof(serv_addr);
  *size = Fix::recvfrom(sockfd, data, maxdata, 0,
		   (struct Fix::sockaddr *)&serv_addr, &servlen);

  if (*size < 0)
    if ( *size == -1 && errno == EWOULDBLOCK) {
      data[0] = '\0';
      return SOCK_NODATA;
    } else {
      //      cout << "[Socket] EINTR = " << EINTR << " Errorcode = " << errno << endl;
      return SOCK_ERROR;
    }
  else {
    data[*size] = '\0';

    server_addr.sin_port = serv_addr.sin_port;

    if (*size == 0)
      return SOCK_NODATA;
    else
      return SOCK_OK;
  }
}

CSocket& CSocket::operator=(const CSocket& tmp)
{
  if ( this != &tmp ) {
    sockfd      = tmp.sockfd;
    port        = tmp.port;
    host_ent    = tmp.host_ent;
    addr_ptr    = tmp.addr_ptr;
    client_addr = tmp.client_addr;
    server_addr = tmp.server_addr;
    sock_type   = tmp.sock_type;
    maxdata     = tmp.maxdata;
    addr = tmp.addr;
  }

  return *this;
}


