#ifndef _CSOCKET_H_
#define _CSOCKET_H_

#include <unistd.h>
namespace Fix {
#include <sys/socket.h>
#include <sys/types.h>
#include <netinet/in.h>
#include <inttypes.h>
#include <arpa/inet.h>
#include <netdb.h>
}
#include <fcntl.h>
#include <stdio.h>
#include <string>
#include <errno.h>
//#include <iostream>
#include "types.h"

enum t_sock_result{ SOCK_OK, SOCK_ERROR, SOCK_NODATA };
enum t_sock_type{ SOCK_TYPE_UNKOWN, SOCK_TYPE_TCP, SOCK_TYPE_UDP };
static const int_type MAXDATA_DEFAULT = 256; //The default maximum size of a message

class CSocket
{
  
protected:
  // Attributes:
  std::string addr;
  int_type sockfd;
  int_type port;
  int_type maxdata;
  struct Fix::hostent *host_ent;
  struct Fix::in_addr *addr_ptr;
  struct Fix::sockaddr_in client_addr;
  struct Fix::sockaddr_in server_addr;
  t_sock_type sock_type;

public:
  // Constructors / destructors
  CSocket(const int_type = MAXDATA_DEFAULT, const t_sock_type = SOCK_TYPE_TCP);
  CSocket(const std::string&, const int_type = MAXDATA_DEFAULT,
	  const t_sock_type = SOCK_TYPE_TCP);
  CSocket(const std::string&, const int_type port, const int_type = MAXDATA_DEFAULT,
	  const t_sock_type = SOCK_TYPE_TCP);
  ~CSocket();

  CSocket& operator=(const CSocket&);

  // Methods
  t_sock_type GetType() const;
  t_sock_type SetType(const t_sock_type);

  int_type GetFD() const; 
  int_type SetFD(const int_type);

  int_type GetFlags() const;
  int_type SetFlags(const int_type);

  int_type GetOwner() const;
  int_type SetOwner(const int_type);
  
  int_type GetMaxData() const; 
  int_type SetMaxData(const int_type);

  t_sock_result Open();
  t_sock_result Open(const int_type);
  t_sock_result Open(const std::string&, const int_type);
  t_sock_result Close();

  t_sock_result WriteData(const std::string&, const int_type);
  t_sock_result ReadData(char*, int_type*);
  
};

inline t_sock_type CSocket::GetType() const
{
  return sock_type;
}

inline int_type CSocket::GetFD() const
{
  return sockfd;
}

inline int_type CSocket::GetFlags() const
{
  return fcntl(sockfd, F_GETFL, 0);
}

inline int_type CSocket::GetOwner() const
{
  return fcntl(sockfd, F_GETOWN, 0);
}

inline int_type CSocket::GetMaxData() const
{
  return maxdata;
}

#endif
