#ifndef _CTHEGAMEOBJECT_H_
#define _CTHEGAMEOBJECT_H_

#include "soc_types.h"
#include <string>
#include "CTime.h"

class CTheGameObject
{
public:
  CTheGameObject();
  CTheGameObject(const CTheGameObject& obj);
  // ~CTheGameObject();

  CTheGameObject& operator= (const CTheGameObject& obj);
  
  void ResetAll();
  void Reset();

  void SetOurTeamName(const std::string& new_name);
  const std::string GetOurTeamName() const;
  int_type GetOurTeamNameLength() const;
  void SetTheirTeamName(const std::string& new_name);
  const std::string GetTheirTeamName() const;
  int_type GetTheirTeamNameLength() const;
  
  void SetOurSide(const t_side new_side);
  t_side GetOurSide() const;
  void SetTheirSide(const t_side new_side);
  t_side GetTheirSide() const;

  void SetOurScore(const int_type new_score);
  int_type GetOurScore() const;
  void SetTheirScore(const int_type new_score);
  int_type GetTheirScore() const;

  void SetCurrentTime(const CTime& new_time);
  CTime GetCurrentTime() const;

  void SetPlayMode(const t_play_mode new_play_mode);
  t_play_mode GetPlayMode() const;

  void SetKickOffMode(const t_kick_off_mode new_kick_off_mode);
  t_kick_off_mode GetKickOffMode() const;
  
protected:
  std::string our_team_name;
  std::string their_team_name;

  t_side our_side;
  t_side their_side;

  int_type our_score;
  int_type their_score;

  // This is a reference to the CTime::current_time
  // class variable. The reason is that I rather want
  // the user to use this class than CTime directly.
  CTime& current_time;

  t_play_mode play_mode;
  t_kick_off_mode kick_off_mode;

  static const std::string DEFAULT_OUR_TEAM_NAME;
  static const std::string DEFAULT_THEIR_TEAM_NAME;
  static const t_side DEFAULT_OUR_SIDE;
  static const t_side DEFAULT_THEIR_SIDE;
  static const t_play_mode DEFAULT_PLAY_MODE;
  static const t_kick_off_mode DEFAULT_KICK_OFF_MODE;
  
  void CopyData(const CTheGameObject&);
  
};

inline void CTheGameObject::SetOurTeamName(const std::string& new_name)
{
  our_team_name = new_name;
}

inline const std::string CTheGameObject::GetOurTeamName() const
{
  return our_team_name;
}

inline int_type CTheGameObject::GetOurTeamNameLength() const
{
  return our_team_name.length();
}

inline void CTheGameObject::SetTheirTeamName(const std::string& new_name)
{
  their_team_name = new_name;
}

inline const std::string CTheGameObject::GetTheirTeamName() const
{
  return their_team_name;
}

inline int_type CTheGameObject::GetTheirTeamNameLength() const
{
  return their_team_name.length();
}

inline t_side CTheGameObject::GetOurSide() const
{
  return our_side;
}

inline t_side CTheGameObject::GetTheirSide() const
{
  return their_side;
}

inline void CTheGameObject::SetOurScore(const int_type new_score)
{
  our_score = new_score;
}

inline int_type CTheGameObject::GetOurScore() const
{
  return our_score;
}

inline void CTheGameObject::SetTheirScore(const int_type new_score)
{
  their_score = new_score;
}

inline int_type CTheGameObject::GetTheirScore() const
{
  return their_score;
}

inline void CTheGameObject::SetCurrentTime(const CTime& new_time)
{
  current_time = new_time;
}

inline CTime CTheGameObject::GetCurrentTime() const
{
  return current_time;
}

inline void CTheGameObject::SetPlayMode(const t_play_mode new_play_mode)
{
  play_mode = new_play_mode;
}

inline t_play_mode CTheGameObject::GetPlayMode() const
{
  return play_mode;
}

inline void CTheGameObject::SetKickOffMode(const t_kick_off_mode new_kick_off_mode)
{
  kick_off_mode = new_kick_off_mode;
}

inline t_kick_off_mode CTheGameObject::GetKickOffMode() const
{
  return kick_off_mode;
}

#endif
