#include "CTime.h"

CTime CTime::current_time = CTime(0,0);
CTime CTime::last_stop_clock_time = CTime(0,0);
CTime CTime::second_last_stop_clock_time = CTime(0,0);

CTime::CTime(const int_type vt, const int_type vs)
  : t(vt),
    s(vs)
{
}

CTime::CTime(const CTime& obj)
  : t(obj.t),
    s(obj.s)
{
}

// What to do about the connection between CTime and
// the actual starting and stopping of the game?
// Does CTime have to depend on something else?
// Another alternative (from RoboLog) is to define current_time,
// last_stop_clock_time and second_last_stop_clock_time
// as global or class variables. Not pretty, but solves the problem...
// The current solution is that CTime is separate from the rest of
// the program, but that CBasicIP and CTheGameObject have references
// to the CTime class variables.
// The update of the time and tracking of the clock will be done in
// CBasicIP.

CTime CTime::operator-(const int_type a) const
{ 
  if ( s==0 && t-a > last_stop_clock_time.t ) // default case
    return CTime(t-a,0);

  if ( s>0 ){ 
    if ( a <= s ) 
      return CTime(t,s-a); // Just take off from stopped time 
    else{ // a > s 
      CTime new_time = CTime( t-(a-s),0 );  // take off from stopped time, then server time 
      /*
	if ( new_time < last_stop_clock_time ) 
	  my_error ("can't be sure of this subtraction (1)");
       */
      return new_time;
    }
  }
  else{ // t-a <= last_stop_clock_time.t 
    int_type stopped_time =  a - (t - last_stop_clock_time.t);  
    if ( stopped_time > last_stop_clock_time.s ) {
      if ( !last_stop_clock_time.t ) // if ==0, OK---account for players starting at different times 
	return CTime(0,0);
      int_type tmp = last_stop_clock_time.t - (stopped_time - last_stop_clock_time.s);
      /*
	if ( tmp <= second_last_stop_clock_time.t ) 
	  my_error("can't be sure of this subtraction (2) %d %d %d  %d",t,s,a,last_stop_clock_time.s);
       */
      return CTime( tmp, 0 );
    }
    return CTime( last_stop_clock_time.t, last_stop_clock_time.s - stopped_time );
  }
}

int CTime::operator -(const CTime& a) const
{
  if ( s==0 ){
    if ( a.t < last_stop_clock_time.t ){
      /*
	if ( a.t <= second_last_stop_clock_time.t )
	  my_error("Can't be sure of this subtraction (3) %d %d  -  %d %d",t,s,a.t,a.s);
       */
      return (t - a.t) + last_stop_clock_time.s;
    }
    if ( a.t > last_stop_clock_time.t )
      return (t - a.t);
    if ( a.t == last_stop_clock_time.t )
      return (t - a.t) + (last_stop_clock_time.s - a.s);
  }
  else if ( s > 0 ){
    /*
      if ( a.t <= second_last_stop_clock_time.t )   // If they're equal, it's OK 
        my_error("Can't be sure of this subtraction (4) %d %d  %d %d",t,s,a.t,a.s);
     */
    if ( a.t < last_stop_clock_time.t )
      return last_stop_clock_time.s + (t - a.t);
    else if ( a.t == last_stop_clock_time.t && a.t != t ) // a is during the last stopped interval 
      return ( s + ( last_stop_clock_time.s - a.s ) ) + (t - a.t);
    return (s - a.s) + (t - a.t);
  }
  /*
    else // s<0 
      my_error("s shouldn't be <0");
   */
  return 0;
}

CTime CTime::operator +(const int_type a) const
{
  
  if ( s==0 && t > last_stop_clock_time.t && t+a < current_time.t ) // default case 
    return CTime(t+a,s);

  CTime new_time;
  
  if ( s==0 ){
    if ( last_stop_clock_time.t > t ){ // Could've missed one already 
      // my_error("Can't be sure of this addition (1) %d %d",last_stop_clock_time.t,t);
      new_time = CTime(t+a,0);
    }
    if ( t+a < current_time.t )
      new_time = CTime( t+a,0 );
    else  // t+a >= current_time.t 
      new_time = CTime( current_time.t, a-(current_time.t-t) );
  }
  else if ( s>0 ){
    if ( t == current_time.t ) // clock still stopped 
      new_time = CTime( t,s+a );
    else{
      /*
	if ( last_stop_clock_time.t != t ) // Could've missed one already 
	  my_error("Can't be sure of this addition (2)");
       */
      new_time = CTime ( t+(a-(last_stop_clock_time.s - s)),0 );
    }
  }
  /*  else // s<0 
     my_error("s shouldn't be <0");
   */
  /*
    if ( new_time > current_time ) // clock might stop 
      my_error("Can't be sure of this addition (3)");
   */
  return new_time;
}

CTime& CTime::operator =(const int_type a)
{
  t = a;
  s = 0;
  return *this;
}

CTime& CTime::operator =(const CTime& a)
{
  if ( this != &a ) {
    t = a.t;
    s = a.s;
  }
  return *this;
}

std::istream& operator>>(std::istream& is, CTime& ct)
{
  /* Input formats: i is an int_typeeger
        i     = (i,0)
	(i)   = (i,0)
	(i,i) = (i,i)
  */

  int_type t = 0;
  int_type s = 0;
  char c = 0;

  is >> c;
  if (c == '(') {
    is >> t >> c;
    if (c == ',') is >> s >> c;
    if (c != ')') is.clear(std::ios::badbit); //error...;
  }
  else {
    is.putback(c);
    is >> t;
  }

  if (is) ct = CTime(t, s);

  return is;
}

