// This class is based on CMUnited's implementation created by P. Stone & Co

#ifndef _CTIME_H_
#define _CTIME_H_

#include "types.h"
#include <iostream>

class CTime {
public:

  CTime(const int_type vt = 0, const int_type vs = 0);
  CTime(const CTime& obj);
  
  CTime    operator - (const int_type) const;
  int_type operator - (const CTime&)   const;
  CTime    operator + (const int_type) const;
  CTime&   operator -=(const int_type);
  CTime&   operator -=(const CTime&);
  CTime&   operator +=(const int_type);
  CTime&   operator ++();
  CTime&   operator --();
  CTime&   operator  =(const int_type);
  CTime&   operator  =(const CTime&);
  bool     operator ==(const CTime&)   const;
  bool     operator ==(const int_type) const;
  bool     operator !=(const CTime&)   const;
  bool     operator !=(const int_type) const;
  bool     operator < (const CTime&)   const;
  bool     operator < (const int_type) const;
  bool     operator <=(const CTime&)   const;
  bool     operator <=(const int_type) const;
  bool     operator > (const CTime&)   const;
  bool     operator > (const int_type) const;
  bool     operator >=(const CTime&)   const;
  bool     operator >=(const int_type) const;
  bool     operator ! ()               const;
  
  friend std::ostream& operator<<(std::ostream&, const CTime&);
  friend std::istream& operator>>(std::istream&, CTime&);

  int_type  GetS() const;
  CTime& SetS(const int_type);
  int_type  GetT() const;
  CTime& SetT(const int_type);

  static CTime current_time;
  static CTime last_stop_clock_time;
  static CTime second_last_stop_clock_time;
  
private:
  int_type t; // time from the server, this is the game time
  int_type s; // stopped clock cycles, this is how many server cycles
              // the clock has been stopped

};

static const CTime UNKNOWN_TIME(UNKNOWN_INT_TYPE, UNKNOWN_INT_TYPE);

inline CTime& CTime::operator -=(const int_type a)
{
  return *this = *this - a;
}

inline CTime& CTime::operator -=(const CTime& a)
{
  return *this = *this - a;
}

inline CTime& CTime::operator +=(const int_type a)
{
  return *this = *this + a;
}

inline CTime& CTime::operator ++()
{
  return *this += 1;
}

inline CTime& CTime::operator --()
{
  return *this -= 1;
}

inline bool CTime::operator ==(const CTime& a) const
{
  return (s == a.s) && (t == a.t);
}

inline bool CTime::operator ==(const int_type a) const
{
  return t == a;
}

inline bool CTime::operator !=(const CTime& a) const
{
  return (s != a.s) || (t != a.t);
}

inline bool CTime::operator !=(const int_type a) const
{
  return t != a;
}

inline bool CTime::operator < (const CTime& a) const
{
  return ( t < a.t ) || ( t == a.t && s < a.s );
}

inline bool CTime::operator < (const int_type a) const
{
  return t < a;
}

inline bool CTime::operator <=(const CTime& a) const
{
  return ( t < a.t ) || ( t == a.t && s <= a.s );
}

inline bool CTime::operator <=(const int_type a) const
{
  return t <= a;
}

inline bool CTime::operator > (const CTime& a) const
{
  return ( t > a.t ) || ( t == a.t && s > a.s );
}

inline bool CTime::operator > (const int_type a) const
{
  return t > a;
}

inline bool CTime::operator >=(const CTime& a) const
{
  return ( t > a.t ) || ( t == a.t && s >= a.s );
}

inline bool CTime::operator >=(const int_type a) const
{
  return t >= a;
}

inline bool CTime::operator !() const
{
  return (s == 0) && (t == 0);
}
  
inline std::ostream& operator<<(std::ostream&  os, const CTime& ct)
{
  return os << "(" << ct.t << ", " << ct.s << ")";
};


inline int_type CTime::GetS() const
{
  return s;
}

inline CTime& CTime::SetS(const int_type i)
{
  s = i;
  return *this;
}

inline int_type CTime::GetT() const
{
  return t;
}

inline CTime& CTime::SetT(const int_type i)
{
  t = i;
  return *this;
}

#endif
