#include "CTurnCommand.h"
#include "CKnowledge.h"
#include "utils.h"

CTurnCommand::CTurnCommand(CKnowledge* const kb, const CAngleDeg& ang,
			   const t_priority prio,
			   const CTime& s_time, const CTime& f_time)
  : CCommand(kb, CMD_Turn, prio, s_time, f_time),
    turn_type(TURN_Angle),
    angle(ang)
{
}

CTurnCommand::CTurnCommand(CKnowledge* const kb, const CPoint& dest,
			   const t_priority prio,
			   const CTime& s_time, const CTime& f_time)
  : CCommand(kb, CMD_Turn, prio, s_time, f_time),
    turn_type(TURN_Dest),
    destination(dest)
{
}

CTurnCommand::CTurnCommand(CKnowledge* const kb, const CVector& dir,
			   const t_priority prio,
			   const CTime& s_time, const CTime& f_time)
  : CCommand(kb, CMD_Turn, prio, s_time, f_time),
    turn_type(TURN_Dir),
    direction(dir)
{
}

CTurnCommand::CTurnCommand(const CTurnCommand& obj)
  : CCommand(obj),
    turn_type(obj.turn_type),
    angle(obj.angle),
    destination(obj.destination),
    direction(obj.direction)
{
}

CTurnCommand::~CTurnCommand()
{
}

void CTurnCommand::CopyCommand(const CTurnCommand& obj)
{
  CCommand::CopyCommand(obj);
  turn_type = obj.turn_type;
  angle = obj.angle;
  destination = obj.destination;
  direction = obj.direction;
}

CTurnCommand& CTurnCommand::operator =(const CTurnCommand& obj)
{
  if ( this != &obj ) {
    CopyCommand(obj);
  }

  return *this;
}

void CTurnCommand::TurnAngle(const CAngleDeg& ang)
{
  turn_type = TURN_Angle;
  angle = ang;
}

void CTurnCommand::TurnDestination(const CPoint& dest)
{
  turn_type = TURN_Dest;
  destination = dest;
}

void CTurnCommand::TurnDirection(const CVector& dir)
{
  turn_type = TURN_Dir;
  direction = dir;
}

void CTurnCommand::UpdateAfterCommand(const int_type from,
				      const int_type to) const
{
  CAngleDeg actual_angle = angle / (1.0 + k->GetParameters()->SP_inertia_moment
				          * k->theAgent().GetSpeed(from));
  //std::cout << "Angle: " << angle << " actual_angle: " << actual_angle
  //   << " BodyDir: " << k->theAgent().GetBodyDir(from) << std::endl;
  k->theAgent().SetBodyDir(k->theAgent().GetBodyDir(from) + actual_angle, to);
  k->theAgent().SetFaceDir(k->theAgent().GetFaceDir(from) + actual_angle, to);
}

bool CTurnCommand::Validate(const int_type hist)
{
  switch(turn_type) {
  case TURN_Angle : // Turn with angle already set
    break;
  case TURN_Dest : // Turn with an intended destination
    TurnDestination(hist);
    break;
  case TURN_Dir : // Turn with an intended direction
    TurnDirection(hist);
    break;
  default : // Error, what should I do??
    std::cout << "Unknown turn_type in TurnCommand!\n";
    return false;
  }

  //std::cout << "Angle: " << angle << " Speed: " << k->theAgent().GetSpeed(hist);
  angle *= 1.0 + k->GetParameters()->SP_inertia_moment * k->theAgent().GetSpeed(hist);
  angle = MinMax(k->GetParameters()->SP_minmoment, angle,
		 k->GetParameters()->SP_maxmoment);

  if ( Abs(angle) < 0.15 )
    angle = 0;

  //std::cout << " actual_angle: " << angle << std::endl;
  return true;
}

void CTurnCommand::TurnDestination(const int_type hist)
{
  angle = k->theAgent().GetRelativeDirectionTo(destination, hist);
}

void CTurnCommand::TurnDirection(const int_type hist)
{
  angle = direction.GetAngle() - k->theAgent().GetBodyDir(hist);
}

const std::string CTurnCommand::ServerString() const
{
  return "(turn " + ToString(angle) + ")";
}
