#ifndef _CTURNCOMMAND_H_
#define _CTURNCOMMAND_H_

#include "CCommand.h"
#include "CAngleDeg.h"
#include "CPoint.h"
#include "CVector.h"

enum t_turn_type {
  TURN_Unknown,
  TURN_Angle,
  TURN_Dest,
  TURN_Dir
};

class CTurnCommand : public CCommand
{
public:
  CTurnCommand(CKnowledge* const kb, const CAngleDeg& ang = 0,
	       const t_priority prio = PRIO_NORMAL,
	       const CTime& s_time = 0, const CTime& f_time = 0);
  CTurnCommand(CKnowledge* const kb, const CPoint&,
	       const t_priority prio = PRIO_NORMAL,
	       const CTime& s_time = 0, const CTime& f_time = 0);
  CTurnCommand(CKnowledge* const kb, const CVector&,
	       const t_priority prio = PRIO_NORMAL,
	       const CTime& s_time = 0, const CTime& f_time = 0);
  CTurnCommand(const CTurnCommand& obj);
  virtual ~CTurnCommand();

  CTurnCommand& operator =(const CTurnCommand& obj);
  
  virtual const CTurnCommand* const GetTurnCommand() const;
  virtual CTurnCommand* GetTurnCommand();
  
  void TurnAngle(const CAngleDeg& angle);
  void TurnDestination(const CPoint& dest);
  void TurnDirection(const CVector& dir);
  
  t_turn_type GetTurnType() const;
  void SetTurnType(const t_turn_type);  
  
  CAngleDeg GetAngle() const;
  void SetAngle(const CAngleDeg& angle);

  CPoint GetDestination() const;
  void SetDestination(const CPoint& destination);

  CVector GetDirection() const;
  void SetDirection(const CVector& direction);

  virtual void UpdateAfterCommand(const int_type from, const int_type to) const;
  virtual bool Validate(const int_type hist);

  virtual const std::string ServerString() const;
  
private:
  t_turn_type turn_type;
  CAngleDeg angle;
  CPoint destination;
  CVector direction;

  void TurnDestination(const int_type hist);
  void TurnDirection(const int_type hist);

  void CopyCommand(const CTurnCommand& obj);
  
};
  
inline const CTurnCommand* const CTurnCommand::GetTurnCommand() const
{
  return this;
}

inline CTurnCommand* CTurnCommand::GetTurnCommand()
{
  return this;
}

inline t_turn_type CTurnCommand::GetTurnType() const
{
  return turn_type;
}
  
inline CAngleDeg CTurnCommand::GetAngle() const
{
  return angle;
}

inline void CTurnCommand::SetAngle(const CAngleDeg& new_angle)
{
  angle = new_angle;
}

inline CPoint CTurnCommand::GetDestination() const
{
  return destination;
}

inline void CTurnCommand::SetDestination(const CPoint& new_destination)
{
  destination = new_destination;
}

inline CVector CTurnCommand::GetDirection() const
{
  return direction;
}

inline void CTurnCommand::SetDirection(const CVector& new_direction)
{
  direction = new_direction;
}

#endif
