#include "CTurnNeckCommand.h"
#include "CKnowledge.h"
#include "utils.h"

CTurnNeckCommand::CTurnNeckCommand(CKnowledge* const kb, const CAngleDeg& ang,
				   const t_priority prio,
				   const CTime& s_time, const CTime& f_time)
  : CCommand(kb, CMD_TurnNeck, prio, s_time, f_time),
    turn_neck_type(TURN_NECK_Angle),
    angle(ang)
{
}

CTurnNeckCommand::CTurnNeckCommand(const CTurnNeckCommand& obj)
  : CCommand(obj),
    turn_neck_type(obj.turn_neck_type),
    angle(obj.angle),
    destination(obj.destination),
    direction(obj.direction)
{
}

CTurnNeckCommand::~CTurnNeckCommand()
{
}

void CTurnNeckCommand::CopyCommand(const CTurnNeckCommand& obj)
{
  CCommand::CopyCommand(obj);
  turn_neck_type = obj.turn_neck_type;
  angle = obj.angle;
  destination = obj.destination;
  direction = obj.direction;
}

CTurnNeckCommand& CTurnNeckCommand::operator =(const CTurnNeckCommand& obj)
{
  if ( this != &obj ) {
    CopyCommand(obj);
  }

  return *this;
}

void CTurnNeckCommand::TurnNeckAngle(const CAngleDeg& ang)
{
  turn_neck_type = TURN_NECK_Angle;
  angle = ang;
}

void CTurnNeckCommand::TurnNeckDestination(const CPoint& dest)
{
  turn_neck_type = TURN_NECK_Dest;
  destination = dest;
}

void CTurnNeckCommand::TurnNeckDirection(const CVector& dir)
{
  turn_neck_type = TURN_NECK_Dir;
  direction = dir;
}

void CTurnNeckCommand::UpdateAfterCommand(const int_type from,
					  const int_type to) const
{
  CAngleDeg new_neck_dir = MinMax(k->GetParameters()->SP_minneckang,
				  k->theAgent().GetNeckDir(from) + angle,
				  k->GetParameters()->SP_maxneckang);
  k->theAgent().SetNeckDir(new_neck_dir, to);
  k->theAgent().SetFaceDir(k->theAgent().GetBodyDir(from) + new_neck_dir, to);
}

bool CTurnNeckCommand::Validate(const int_type hist)
{
  switch(turn_neck_type) {
  case TURN_NECK_Angle : // Turn with angle already set
    break;
  case TURN_NECK_Dest : // Turn with an intended destination
    TurnNeckDestination(hist);
    break;
  case TURN_NECK_Dir : // Turn with an intended direction
    TurnNeckDirection(hist);
    break;
  default : // Error, what should I do??
    std::cout << "Unknown turn_neck_type in TurnNeckCommand!\n";
    return false;
  }

  CAngleDeg max_turn_neck = MinMax(k->GetParameters()->SP_minneckang,
				   k->theAgent().GetNeckDir(hist) + angle,
				   k->GetParameters()->SP_maxneckang);
  angle = MinMax(k->GetParameters()->SP_minneckmoment,
		 max_turn_neck - k->theAgent().GetNeckDir(hist),
		 k->GetParameters()->SP_maxneckmoment);
  return true;
}

void CTurnNeckCommand::TurnNeckDestination(const int_type hist)
{
  angle = k->theAgent().GetRelativeDirectionTo(destination, hist+1);
}

void CTurnNeckCommand::TurnNeckDirection(const int_type hist)
{
  angle = direction.GetAngle() - k->theAgent().GetFaceDir(hist+1);
}

const std::string CTurnNeckCommand::ServerString() const
{
  return "(turn_neck " + ToString(angle) + ")";
}
