#ifndef _CTURNNECKCOMMAND_H_
#define _CTURNNECKCOMMAND_H_

#include "CCommand.h"
#include "CAngleDeg.h"
#include "CPoint.h"
#include "CVector.h"

// The only place I know k can be set to NULL is in CAgentObject and
// DEFAULT_NEXT_TURN_NECK.
// If you get segfault that's a probable cause!

enum t_turn_neck_type {
  TURN_NECK_Unknown,
  TURN_NECK_Angle,
  TURN_NECK_Dest,
  TURN_NECK_Dir
};

class CTurnNeckCommand : public CCommand
{
public:
  CTurnNeckCommand(CKnowledge* const kb, const CAngleDeg& ang = 0,
		   const t_priority prio = PRIO_NORMAL,
		   const CTime& s_time = 0, const CTime& f_time = 0);
  CTurnNeckCommand(const CTurnNeckCommand& obj);
  virtual ~CTurnNeckCommand();

  CTurnNeckCommand& operator =(const CTurnNeckCommand& obj);
  
  virtual const CTurnNeckCommand* const GetTurnNeckCommand() const;
  virtual CTurnNeckCommand* GetTurnNeckCommand();
  
  void TurnNeckAngle(const CAngleDeg& angle);
  void TurnNeckDestination(const CPoint& dest);
  void TurnNeckDirection(const CVector& dir);
  
  t_turn_neck_type GetTurnNeckType() const;
  void SetTurnNeckType(const t_turn_neck_type);  
  
  CAngleDeg GetAngle() const;
  void SetAngle(const CAngleDeg& angle);

  CPoint GetDestination() const;
  void SetDestination(const CPoint& destination);

  CVector GetDirection() const;
  void SetDirection(const CVector& direction);

  virtual void UpdateAfterCommand(const int_type from, const int_type to) const;
  virtual bool Validate(const int_type hist); 

  virtual const std::string ServerString() const;
  
private:
  t_turn_neck_type turn_neck_type;
  CAngleDeg angle;
  CPoint destination;
  CVector direction;

  void TurnNeckDestination(const int_type hist);
  void TurnNeckDirection(const int_type hist);

  void CopyCommand(const CTurnNeckCommand& obj);
  
};

inline const CTurnNeckCommand* const CTurnNeckCommand::GetTurnNeckCommand() const
{
  return this;
}

inline CTurnNeckCommand* CTurnNeckCommand::GetTurnNeckCommand()
{
  return this;
}

inline t_turn_neck_type CTurnNeckCommand::GetTurnNeckType() const
{
  return turn_neck_type;
}
  
inline CAngleDeg CTurnNeckCommand::GetAngle() const
{
  return angle;
}

inline void CTurnNeckCommand::SetAngle(const CAngleDeg& new_angle)
{
  angle = new_angle;
}

inline CPoint CTurnNeckCommand::GetDestination() const
{
  return destination;
}

inline void CTurnNeckCommand::SetDestination(const CPoint& new_destination)
{
  destination = new_destination;
}

inline CVector CTurnNeckCommand::GetDirection() const
{
  return direction;
}

inline void CTurnNeckCommand::SetDirection(const CVector& new_direction)
{
  direction = new_direction;
}

#endif
