#include "CVector.h"

CVector::CVector(const CAngleDeg& vdir, const float_type vdist)
  : dir(vdir),
    dist(vdist)
{
  if (dist < 0) {
    dir -= 180;
    dist = -dist;
  }
}

CVector::CVector(const CVector& v)
  : dir(v.dir),
    dist(v.dist)
{
}

CVector::CVector(const CPoint& pt1, const CPoint& pt2)
{
  if (pt1 == pt2) {
    dir = 0;
    dist = 0;
  } else {
    dir = ATan2((pt1.GetY()-pt2.GetY()), (pt1.GetX()-pt2.GetX())) - 180;
    dist = sqrt(Sqr(pt1.GetX()-pt2.GetX()) + Sqr(pt1.GetY()-pt2.GetY()));
  }
}

CVector::CVector(const CPoint& pt1)
{
  *this = CVector(CPoint(), pt1);
}

CVector& CVector::operator =(const CVector& v)
{
  if (this != &v) {
    dist = v.dist;
    dir = v.dir;
  }

  return *this;
}

CVector CVector::operator +(const CVector& v) const
{
  return CVector(CPoint(v) + CPoint(*this));
}

CPoint CVector::operator+(const CPoint& pt1) const
{
  return CPoint(dist*Cos(dir), dist*Sin(dir)) + pt1;
}
 
void CVector::SetLength(const float_type len)
{
  dist = len;
}

void CVector::SetAngle(const CAngleDeg& ang)
{
  dir = ang;
}

std::istream& operator>>(std::istream& is, CVector& v)
{
  /* Input formats: f is a floating-point number
        f     = (0,f)
	(f)   = (0,f)
	(f,f) = (f,f)
  */

  float_type dist=0.0;
  CAngleDeg dir(0.0);
  char c = 0;

  is >> c;
  if (c == '(') {
    is >> dir >> c;
    if (c == ',') is >> dist >> c;
    if (c != ')') is.clear(std::ios::badbit); //error...;
  }
  else {
    is.putback(c);
    is >> dist;
  }

  if (dist == 0.0) {
    dist = float_type(dir);
    dir = 0.0;
  }
  
  if (is) v = CVector(dir, dist);

  return is;

}



