#ifndef _CVECTOR_H_
#define _CVECTOR_H_

class CVector;

#include "types.h"
#include "CAngleDeg.h"
#include "CPoint.h"
#include "MathUtils.h"
#include <iostream>

class CVector {
public:
  CVector(const CAngleDeg& = 0, const float_type = 0);
  CVector(const CVector&);
  CVector(const CPoint&);
  CVector(const CPoint&, const CPoint&);

  CVector  operator - ()                 const;
  CVector  operator - (const CVector&)   const;
  CVector  operator + (const CVector&)   const; 
  CPoint   operator + (const CPoint&)    const;
  CVector  operator * (const float_type) const;
  CVector  operator / (const float_type) const;
  CVector& operator  =(const CVector&);
  CVector& operator -=(const CVector&);
  CVector& operator +=(const CVector&);
  CVector& operator *=(const float_type);
  CVector& operator /=(const float_type);
  bool     operator ==(const CVector&)   const;
  bool     operator !=(const CVector&)   const;
  // These compares the length of the vectors
  bool     operator < (const CVector&)   const;
  bool     operator <=(const CVector&)   const;
  bool     operator > (const CVector&)   const;
  bool     operator >=(const CVector&)   const;
  // Is the vector the zero vector?
  bool     operator ! ()                 const;

  // Shoule I include these? Instead of the above operators?
  // bool SameLength
  // bool Longer
  // bool Shorter

  // Adds to the angle resp. to the distance of the vector.
  CVector& Rotate(const CAngleDeg&);
  // Bad name! I should have thought this was multiplication not addition.
  CVector& Scale (const float_type);

  float_type GetLength() const;
  void SetLength(const float_type);
  CAngleDeg GetAngle() const;
  void SetAngle(const CAngleDeg&);

  friend std::ostream& operator<<(std::ostream&, const CVector&);
  friend std::istream& operator>>(std::istream&, CVector&);

  friend CVector operator*(const float_type, const CVector&);

protected:
  CAngleDeg dir; /* The direction of the vector */
  float_type dist; /* The distance/length of the vector */

};

#if CHECK_DIVISION_BY_ZERO == 1
static const CVector CVECTOR_DIVISION_BY_ZERO_RESULT(FLOAT_TYPE_DIVISION_BY_ZERO_RESULT,
						     FLOAT_TYPE_DIVISION_BY_ZERO_RESULT);
#endif CHECK_DIVISION_BY_ZERO

static const CVector UNKNOWN_VECTOR(UNKNOWN_DEG, UNKNOWN_FLOAT_TYPE);

inline CVector CVector::operator -() const
{
  return CVector(dir - 180, dist);
}

inline CVector CVector::operator -(const CVector& v) const
{
  return *this+CVector(v.dir - 180, v.dist);
}

inline CVector CVector::operator *(const float_type a) const
{
  return CVector(dir, dist * a);
}

inline CVector CVector::operator /(const float_type a) const
{
#if CHECK_DIVISION_BY_ZERO == 1
  if (a == 0)
    return CVECTOR_DIVISION_BY_ZERO_RESULT;
  else
#endif
    return CVector(dir, dist / a);
}

inline CVector& CVector::operator-=(const CVector& v)
{
  return *this = *this - v;
}

inline CVector& CVector::operator+=(const CVector& v)
{
  return *this = *this + v;
}

inline CVector& CVector::operator*=(const float_type a)
{
  dist *= a;
  return *this;
}

inline CVector& CVector::operator/=(const float_type a)
{
#if CHECK_DIVISION_BY_ZERO == 1
  if (a == 0)
    dist = FLOAT_TYPE_DIVISION_BY_ZERO_RESULT;
  else
#endif
    dist /= a;
  
  return *this;
}

inline bool CVector::operator ==(const CVector& v) const
{
  return (dist == v.dist) && (dir == v.dir);
}

inline bool CVector::operator !=(const CVector& v) const
{
  return (dist != v.dist) || (dir != v.dir);
}

inline bool CVector::operator < (const CVector& v) const
{
  return (dist <  v.dist);
}

inline bool CVector::operator <=(const CVector& v) const
{
  return (dist <= v.dist);
}

inline bool CVector::operator > (const CVector& v) const
{
  return (dist >  v.dist);
}

inline bool CVector::operator >=(const CVector& v) const
{
  return (dist >= v.dist);
}

inline bool CVector::operator !() const
{
  return (dist != 0) && (dir != 0);
}

inline CVector& CVector::Rotate(const CAngleDeg& vdir)
{
  dir += vdir;
  return *this;
}

inline CVector& CVector::Scale(const float_type vdist)
{
  dist += vdist;
  return *this;
}

inline float_type CVector::GetLength() const
{
  return dist;
}

inline CAngleDeg CVector::GetAngle() const
{
  return dir;
}

inline std::ostream& operator<<(std::ostream& os, const CVector& v)
{
  return os << "(" << v.dir << ", " << v.dist << ")";
}

inline CVector operator*(const float_type a, const CVector& v)
{
  return CVector(v.dir, v.dist * a);
}

#endif
