#ifndef _MATHUTILS_H_
#define _MATHUTILS_H_

#include <cmath>
#include "types.h"

using std::sqrt;
using std::exp;
using std::log;
using std::sin;
using std::cos;
using std::tan;
using std::asin;
using std::acos;
using std::atan;
using std::atan2;

// This file contain some useful mathematical functions.
// Many of them are already implemented by C++.
// Some should be faster, like pow...

static const float_type m_pi = 3.14159265358979323846;
static const float_type m_float_type_eps = 0.0001;

static const float_type m_deg2rad = m_pi / 180.0;
static const float_type m_rad2deg = 180.0 / m_pi;

static const float_type m_two_pi = m_pi + m_pi;

inline float_type Deg2Rad(const float_type deg)
{ return deg * m_deg2rad; }

inline float_type Rad2Deg(const float_type rad)
{ return rad * m_rad2deg; }

inline float_type Max(const float_type d1, const float_type d2)
{ return d1 > d2 ? d1 : d2; }  

inline int_type Max(const int_type i1, const int_type i2)
{ return i1 > i2 ? i1 : i2; }  

inline float_type Min(const float_type d1, const float_type d2)
{ return d1 < d2 ? d1 : d2; }

inline int_type Min(const int_type i1, const int_type i2)
{ return i1 < i2 ? i1 : i2; }

inline float_type MinMax(const float_type min_val, const float_type val, const float_type max_val)
{ return Min(Max(min_val, val), max_val); }

inline int_type MinMax(const int_type min_val, const int_type val, const int_type max_val)
{ return Min(Max(min_val, val), max_val); }

inline float_type Sqr(const float_type d) 
{ return d * d; }

inline int_type Sqr(const int_type i)
{ return i * i; }

inline float_type Abs(const float_type d)
{ return ((d < 0) ? -d : d); }

inline int_type Abs(const int_type i)
{ return ((i < 0) ? -i : i); }

inline int_type Sign(const float_type d) 
{ return ((d < 0) ? -1 : 1); }

inline int_type Sign(const int_type i) 
{ return ((i < 0) ? -1 : 1); }

// Don't know if these are correct!
// Or needed...
inline float_type Mod(const float_type d1, const float_type d2)
{ return d1 - int_type(d1/d2)*d2; }
//{ float_type res; modf(d1/d2, &res); return res; }

inline int_type Mod(const int_type i1, const int_type i2)
{ return i1 % i2; }

inline int_type Div(const float_type d1, const float_type d2)
{ return int_type(d1/d2); }
// { div_t res = div(i1, i2); return res.quot; }

inline int_type Div(const int_type i1, const int_type i2)
{ return int_type(i1/i2); }

// This works but is not the most efficient
// Do a while loop and shift left if even otherwise decrease with one.
// Maybe not as efficient since we need to test for odd or even.
// without lowlevel tests, it's not as efficient...
int Power(const int_type x, const int_type y);

float_type Power(const float_type x, const int_type y);

inline float_type Power(const float_type x, const float_type y)
{ return exp(y * log(x)); }  

#endif
