#ifndef _SOC_TYPES_H_
#define _SOC_TYPES_H_

#include "types.h"
#include "constants.h"
#include "CAngleDeg.h"

typedef float_type t_speed;
typedef int_type t_unum;  // Uniform number
typedef float_type t_power;

// This is NOT a good way to handle unknown values!
static const t_speed UNKNOWN_SPEED = UNKNOWN_FLOAT_TYPE;
static const t_unum UNKNOWN_UNUM = 9996;
static const t_power UNKNOWN_POWER = 9995;

typedef struct t_kick_data {
  t_power power;
  CAngleDeg angle;
};

enum t_view_width {
  VW_Normal,
  VW_Wide,
  VW_Narrow
};

enum t_view_quality {
  VQ_High,
  VQ_Low
};

enum t_team {
  T_Our_Team,
  T_Their_Team,
  T_Unknown
};

enum t_sense{
  SENSE_See,
  SENSE_Hear,
  SENSE_Sense // SENSE_Body, SENSE_Feel?
};

// Needed?
enum t_object{
  OBJ_Line,
  OBJ_Ball,
  OBJ_Marker,
  OBJ_Marker_Behind,  // Not seen
  OBJ_Player
};

enum t_side_line{
  SL_Left,
  SL_Right,
  SL_Top,
  SL_Bottom,
  SL_Unknown,
  SL_No_Line // Should I not use Unknonw?
};

enum t_marker{
  Goal_L,
  Goal_R,

  Flag_C,
  Flag_CT,
  Flag_CB,
  Flag_LT,
  Flag_LB,
  Flag_RT,
  Flag_RB,

  Flag_PLT,
  Flag_PLC,
  Flag_PLB,
  Flag_PRT,
  Flag_PRC,
  Flag_PRB,

  Flag_GLT,
  Flag_GLB,
  Flag_GRT,
  Flag_GRB,

  Flag_TL50,
  Flag_TL40,
  Flag_TL30,
  Flag_TL20,
  Flag_TL10,
  Flag_T0,
  Flag_TR10,
  Flag_TR20,
  Flag_TR30,
  Flag_TR40,
  Flag_TR50,

  Flag_BL50,
  Flag_BL40,
  Flag_BL30,
  Flag_BL20,
  Flag_BL10,
  Flag_B0,
  Flag_BR10,
  Flag_BR20,
  Flag_BR30,
  Flag_BR40,
  Flag_BR50,

  Flag_LT30,
  Flag_LT20,
  Flag_LT10,
  Flag_L0,
  Flag_LB10,
  Flag_LB20,
  Flag_LB30,

  Flag_RT30,
  Flag_RT20,
  Flag_RT10,
  Flag_R0,
  Flag_RB10,
  Flag_RB20,
  Flag_RB30,

  No_Marker
};

typedef char t_side;
static const t_side S_Right = 'r';
static const t_side S_Left = 'l';
static const t_side S_Unknown = 'u';

enum t_kick_off_mode{
  KO_Ours,
  KO_Theirs,
  KO_Unknown
};

enum t_play_mode {
  PM_Unknown,
  PM_Before_Kick_Off,
  PM_Time_Over,
  PM_Play_On,
  PM_Drop_Ball,
  PM_Our_Offside_Kick,
  PM_Their_Offside_Kick,
  PM_Half_Time,
  PM_Time_Up,
  PM_Extended_Time,
  PM_Our_Kick_Off,
  PM_Their_Kick_Off,
  PM_Our_Kick_In,
  PM_Their_Kick_In,
  PM_Our_Free_Kick,
  PM_Their_Free_Kick,
  PM_Our_Corner_Kick,
  PM_Their_Corner_Kick,
  PM_Our_Goal_Kick,
  PM_Their_Goal_Kick,
  PM_Our_Goal,
  PM_Their_Goal,
  PM_Our_Goalie_Got_Ball,
  PM_Their_Goalie_Got_Ball
};

#endif
