#include "CServerIFace.h"
#include "CTurnCommand.h"
#include "CDashCommand.h"
#include "CKickCommand.h"
#include "CMoveCommand.h"
#include "CLogFile.h"

#include <iostream>

int main(int argc, char** argv)
{

  std::cout << "Nu brjar det..." << std::endl;
  CRSParameters p(argc, argv);
  CKnowledge    k(&p, CTheGameObject(), CAgentObject(&p), CBallObject(&p),
		  CSetofPlayersObject(&p, 22, false));
  CLogFile      logf(&k, "Test.log", false);
  CBasicIP      ip(&k, &logf);
  CCommandQueue command_queue;
  CServerIFace  server_interface(&ip, &command_queue);

  logf.Open();
  k.theGame().SetOurTeamName(p.RS_team_name);
  k.theAgent().SetConfidenceDecay(p.RS_confidence_decay_player);
  k.theAgent().SetLastActTime(CTime(0,0));
  k.theAgent().SetBodyDir(0, NOW);
  k.theAgent().SetFaceDir(0, NOW);
  k.theAgent().SetNeckDir(0, NOW);
  k.theAgent().SetAcceleration(CVector(0,0), NOW);
  k.theAgent().SetAbsoluteSpeedVector(CVector(0,0), NOW);
  k.theAgent().Copy(NOW, CURRENT_EST);
  k.theAgent().Copy(NOW, PREV);
  k.theAgent().Copy(NOW, NEXT);

  k.theBall().SetConfidenceDecay(p.RS_confidence_decay_ball);
  k.theBall().SetPosition(CPoint(0,0), NOW);
  k.theBall().SetAcceleration(CVector(0,0), NOW);
  k.theBall().SetAbsoluteSpeedVector(CVector(0,0), NOW);
  k.theBall().Copy(NOW, CURRENT_EST);
  k.theBall().Copy(NOW, PREV);
  k.theBall().Copy(NOW, NEXT);
  //k.theBall().Print();
  
  server_interface.Init("aste3.ida.liu.se", 6000);
  server_interface.Start();

  CCommand* cmd;
  bool moved = false;
  CVector vec;
  CAngleDeg ang;
  
  server_interface.WaitForNewCycle();
  while (server_interface.ServerAlive()) {
    if ( server_interface.WaitForNewCycle() ) {
      logf.LogText("new_cycle");
      logf.LogAgentLocation(CURRENT_EST);
      logf.LogBallLocation(CURRENT_EST);
      logf.LogAgentLocation(NEXT_EST);
      logf.LogBallLocation(NEXT_EST);
      command_queue.MakeEmpty();
      if ( k.theGame().GetCurrentTime().GetT() == 0 ) {
	if ( !moved ) {
	  cmd = new CMoveCommand(&k, CPoint(-10, -10));
	  moved = true;
	} else {
	  vec = k.theAgent().GetRelativeVectorTo(CPoint(0,0), NEXT_EST);
	  ang = vec.GetAngle();
	  cmd = new CTurnCommand(&k, ang);
	  //std::cout << "Vector to (0,0): " << vec
	  //    << "relative direction: " << ang << std::endl;	  
	}
	command_queue += cmd;
	delete cmd;
      } else {
	/*
	  std::cout << k.theGame().GetCurrentTime()
		<< " my_pos " << k.theAgent().GetPosition(NEXT_EST)
		<< " my_bodydir " << k.theAgent().GetBodyDir(NEXT_EST)
		<< " my_speed " << k.theAgent().GetAbsoluteSpeedVector(NEXT_EST)
		<< "\n" << k.theGame().GetCurrentTime()
		<< " ball_conf " << k.theBall().GetConfidence(NEXT_EST)
		<< " ball_pos: " << k.theBall().GetPosition(NEXT_EST)
		<< " ball_vector: " << k.theBall().GetVector(NEXT_EST)
		<< " ball_speed: " << k.theBall().GetAbsoluteSpeedVector(NEXT_EST)
		<< std::endl;
		*/
	if ( !moved ) {
	  cmd = new CMoveCommand(&k, CPoint(-10, -10));
	  moved = true;
	  //k.theAgent().SetDoTurnNeck(true);
	  //k.theAgent().SetNextTurnNeck(CTurnNeckCommand(k.theBall().GetDirection(NEXT_EST)));
	} else if ( k.theBall().GetConfidence(CURRENT_EST) < 0.6 )
	  cmd = new CTurnCommand(&k, CAngleDeg(30));
	else if ( Abs(k.theBall().GetDirection(CURRENT_EST)) > 4.0 ) {
	  cmd = new CTurnCommand(&k, k.theBall().GetDirection(NEXT_EST));
	  //command_queue += &CTurnCommand(k.theBall().GetDirection());
	} else if ( k.theBall().GetDistance(CURRENT_EST) > 4 ) {
	  cmd = new CDashCommand(&k, 50);
	} else if ( k.theBall().GetDistance(CURRENT_EST) > p.SP_kickable_area*0.9 ) {
	  cmd = new CDashCommand(&k, k.theBall().GetPosition(NEXT_EST) - p.SP_player_size);
	  //k.theAgent().SetDoTurnNeck(true);
	  //k.theAgent().SetNextTurnNeck(CTurnNeckCommand(k.theBall().GetDirection(NEXT_EST)));
	  // command_queue += &CDashCommand(50);
	} else {
	  // This is not 100% correct, it should be the direction from the ball
	  // to the goal!
	  //cmd = new CKickCommand(&k, 100, k.theirGoal.GetDirection(CURRENT_EST));
	  //cmd = new CKickCommand(&k, 100, k.theirGoal.GetDirection(CURRENT_EST) -
	  //		 k.theBall().GetDirection(CURRENT_EST));
	  //	  cout << "TheirGoal pos: " << k.theirGoal.GetPosition(CURRENT_EST)
	  //	       << std::endl;
	  cmd = new CKickCommand(&k, k.theirGoal.GetPosition());
	  // command_queue += &CKickCommand(100, k.theirGoal.GetDirection());
	  //k.theAgent().SetDoTurnNeck(true);
	  //k.theAgent().SetNextTurnNeck(CTurnNeckCommand(k.theBall().GetDirection(NEXT_EST)));
	}
	command_queue += cmd;
	delete cmd;
      }
    } else
      std::cout << "New cycle, failed!" << std::endl;
    
    /*
      if ( server_interface.WaitForNewInformation() )
      std::cout << "New info!" << std::endl;
      else
      std::cout << "New info, failed!" << std::endl;
      */
    /*
      if ( server_interface.NewInformation() )
      std::cout << "Got new info!" << std::endl;
      else if ( server_interface.NewCycle() )
      std::cout << "New cycle!" << std::endl;
      //    else
      //      std::cout << "Something wrong..." << std::endl;
      */
  }
  /*
  cout << argc;
  for (int i = 0; i < argc; i++)
    cout << " " << argv[i];
  cout << endl;

  CParameters p;
  CRSParameters rsp;

  cout << p.SP_host << " " << p.SP_hear_max << " " << p.SP_kickable_margin << endl;
  cout << rsp.SP_host << " " << rsp.RS_min_score_dist << " " << rsp.RS_timer_interval << endl;

  // p.ReadCommandLine(argc, argv);
  // cout << p.SP_host << " " << p.SP_hear_max << " " << p.SP_kickable_margin << endl;
  
  rsp.ReadCommandLine(argc, argv);
  cout << rsp.SP_host << " " << rsp.RS_min_score_dist << " " << rsp.RS_timer_interval << endl;
  */

  logf.Close();

  return 0;
}
